@@ -8,8 +8,8 @@
  * You can adjust these values to suit your needs by setting the two variables below.
  * Set them to -1 if you want to completely disable flashing at the end of track.
  **************************/
- var JogFlashWarningTime = 30; // number of seconds to slowly blink at the end of track
- var JogFlashCriticalTime = 15; // number of seconds to quickly blink at the end of track
+const JogFlashWarningTime = 30; // number of seconds to slowly blink at the end of track
+const JogFlashCriticalTime = 15; // number of seconds to quickly blink at the end of track
 
 
 /************************  GPL v2 licence  *****************************
@@ -58,76 +58,73 @@
 ////////////////////////////////////////////////////////////////////////
 // JSHint configuration                                               //
 ////////////////////////////////////////////////////////////////////////
-/* global engine                                                      */
-/* global script                                                      */
 /* global print                                                       */
-/* global midi                                                        */
 ////////////////////////////////////////////////////////////////////////
 
 // Global variables and declarations.
 // ========================================================
 var ReloopBeatmix24 = {};
 
-var RateRangeArray = [0.08, 0.10, 0.12, 0.16];
+const RateRangeArray = [0.08, 0.10, 0.12, 0.16];
 
 // Timers & long press state
-var jogWheelTimers = [];
-var loadButtonTimers = [];
-var loadButtonLongPressed = [];
-var FxModeTimers = [];
-var FxModeLongPressed = [];
+const jogWheelTimers = [];
+const loadButtonTimers = [];
+const loadButtonLongPressed = [];
+const FxModeTimers = [];
+const FxModeLongPressed = [];
 
 // A variable to store previous value of some connectControls
-var previousValue = [];
+const previousValue = [];
 
 // Trax mode
 // 1 for playlist mode
 // 2 for track mode
 // 3 for preview mode
-var traxMode = 2;
+let traxMode = 2;
 
 // Effects mode
 // 1 for single effect mode (1 effect controlled in each EffectUnit)
 // 2 for multi-effect mode (3 effects controlled in each EffectUnit)
 // SHIFT + long press on pitchbend +/- to change mode
-var FxMode = 1; // Single effect mode by default
+let FxMode = 1; // Single effect mode by default
 
 // Jog Led variables
 // set JogFlashWarningTime and JogFlashCriticalTime to -1 to disable jog wheel flash
-var JogRPM = 33.0 + 1/3, // Jog Wheel simulates a 33.3RPM turntable
-    RoundTripTime = 60.0 / JogRPM, // Time in seconds for a complete turn
-    JogLedNumber = 16, // number of leds (sections) on the jog wheel
-    JogBaseLed = 0x3f, // Midino of last led (we count backward to turn in the right side)
-    JogFlashWarningInterval = 400, // number of ms to wait when flashing slowly
-    JogFlashCriticalInterval = 200; // number of ms to wait when flashing quickly
+const JogRPM = 33.0 + 1/3; // Jog Wheel simulates a 33.3RPM turntable
+const RoundTripTime = 60.0 / JogRPM; // Time in seconds for a complete turn
+const JogLedNumber = 16; // number of leds (sections) on the jog wheel
+const JogBaseLed = 0x3f; // Midino of last led (we count backward to turn in the right side)
+const JogFlashWarningInterval = 400; // number of ms to wait when flashing slowly
+const JogFlashCriticalInterval = 200; // number of ms to wait when flashing quickly
 
-var JogLedLit = [];
-var channelPlaying = []; // Keeping track of channel playing
-var JogBlinking = [];
+const JogLedLit = [];
+const channelPlaying = []; // Keeping track of channel playing
+const JogBlinking = [];
 
 // Buttons and Led Variables
-var ON = 0x7F,
-    OFF = 0x00,
-    RED = 0x7F,
-    BLUE = 0x55,
-    VIOLET = 0x2A,
-    SHIFT = 0x40,
-    DOWN = 0x7F,
-    UP = 0x00;
+const ON = 0x7F;
+const OFF = 0x00;
+const RED = 0x7F;
+//const BLUE = 0x55;
+const VIOLET = 0x2A;
+const SHIFT = 0x40;
+const DOWN = 0x7F;
+const UP = 0x00;
 
 // The SysEx message to send to the controller to force the midi controller
 // to send the status of every item on the control surface.
-var ControllerStatusSysex = [0xF0, 0x00, 0x20, 0x7F, 0x03, 0x01, 0xF7];
+const ControllerStatusSysex = [0xF0, 0x00, 0x20, 0x7F, 0x03, 0x01, 0xF7];
 
 // Some useful regex
-var channelRegEx = /\[Channel(\d+)\]/;
-var samplerRegEx = /\[Sampler(\d+)\]/;
+const channelRegEx = /\[Channel(\d+)\]/;
+const samplerRegEx = /\[Sampler(\d+)\]/;
 
 // Initialise and shutdown stuff.
 // ========================================================
 ReloopBeatmix24.TurnLEDsOff = function() {
     // Turn all LEDS off
-    var i, j;
+    let i, j;
     for (i = 0x91; i <= 0x94; i++) { // 4 decks
 
         // Pads
@@ -156,10 +153,10 @@ ReloopBeatmix24.TurnLEDsOff = function() {
 };
 
 ReloopBeatmix24.connectControls = function() {
-    var group;
+    let group;
 
     // Channels controls
-    for (var i = 1; i <= 4; i++) {
+    for (let i = 1; i <= 4; i++) {
         group = "[Channel" + i + "]";
         engine.connectControl(group, "track_samples",
             "ReloopBeatmix24.deckLoaded");
@@ -174,12 +171,12 @@ ReloopBeatmix24.connectControls = function() {
             "group_" + group + "_enable", 0);
         engine.setValue("[EffectRack1_EffectUnit2]",
             "group_" + group + "_enable", 0);
-        channelPlaying[group] = engine.getValue(group, "play") ? true : false;
+        channelPlaying[group] = !!engine.getValue(group, "play");
         JogBlinking[group] = false;
     }
 
     // Samplers controls
-    for (i = 1; i <= 8; i++) {
+    for (let i = 1; i <= 8; i++) {
         group = "[Sampler" + i + "]";
         engine.connectControl(group, "track_samples",
             "ReloopBeatmix24.deckLoaded");
@@ -192,12 +189,15 @@ ReloopBeatmix24.connectControls = function() {
     engine.setValue("[EffectRack1_EffectUnit2]", "group_[Master]_enable", 0);
 };
 
-ReloopBeatmix24.init = function(id, debug) {
+ReloopBeatmix24.init = function(id, _debug) {
     ReloopBeatmix24.id = id;
     ReloopBeatmix24.TurnLEDsOff(); // Turn off all LEDs
+    if (engine.getValue("[App]", "num_samplers") < 8) {
+        engine.setValue("[App]", "num_samplers", 8);
+    }
     ReloopBeatmix24.connectControls(false);
 
-    for (var i = 1; i <= 4; i++) {
+    for (let i = 1; i <= 4; i++) {
         engine.trigger("[Channel" + i + "]", "loop_end_position");
     }
 
@@ -217,8 +217,8 @@ ReloopBeatmix24.shutdown = function() {
 // Button functions.
 // ========================================================
 ReloopBeatmix24.GetNextRange = function(previous) {
-    var len = RateRangeArray.length;
-    var pos = RateRangeArray.indexOf(previous);
+    const len = RateRangeArray.length;
+    const pos = RateRangeArray.indexOf(previous);
     // If 'previous' is not found in the array, pos == -1 and (pos+1) == 0,
     // so this function will return the first element
     return RateRangeArray[(pos + 1) % len];
@@ -226,7 +226,7 @@ ReloopBeatmix24.GetNextRange = function(previous) {
 
 ReloopBeatmix24.Range = function(channel, control, value, status, group) {
     if (value === DOWN) {
-        var oldvalue = engine.getValue(group, "rateRange");
+        const oldvalue = engine.getValue(group, "rateRange");
         engine.setValue(group, "rateRange", ReloopBeatmix24.GetNextRange(
             oldvalue));
         engine.softTakeoverIgnoreNextValue(group, "rate");
@@ -255,70 +255,69 @@ ReloopBeatmix24.PitchSlider = function(channel, control, value, status, group) {
 // ========================================================
 ReloopBeatmix24.traxSelect = function(value, step) {
     switch (traxMode) {
-        case 1: // Playlist mode
-            var i, j;
-            for (i = 0; i < Math.abs(value); i++) {
-                for (j = 0; j < step; j++) {
-                    if (value < 0) {
-                        engine.setValue("[Playlist]", "SelectPrevPlaylist",
-                            true);
-                    } else {
-                        engine.setValue("[Playlist]", "SelectNextPlaylist",
-                            true);
-                    }
+    case 1: // Playlist mode
+        for (let i = 0; i < Math.abs(value); i++) {
+            for (let j = 0; j < step; j++) {
+                if (value < 0) {
+                    engine.setValue("[Playlist]", "SelectPrevPlaylist",
+                        true);
+                } else {
+                    engine.setValue("[Playlist]", "SelectNextPlaylist",
+                        true);
                 }
             }
-            break;
-        case 2: // Track mode
-            engine.setValue("[Playlist]", "SelectTrackKnob", value * step);
-            break;
-        case 3: // Preview mode
-            engine.setValue("[PreviewDeck1]", "playposition", Math.max(0,
-                Math.min(1, engine.getValue("[PreviewDeck1]",
-                    "playposition") + 0.02 * value * step)));
-            break;
+        }
+        break;
+    case 2: // Track mode
+        engine.setValue("[Playlist]", "SelectTrackKnob", value * step);
+        break;
+    case 3: // Preview mode
+        engine.setValue("[PreviewDeck1]", "playposition", Math.max(0,
+            Math.min(1, engine.getValue("[PreviewDeck1]",
+                "playposition") + 0.02 * value * step)));
+        break;
     }
 };
 
-ReloopBeatmix24.TraxTurn = function(channel, control, value, status, group) {
+ReloopBeatmix24.TraxTurn = function(channel, control, value, _status, _group) {
     ReloopBeatmix24.traxSelect(value - 0x40, 1);
 };
 
-ReloopBeatmix24.ShiftTraxTurn = function(channel, control, value, status, group) {
+ReloopBeatmix24.ShiftTraxTurn = function(channel, control, value, _status, _group) {
     ReloopBeatmix24.traxSelect(value - 0x40, 10);
 };
 
-ReloopBeatmix24.TraxPush = function(channel, control, value, status, group) {
+ReloopBeatmix24.TraxPush = function(channel, control, value, _status, _group) {
     switch (traxMode) {
-        case 1: // Playlist mode
-            engine.setValue("[Playlist]", "ToggleSelectedSidebarItem",
-                value);
-            break;
-        case 2: // Track mode
-            engine.setValue("[PreviewDeck1]", "LoadSelectedTrackAndPlay",
-                value);
-            traxMode = 3;
-            break;
-        case 3: // Preview mode
-            if (value == DOWN) {
-                script.toggleControl("[PreviewDeck1]", "play");
-            }
-            break;
+    case 1: // Playlist mode
+        engine.setValue("[Playlist]", "ToggleSelectedSidebarItem",
+            value);
+        break;
+    case 2: // Track mode
+        engine.setValue("[PreviewDeck1]", "LoadSelectedTrackAndPlay",
+            value);
+        traxMode = 3;
+        break;
+    case 3: // Preview mode
+        if (value === DOWN) {
+            script.toggleControl("[PreviewDeck1]", "play");
+        }
+        break;
     }
 };
 
-ReloopBeatmix24.BackButton = function(channel, control, value, status, group) {
+ReloopBeatmix24.BackButton = function(channel, control, value, _status, _group) {
     if (value === DOWN) {
         switch (traxMode) {
-            case 1: // Playlist mode
-                traxMode = 2; // Switch to track mode
-                break;
-            case 2: // Track mode
-                traxMode = 1; // Switch to playlist mode
-                break;
-            case 3: // Preview mode
-                traxMode = 2; // Switch to track mode
-                break;
+        case 1: // Playlist mode
+            traxMode = 2; // Switch to track mode
+            break;
+        case 2: // Track mode
+            traxMode = 1; // Switch to playlist mode
+            break;
+        case 3: // Preview mode
+            traxMode = 2; // Switch to track mode
+            break;
         }
     }
 };
@@ -333,7 +332,7 @@ ReloopBeatmix24.LoadButton = function(channel, control, value, status, group) {
     if (value === DOWN) {
         loadButtonLongPressed[group] = false;
         loadButtonTimers[group] = engine.beginTimer(1000,
-            "ReloopBeatmix24.LoadButtonEject(\"" + group + "\")", true);
+            () => {RegloopBeatmix24.LoadButtonEject(group); }, true);
     } else { // UP
         if (!loadButtonLongPressed[group]) { // Short press
             engine.stopTimer(loadButtonTimers[group]);
@@ -379,8 +378,8 @@ ReloopBeatmix24.ShiftSamplerPad = function(channel, control, value, status,
     }
 };
 
-ReloopBeatmix24.SamplerVol = function(channel, control, value, status, group) {
-    for (var i = 1; i <= engine.getValue("[Master]", "num_samplers"); i++) {
+ReloopBeatmix24.SamplerVol = function(channel, control, value, _status, _group) {
+    for (let i = 1; i <= engine.getValue("[App]", "num_samplers"); i++) {
         engine.setValue("[Sampler" + i + "]", "volume", value / 127.0);
     }
 };
@@ -389,10 +388,10 @@ ReloopBeatmix24.SamplerVol = function(channel, control, value, status, group) {
 // ========================================================
 
 ReloopBeatmix24.WheelTouch = function(channel, control, value, status, group) {
-    var deck = parseInt(group.substr(8, 1), 10);
+    const deck = parseInt(group.substr(8, 1), 10);
     if (value === DOWN) {
-        var alpha = 1.0 / 8;
-        var beta = alpha / 32;
+        const alpha = 1.0 / 8;
+        const beta = alpha / 32;
         engine.scratchEnable(deck, 800, JogRPM, alpha, beta);
     } else {
         engine.scratchDisable(deck);
@@ -400,33 +399,34 @@ ReloopBeatmix24.WheelTouch = function(channel, control, value, status, group) {
 };
 
 ReloopBeatmix24.WheelTurn = function(channel, control, value, status, group) {
-    var newValue = value - 64;
-    var deck = parseInt(group.substr(8, 1), 10);
+    const newValue = value - 64;
+    const deck = parseInt(group.substr(8, 1), 10);
 
     // In either case, register the movement
     if (engine.isScratching(deck)) {
         engine.scratchTick(deck, newValue); // Scratch!
     } else {
-        engine.setValue(group, 'jog', newValue / 5); // Pitch bend
+        engine.setValue(group, "jog", newValue / 5); // Pitch bend
     }
 };
 
 // Led Feedback functions
 // ========================================================
 ReloopBeatmix24.AllJogLEDsToggle = function(deck, state, step) {
-    step = typeof step !== 'undefined' ? step : 1; // default value
-    for (var j = 0x30; j <= 0x3F; j += step) {
+    step = typeof step !== "undefined" ? step : 1; // default value
+    for (let j = 0x30; j <= 0x3F; j += step) {
         midi.sendShortMsg(deck, j, state);
     }
 };
 
-ReloopBeatmix24.deckLoaded = function(value, group, control) {
-    var i;
+ReloopBeatmix24.deckLoaded = function(value, group, _control) {
+    let i;
     switch (group.substr(1, 7)) {
-        case "Channel":
-            var channelChan = parseInt(channelRegEx.exec(group)[1]);
+    case "Channel":
+        {
+            const channelChan = parseInt(channelRegEx.exec(group)[1]);
             if (channelChan <= 4) {
-                // shut down load button
+            // shut down load button
                 midi.sendShortMsg(0x90 + channelChan, 0x50,
                     value ? ON : OFF);
 
@@ -438,19 +438,21 @@ ReloopBeatmix24.deckLoaded = function(value, group, control) {
                     delete JogLedLit[group];
                 }
             }
-            break;
-        case "Sampler":
-            var samplerChan = parseInt(samplerRegEx.exec(group)[1]);
+        }
+        break;
+    case "Sampler":
+        {
+            const samplerChan = parseInt(samplerRegEx.exec(group)[1]);
             if (samplerChan <= 8) { // We only handle 8 samplers (1 per pad)
                 for (i = 0x91; i <= 0x94; i++) {
-                    // PAD1 Mode A
+                // PAD1 Mode A
                     midi.sendShortMsg(i, 0x08 - 1 + samplerChan, value ?
                         RED : OFF);
                     // SHIFT+PAD1 Mode A
                     midi.sendShortMsg(i, 0x48 - 1 + samplerChan, value ?
                         RED : OFF);
                     if (samplerChan <= 4) { // Handle first 4 samplers in split mode
-                        // PAD5 Mode A+B (sampler 1 in split mode)
+                    // PAD5 Mode A+B (sampler 1 in split mode)
                         midi.sendShortMsg(i, 0x14 - 1 + samplerChan, value ?
                             RED : OFF);
                         // SHIFT+PAD5 Mode A+B (sampler 1 in split mode)
@@ -459,14 +461,15 @@ ReloopBeatmix24.deckLoaded = function(value, group, control) {
                     }
                 }
             }
-            break;
+        }
+        break;
     }
 };
 
-ReloopBeatmix24.SamplerPlay = function(value, group, control) {
-    var samplerChan = parseInt(samplerRegEx.exec(group)[1]);
+ReloopBeatmix24.SamplerPlay = function(value, group, _control) {
+    const samplerChan = parseInt(samplerRegEx.exec(group)[1]);
     if (samplerChan <= 8) { // We only handle 8 samplers (1 per pad)
-        var ledColor;
+        let ledColor;
         if (value) {
             ledColor = VIOLET;
         } else {
@@ -475,7 +478,7 @@ ReloopBeatmix24.SamplerPlay = function(value, group, control) {
 
         // We need to switch off pad lights before changing color otherwise
         // VIOLET to RED transition does not work well. (???)
-        for (var i = 0x91; i <= 0x94; i++) {
+        for (let i = 0x91; i <= 0x94; i++) {
             // PAD1 Mode A
             midi.sendShortMsg(i, 0x08 - 1 + samplerChan, OFF);
             midi.sendShortMsg(i, 0x08 - 1 + samplerChan, ledColor);
@@ -494,15 +497,15 @@ ReloopBeatmix24.SamplerPlay = function(value, group, control) {
     }
 };
 
-ReloopBeatmix24.loopDefined = function(value, group, control) {
-    var channelChan = parseInt(channelRegEx.exec(group)[1]);
+ReloopBeatmix24.loopDefined = function(value, group, _control) {
+    const channelChan = parseInt(channelRegEx.exec(group)[1]);
     if (channelChan <= 4) {
         midi.sendShortMsg(0x90 + channelChan, 0x44, value < 0 ? OFF :
             VIOLET);
     }
 };
 
-ReloopBeatmix24.ChannelPlay = function(value, group, control) {
+ReloopBeatmix24.ChannelPlay = function(value, group, _control) {
     // Keep track of the playing state of each channel to avoid
     // calling engine.getValue(group, "play") too often
     if (value) {
@@ -513,7 +516,7 @@ ReloopBeatmix24.ChannelPlay = function(value, group, control) {
             engine.stopTimer(jogWheelTimers[group]);
             delete jogWheelTimers[group];
             JogBlinking[group] = false;
-            var channelChan = parseInt(channelRegEx.exec(group)[1]);
+            const channelChan = parseInt(channelRegEx.exec(group)[1]);
             ReloopBeatmix24.AllJogLEDsToggle(0x90 + channelChan, OFF, 2);
             engine.trigger(group, "playposition"); // light up jog position led
         }
@@ -524,12 +527,11 @@ ReloopBeatmix24.ChannelPlay = function(value, group, control) {
 // This function will light jog led and is connected to playposition,
 // so value here represent the position in the track in the range [-0.14..1.14]
 // (0 = beginning, 1 = end)
-ReloopBeatmix24.JogLed = function(value, group, control) {
-
+ReloopBeatmix24.JogLed = function(value, group, _control) {
     // time computation
-    var trackDuration = engine.getValue(group, "duration");
-    var timeLeft = trackDuration * (1.0 - value);
-    var channelChan = parseInt(channelRegEx.exec(group)[1]);
+    const trackDuration = engine.getValue(group, "duration");
+    const timeLeft = trackDuration * (1.0 - value);
+    const channelChan = parseInt(channelRegEx.exec(group)[1]);
 
     // Start JogWheel blinking if playing and time left < warning time
     if (channelPlaying[group] && timeLeft <= JogFlashWarningTime) {
@@ -547,17 +549,17 @@ ReloopBeatmix24.JogLed = function(value, group, control) {
             jogWheelTimers[group] = engine.beginTimer(
                 timeLeft <= JogFlashCriticalTime ?
                     JogFlashCriticalInterval : JogFlashWarningInterval,
-                "ReloopBeatmix24.jogLedFlash(\"" + group + "\", " + ON + ")",
+                () => {ReloopBeatmix24.jogLedFlash(group, ON); },
                 true);
             JogBlinking[group] = true;
         }
         return;
     }
 
-    var timePosition = trackDuration * value;
-    var rotationNumber = timePosition / RoundTripTime; // number of turn since beginning
-    var positionInCircle = rotationNumber - Math.floor(rotationNumber); // decimal part
-    var ledToLight = Math.round(positionInCircle * JogLedNumber);
+    const timePosition = trackDuration * value;
+    const rotationNumber = timePosition / RoundTripTime; // number of turn since beginning
+    const positionInCircle = rotationNumber - Math.floor(rotationNumber); // decimal part
+    const ledToLight = Math.round(positionInCircle * JogLedNumber);
     if (JogLedLit[group] === ledToLight) { // exit if there is no change
         return;
     }
@@ -572,21 +574,21 @@ ReloopBeatmix24.JogLed = function(value, group, control) {
 };
 
 ReloopBeatmix24.jogLedFlash = function(group, state) {
-    var chan = parseInt(group.substr(8, 1), 10);
+    const chan = parseInt(group.substr(8, 1), 10);
 
     // toggle all jog leds
     ReloopBeatmix24.AllJogLEDsToggle(0x90 + chan, state ? OFF : ON, 2);
 
-    var timeleft = engine.getValue(group, "duration") * (1.0 - engine.getValue(
+    const timeleft = engine.getValue(group, "duration") * (1.0 - engine.getValue(
         group, "playposition"));
 
     if (timeleft < JogFlashWarningTime) {
         // Set timer for leds shut off
-        var nextTime = (timeleft < JogFlashCriticalTime ?
+        const nextTime = (timeleft < JogFlashCriticalTime ?
             JogFlashCriticalInterval : JogFlashWarningInterval);
         jogWheelTimers[group] = engine.beginTimer(nextTime,
-            "ReloopBeatmix24.jogLedFlash(\"" + group + "\", " +
-            (state ? OFF : ON) + ")", true);
+            () => {ReloopBeatmix24.jogLedFlash(group, state ? OFF : ON); },
+             true);
     } else { // Back in time ?
         // shut off all jog leds
         ReloopBeatmix24.AllJogLEDsToggle(0x90 + chan, OFF);
@@ -599,8 +601,8 @@ ReloopBeatmix24.jogLedFlash = function(group, state) {
 // Effects functions
 // ========================================================
 ReloopBeatmix24.FxModeLedFlash = function(step, mode) {
-    var i;
-    var ledValue = (step % 2) ? ON : OFF;
+    let i;
+    const ledValue = (step % 2) ? ON : OFF;
     if (step >= 7) {
         for (i = 1; i <= 4; i++) {
             // engine.trigger should be sufficient, but...
@@ -608,8 +610,8 @@ ReloopBeatmix24.FxModeLedFlash = function(step, mode) {
                 i + "]_enable");
             engine.trigger("[EffectRack1_EffectUnit2]", "group_[Channel" +
                 i + "]_enable");
-            // Workaround for bug #1607277 as engine.trigger doesn't work well
-            var newValue = engine.getValue("[EffectRack1_EffectUnit1]",
+            // Workaround for issue #8620 as engine.trigger doesn't work well
+            let newValue = engine.getValue("[EffectRack1_EffectUnit1]",
                 "group_[Channel" + i + "]_enable");
             midi.sendShortMsg(0x90 + i, 0x25, newValue ? ON : OFF);
             midi.sendShortMsg(0x90 + i, 0x25 + SHIFT, newValue ? ON : OFF);
@@ -623,8 +625,7 @@ ReloopBeatmix24.FxModeLedFlash = function(step, mode) {
             midi.sendShortMsg(i, 0x26 - mode, ledValue);
             midi.sendShortMsg(i, 0x26 + SHIFT - mode, ledValue);
         }
-        engine.beginTimer(150, "ReloopBeatmix24.FxModeLedFlash(" + (step + 1) +
-            ", " + mode + ")", true);
+        engine.beginTimer(150, () => { ReloopBeatmix24.FxModeLedFlash(step + 1, mode); }, true);
     }
 };
 
@@ -633,11 +634,11 @@ ReloopBeatmix24.FxModeCallback = function(group, mode) {
     FxModeLongPressed[group] = true;
     delete FxModeTimers[group];
     // give some visual feedback (blink led 3 times)
-    for (var i = 0x91; i <= 0x94; i++) {
+    for (let i = 0x91; i <= 0x94; i++) {
         midi.sendShortMsg(i, 0x26 - mode, OFF);
         midi.sendShortMsg(i, 0x26 + SHIFT - mode, OFF);
     }
-    engine.beginTimer(150, "ReloopBeatmix24.FxModeLedFlash(1, " + mode + ")",
+    engine.beginTimer(150, () => { ReloopBeatmix24.FxModeLedFlash(1, mode); },
         true);
 };
 
@@ -646,7 +647,7 @@ ReloopBeatmix24.FxModeCallback = function(group, mode) {
 // It is mapped to SHIFT + PITCHBEND+/- (FX1 and FX2)
 ReloopBeatmix24.ActivateFx = function(channel, control, value, status, group) {
     // Calculate Fx num based on midi control (0x66 for Fx1 and 0x67 for Fx2)
-    var FxNum = control - 0x65;
+    const FxNum = control - 0x65;
     if (value === DOWN) {
         if (FxModeTimers[group]) {
             engine.stopTimer(FxModeTimers[group]);
@@ -654,8 +655,7 @@ ReloopBeatmix24.ActivateFx = function(channel, control, value, status, group) {
         }
         FxModeLongPressed[group] = false;
         FxModeTimers[group] = engine.beginTimer(1000,
-            "ReloopBeatmix24.FxModeCallback(\"" + group + "\", " + FxNum + ")",
-            true);
+            () => {ReloopBeatmix24.FxModeCallback(group, FxNum); }, true);
     } else { // UP
         if (FxModeLongPressed[group]) { // long press
             // Nothing to do, this has already been done in callback function
@@ -671,8 +671,8 @@ ReloopBeatmix24.ActivateFx = function(channel, control, value, status, group) {
 };
 
 ReloopBeatmix24.FxKnobTurn = function(channel, control, value, status, group) {
-    if (FxMode == 1) {
-        var parameter = control;
+    if (FxMode === 1) {
+        const parameter = control;
         engine.setParameter(group, "parameter" + parameter.toString(),
             script.absoluteLin(value, 0, 1));
     }
@@ -681,14 +681,14 @@ ReloopBeatmix24.FxKnobTurn = function(channel, control, value, status, group) {
 
 ReloopBeatmix24.ShiftFxKnobTurn = function(channel, control, value, status,
     group) {
-    if (FxMode == 1) {
-        var parameter = 3 + control - SHIFT;
+    if (FxMode === 1) {
+        const parameter = 3 + control - SHIFT;
         engine.setParameter(group, "parameter" + parameter.toString(),
             script.absoluteLin(value, 0, 1));
     } else {
-        var effectUnit = parseInt(group.substr(23, 1), 10);
-        var Effect = control - SHIFT;
-        var storeIndex = "FX" + Effect.toString() + "U" + effectUnit.toString();
+        const effectUnit = parseInt(group.substr(23, 1), 10);
+        const Effect = control - SHIFT;
+        const storeIndex = "FX" + Effect.toString() + "U" + effectUnit.toString();
         if (storeIndex in previousValue) {
             if (value - previousValue[storeIndex] > 5) {
                 engine.setValue("[EffectRack1_EffectUnit" + effectUnit +
@@ -727,16 +727,15 @@ ReloopBeatmix24.ShiftFxKnobOnOff = function(channel, control, value, status, gro
         if (value === UP) {
             engine.setValue(group, "clear", 1);
             engine.beginTimer(100,
-                "ReloopBeatmix24.EffectClearTimerCallBack(\"" + group +
-                "\")", true);
+                () => {ReloopBeatmix24.EffectClearTimerCallBack(group); }, true);
         }
     }
     // Nothing in single-effect mode
 };
 
 ReloopBeatmix24.FxEncoderTurn = function(channel, control, value, status, group) {
-    var newValue = value - 0x40;
-    if (FxMode == 1) {
+    const newValue = value - 0x40;
+    if (FxMode === 1) {
         engine.setValue(group, newValue > 0 ? "mix_up" : "mix_down", 1);
     } else {
         engine.setValue(group, newValue > 0 ? "super1_up" : "super1_down", 1);
@@ -745,8 +744,8 @@ ReloopBeatmix24.FxEncoderTurn = function(channel, control, value, status, group)
 
 ReloopBeatmix24.ShiftFxEncoderTurn = function(channel, control, value, status,
     group) {
-    var newValue = value - 0x40;
-    if (FxMode == 1) {
+    const newValue = value - 0x40;
+    if (FxMode === 1) {
         engine.setValue(group, "chain_selector", newValue);
     } else {
         engine.setValue(group, newValue > 0 ? "mix_up" : "mix_down", 1);
