@@ -7,7 +7,6 @@
 #include "analyzer/analyzer.h"
 #include "analyzer/plugins/analyzerplugin.h"
 #include "preferences/keydetectionsettings.h"
-#include "preferences/usersettings.h"
 #include "track/track_decl.h"
 #include "util/memory.h"
 
@@ -19,7 +18,7 @@ class AnalyzerKey : public Analyzer {
     static QList<mixxx::AnalyzerPluginInfo> availablePlugins();
     static mixxx::AnalyzerPluginInfo defaultPlugin();
 
-    bool initialize(TrackPointer pTrack,
+    bool initialize(const AnalyzerTrack& track,
             mixxx::audio::SampleRate sampleRate,
             SINT frameLength) override;
     bool processSamples(const CSAMPLE* pIn, SINT count) override;
@@ -35,7 +34,7 @@ class AnalyzerKey : public Analyzer {
     KeyDetectionSettings m_keySettings;
     std::unique_ptr<mixxx::AnalyzerKeyPlugin> m_pPlugin;
     QString m_pluginId;
-    int m_sampleRate;
+    mixxx::audio::SampleRate m_sampleRate;
     SINT m_totalFrames;
     SINT m_maxFramesToProcess;
     SINT m_currentFrame;
