@@ -1,26 +1,28 @@
 #pragma once
 
-#include <QAbstractTableModel>
-#include <QVariant>
 #include <QModelIndex>
-#include <QAbstractItemDelegate>
 
 #include "controllers/controllermappingtablemodel.h"
 #include "controllers/midi/midimessage.h"
 
+class QTableView;
+class QAbstractItemDelegate;
+
 /// Table Model for the "Outputs" table view in the preferences dialog.
 ///
-/// This allows editing the output mappings for a MIDI preset.
+/// This allows editing the output mappings for a MIDI mapping.
 class ControllerOutputMappingTableModel : public ControllerMappingTableModel {
     Q_OBJECT
   public:
-    ControllerOutputMappingTableModel(QObject* pParent, ControlPickerMenu* pControlPickerMenu);
+    ControllerOutputMappingTableModel(QObject* pParent,
+            ControlPickerMenu* pControlPickerMenu,
+            QTableView* pTableView);
     ~ControllerOutputMappingTableModel() override;
 
-    // Apply the changes to the loaded preset.
+    // Apply the changes to the loaded mapping.
     void apply();
 
-    // Clears all output mappings in the preset.
+    // Clears all output mappings in the mapping.
     void clear();
 
     // Adds an empty output mapping.
@@ -39,11 +41,12 @@ class ControllerOutputMappingTableModel : public ControllerMappingTableModel {
     int rowCount(const QModelIndex& parent = QModelIndex()) const override;
     int columnCount(const QModelIndex& parent = QModelIndex()) const override;
     QVariant data(const QModelIndex& index, int role = Qt::DisplayRole) const override;
+    QString getDisplayString(const QModelIndex& index) const override;
     bool setData(const QModelIndex& index, const QVariant& value,
                  int role = Qt::EditRole) override;
 
   protected:
-    void onPresetLoaded() override;
+    void onMappingLoaded() override;
 
   private:
     enum MidiColumn {
