@@ -4,6 +4,7 @@
 
 #include "control/controlobject.h"
 #include "controllers/learningutils.h"
+#include "controllers/midi/midicontroller.h"
 #include "controllers/midi/midiutils.h"
 #include "moc_dlgcontrollerlearning.cpp"
 #include "util/versionstore.h"
@@ -20,12 +21,12 @@ DlgControllerLearning::DlgControllerLearning(QWidget* parent,
         ControlPickerMenu* pControlPickerMenu)
         : QDialog(parent),
           m_pController(controller),
-          m_pMidiController(nullptr),
           m_pControlPickerMenu(pControlPickerMenu),
           m_messagesLearned(false) {
     qRegisterMetaType<MidiInputMappings>("MidiInputMappings");
 
     setupUi(this);
+    labelDescription->setWordWrap(true);
     labelMappedTo->setText("");
 
     QString helpTitle(tr("Click anywhere in Mixxx or choose a control to learn"));
@@ -206,9 +207,46 @@ void DlgControllerLearning::slotChooseControlPressed() {
 void DlgControllerLearning::startListening() {
     // Start listening as soon as we're on this page -- that way advanced
     // users don't have to specifically click the "Learn" button.
-    // Get the underlying type of the Controller. This will call
-    // one of the visit() methods below immediately.
-    m_pController->accept(this);
+    // Disconnect everything in both directions so we don't end up with duplicate connections
+    // after pressing the "Learn Another" button
+    MidiController* pMidiController = qobject_cast<MidiController*>(m_pController);
+    VERIFY_OR_DEBUG_ASSERT(pMidiController) {
+        // DlgControllerLearning should have only been created by DlgController if
+        // the Controller was a MidiController.
+        qWarning() << "Only MIDI controllers are supported by the learning wizard.";
+        return;
+    }
+    pMidiController->disconnect(this);
+    this->disconnect(pMidiController);
+
+    connect(pMidiController,
+            &MidiController::messageReceived,
+            this,
+            &DlgControllerLearning::slotMessageReceived);
+
+    connect(this,
+            &DlgControllerLearning::learnTemporaryInputMappings,
+            pMidiController,
+            &MidiController::learnTemporaryInputMappings);
+    connect(this,
+            &DlgControllerLearning::clearTemporaryInputMappings,
+            pMidiController,
+            &MidiController::clearTemporaryInputMappings);
+
+    connect(this,
+            &DlgControllerLearning::commitTemporaryInputMappings,
+            pMidiController,
+            &MidiController::commitTemporaryInputMappings);
+    connect(this,
+            &DlgControllerLearning::startLearning,
+            pMidiController,
+            &MidiController::startLearning);
+    connect(this,
+            &DlgControllerLearning::stopLearning,
+            pMidiController,
+            &MidiController::stopLearning);
+
+    emit startLearning();
     emit listenForClicks();
 }
 
@@ -222,11 +260,11 @@ void DlgControllerLearning::slotStartLearningPressed() {
 
 #ifdef CONTROLLERLESSTESTING
 void DlgControllerLearning::DEBUGFakeMidiMessage() {
-    slotMessageReceived(MIDI_CC, 0x20, 0x41);
+    slotMessageReceived(MidiOpCode::ControlChange, 0x20, 0x41);
 }
 
 void DlgControllerLearning::DEBUGFakeMidiMessage2() {
-    slotMessageReceived(MIDI_CC, 0x20, 0x3F);
+    slotMessageReceived(MidiOpCode::ControlChange, 0x20, 0x3F);
 }
 #endif
 
@@ -245,7 +283,7 @@ void DlgControllerLearning::slotMessageReceived(unsigned char status,
 
     // NOTE(rryan): We intend to use MidiKey(status, control) here rather than
     // setting fields individually since we will use the MidiKey with an input
-    // mapping. See Bug #1532297
+    // mapping. See Issue #8432
     MidiKey key(status, control);
 
     // Ignore all standard MIDI System Real-Time Messages because they
@@ -276,11 +314,11 @@ void DlgControllerLearning::slotMessageReceived(unsigned char status,
     // We got a message, so we can cancel the taking-too-long timeout.
     m_firstMessageTimer.stop();
 
-    // Unless this is a MIDI_CC and the progress bar is full, restart the
+    // Unless this is a MidiOpCode::ControlChange and the progress bar is full, restart the
     // timer.  That way the user won't just push buttons forever and wonder
     // why the wizard never advances.
-    unsigned char opCode = MidiUtils::opCodeFromStatus(status);
-    if (opCode != MIDI_CC || progressBarWiggleFeedback->value() != 10) {
+    MidiOpCode opCode = MidiUtils::opCodeFromStatus(status);
+    if (opCode != MidiOpCode::ControlChange || progressBarWiggleFeedback->value() != 10) {
         m_lastMessageTimer.start();
     }
 }
@@ -326,28 +364,34 @@ void DlgControllerLearning::slotTimerExpired() {
     QString midiControl = "";
     bool first = true;
     foreach (const MidiInputMapping& mapping, m_mappings) {
-        unsigned char opCode = MidiUtils::opCodeFromStatus(mapping.key.status);
+        MidiOpCode opCode = MidiUtils::opCodeFromStatus(mapping.key.status);
         bool twoBytes = MidiUtils::isMessageTwoBytes(opCode);
-        QString mappingStr = twoBytes ? QString("Status: 0x%1 Control: 0x%2 Options: 0x%03")
-                .arg(QString::number(mapping.key.status, 16).toUpper(),
-                     QString::number(mapping.key.control, 16).toUpper()
-                     .rightJustified(2, '0'),
-                     QString::number(mapping.options.all, 16).toUpper()
-                     .rightJustified(2, '0')) :
-                QString("0x%1 0x%2")
-                .arg(QString::number(mapping.key.status, 16).toUpper(),
-                     QString::number(mapping.options.all, 16).toUpper()
-                     .rightJustified(2, '0'));
+        QString mappingStr = twoBytes
+                ? QString("Status: 0x%1 Control: 0x%2 Options: 0x%03")
+                          .arg(QString::number(mapping.key.status, 16)
+                                          .toUpper(),
+                                  QString::number(mapping.key.control, 16)
+                                          .toUpper()
+                                          .rightJustified(2, '0'),
+                                  QString::number(mapping.options, 16)
+                                          .toUpper()
+                                          .rightJustified(2, '0'))
+                : QString("0x%1 0x%2")
+                          .arg(QString::number(mapping.key.status, 16)
+                                          .toUpper(),
+                                  QString::number(mapping.options, 16)
+                                          .toUpper()
+                                          .rightJustified(2, '0'));
 
         // Set the debug string and "Advanced MIDI Options" group using the
         // first mapping.
         if (first) {
             midiControl = mappingStr;
             MidiOptions options = mapping.options;
-            midiOptionInvert->setChecked(options.invert);
-            midiOptionSelectKnob->setChecked(options.selectknob);
-            midiOptionSoftTakeover->setChecked(options.soft_takeover);
-            midiOptionSwitchMode->setChecked(options.sw);
+            midiOptionInvert->setChecked(options.testFlag(MidiOption::Invert));
+            midiOptionSelectKnob->setChecked(options.testFlag(MidiOption::SelectKnob));
+            midiOptionSoftTakeover->setChecked(options.testFlag(MidiOption::SoftTakeover));
+            midiOptionSwitchMode->setChecked(options.testFlag(MidiOption::Switch));
             first = false;
         }
 
@@ -381,10 +425,10 @@ void DlgControllerLearning::slotMidiOptionsChanged() {
     for (MidiInputMappings::iterator it = m_mappings.begin();
          it != m_mappings.end(); ++it) {
         MidiOptions& options = it->options;
-        options.sw = midiOptionSwitchMode->isChecked();
-        options.soft_takeover = midiOptionSoftTakeover->isChecked();
-        options.invert = midiOptionInvert->isChecked();
-        options.selectknob = midiOptionSelectKnob->isChecked();
+        options.setFlag(MidiOption::Switch, midiOptionSwitchMode->isChecked());
+        options.setFlag(MidiOption::SoftTakeover, midiOptionSoftTakeover->isChecked());
+        options.setFlag(MidiOption::Invert, midiOptionInvert->isChecked());
+        options.setFlag(MidiOption::SelectKnob, midiOptionSelectKnob->isChecked());
     }
 
     emit learnTemporaryInputMappings(m_mappings);
@@ -395,54 +439,6 @@ void DlgControllerLearning::commitMapping() {
     emit inputMappingsLearned(m_mappings);
 }
 
-void DlgControllerLearning::visit(MidiController* pMidiController) {
-    // Disconnect everything in both directions so we don't end up with duplicate connections
-    // after pressing the "Learn Another" button
-    pMidiController->disconnect(this);
-    this->disconnect(pMidiController);
-
-    m_pMidiController = pMidiController;
-
-    connect(m_pMidiController,
-            &MidiController::messageReceived,
-            this,
-            &DlgControllerLearning::slotMessageReceived);
-
-    connect(this,
-            &DlgControllerLearning::learnTemporaryInputMappings,
-            m_pMidiController,
-            &MidiController::learnTemporaryInputMappings);
-    connect(this,
-            &DlgControllerLearning::clearTemporaryInputMappings,
-            m_pMidiController,
-            &MidiController::clearTemporaryInputMappings);
-
-    connect(this,
-            &DlgControllerLearning::commitTemporaryInputMappings,
-            m_pMidiController,
-            &MidiController::commitTemporaryInputMappings);
-    connect(this,
-            &DlgControllerLearning::startLearning,
-            m_pMidiController,
-            &MidiController::startLearning);
-    connect(this,
-            &DlgControllerLearning::stopLearning,
-            m_pMidiController,
-            &MidiController::stopLearning);
-
-    emit startLearning();
-}
-
-void DlgControllerLearning::visit(HidController* pHidController) {
-    qWarning() << "ERROR: DlgControllerLearning does not support HID devices.";
-    Q_UNUSED(pHidController);
-}
-
-void DlgControllerLearning::visit(BulkController* pBulkController) {
-    qWarning() << "ERROR: DlgControllerLearning does not support Bulk devices.";
-    Q_UNUSED(pBulkController);
-}
-
 DlgControllerLearning::~DlgControllerLearning() {
     // If the user hit done, we should save any pending mappings.
     if (m_messagesLearned) {
