@@ -7,23 +7,15 @@
 
 #include "controllers/ui_dlgcontrollerlearning.h"
 #include "controllers/controlpickermenu.h"
-#include "controllers/midi/midicontroller.h"
-#ifdef __HID__
-#include "controllers/hid/hidcontroller.h"
-#endif
-#include "controllers/bulk/bulkcontroller.h"
 #include "controllers/midi/midimessage.h"
-#include "controllers/controller.h"
-#include "controllers/controllervisitor.h"
-#include "preferences/usersettings.h"
 
-class ControllerPreset;
+class Controller;
+class ControlObject;
 
 //#define CONTROLLERLESSTESTING
 
 /// The controller mapping learning wizard
 class DlgControllerLearning : public QDialog,
-                              public ControllerVisitor,
                               public Ui::DlgControllerLearning {
     Q_OBJECT
 
@@ -33,10 +25,6 @@ class DlgControllerLearning : public QDialog,
             ControlPickerMenu* pControlPickerMenu);
     virtual ~DlgControllerLearning();
 
-    void visit(MidiController* pController);
-    void visit(HidController* pController);
-    void visit(BulkController* pController);
-
   signals:
     void learnTemporaryInputMappings(const MidiInputMappings& mappings);
     void clearTemporaryInputMappings();
@@ -74,7 +62,7 @@ class DlgControllerLearning : public QDialog,
     void showControlMenu();
 #ifdef CONTROLLERLESSTESTING
     void DEBUGFakeMidiMessage();
-    void DEBUGFakeMidiMessage2();
+    void DEBUGFakeMidiMessage();
 #endif
 
   private:
@@ -85,12 +73,11 @@ class DlgControllerLearning : public QDialog,
     void populateComboBox();
 
     Controller* m_pController;
-    MidiController* m_pMidiController;
     ControlPickerMenu* m_pControlPickerMenu;
     ConfigKey m_currentControl;
     bool m_messagesLearned;
     QTimer m_firstMessageTimer;
     QTimer m_lastMessageTimer;
-    QList<QPair<MidiKey, unsigned char> > m_messages;
+    QList<QPair<MidiKey, unsigned char>> m_messages;
     MidiInputMappings m_mappings;
 };
