@@ -2,7 +2,7 @@
 
 #include <portmidi.h>
 
-#include <QRegExp>
+#include <QRegularExpression>
 
 #include "controllers/midi/portmidicontroller.h"
 #include "moc_portmidienumerator.cpp"
@@ -16,109 +16,61 @@ bool recognizeDevice(const PmDeviceInfo& deviceInfo) {
     // In developer mode we show the MIDI Through Port, otherwise ignore it
     // since it routinely causes trouble.
     return CmdlineArgs::Instance().getDeveloper() ||
-#if QT_VERSION >= QT_VERSION_CHECK(5, 10, 0)
             !QLatin1String(deviceInfo.name)
-#else
-            !QString::fromLatin1(deviceInfo.name)
-#endif
                      .startsWith(kMidiThroughPortPrefix, Qt::CaseInsensitive);
 }
 
-} // namespace
+// Some platforms format MIDI device names as "deviceName MIDI ###" where
+// ### is the instance # of the device. Therefore we want to link two
+// devices that have an equivalent "deviceName" and ### section.
+const QRegularExpression kMidiDeviceNameRegex(QStringLiteral("^(.*) MIDI (\\d+)( .*)?$"));
 
-PortMidiEnumerator::PortMidiEnumerator(UserSettingsPointer pConfig)
-        : m_pConfig(pConfig) {
-    PmError err = Pm_Initialize();
-    // Based on reading the source, it's not possible for this to fail.
-    if (err != pmNoError) {
-        qWarning() << "PortMidi error:" << Pm_GetErrorText(err);
-    }
-}
+const QRegularExpression kInputRegex(QStringLiteral("^(.*) in( \\d+)?( .*)?$"),
+        QRegularExpression::CaseInsensitiveOption);
+const QRegularExpression kOutputRegex(QStringLiteral("^(.*) out( \\d+)?( .*)?$"),
+        QRegularExpression::CaseInsensitiveOption);
 
-PortMidiEnumerator::~PortMidiEnumerator() {
-    qDebug() << "Deleting PortMIDI devices...";
-    QListIterator<Controller*> dev_it(m_devices);
-    while (dev_it.hasNext()) {
-        delete dev_it.next();
-    }
-    PmError err = Pm_Terminate();
-    // Based on reading the source, it's not possible for this to fail.
-    if (err != pmNoError) {
-        qWarning() << "PortMidi error:" << Pm_GetErrorText(err);
+// This is a broad pattern that matches a text blob followed by a numeral
+// potentially followed by non-numeric text. The non-numeric requirement is
+// meant to avoid corner cases around devices with names like "Hercules RMX
+// 2" where we would potentially confuse the number in the device name as
+// the ordinal index of the device.
+const QRegularExpression kDeviceNameRegex(QStringLiteral("^(.*) (\\d+)( [^0-9]+)?$"));
+
+bool namesMatchRegexes(const QRegularExpression& kInputRegex,
+        const QString& input_name,
+        const QRegularExpression& kOutputRegex,
+        const QString& output_name) {
+    QRegularExpressionMatch inputMatch = kInputRegex.match(input_name);
+    if (inputMatch.hasMatch()) {
+        QString inputDeviceName = inputMatch.captured(1);
+        QString inputDeviceIndex = inputMatch.captured(2);
+        QRegularExpressionMatch outputMatch = kOutputRegex.match(output_name);
+        if (outputMatch.hasMatch()) {
+            QString outputDeviceName = outputMatch.captured(1);
+            QString outputDeviceIndex = outputMatch.captured(2);
+            if (outputDeviceName.compare(inputDeviceName, Qt::CaseInsensitive) == 0 &&
+                outputDeviceIndex == inputDeviceIndex) {
+                return true;
+            }
+        }
     }
+    return false;
 }
 
 bool namesMatchMidiPattern(const QString& input_name,
         const QString& output_name) {
-    // Some platforms format MIDI device names as "deviceName MIDI ###" where
-    // ### is the instance # of the device. Therefore we want to link two
-    // devices that have an equivalent "deviceName" and ### section.
-    QRegExp deviceNamePattern("^(.*) MIDI (\\d+)( .*)?$");
-
-    int inputMatch = deviceNamePattern.indexIn(input_name);
-    if (inputMatch == 0) {
-        QString inputDeviceName = deviceNamePattern.cap(1);
-        QString inputDeviceIndex = deviceNamePattern.cap(2);
-        int outputMatch = deviceNamePattern.indexIn(output_name);
-        if (outputMatch == 0) {
-            QString outputDeviceName = deviceNamePattern.cap(1);
-            QString outputDeviceIndex = deviceNamePattern.cap(2);
-            if (outputDeviceName.compare(inputDeviceName, Qt::CaseInsensitive) == 0 &&
-                outputDeviceIndex == inputDeviceIndex) {
-                return true;
-            }
-        }
-    }
-    return false;
+    return namesMatchRegexes(kMidiDeviceNameRegex, input_name, kMidiDeviceNameRegex, output_name);
 }
 
 bool namesMatchInOutPattern(const QString& input_name,
         const QString& output_name) {
-    QString basePattern = "^(.*) %1 (\\d+)( .*)?$";
-    QRegExp inputPattern(basePattern.arg("in"));
-    QRegExp outputPattern(basePattern.arg("out"));
-
-    int inputMatch = inputPattern.indexIn(input_name);
-    if (inputMatch == 0) {
-        QString inputDeviceName = inputPattern.cap(1);
-        QString inputDeviceIndex = inputPattern.cap(2);
-        int outputMatch = outputPattern.indexIn(output_name);
-        if (outputMatch == 0) {
-            QString outputDeviceName = outputPattern.cap(1);
-            QString outputDeviceIndex = outputPattern.cap(2);
-            if (outputDeviceName.compare(inputDeviceName, Qt::CaseInsensitive) == 0 &&
-                outputDeviceIndex == inputDeviceIndex) {
-                return true;
-            }
-        }
-    }
-    return false;
+    return namesMatchRegexes(kInputRegex, input_name, kOutputRegex, output_name);
 }
 
 bool namesMatchPattern(const QString& input_name,
         const QString& output_name) {
-    // This is a broad pattern that matches a text blob followed by a numeral
-    // potentially followed by non-numeric text. The non-numeric requirement is
-    // meant to avoid corner cases around devices with names like "Hercules RMX
-    // 2" where we would potentially confuse the number in the device name as
-    // the ordinal index of the device.
-    QRegExp deviceNamePattern("^(.*) (\\d+)( [^0-9]+)?$");
-
-    int inputMatch = deviceNamePattern.indexIn(input_name);
-    if (inputMatch == 0) {
-        QString inputDeviceName = deviceNamePattern.cap(1);
-        QString inputDeviceIndex = deviceNamePattern.cap(2);
-        int outputMatch = deviceNamePattern.indexIn(output_name);
-        if (outputMatch == 0) {
-            QString outputDeviceName = deviceNamePattern.cap(1);
-            QString outputDeviceIndex = deviceNamePattern.cap(2);
-            if (outputDeviceName.compare(inputDeviceName, Qt::CaseInsensitive) == 0 &&
-                outputDeviceIndex == inputDeviceIndex) {
-                return true;
-            }
-        }
-    }
-    return false;
+    return namesMatchRegexes(kDeviceNameRegex, input_name, kDeviceNameRegex, output_name);
 }
 
 bool namesMatchAllowableEdgeCases(const QString& input_name,
@@ -139,6 +91,29 @@ bool namesMatchAllowableEdgeCases(const QString& input_name,
     return false;
 }
 
+} // namespace
+
+PortMidiEnumerator::PortMidiEnumerator() {
+    PmError err = Pm_Initialize();
+    // Based on reading the source, it's not possible for this to fail.
+    if (err != pmNoError) {
+        qWarning() << "PortMidi error:" << Pm_GetErrorText(err);
+    }
+}
+
+PortMidiEnumerator::~PortMidiEnumerator() {
+    qDebug() << "Deleting PortMIDI devices...";
+    QListIterator<Controller*> dev_it(m_devices);
+    while (dev_it.hasNext()) {
+        delete dev_it.next();
+    }
+    PmError err = Pm_Terminate();
+    // Based on reading the source, it's not possible for this to fail.
+    if (err != pmNoError) {
+        qWarning() << "PortMidi error:" << Pm_GetErrorText(err);
+    }
+}
+
 bool shouldLinkInputToOutput(const QString& input_name,
         const QString& output_name) {
     // Early exit.
@@ -279,8 +254,7 @@ QList<Controller*> PortMidiEnumerator::queryDevices() {
                 new PortMidiController(inputDeviceInfo,
                         outputDeviceInfo,
                         inputDevIndex,
-                        outputDevIndex,
-                        m_pConfig);
+                        outputDevIndex);
         m_devices.push_back(currentDevice);
     }
     return m_devices;
