@@ -1,16 +1,16 @@
 #pragma once
 
 #include <QMutex>
-#include <QSemaphore>
 #include <QString>
-#include <QThread>
-#include <QtDebug>
 
+#include "audio/frame.h"
 #include "engine/cachingreader/cachingreaderchunk.h"
 #include "engine/engineworker.h"
 #include "sources/audiosource.h"
 #include "track/track_decl.h"
-#include "util/fifo.h"
+
+template<class DataType>
+class FIFO;
 
 // POD with trivial ctor/dtor/copy for passing through FIFO
 typedef struct CachingReaderChunkReadRequest {
@@ -147,9 +147,13 @@ class CachingReaderWorker : public EngineWorker {
     ReaderStatusUpdate processReadRequest(
             const CachingReaderChunkReadRequest& request);
 
+    void verifyFirstSound(const CachingReaderChunk* pChunk);
+
     // The current audio source of the track loaded
     mixxx::AudioSourcePointer m_pAudioSource;
 
+    mixxx::audio::FramePos m_firstSoundFrameToVerify;
+
     // Temporary buffer for reading samples from all channels
     // before conversion to a stereo signal.
     mixxx::SampleBuffer m_tempReadBuffer;
