@@ -1,6 +1,16 @@
 #pragma once
+
+#include <QHash>
+#include <QString>
+#include <QVarLengthArray>
+#include <QtDebug>
+#include <memory>
+
+#include "util/assert.h"
+#include "util/compatibility/qhash.h"
+
 // ChannelHandle defines a unique identifier for channels of audio in the engine
-// (e.g. headphone output, master output, deck 1, microphone 3). Previously we
+// (e.g. headphone output, main output, deck 1, microphone 3). Previously we
 // used the group string of the channel in the engine to uniquely identify it
 // and key associative containers (e.g. QMap, QHash) but the downside to this is
 // that we waste a lot of callback time hashing and re-hashing the strings.
@@ -11,21 +21,13 @@
 // and equality of ChannelHandles are simple to calculate and a QVarLengthArray
 // can be used to create a fast associative container backed by a simple array
 // (since the keys are numbered [0, num_channels]).
-//
-// A helper class, ChannelHandleFactory, keeps a running count of handles that
-// have been assigned.
 
-#include <QHash>
-#include <QString>
-#include <QVarLengthArray>
-#include <QtDebug>
-#include <memory>
-
-#include "util/assert.h"
-
-// A wrapper around an integer handle. Used to uniquely identify and refer to
-// channels (headphone output, master output, deck 1, microphone 4, etc.) of
-// audio in the engine.
+/// A wrapper around an integer handle. Used to uniquely identify and refer to
+/// channels (headphone output, main output, deck 1, microphone 4, etc.) while
+/// avoiding slow QString comparisons incurred when using the group.
+///
+/// A helper class, ChannelHandleFactory, keeps a running count of handles that
+/// have been assigned.
 class ChannelHandle {
   public:
     ChannelHandle() : m_iHandle(-1) {
@@ -39,6 +41,10 @@ class ChannelHandle {
         return m_iHandle;
     }
 
+    operator int() const {
+        return m_iHandle;
+    }
+
   private:
     ChannelHandle(int iHandle)
             : m_iHandle(iHandle) {
@@ -53,10 +59,22 @@ class ChannelHandle {
     friend class ChannelHandleFactory;
 };
 
+inline bool operator>(const ChannelHandle& h1, const ChannelHandle& h2) {
+    return h1.handle() > h2.handle();
+}
+
+inline bool operator<(const ChannelHandle& h1, const ChannelHandle& h2) {
+    return h1.handle() < h2.handle();
+}
+
 inline bool operator==(const ChannelHandle& h1, const ChannelHandle& h2) {
     return h1.handle() == h2.handle();
 }
 
+inline bool operator==(const int& i, const ChannelHandle& h2) {
+    return i == h2.handle();
+}
+
 inline bool operator!=(const ChannelHandle& h1, const ChannelHandle& h2) {
     return h1.handle() != h2.handle();
 }
@@ -66,8 +84,10 @@ inline QDebug operator<<(QDebug stream, const ChannelHandle& h) {
     return stream;
 }
 
-inline uint qHash(const ChannelHandle& handle) {
-    return qHash(handle.handle());
+inline qhash_seed_t qHash(
+        const ChannelHandle& handle,
+        qhash_seed_t seed = 0) {
+    return qHash(handle.handle(), seed);
 }
 
 // Convenience class that mimics QPair<ChannelHandle, QString> except with
@@ -83,7 +103,7 @@ class ChannelHandleAndGroup {
         return m_name;
     }
 
-    inline const ChannelHandle& handle() const {
+    inline ChannelHandle handle() const {
         return m_handle;
     }
 
@@ -104,15 +124,17 @@ inline QDebug operator<<(QDebug stream, const ChannelHandleAndGroup& g) {
     return stream;
 }
 
-inline uint qHash(const ChannelHandleAndGroup& handle_group) {
-    return qHash(handle_group.handle());
+inline qhash_seed_t qHash(
+        const ChannelHandleAndGroup& handleGroup,
+        qhash_seed_t seed = 0) {
+    return qHash(handleGroup.handle(), seed);
 }
 
-// A helper class used by EngineMaster to assign ChannelHandles to channel group
+// A helper class used by EngineMixer to assign ChannelHandles to channel group
 // strings. Warning: ChannelHandles produced by different ChannelHandleFactory
 // objects are not compatible and will produce incorrect results when compared,
 // stored in the same container, etc. In practice we only use one instance in
-// EngineMaster.
+// EngineMixer.
 class ChannelHandleFactory {
   public:
     ChannelHandleFactory() : m_iNextHandle(0) {
@@ -153,7 +175,7 @@ typedef std::shared_ptr<ChannelHandleFactory> ChannelHandleFactoryPointer;
 // integer value.
 template <class T>
 class ChannelHandleMap {
-    static const int kMaxExpectedGroups = 256;
+    static constexpr int kMaxExpectedGroups = 256;
     typedef QVarLengthArray<T, kMaxExpectedGroups> container_type;
   public:
     typedef typename QVarLengthArray<T, kMaxExpectedGroups>::const_iterator const_iterator;
@@ -167,7 +189,7 @@ class ChannelHandleMap {
         if (!handle.valid()) {
             return m_dummy;
         }
-        return m_data.at(handle.handle());
+        return m_data.at(handle);
     }
 
     void insert(const ChannelHandle& handle, const T& value) {
@@ -175,24 +197,30 @@ class ChannelHandleMap {
             return;
         }
 
-        int iHandle = handle.handle();
-        maybeExpand(iHandle + 1);
-        m_data[iHandle] = value;
+        maybeExpand(static_cast<int>(handle) + 1);
+        m_data[handle] = value;
     }
 
     T& operator[](const ChannelHandle& handle) {
         if (!handle.valid()) {
             return m_dummy;
         }
-        int iHandle = handle.handle();
-        maybeExpand(iHandle + 1);
-        return m_data[iHandle];
+        maybeExpand(static_cast<int>(handle) + 1);
+        return m_data[handle];
     }
 
     void clear() {
         m_data.clear();
     }
 
+    int size() const {
+        return m_data.size();
+    }
+
+    bool isEmpty() {
+        return m_data.isEmpty();
+    }
+
     typename container_type::iterator begin() {
         return m_data.begin();
     }
