@@ -4,10 +4,10 @@
 #include "util/defs.h"
 #include "util/sample.h"
 
-EngineEffectChain::EngineEffectChain(const QString& id,
-                                     const QSet<ChannelHandleAndGroup>& registeredInputChannels,
-                                     const QSet<ChannelHandleAndGroup>& registeredOutputChannels)
-        : m_id(id),
+EngineEffectChain::EngineEffectChain(const QString& group,
+        const QSet<ChannelHandleAndGroup>& registeredInputChannels,
+        const QSet<ChannelHandleAndGroup>& registeredOutputChannels)
+        : m_group(group),
           m_enableState(EffectEnableState::Enabled),
           m_mixMode(EffectChainMixMode::DrySlashWet),
           m_dMix(0),
@@ -46,7 +46,7 @@ bool EngineEffectChain::addEffect(EngineEffect* pEffect, int iIndex) {
     }
 
     while (iIndex >= m_effects.size()) {
-        m_effects.append(NULL);
+        m_effects.append(nullptr);
     }
     m_effects.replace(iIndex, pEffect);
     return true;
@@ -69,7 +69,7 @@ bool EngineEffectChain::removeEffect(EngineEffect* pEffect, int iIndex) {
         return false;
     }
 
-    m_effects.replace(iIndex, NULL);
+    m_effects.replace(iIndex, nullptr);
     return true;
 }
 
@@ -88,150 +88,97 @@ bool EngineEffectChain::updateParameters(const EffectsRequest& message) {
 }
 
 bool EngineEffectChain::processEffectsRequest(EffectsRequest& message,
-                                              EffectsResponsePipe* pResponsePipe) {
+        EffectsResponsePipe* pResponsePipe) {
     EffectsResponse response(message);
     switch (message.type) {
-        case EffectsRequest::ADD_EFFECT_TO_CHAIN:
-            if (kEffectDebugOutput) {
-                qDebug() << debugString() << this << "ADD_EFFECT_TO_CHAIN"
-                         << message.AddEffectToChain.pEffect
-                         << message.AddEffectToChain.iIndex;
-            }
-            response.success = addEffect(message.AddEffectToChain.pEffect,
-                                         message.AddEffectToChain.iIndex);
-            break;
-        case EffectsRequest::REMOVE_EFFECT_FROM_CHAIN:
-            if (kEffectDebugOutput) {
-                qDebug() << debugString() << this << "REMOVE_EFFECT_FROM_CHAIN"
-                         << message.RemoveEffectFromChain.pEffect
-                         << message.RemoveEffectFromChain.iIndex;
-            }
-            response.success = removeEffect(message.RemoveEffectFromChain.pEffect,
-                                            message.RemoveEffectFromChain.iIndex);
-            break;
-        case EffectsRequest::SET_EFFECT_CHAIN_PARAMETERS:
-            if (kEffectDebugOutput) {
-                qDebug() << debugString() << this << "SET_EFFECT_CHAIN_PARAMETERS"
-                         << "enabled" << message.SetEffectChainParameters.enabled
-                         << "mix" << message.SetEffectChainParameters.mix;
-            }
-            response.success = updateParameters(message);
-            break;
-        case EffectsRequest::ENABLE_EFFECT_CHAIN_FOR_INPUT_CHANNEL:
-            if (kEffectDebugOutput) {
-                qDebug() << debugString() << this
-                         << "ENABLE_EFFECT_CHAIN_FOR_INPUT_CHANNEL"
-                         << message.pTargetChain
-                         << *message.EnableInputChannelForChain.pChannelHandle;
-            }
-            response.success = enableForInputChannel(
-                  message.EnableInputChannelForChain.pChannelHandle,
-                  message.EnableInputChannelForChain.pEffectStatesMapArray);
-            break;
-        case EffectsRequest::DISABLE_EFFECT_CHAIN_FOR_INPUT_CHANNEL:
-            if (kEffectDebugOutput) {
-                qDebug() << debugString() << this
-                         << "DISABLE_EFFECT_CHAIN_FOR_INPUT_CHANNEL"
-                         << message.pTargetChain
-                         << *message.DisableInputChannelForChain.pChannelHandle;
-            }
-            response.success = disableForInputChannel(
-                    message.DisableInputChannelForChain.pChannelHandle);
-            break;
-        default:
-            return false;
+    case EffectsRequest::ADD_EFFECT_TO_CHAIN:
+        if (kEffectDebugOutput) {
+            qDebug() << debugString() << this << "ADD_EFFECT_TO_CHAIN"
+                     << message.AddEffectToChain.pEffect
+                     << message.AddEffectToChain.iIndex;
+        }
+        response.success = addEffect(message.AddEffectToChain.pEffect,
+                message.AddEffectToChain.iIndex);
+        break;
+    case EffectsRequest::REMOVE_EFFECT_FROM_CHAIN:
+        if (kEffectDebugOutput) {
+            qDebug() << debugString() << this << "REMOVE_EFFECT_FROM_CHAIN"
+                     << message.RemoveEffectFromChain.pEffect
+                     << message.RemoveEffectFromChain.iIndex;
+        }
+        response.success = removeEffect(message.RemoveEffectFromChain.pEffect,
+                message.RemoveEffectFromChain.iIndex);
+        break;
+    case EffectsRequest::SET_EFFECT_CHAIN_PARAMETERS:
+        if (kEffectDebugOutput) {
+            qDebug() << debugString() << this << "SET_EFFECT_CHAIN_PARAMETERS"
+                     << "enabled" << message.SetEffectChainParameters.enabled
+                     << "mix" << message.SetEffectChainParameters.mix
+                     << "mix_mode" << static_cast<int>(message.SetEffectChainParameters.mix_mode);
+        }
+        response.success = updateParameters(message);
+        break;
+    case EffectsRequest::ENABLE_EFFECT_CHAIN_FOR_INPUT_CHANNEL:
+        if (kEffectDebugOutput) {
+            qDebug() << debugString() << this
+                     << "ENABLE_EFFECT_CHAIN_FOR_INPUT_CHANNEL"
+                     << message.pTargetChain
+                     << message.EnableInputChannelForChain.channelHandle;
+        }
+        response.success = enableForInputChannel(
+                message.EnableInputChannelForChain.channelHandle);
+        break;
+    case EffectsRequest::DISABLE_EFFECT_CHAIN_FOR_INPUT_CHANNEL:
+        if (kEffectDebugOutput) {
+            qDebug() << debugString() << this
+                     << "DISABLE_EFFECT_CHAIN_FOR_INPUT_CHANNEL"
+                     << message.pTargetChain
+                     << message.DisableInputChannelForChain.channelHandle;
+        }
+        response.success = disableForInputChannel(
+                message.DisableInputChannelForChain.channelHandle);
+        break;
+    default:
+        return false;
     }
     pResponsePipe->writeMessage(response);
     return true;
 }
 
-bool EngineEffectChain::enableForInputChannel(const ChannelHandle* inputHandle,
-        EffectStatesMapArray* statesForEffectsInChain) {
+bool EngineEffectChain::enableForInputChannel(ChannelHandle inputHandle) {
     if (kEffectDebugOutput) {
         qDebug() << "EngineEffectChain::enableForInputChannel" << this << inputHandle;
     }
-    auto& outputMap = m_chainStatusForChannelMatrix[*inputHandle];
+    auto& outputMap = m_chainStatusForChannelMatrix[inputHandle];
     for (auto&& outputChannelStatus : outputMap) {
-        VERIFY_OR_DEBUG_ASSERT(outputChannelStatus.enableState !=
-                EffectEnableState::Enabled) {
-            for (auto&& pStatesMap : *statesForEffectsInChain) {
-                for (auto&& pState : pStatesMap) {
-                    delete pState;
-                }
-            }
-            return false;
-        }
+        DEBUG_ASSERT(outputChannelStatus.enableState != EffectEnableState::Enabled);
         outputChannelStatus.enableState = EffectEnableState::Enabling;
     }
-    for (int i = 0; i < m_effects.size(); ++i) {
-        if (m_effects[i] != nullptr) {
-            if (kEffectDebugOutput) {
-                qDebug() << "EngineEffectChain::enableForInputChannel" << this
-                         << "loading states for effect" << i;
-            }
-            EffectStatesMap* pStatesMap = &(*statesForEffectsInChain)[i];
-            VERIFY_OR_DEBUG_ASSERT(pStatesMap) {
-                return false;
-            }
-            m_effects[i]->loadStatesForInputChannel(inputHandle, pStatesMap);
-        }
-    }
     return true;
 }
 
-bool EngineEffectChain::disableForInputChannel(const ChannelHandle* inputHandle) {
-    auto& outputMap = m_chainStatusForChannelMatrix[*inputHandle];
+bool EngineEffectChain::disableForInputChannel(ChannelHandle inputHandle) {
+    auto& outputMap = m_chainStatusForChannelMatrix[inputHandle];
     for (auto&& outputChannelStatus : outputMap) {
-        if (outputChannelStatus.enableState != EffectEnableState::Disabled) {
+        if (outputChannelStatus.enableState == EffectEnableState::Enabling) {
+            // Channel has never been processed and can be disabled immediately
+            outputChannelStatus.enableState = EffectEnableState::Disabled;
+        } else if (outputChannelStatus.enableState == EffectEnableState::Enabled) {
+            // Channel was enabled, fade effect out via Disabling state
             outputChannelStatus.enableState = EffectEnableState::Disabling;
         }
     }
-    // Do not call deleteStatesForInputChannel here because the EngineEffects'
-    // process() method needs to run one last time before deleting the states.
-    // deleteStatesForInputChannel needs to be called from the main thread after
-    // the successful EffectsResponse is returned by the MessagePipe FIFO.
     return true;
 }
 
-// Called from the main thread for garbage collection after an input channel is disabled
-void EngineEffectChain::deleteStatesForInputChannel(const ChannelHandle* inputChannel) {
-    // If an output channel is not presently being processed, for example when
-    // PFL is not active, then process() cannot be relied upon to set this
-    // chain's EffectEnableState from Disabling to Disabled. This must be done
-    // before the next time process() is called for that output channel,
-    // otherwise, if any EngineEffects are Enabled,
-    // EffectProcessorImpl::processChannel will try to run
-    // with an EffectState that has already been deleted and cause a crash.
-    // Refer to https://bugs.launchpad.net/mixxx/+bug/1741213
-    // NOTE: ChannelHandleMap is like a map in that it associates an object with
-    // a ChannelHandle key, but it actually backed by a QVarLengthArray, not a
-    // QMap. So it is okay that m_chainStatusForChannelMatrix may be
-    // accessed concurrently in the audio engine thread in process(),
-    // enableForInputChannel(), or disableForInputChannel().
-    auto& outputMap = m_chainStatusForChannelMatrix[*inputChannel];
-    for (auto&& outputChannelStatus : outputMap) {
-        outputChannelStatus.enableState = EffectEnableState::Disabled;
-    }
-    for (EngineEffect* pEffect : qAsConst(m_effects)) {
-        if (pEffect != nullptr) {
-            pEffect->deleteStatesForInputChannel(inputChannel);
-        }
-    }
-}
-
-EngineEffectChain::ChannelStatus& EngineEffectChain::getChannelStatus(
-        const ChannelHandle& inputHandle,
-        const ChannelHandle& outputHandle) {
-    ChannelStatus& status = m_chainStatusForChannelMatrix[inputHandle][outputHandle];
-    return status;
-}
-
 bool EngineEffectChain::process(const ChannelHandle& inputHandle,
-                                const ChannelHandle& outputHandle,
-                                CSAMPLE* pIn, CSAMPLE* pOut,
-                                const unsigned int numSamples,
-                                const unsigned int sampleRate,
-                                const GroupFeatureState& groupFeatures) {
+        const ChannelHandle& outputHandle,
+        CSAMPLE* pIn,
+        CSAMPLE* pOut,
+        const unsigned int numSamples,
+        const unsigned int sampleRate,
+        const GroupFeatureState& groupFeatures,
+        bool fadeout) {
     // Compute the effective enable state from the channel input routing switch and
     // the chain's enable state. When either of these are turned on/off, send the
     // effects the intermediate enabling/disabling signal.
@@ -243,6 +190,13 @@ bool EngineEffectChain::process(const ChannelHandle& inputHandle,
     ChannelStatus& channelStatus = m_chainStatusForChannelMatrix[inputHandle][outputHandle];
     EffectEnableState effectiveChainEnableState = channelStatus.enableState;
 
+    if (fadeout && channelStatus.enableState == EffectEnableState::Enabled) {
+        // This is the last callback before pause
+        // It can start again without further notice
+        // make use the effect is paused
+        effectiveChainEnableState = EffectEnableState::Disabling;
+    }
+
     // If the channel is fully disabled, do not let intermediate
     // enabling/disabling signals from the chain's enable switch override
     // the channel's state.
@@ -263,9 +217,10 @@ bool EngineEffectChain::process(const ChannelHandle& inputHandle,
         // requires that the input buffer does not get modified.
         CSAMPLE* pIntermediateInput = pIn;
         CSAMPLE* pIntermediateOutput;
+        SINT effectChainGroupDelayFrames = 0;
         bool firstAddDryToWetEffectProcessed = false;
 
-        for (EngineEffect* pEffect : qAsConst(m_effects)) {
+        for (EngineEffect* pEffect : std::as_const(m_effects)) {
             if (pEffect != nullptr) {
                 // Select an unused intermediate buffer for the next output
                 if (pIntermediateInput == m_buffer1.data()) {
@@ -274,10 +229,14 @@ bool EngineEffectChain::process(const ChannelHandle& inputHandle,
                     pIntermediateOutput = m_buffer1.data();
                 }
 
-                if (pEffect->process(inputHandle, outputHandle,
-                                     pIntermediateInput, pIntermediateOutput,
-                                     numSamples, sampleRate,
-                                     effectiveChainEnableState, groupFeatures)) {
+                if (pEffect->process(inputHandle,
+                            outputHandle,
+                            pIntermediateInput,
+                            pIntermediateOutput,
+                            numSamples,
+                            sampleRate,
+                            effectiveChainEnableState,
+                            groupFeatures)) {
                     if (pEffect->getManifest()->addDryToWet()) {
                         // Skip adding the dry signal to the effect's wet output
                         // when it is the first addDryToWet type effect in
@@ -289,8 +248,8 @@ bool EngineEffectChain::process(const ChannelHandle& inputHandle,
                         // with the input to Echo. The dry signal that entered
                         // the first effect in the chain will be mixed back in
                         // below after all effects in the chain have been processed.
-                        bool skipAddingDry = !firstAddDryToWetEffectProcessed
-                                && m_mixMode == EffectChainMixMode::DryPlusWet;
+                        bool skipAddingDry = !firstAddDryToWetEffectProcessed &&
+                                m_mixMode == EffectChainMixMode::DryPlusWet;
 
                         if (!skipAddingDry) {
                             for (SINT i = 0; i <= static_cast<SINT>(numSamples); ++i) {
@@ -302,12 +261,17 @@ bool EngineEffectChain::process(const ChannelHandle& inputHandle,
                     }
 
                     processingOccured = true;
+                    effectChainGroupDelayFrames += pEffect->getGroupDelayFrames();
+
                     // Output of this effect becomes the input of the next effect
                     pIntermediateInput = pIntermediateOutput;
                 }
             }
         }
 
+        m_effectsDelay.setDelayFrames(effectChainGroupDelayFrames);
+        m_effectsDelay.process(pIn, numSamples);
+
         if (processingOccured) {
             // pIntermediateInput is the output of the last processed effect. It would be the
             // intermediate input of the next effect if there was one.
@@ -343,11 +307,15 @@ bool EngineEffectChain::process(const ChannelHandle& inputHandle,
     // enabling/disabling state, set the channel state or chain state
     // to the fully enabled/disabled state for the next engine callback.
 
-    EffectEnableState& chainOnChannelEnableState = channelStatus.enableState;
-    if (chainOnChannelEnableState == EffectEnableState::Disabling) {
-        chainOnChannelEnableState = EffectEnableState::Disabled;
-    } else if (chainOnChannelEnableState == EffectEnableState::Enabling) {
-        chainOnChannelEnableState = EffectEnableState::Enabled;
+    if (channelStatus.enableState == EffectEnableState::Disabling) {
+        channelStatus.enableState = EffectEnableState::Disabled;
+    } else if (channelStatus.enableState == EffectEnableState::Enabling) {
+        channelStatus.enableState = EffectEnableState::Enabled;
+    }
+
+    if (fadeout && channelStatus.enableState == EffectEnableState::Enabled) {
+        // Effect is paused now, ramp up next callback which may happen later
+        channelStatus.enableState = EffectEnableState::Enabling;
     }
 
     if (m_enableState == EffectEnableState::Disabling) {
