@@ -1,15 +1,13 @@
 #pragma once
 
+#include "control/controlobject.h"
+#include "control/pollingcontrolproxy.h"
 #include "engine/engineobject.h"
 
-class ControlPotmeter;
-class ControlProxy;
-
 class EngineVuMeter : public EngineObject {
     Q_OBJECT
   public:
-    EngineVuMeter(const QString& group);
-    virtual ~EngineVuMeter();
+    EngineVuMeter(const QString& group, const QString& legacyGroup = QString());
 
     virtual void process(CSAMPLE* pInOut, const int iBufferSize);
 
@@ -18,20 +16,20 @@ class EngineVuMeter : public EngineObject {
   private:
     void doSmooth(CSAMPLE &currentVolume, CSAMPLE newVolume);
 
-    ControlPotmeter* m_ctrlVuMeter;
-    ControlPotmeter* m_ctrlVuMeterL;
-    ControlPotmeter* m_ctrlVuMeterR;
+    ControlObject m_vuMeter;
+    ControlObject m_vuMeterLeft;
+    ControlObject m_vuMeterRight;
     CSAMPLE m_fRMSvolumeL;
     CSAMPLE m_fRMSvolumeSumL;
     CSAMPLE m_fRMSvolumeR;
     CSAMPLE m_fRMSvolumeSumR;
     int m_iSamplesCalculated;
 
-    ControlPotmeter* m_ctrlPeakIndicator;
-    ControlPotmeter* m_ctrlPeakIndicatorL;
-    ControlPotmeter* m_ctrlPeakIndicatorR;
+    ControlObject m_peakIndicator;
+    ControlObject m_peakIndicatorLeft;
+    ControlObject m_peakIndicatorRight;
     int m_peakDurationL;
     int m_peakDurationR;
 
-    ControlProxy* m_pSampleRate;
+    PollingControlProxy m_sampleRate;
 };
