@@ -1,3 +1,7 @@
+#include "engine/sidechain/shoutconnection.h"
+
+#include <QRegularExpression>
+#include <QTextCodec>
 #include <QUrl>
 
 // These includes are only required by ignoreSigpipe, which is unix-only
@@ -16,27 +20,30 @@
 #endif
 
 #include "broadcast/defs_broadcast.h"
-#include "control/controlpushbutton.h"
 #include "encoder/encoder.h"
 #include "encoder/encoderbroadcastsettings.h"
 #ifdef __OPUS__
 #include "encoder/encoderopus.h"
 #endif
-#include "engine/sidechain/shoutconnection.h"
+#include "errordialoghandler.h"
 #include "mixer/playerinfo.h"
 #include "moc_shoutconnection.cpp"
 #include "preferences/usersettings.h"
 #include "recording/defs_recording.h"
 #include "track/track.h"
+#include "util/compatibility/qatomic.h"
 #include "util/logger.h"
 
 namespace {
 
-const int kConnectRetries = 30;
-const int kMaxNetworkCache = 491520;  // 10 s mp3 @ 192 kbit/s
+constexpr int kConnectRetries = 30;
+constexpr int kMaxNetworkCache = 491520; // 10 s mp3 @ 192 kbit/s
 // Shoutcast default receive buffer 1048576 and autodumpsourcetime 30 s
 // http://wiki.shoutcast.com/wiki/SHOUTcast_DNAS_Server_2
-const int kMaxShoutFailures = 3;
+constexpr int kMaxShoutFailures = 3;
+
+const QRegularExpression kArtistOrTitleRegex(QStringLiteral("\\$artist|\\$title"));
+const QRegularExpression kArtistRegex(QStringLiteral("\\$artist"));
 
 const mixxx::Logger kLogger("ShoutConnection");
 
@@ -54,8 +61,8 @@ ShoutConnection::ShoutConnection(BroadcastProfilePtr profile,
           m_pConfig(pConfig),
           m_pProfile(profile),
           m_encoder(nullptr),
-          m_pMasterSamplerate(new ControlProxy("[Master]", "samplerate", this)),
-          m_pBroadcastEnabled(new ControlProxy(BROADCAST_PREF_KEY, "enabled", this)),
+          m_mainSamplerate(QStringLiteral("[App]"), QStringLiteral("samplerate")),
+          m_broadcastEnabled(BROADCAST_PREF_KEY, "enabled"),
           m_custom_metadata(false),
           m_firstCall(false),
           m_format_is_mp3(false),
@@ -96,7 +103,7 @@ ShoutConnection::ShoutConnection(BroadcastProfilePtr profile,
 #ifdef SHOUT_TLS
     // Libshout defaults to SHOUT_TLS_AUTO if build with SHOUT_TLS
     // Sometimes autodetection fails, resulting into no metadata send
-    // https://bugs.launchpad.net/mixxx/+bug/1817395
+    // https://github.com/mixxxdj/mixxx/issues/9599
     if (shout_set_tls(m_pShout, SHOUT_TLS_DISABLED) != SHOUTERR_SUCCESS) {
         errorDialog(tr("Error setting tls mode:"),
                 shout_get_error(m_pShout));
@@ -105,8 +112,6 @@ ShoutConnection::ShoutConnection(BroadcastProfilePtr profile,
 }
 
 ShoutConnection::~ShoutConnection() {
-    delete m_pMasterSamplerate;
-
     if (m_pShoutMetaData) {
         shout_metadata_free(m_pShoutMetaData);
     }
@@ -140,7 +145,7 @@ bool ShoutConnection::isConnected() {
 // Only called when applying settings while broadcasting is active
 void ShoutConnection::applySettings() {
     // Do nothing if profile or Live Broadcasting is disabled
-    if (!m_pBroadcastEnabled->toBool() || !m_pProfile->getEnabled()) {
+    if (!m_broadcastEnabled.toBool() || !m_pProfile->getEnabled()) {
         return;
     }
 
@@ -394,23 +399,23 @@ void ShoutConnection::updateFromPreferences() {
         qWarning() << "Error: unknown bit rate:" << iBitrate;
     }
 
-    auto masterSamplerate = mixxx::audio::SampleRate::fromDouble(m_pMasterSamplerate->get());
-    VERIFY_OR_DEBUG_ASSERT(masterSamplerate.isValid()) {
-        qWarning() << "Invalid sample rate!" << masterSamplerate;
+    auto mainSamplerate = mixxx::audio::SampleRate::fromDouble(m_mainSamplerate.get());
+    VERIFY_OR_DEBUG_ASSERT(mainSamplerate.isValid()) {
+        qWarning() << "Invalid sample rate!" << mainSamplerate;
         return;
     }
 
-    if (m_format_is_ov && masterSamplerate == 96000) {
+    if (m_format_is_ov && mainSamplerate == 96000) {
         errorDialog(tr("Broadcasting at 96 kHz with Ogg Vorbis is not currently "
                        "supported. Please try a different sample rate or switch "
                        "to a different encoding."),
-                    tr("See https://bugs.launchpad.net/mixxx/+bug/686212 for more "
+                    tr("See https://github.com/mixxxdj/mixxx/issues/5701 for more "
                        "information."));
         return;
     }
 
 #ifdef __OPUS__
-    if (m_format_is_opus && masterSamplerate != EncoderOpus::getMasterSamplerate()) {
+    if (m_format_is_opus && mainSamplerate != EncoderOpus::getMainSampleRate()) {
         errorDialog(
             EncoderOpus::getInvalidSamplerateMessage(),
             tr("Unsupported sample rate")
@@ -461,7 +466,7 @@ void ShoutConnection::updateFromPreferences() {
     QString userErrorMsg;
     int ret = -1;
     if (m_encoder) {
-        ret = m_encoder->initEncoder(static_cast<int>(masterSamplerate), &userErrorMsg);
+        ret = m_encoder->initEncoder(mainSamplerate, &userErrorMsg);
     }
 
     // TODO(XXX): Use mixxx::audio::SampleRate instead of int in initEncoder
@@ -848,13 +853,12 @@ void ShoutConnection::updateMetaData() {
                 do {
                     // find the next occurrence
                     replaceIndex = metadataFinal.indexOf(
-                                      QRegExp("\\$artist|\\$title"),
-                                      replaceIndex);
+                            kArtistOrTitleRegex,
+                            replaceIndex);
 
                     if (replaceIndex != -1) {
                         if (metadataFinal.indexOf(
-                                          QRegExp("\\$artist"), replaceIndex)
-                                          == replaceIndex) {
+                                    kArtistRegex, replaceIndex) == replaceIndex) {
                             metadataFinal.replace(replaceIndex, 7, artist);
                             // skip to the end of the replacement
                             replaceIndex += artist.length();
@@ -1022,10 +1026,9 @@ void ShoutConnection::run() {
 
     while(true) {
         // Stop the thread if broadcasting is turned off
-        if (!m_pProfile->getEnabled()
-                || !m_pBroadcastEnabled->toBool()
-                || getStatus() == BroadcastProfile::STATUS_FAILURE
-                || getStatus() == BroadcastProfile::STATUS_UNCONNECTED) {
+        if (!m_pProfile->getEnabled() || !m_broadcastEnabled.toBool() ||
+                getStatus() == BroadcastProfile::STATUS_FAILURE ||
+                getStatus() == BroadcastProfile::STATUS_UNCONNECTED) {
             m_threadWaiting = false;
             kLogger.debug() << "run: Connection disabled or failed. Disconnecting";
             if(processDisconnect()) {
