@@ -1,24 +1,19 @@
 #include "library/autodj/autodjfeature.h"
 
 #include <QMenu>
-#include <QMetaObject>
 #include <QtDebug>
 
-#include "controllers/keyboard/keyboardeventfilter.h"
 #include "library/autodj/autodjprocessor.h"
 #include "library/autodj/dlgautodj.h"
-#include "library/crate/cratestorage.h"
 #include "library/library.h"
 #include "library/parser.h"
 #include "library/trackcollection.h"
 #include "library/trackcollectionmanager.h"
+#include "library/trackset/crate/cratestorage.h"
 #include "library/treeitem.h"
-#include "mixer/playermanager.h"
 #include "moc_autodjfeature.cpp"
 #include "sources/soundsourceproxy.h"
 #include "track/track.h"
-#include "util/compatibility.h"
-#include "util/dnd.h"
 #include "widget/wlibrary.h"
 #include "widget/wlibrarysidebar.h"
 
@@ -29,7 +24,7 @@ const QString kViewName = QStringLiteral("Auto DJ");
 } // namespace
 
 namespace {
-    const int kMaxRetrieveAttempts = 3;
+constexpr int kMaxRetrieveAttempts = 3;
 
     int findOrCrateAutoDjPlaylistId(PlaylistDAO& playlistDAO) {
         int playlistId = playlistDAO.getPlaylistIdFromName(AUTODJ_TABLE);
@@ -46,23 +41,25 @@ namespace {
 } // anonymous namespace
 
 AutoDJFeature::AutoDJFeature(Library* pLibrary,
-                             UserSettingsPointer pConfig,
-                             PlayerManagerInterface* pPlayerManager)
-        : LibraryFeature(pLibrary, pConfig),
-          m_pTrackCollection(pLibrary->trackCollections()->internalCollection()),
+        UserSettingsPointer pConfig,
+        PlayerManagerInterface* pPlayerManager)
+        : LibraryFeature(pLibrary, pConfig, QStringLiteral("autodj")),
+          m_pTrackCollection(pLibrary->trackCollectionManager()->internalCollection()),
           m_playlistDao(m_pTrackCollection->getPlaylistDAO()),
           m_iAutoDJPlaylistId(findOrCrateAutoDjPlaylistId(m_playlistDao)),
           m_pAutoDJProcessor(nullptr),
+          m_pSidebarModel(make_parented<TreeItemModel>(this)),
           m_pAutoDJView(nullptr),
-          m_autoDjCratesDao(m_iAutoDJPlaylistId, m_pTrackCollection, m_pConfig),
-          m_icon(":/images/library/ic_library_autodj.svg") {
-
+          m_autoDjCratesDao(m_iAutoDJPlaylistId, pLibrary->trackCollectionManager(), m_pConfig) {
     qRegisterMetaType<AutoDJProcessor::AutoDJState>("AutoDJState");
-    m_pAutoDJProcessor = new AutoDJProcessor(
-            this, m_pConfig, pPlayerManager, pLibrary->trackCollections(), m_iAutoDJPlaylistId);
+    m_pAutoDJProcessor = new AutoDJProcessor(this,
+            m_pConfig,
+            pPlayerManager,
+            pLibrary->trackCollectionManager(),
+            m_iAutoDJPlaylistId);
 
     // Connect loadTrackToPlayer signal as a queued connection to make sure all callbacks of a
-    // previous load attempt have been called (lp1941743)
+    // previous load attempt have been called #10504.
     connect(m_pAutoDJProcessor,
             &AutoDJProcessor::loadTrackToPlayer,
             this,
@@ -79,7 +76,7 @@ AutoDJFeature::AutoDJFeature(Library* pLibrary,
     // Create tree-items under "Crates".
     constructCrateChildModel();
 
-    m_childModel.setRootItem(std::move(pRootItem));
+    m_pSidebarModel->setRootItem(std::move(pRootItem));
 
     // Be notified when the status of crates changes.
     connect(m_pTrackCollection,
@@ -113,10 +110,6 @@ QVariant AutoDJFeature::title() {
     return tr("Auto DJ");
 }
 
-QIcon AutoDJFeature::getIcon() {
-    return m_icon;
-}
-
 void AutoDJFeature::bindLibraryWidget(
         WLibrary* libraryWidget,
         KeyboardEventFilter* keyboard) {
@@ -147,7 +140,7 @@ void AutoDJFeature::bindLibraryWidget(
             this,
             &AutoDJFeature::slotRandomQueue);
     connect(m_pAutoDJView,
-            &DlgAutoDJ::addRandomButton,
+            &DlgAutoDJ::addRandomTrackButton,
             this,
             &AutoDJFeature::slotAddRandomTrack);
 }
@@ -157,8 +150,8 @@ void AutoDJFeature::bindSidebarWidget(WLibrarySidebar* pSidebarWidget) {
     m_pSidebarWidget = pSidebarWidget;
 }
 
-TreeItemModel* AutoDJFeature::getChildModel() {
-    return &m_childModel;
+TreeItemModel* AutoDJFeature::sidebarModel() const {
+    return m_pSidebarModel;
 }
 
 void AutoDJFeature::activate() {
@@ -174,7 +167,7 @@ bool AutoDJFeature::dropAccept(const QList<QUrl>& urls, QObject* pSource) {
     // Auto DJ playlist.
     // pSource != nullptr it is a drop from inside Mixxx and indicates all
     // tracks already in the DB
-    QList<TrackId> trackIds = m_pTrackCollection->resolveTrackIdsFromUrls(urls,
+    QList<TrackId> trackIds = m_pLibrary->trackCollectionManager()->resolveTrackIdsFromUrls(urls,
             !pSource);
     if (trackIds.isEmpty()) {
         return false;
@@ -190,8 +183,8 @@ bool AutoDJFeature::dragMoveAccept(const QUrl& url) {
 }
 
 // Add a crate to the auto-DJ queue.
-void AutoDJFeature::slotAddCrateToAutoDj(int iCrateId) {
-    m_pTrackCollection->updateAutoDjCrate(CrateId(iCrateId), true);
+void AutoDJFeature::slotAddCrateToAutoDj(CrateId crateId) {
+    m_pTrackCollection->updateAutoDjCrate(crateId, true);
 }
 
 void AutoDJFeature::slotRemoveCrateFromAutoDj() {
@@ -207,9 +200,9 @@ void AutoDJFeature::slotCrateChanged(CrateId crateId) {
         // -> Find and update the corresponding child item
         for (int i = 0; i < m_crateList.length(); ++i) {
             if (m_crateList[i].getId() == crateId) {
-                QModelIndex parentIndex = m_childModel.index(0, 0);
-                QModelIndex childIndex = m_childModel.index(i, 0, parentIndex);
-                m_childModel.setData(childIndex, crate.getName(), Qt::DisplayRole);
+                QModelIndex parentIndex = m_pSidebarModel->index(0, 0);
+                QModelIndex childIndex = m_pSidebarModel->index(i, 0, parentIndex);
+                m_pSidebarModel->setData(childIndex, crate.getName(), Qt::DisplayRole);
                 m_crateList[i] = crate;
                 return; // early exit
             }
@@ -220,16 +213,16 @@ void AutoDJFeature::slotCrateChanged(CrateId crateId) {
         // std::vector for a single element.
         std::vector<std::unique_ptr<TreeItem>> rows;
         rows.push_back(std::make_unique<TreeItem>(crate.getName(), crate.getId().toVariant()));
-        QModelIndex parentIndex = m_childModel.index(0, 0);
-        m_childModel.insertTreeItemRows(std::move(rows), m_crateList.length(), parentIndex);
+        QModelIndex parentIndex = m_pSidebarModel->index(0, 0);
+        m_pSidebarModel->insertTreeItemRows(std::move(rows), m_crateList.length(), parentIndex);
         m_crateList.append(crate);
     } else {
         // Crate does not exist or is not a source for AutoDJ
         // -> Find and remove the corresponding child item
         for (int i = 0; i < m_crateList.length(); ++i) {
             if (m_crateList[i].getId() == crateId) {
-                QModelIndex parentIndex = m_childModel.index(0, 0);
-                m_childModel.removeRows(i, 1, parentIndex);
+                QModelIndex parentIndex = m_pSidebarModel->index(0, 0);
+                m_pSidebarModel->removeRows(i, 1, parentIndex);
                 m_crateList.removeAt(i);
                 return; // early exit
             }
@@ -256,13 +249,13 @@ void AutoDJFeature::slotAddRandomTrack() {
             }
 
             if (randomTrackId.isValid()) {
-                pRandomTrack = m_pTrackCollection->getTrackById(randomTrackId);
+                pRandomTrack = m_pLibrary->trackCollectionManager()->getTrackById(randomTrackId);
                 VERIFY_OR_DEBUG_ASSERT(pRandomTrack) {
                     qWarning() << "Track does not exist:"
                             << randomTrackId;
                     continue;
                 }
-                if (!pRandomTrack->checkFileExists()) {
+                if (!pRandomTrack->getFileInfo().checkFileExists()) {
                     qWarning() << "Track does not exist:"
                                << pRandomTrack->getInfo()
                                << pRandomTrack->getLocation();
@@ -304,9 +297,10 @@ void AutoDJFeature::onRightClickChild(const QPoint& globalPos,
         Crate crate;
         while (nonAutoDjCrates.populateNext(&crate)) {
             auto pAction = std::make_unique<QAction>(crate.getName(), &crateMenu);
-            int iCrateId = crate.getId().value();
-            connect(pAction.get(), &QAction::triggered,
-                    this, [this, iCrateId] { slotAddCrateToAutoDj(iCrateId); });
+            auto crateId = crate.getId();
+            connect(pAction.get(), &QAction::triggered, this, [this, crateId] {
+                slotAddCrateToAutoDj(crateId);
+            });
             crateMenu.addAction(pAction.get());
             pAction.release();
         }
