@@ -2,8 +2,8 @@
 
 #include "control/controlproxy.h"
 #include "control/controlpushbutton.h"
-#include "engine/engine.h"
-#include "library/trackcollection.h"
+#include "engine/channels/enginedeck.h"
+#include "library/playlisttablemodel.h"
 #include "mixer/basetrackplayer.h"
 #include "mixer/playermanager.h"
 #include "moc_autodjprocessor.cpp"
@@ -14,12 +14,11 @@
 namespace {
 const char* kTransitionPreferenceName = "Transition";
 const char* kTransitionModePreferenceName = "TransitionMode";
-const double kTransitionPreferenceDefault = 10.0;
-const double kKeepPosition = -1.0;
-// A track needs to be longer than two callbacks to not stop AutoDJ
-const double kMinimumTrackDurationSec = 0.2;
+constexpr double kTransitionPreferenceDefault = 10.0;
+constexpr double kKeepPosition = -1.0;
 
-const mixxx::audio::ChannelCount kChannelCount = mixxx::kEngineChannelCount;
+// A track needs to be longer than two callbacks to not stop AutoDJ
+constexpr double kMinimumTrackDurationSec = 0.2;
 
 constexpr bool sDebug = false;
 } // anonymous namespace
@@ -117,14 +116,13 @@ AutoDJProcessor::AutoDJProcessor(
         int iAutoDJPlaylistId)
         : QObject(pParent),
           m_pConfig(pConfig),
-          m_pPlayerManager(pPlayerManager),
           m_pAutoDJTableModel(nullptr),
           m_eState(ADJ_DISABLED),
           m_transitionProgress(0.0),
           m_transitionTime(kTransitionPreferenceDefault) {
     m_pAutoDJTableModel = new PlaylistTableModel(this, pTrackCollectionManager,
                                                  "mixxx.db.model.autodj");
-    m_pAutoDJTableModel->setTableModel(iAutoDJPlaylistId);
+    m_pAutoDJTableModel->selectPlaylist(iAutoDJPlaylistId);
     m_pAutoDJTableModel->select();
 
     m_pShufflePlaylist = new ControlPushButton(
@@ -137,6 +135,13 @@ AutoDJProcessor::AutoDJProcessor(
     connect(m_pSkipNext, &ControlObject::valueChanged,
             this, &AutoDJProcessor::controlSkipNext);
 
+    m_pAddRandomTrack = new ControlPushButton(
+            ConfigKey("[AutoDJ]", "add_random_track"));
+    connect(m_pAddRandomTrack,
+            &ControlObject::valueChanged,
+            this,
+            &AutoDJProcessor::controlAddRandomTrack);
+
     m_pFadeNow = new ControlPushButton(
             ConfigKey("[AutoDJ]", "fade_now"));
     connect(m_pFadeNow, &ControlObject::valueChanged,
@@ -184,6 +189,7 @@ AutoDJProcessor::~AutoDJProcessor() {
     delete m_pCOCrossfaderReverse;
 
     delete m_pSkipNext;
+    delete m_pAddRandomTrack;
     delete m_pShufflePlaylist;
     delete m_pEnabledAutoDJ;
     delete m_pFadeNow;
@@ -600,6 +606,12 @@ void AutoDJProcessor::controlSkipNext(double value) {
     }
 }
 
+void AutoDJProcessor::controlAddRandomTrack(double value) {
+    if (value > 0.0) {
+        emit randomTrackRequested(1);
+    }
+}
+
 void AutoDJProcessor::crossfaderChanged(double value) {
     if (m_eState == ADJ_IDLE) {
         // The user is changing the crossfader manually. If the user has
@@ -880,7 +892,7 @@ TrackPointer AutoDJProcessor::getNextTrackFromQueue() {
             m_pAutoDJTableModel->index(0, 0));
 
         if (nextTrack) {
-            if (nextTrack->checkFileExists()) {
+            if (nextTrack->getFileInfo().checkFileExists()) {
                 return nextTrack;
             } else {
                 // Remove missing song from auto DJ playlist.
@@ -1079,47 +1091,76 @@ void AutoDJProcessor::playerOutroEndChanged(DeckAttributes* pAttributes, double
 }
 
 double AutoDJProcessor::getIntroStartSecond(DeckAttributes* pDeck) {
-    double trackEndSample = pDeck->trackSamples();
-    double introStartSample = pDeck->introStartPosition();
-    if (introStartSample == Cue::kNoPosition || introStartSample > trackEndSample) {
-        return getFirstSoundSecond(pDeck);
+    const mixxx::audio::FramePos trackEndPosition = pDeck->trackEndPosition();
+    const mixxx::audio::FramePos introStartPosition = pDeck->introStartPosition();
+    const mixxx::audio::FramePos introEndPosition = pDeck->introEndPosition();
+    if (!introStartPosition.isValid() || introStartPosition > trackEndPosition) {
+        double firstSoundSecond = getFirstSoundSecond(pDeck);
+        if (!introEndPosition.isValid() || introEndPosition > trackEndPosition) {
+            // No intro start and intro end set, use First Sound.
+            return firstSoundSecond;
+        }
+        double introEndSecond = framePositionToSeconds(introEndPosition, pDeck);
+        if (m_transitionTime >= 0) {
+            return introEndSecond - m_transitionTime;
+        }
+        return introEndSecond;
     }
-    return samplePositionToSeconds(introStartSample, pDeck);
+    return framePositionToSeconds(introStartPosition, pDeck);
 }
 
 double AutoDJProcessor::getIntroEndSecond(DeckAttributes* pDeck) {
-    double trackEndSample = pDeck->trackSamples();
-    double introEndSample = pDeck->introEndPosition();
-    if (introEndSample == Cue::kNoPosition || introEndSample > trackEndSample) {
+    const mixxx::audio::FramePos trackEndPosition = pDeck->trackEndPosition();
+    const mixxx::audio::FramePos introEndPosition = pDeck->introEndPosition();
+    if (!introEndPosition.isValid() || introEndPosition > trackEndPosition) {
         // Assume a zero length intro if introEnd is not set.
         // The introStart is automatically placed by AnalyzerSilence, so use
         // that as a fallback if the user has not placed outroStart. If it has
         // not been placed, getIntroStartPosition will return 0:00.
         return getIntroStartSecond(pDeck);
     }
-    return samplePositionToSeconds(introEndSample, pDeck);
+    return framePositionToSeconds(introEndPosition, pDeck);
 }
 
 double AutoDJProcessor::getOutroStartSecond(DeckAttributes* pDeck) {
-    double trackEndSample = pDeck->trackSamples();
-    double outroStartSample = pDeck->outroStartPosition();
-    if (outroStartSample == Cue::kNoPosition || outroStartSample > trackEndSample) {
+    const mixxx::audio::FramePos trackEndPosition = pDeck->trackEndPosition();
+    const mixxx::audio::FramePos outroStartPosition = pDeck->outroStartPosition();
+    if (!outroStartPosition.isValid() || outroStartPosition > trackEndPosition) {
         // Assume a zero length outro if outroStart is not set.
         // The outroEnd is automatically placed by AnalyzerSilence, so use
         // that as a fallback if the user has not placed outroStart. If it has
         // not been placed, getOutroEndPosition will return the end of the track.
         return getOutroEndSecond(pDeck);
     }
-    return samplePositionToSeconds(outroStartSample, pDeck);
+    return framePositionToSeconds(outroStartPosition, pDeck);
 }
 
 double AutoDJProcessor::getOutroEndSecond(DeckAttributes* pDeck) {
-    double trackEndSample = pDeck->trackSamples();
-    double outroEndSample = pDeck->outroEndPosition();
-    if (outroEndSample == Cue::kNoPosition || outroEndSample > trackEndSample) {
-        return getLastSoundSecond(pDeck);
+    const mixxx::audio::FramePos trackEndPosition = pDeck->trackEndPosition();
+    const mixxx::audio::FramePos outroStartPosition = pDeck->outroStartPosition();
+    const mixxx::audio::FramePos outroEndPosition = pDeck->outroEndPosition();
+    if (!outroEndPosition.isValid() || outroEndPosition > trackEndPosition) {
+        double lastSoundSecond = getLastSoundSecond(pDeck);
+        DEBUG_ASSERT(lastSoundSecond <= framePositionToSeconds(trackEndPosition, pDeck));
+        if (!outroStartPosition.isValid() || outroStartPosition > trackEndPosition) {
+            // No outro start and outro end set, use Last Sound.
+            return lastSoundSecond;
+        }
+        // Try to find a better Outro End using Outro Start and transition time
+        double outroStartSecond = framePositionToSeconds(outroStartPosition, pDeck);
+        if (m_transitionTime >= 0 && lastSoundSecond > outroStartSecond) {
+            double outroEndFromTime = outroStartSecond + m_transitionTime;
+            if (outroEndFromTime < lastSoundSecond) {
+                // The outroEnd is automatically placed by AnalyzerSilence at the last sound
+                // Here the user has removed it, but has placed a outro start.
+                // Use the transition time instead of the dismissed last sound position.
+                return outroEndFromTime;
+            }
+            return lastSoundSecond;
+        }
+        return outroStartSecond;
     }
-    return samplePositionToSeconds(outroEndSample, pDeck);;
+    return framePositionToSeconds(outroEndPosition, pDeck);
 }
 
 double AutoDJProcessor::getFirstSoundSecond(DeckAttributes* pDeck) {
@@ -1128,15 +1169,15 @@ double AutoDJProcessor::getFirstSoundSecond(DeckAttributes* pDeck) {
         return 0.0;
     }
 
-    CuePointer pFromTrackAudibleSound = pTrack->findCueByType(mixxx::CueType::AudibleSound);
-    if (pFromTrackAudibleSound) {
-        double firstSound = pFromTrackAudibleSound->getPosition();
-        if (firstSound > 0.0) {
-            double trackEndSample = pDeck->trackSamples();
-            if (firstSound <= trackEndSample) {
-                return samplePositionToSeconds(firstSound, pDeck);
+    CuePointer pFromTrackN60dBSound = pTrack->findCueByType(mixxx::CueType::N60dBSound);
+    if (pFromTrackN60dBSound) {
+        const mixxx::audio::FramePos firstSound = pFromTrackN60dBSound->getPosition();
+        if (firstSound.isValid()) {
+            const mixxx::audio::FramePos trackEndPosition = pDeck->trackEndPosition();
+            if (firstSound <= trackEndPosition) {
+                return framePositionToSeconds(firstSound, pDeck);
             } else {
-                qWarning() << "Audible Sound Cue starts after track end in:"
+                qWarning() << "-60 dB Sound Cue starts after track end in:"
                            << pTrack->getLocation()
                            << "Using the first sample instead.";
             }
@@ -1151,21 +1192,21 @@ double AutoDJProcessor::getLastSoundSecond(DeckAttributes* pDeck) {
         return 0.0;
     }
 
-    double trackEndSample = pDeck->trackSamples();
-    CuePointer pCueAudibleSound = pTrack->findCueByType(mixxx::CueType::AudibleSound);
-    if (pCueAudibleSound && pCueAudibleSound->getLength() > 0) {
-        double lastSound = pCueAudibleSound->getEndPosition();
-        if (lastSound > 0) {
-            if (lastSound <= trackEndSample) {
-                return samplePositionToSeconds(lastSound, pDeck);
+    const mixxx::audio::FramePos trackEndPosition = pDeck->trackEndPosition();
+    CuePointer pFromTrackN60dBSound = pTrack->findCueByType(mixxx::CueType::N60dBSound);
+    if (pFromTrackN60dBSound && pFromTrackN60dBSound->getLengthFrames() > 0.0) {
+        const mixxx::audio::FramePos lastSound = pFromTrackN60dBSound->getEndPosition();
+        if (lastSound > mixxx::audio::FramePos(0.0)) {
+            if (lastSound <= trackEndPosition) {
+                return framePositionToSeconds(lastSound, pDeck);
             } else {
-                qWarning() << "Audible Sound Cue ends after track end in:"
+                qWarning() << "-60 dB Sound Cue ends after track end in:"
                            << pTrack->getLocation()
                            << "Using the last sample instead.";
             }
         }
     }
-    return samplePositionToSeconds(trackEndSample, pDeck);
+    return framePositionToSeconds(trackEndPosition, pDeck);
 }
 
 double AutoDJProcessor::getEndSecond(DeckAttributes* pDeck) {
@@ -1174,17 +1215,18 @@ double AutoDJProcessor::getEndSecond(DeckAttributes* pDeck) {
         return 0.0;
     }
 
-    double trackEndSample = pDeck->trackSamples();
-    return samplePositionToSeconds(trackEndSample, pDeck);
+    mixxx::audio::FramePos trackEndPosition = pDeck->trackEndPosition();
+    return framePositionToSeconds(trackEndPosition, pDeck);
 }
 
-double AutoDJProcessor::samplePositionToSeconds(double samplePosition, DeckAttributes* pDeck) {
-    samplePosition /= kChannelCount;
+double AutoDJProcessor::framePositionToSeconds(
+        mixxx::audio::FramePos position, DeckAttributes* pDeck) {
     mixxx::audio::SampleRate sampleRate = pDeck->sampleRate();
-    if (!sampleRate.isValid()) {
+    if (!sampleRate.isValid() || !position.isValid()) {
         return 0.0;
     }
-    return samplePosition / sampleRate / pDeck->rateRatio();
+
+    return position.value() / sampleRate / pDeck->rateRatio();
 }
 
 void AutoDJProcessor::calculateTransition(DeckAttributes* pFromDeck,
