@@ -28,22 +28,21 @@ const bool sDebug = false;
 // The logic in the following code relies to a track column = 0
 // Do not change it without changing the logic
 // Column 0 is skipped when calculating the the columns of the view table
-const int kIdColumn = 0;
-const int kMaxSortColumns = 3;
+constexpr int kIdColumn = 0;
+constexpr int kMaxSortColumns = 3;
 
 // Constant for getModelSetting(name)
 const QString COLUMNS_SORTING = QStringLiteral("ColumnsSorting");
 
+const QString kModelName = "table:";
+
 } // anonymous namespace
 
 BaseSqlTableModel::BaseSqlTableModel(
         QObject* parent,
         TrackCollectionManager* pTrackCollectionManager,
         const char* settingsNamespace)
-        : BaseTrackTableModel(
-                  settingsNamespace,
-                  pTrackCollectionManager,
-                  parent),
+        : BaseTrackTableModel(parent, pTrackCollectionManager, settingsNamespace),
           m_pTrackCollectionManager(pTrackCollectionManager),
           m_database(pTrackCollectionManager->internalCollection()->database()),
           m_bInitialized(false) {
@@ -122,6 +121,9 @@ void BaseSqlTableModel::initSortColumnMapping() {
     m_columnIndexBySortColumnId[static_cast<int>(
             TrackModel::SortColumnId::TimesPlayed)] =
             fieldIndex(ColumnCache::COLUMN_LIBRARYTABLE_TIMESPLAYED);
+    m_columnIndexBySortColumnId[static_cast<int>(
+            TrackModel::SortColumnId::LastPlayedAt)] =
+            fieldIndex(ColumnCache::COLUMN_LIBRARYTABLE_LAST_PLAYED_AT);
     m_columnIndexBySortColumnId[static_cast<int>(
             TrackModel::SortColumnId::Rating)] =
             fieldIndex(ColumnCache::COLUMN_LIBRARYTABLE_RATING);
@@ -140,6 +142,9 @@ void BaseSqlTableModel::initSortColumnMapping() {
     m_columnIndexBySortColumnId[static_cast<int>(
             TrackModel::SortColumnId::SampleRate)] =
             fieldIndex(ColumnCache::COLUMN_LIBRARYTABLE_SAMPLERATE);
+    m_columnIndexBySortColumnId[static_cast<int>(
+            TrackModel::SortColumnId::PlaylistDateTimeAdded)] =
+            fieldIndex(ColumnCache::COLUMN_PLAYLISTTRACKSTABLE_DATETIMEADDED);
 
     m_sortColumnIdByColumnIndex.clear();
     for (int i = static_cast<int>(TrackModel::SortColumnId::IdMin);
@@ -231,7 +236,7 @@ void BaseSqlTableModel::select() {
     }
 
     // Remove all the rows from the table after(!) the query has been
-    // executed successfully. See Bug #1090888.
+    // executed successfully. See issue #6782.
     // TODO(rryan) we could edit the table in place instead of clearing it?
     clearRows();
 
@@ -247,15 +252,17 @@ void BaseSqlTableModel::select() {
         if (idColumn < 0) {
             idColumn = sqlRecord.indexOf(m_idColumn);
         }
+
+        // TODO(XXX): Can we get rid of the hard-coded assumption that
+        // the the first column always contains the id?
+        DEBUG_ASSERT(idColumn == kIdColumn);
+
         VERIFY_OR_DEBUG_ASSERT(idColumn >= 0) {
             qCritical()
                     << "ID column not available in database query results:"
                     << m_idColumn;
             return;
         }
-        // TODO(XXX): Can we get rid of the hard-coded assumption that
-        // the the first column always contains the id?
-        DEBUG_ASSERT(idColumn == kIdColumn);
 
         TrackId trackId(sqlRecord.value(idColumn));
         trackIds.insert(trackId);
@@ -323,15 +330,15 @@ void BaseSqlTableModel::select() {
     // number of total rows returned by the query
     DEBUG_ASSERT(trackIdToRows.size() <= rowInfos.size());
 
-    // We're done! Issue the update signals and replace the master maps.
+    // We're done! Issue the update signals and replace the main maps.
     replaceRows(
             std::move(rowInfos),
             std::move(trackIdToRows));
     // Both rowInfo and trackIdToRows (might) have been moved and
     // must not be used afterwards!
 
-    qDebug() << this << "select() took" << time.elapsed().debugMillisWithUnit()
-             << m_rowInfo.size();
+    qDebug() << this << "select() returned" << m_rowInfo.size()
+             << "results in" << time.elapsed().debugMillisWithUnit();
 }
 
 void BaseSqlTableModel::setTable(QString tableName,
@@ -356,7 +363,7 @@ void BaseSqlTableModel::setTable(QString tableName,
         // It's important that this not be a direct connection, or else the UI
         // might try to update while a cache operation is in progress, and that
         // will hit the cache again and cause dangerous reentry cycles
-        // See https://bugs.launchpad.net/mixxx/+bug/1365708
+        // See https://github.com/mixxxdj/mixxx/issues/7569
         // TODO: A better fix is to have cache and trackpointers defer saving
         // and deleting, so those operations only take place at the top of
         // the call stack.
@@ -521,7 +528,7 @@ void BaseSqlTableModel::setSort(int column, Qt::SortOrder order) {
         m_sortColumns.prepend(SortColumn(column, order));
     } else if (m_trackSource) {
         bool first = true;
-        for (const SortColumn& sc : qAsConst(m_sortColumns)) {
+        for (const SortColumn& sc : std::as_const(m_sortColumns)) {
             QString sort_field;
             if (sc.m_column < m_tableColumns.size()) {
                 if (sc.m_column == kIdColumn) {
@@ -610,6 +617,15 @@ int BaseSqlTableModel::fieldIndex(const QString& fieldName) const {
     return tableIndex;
 }
 
+QString BaseSqlTableModel::modelKey(bool noSearch) const {
+    if (noSearch) {
+        return kModelName + m_tableName;
+    }
+    return kModelName + m_tableName +
+            QStringLiteral("#") +
+            currentSearch();
+}
+
 QVariant BaseSqlTableModel::rawValue(
         const QModelIndex& index) const {
     DEBUG_ASSERT(index.isValid());
@@ -692,7 +708,7 @@ bool BaseSqlTableModel::setTrackValueForColumn(
     } else if (fieldIndex(ColumnCache::COLUMN_LIBRARYTABLE_YEAR) == column) {
         pTrack->setYear(value.toString());
     } else if (fieldIndex(ColumnCache::COLUMN_LIBRARYTABLE_GENRE) == column) {
-        pTrack->setGenre(value.toString());
+        updateTrackGenre(pTrack.get(), value.toString());
     } else if (fieldIndex(ColumnCache::COLUMN_LIBRARYTABLE_COMPOSER) == column) {
         pTrack->setComposer(value.toString());
     } else if (fieldIndex(ColumnCache::COLUMN_LIBRARYTABLE_GROUPING) == column) {
@@ -721,24 +737,24 @@ bool BaseSqlTableModel::setTrackValueForColumn(
         StarRating starRating = value.value<StarRating>();
         pTrack->setRating(starRating.starCount());
     } else if (fieldIndex(ColumnCache::COLUMN_LIBRARYTABLE_KEY) == column) {
-        pTrack->setKeyText(value.toString(),
+        pTrack->setKeyText(
+                value.toString(),
                 mixxx::track::io::key::USER);
     } else if (fieldIndex(ColumnCache::COLUMN_LIBRARYTABLE_BPM_LOCK) == column) {
         pTrack->setBpmLocked(value.toBool());
     } else {
         // We never should get up to this point!
-        VERIFY_OR_DEBUG_ASSERT(false) {
-            qWarning() << "Column"
-                       << columnNameForFieldIndex(column)
-                       << "is not editable!";
-        }
+        qWarning() << "Column"
+                   << columnNameForFieldIndex(column)
+                   << "is not editable!";
+        DEBUG_ASSERT(false);
         return false;
     }
     return true;
 }
 
 TrackPointer BaseSqlTableModel::getTrack(const QModelIndex& index) const {
-    return m_pTrackCollectionManager->internalCollection()->getTrackById(getTrackId(index));
+    return m_pTrackCollectionManager->getTrackById(getTrackId(index));
 }
 
 TrackId BaseSqlTableModel::getTrackId(const QModelIndex& index) const {
@@ -761,33 +777,54 @@ QString BaseSqlTableModel::getTrackLocation(const QModelIndex& index) const {
     return QDir::fromNativeSeparators(nativeLocation);
 }
 
+QUrl BaseSqlTableModel::getTrackUrl(const QModelIndex& index) const {
+    const QString trackLocation = getTrackLocation(index);
+    DEBUG_ASSERT(trackLocation.trimmed() == trackLocation);
+    if (trackLocation.isEmpty()) {
+        return {};
+    }
+    return QUrl::fromLocalFile(trackLocation);
+}
+
 CoverInfo BaseSqlTableModel::getCoverInfo(const QModelIndex& index) const {
     CoverInfo coverInfo;
-    coverInfo.hash =
+    coverInfo.setImageDigest(
+            index.sibling(index.row(),
+                         fieldIndex(ColumnCache::
+                                         COLUMN_LIBRARYTABLE_COVERART_DIGEST))
+                    .data()
+                    .toByteArray(),
             index.sibling(index.row(),
                          fieldIndex(ColumnCache::
                                          COLUMN_LIBRARYTABLE_COVERART_HASH))
                     .data()
-                    .toUInt();
-    coverInfo.type = static_cast<CoverInfo::Type>(
+                    .toUInt());
+    coverInfo.color = mixxx::RgbColor::fromQVariant(
             index.sibling(index.row(),
                          fieldIndex(ColumnCache::
-                                         COLUMN_LIBRARYTABLE_COVERART_TYPE))
-                    .data()
-                    .toInt());
-    coverInfo.source = static_cast<CoverInfo::Source>(
-            index.sibling(index.row(),
-                         fieldIndex(ColumnCache::
-                                         COLUMN_LIBRARYTABLE_COVERART_SOURCE))
-                    .data()
-                    .toInt());
-    coverInfo.coverLocation =
-            index.sibling(index.row(),
-                         fieldIndex(ColumnCache::
-                                         COLUMN_LIBRARYTABLE_COVERART_LOCATION))
-                    .data()
-                    .toString();
-    coverInfo.trackLocation = getTrackLocation(index);
+                                         COLUMN_LIBRARYTABLE_COVERART_COLOR))
+                    .data());
+    if (coverInfo.hasCacheKey()) {
+        coverInfo.type = static_cast<CoverInfo::Type>(
+                index.sibling(index.row(),
+                             fieldIndex(ColumnCache::
+                                             COLUMN_LIBRARYTABLE_COVERART_TYPE))
+                        .data()
+                        .toInt());
+        coverInfo.source = static_cast<CoverInfo::Source>(
+                index.sibling(index.row(),
+                             fieldIndex(ColumnCache::
+                                             COLUMN_LIBRARYTABLE_COVERART_SOURCE))
+                        .data()
+                        .toInt());
+        coverInfo.coverLocation =
+                index.sibling(index.row(),
+                             fieldIndex(ColumnCache::
+                                             COLUMN_LIBRARYTABLE_COVERART_LOCATION))
+                        .data()
+                        .toString();
+        coverInfo.trackLocation = getTrackLocation(index);
+    }
     return coverInfo;
 }
 
@@ -827,8 +864,8 @@ QList<TrackRef> BaseSqlTableModel::getTrackRefs(
     QList<TrackRef> trackRefs;
     trackRefs.reserve(indices.size());
     foreach (QModelIndex index, indices) {
-        trackRefs.append(TrackRef::fromFileInfo(
-                TrackFile(getTrackLocation(index)),
+        trackRefs.append(TrackRef::fromFilePath(
+                getTrackLocation(index),
                 getTrackId(index)));
     }
     return trackRefs;
