@@ -1,21 +1,17 @@
 #include "library/browse/browsefeature.h"
 
 #include <QAction>
-#include <QDirModel>
 #include <QFileInfo>
 #include <QMenu>
 #include <QPushButton>
-#include <QStandardPaths>
 #include <QStringList>
-#include <QTreeView>
 
-#include "controllers/keyboard/keyboardeventfilter.h"
+#include "library/browse/foldertreemodel.h"
 #include "library/library.h"
 #include "library/trackcollection.h"
 #include "library/trackcollectionmanager.h"
 #include "library/treeitem.h"
 #include "moc_browsefeature.cpp"
-#include "track/track.h"
 #include "util/memory.h"
 #include "widget/wlibrary.h"
 #include "widget/wlibrarysidebar.h"
@@ -31,16 +27,20 @@ BrowseFeature::BrowseFeature(
         Library* pLibrary,
         UserSettingsPointer pConfig,
         RecordingManager* pRecordingManager)
-        : LibraryFeature(pLibrary, pConfig),
-          m_pTrackCollection(pLibrary->trackCollections()->internalCollection()),
-          m_browseModel(this, pLibrary->trackCollections(), pRecordingManager),
-          m_proxyModel(&m_browseModel),
-          m_pLastRightClickedItem(nullptr),
-          m_icon(":/images/library/ic_library_computer.svg") {
+        : LibraryFeature(pLibrary, pConfig, QString("computer")),
+          m_pTrackCollection(pLibrary->trackCollectionManager()->internalCollection()),
+          m_browseModel(this, pLibrary->trackCollectionManager(), pRecordingManager),
+          m_proxyModel(&m_browseModel, true),
+          m_pSidebarModel(new FolderTreeModel(this)),
+          m_pLastRightClickedItem(nullptr) {
     connect(this,
             &BrowseFeature::requestAddDir,
             pLibrary,
             &Library::slotRequestAddDir);
+    connect(&m_browseModel,
+            &BrowseTableModel::restoreModelState,
+            this,
+            &LibraryFeature::restoreModelState);
 
     m_pAddQuickLinkAction = new QAction(tr("Add to Quick Links"),this);
     connect(m_pAddQuickLinkAction,
@@ -93,7 +93,6 @@ BrowseFeature::BrowseFeature(
         if (display_path.endsWith("/")) {
             display_path.chop(1);
         }
-        TreeItem* driveLetter =
         devices_link->appendChild(
                 display_path, // Displays C:
                 drive.filePath()); // Displays C:/
@@ -131,7 +130,7 @@ BrowseFeature::BrowseFeature(
     }
 
     // initialize the model
-    m_childModel.setRootItem(std::move(pRootItem));
+    m_pSidebarModel->setRootItem(std::move(pRootItem));
 }
 
 BrowseFeature::~BrowseFeature() {
@@ -150,12 +149,12 @@ void BrowseFeature::slotAddQuickLink() {
     QString spath = vpath.toString();
     QString name = extractNameFromPath(spath);
 
-    QModelIndex parent = m_childModel.index(m_pQuickLinkItem->parentRow(), 0);
+    QModelIndex parent = m_pSidebarModel->index(m_pQuickLinkItem->parentRow(), 0);
     std::vector<std::unique_ptr<TreeItem>> rows;
     // TODO() Use here std::span to get around the heap allocation of
     // std::vector for a single element.
     rows.push_back(std::make_unique<TreeItem>(name, vpath));
-    m_childModel.insertTreeItemRows(std::move(rows), m_pQuickLinkItem->childRows(), parent);
+    m_pSidebarModel->insertTreeItemRows(std::move(rows), m_pQuickLinkItem->childRows(), parent);
 
     m_quickLinkList.append(spath);
     saveQuickLinks();
@@ -207,19 +206,15 @@ void BrowseFeature::slotRemoveQuickLink() {
     }
 
     m_pLastRightClickedItem = nullptr;
-    QModelIndex parent = m_childModel.index(m_pQuickLinkItem->parentRow(), 0);
-    m_childModel.removeRow(index, parent);
+    QModelIndex parent = m_pSidebarModel->index(m_pQuickLinkItem->parentRow(), 0);
+    m_pSidebarModel->removeRow(index, parent);
 
     m_quickLinkList.removeAt(index);
     saveQuickLinks();
 }
 
-QIcon BrowseFeature::getIcon() {
-    return m_icon;
-}
-
-TreeItemModel* BrowseFeature::getChildModel() {
-    return &m_childModel;
+TreeItemModel* BrowseFeature::sidebarModel() const {
+    return m_pSidebarModel;
 }
 
 void BrowseFeature::bindLibraryWidget(WLibrary* libraryWidget,
@@ -246,28 +241,34 @@ void BrowseFeature::activate() {
 void BrowseFeature::activateChild(const QModelIndex& index) {
     TreeItem *item = static_cast<TreeItem*>(index.internalPointer());
     qDebug() << "BrowseFeature::activateChild " << item->getLabel() << " "
-             << item->getData();
+             << item->getData().toString();
 
     QString path = item->getData().toString();
     if (path == QUICK_LINK_NODE || path == DEVICE_NODE) {
-        m_browseModel.setPath(MDir());
+        emit saveModelState();
+        m_browseModel.setPath({});
     } else {
         // Open a security token for this path and if we do not have access, ask
         // for it.
-        MDir dir(path);
-        if (!dir.canAccess()) {
-            if (Sandbox::askForAccess(path)) {
+        auto dirInfo = mixxx::FileInfo(path);
+        auto dirAccess = mixxx::FileAccess(dirInfo);
+        if (!dirAccess.isReadable()) {
+            if (Sandbox::askForAccess(&dirInfo)) {
                 // Re-create to get a new token.
-                dir = MDir(path);
+                dirAccess = mixxx::FileAccess(dirInfo);
             } else {
                 // TODO(rryan): Activate an info page about sandboxing?
                 return;
             }
         }
-        m_browseModel.setPath(dir);
+        emit saveModelState();
+        m_browseModel.setPath(std::move(dirAccess));
     }
     emit showTrackModel(&m_proxyModel);
-    emit disableSearch();
+    // Search is restored in Library::slotShowTrackModel, disable it where it's useless
+    if (path == QUICK_LINK_NODE || path == DEVICE_NODE) {
+        emit disableSearch();
+    }
     emit enableCoverArtDisplay(false);
 }
 
@@ -370,7 +371,7 @@ void BrowseFeature::onLazyChildExpandation(const QModelIndex& index) {
     // NOTE(2018-04-21, uklotzde): The following checks prevent a crash
     // that would otherwise occur after a quick link has been removed.
     // Especially the check of QVariant::isValid() seems to be essential.
-    // See also: https://bugs.launchpad.net/mixxx/+bug/1510068
+    // See also: https://github.com/mixxxdj/mixxx/issues/8270
     // After migration to Qt5 the implementation of all LibraryFeatures
     // should be revisited, because I consider these checks only as a
     // temporary workaround.
@@ -394,7 +395,7 @@ void BrowseFeature::onLazyChildExpandation(const QModelIndex& index) {
 
     m_pLastRightClickedItem = nullptr;
     // Before we populate the subtree, we need to delete old subtrees
-    m_childModel.removeRows(0, item->childRows(), index);
+    m_pSidebarModel->removeRows(0, item->childRows(), index);
 
     // List of subfolders or drive letters
     std::vector<std::unique_ptr<TreeItem>> folders;
@@ -405,10 +406,10 @@ void BrowseFeature::onLazyChildExpandation(const QModelIndex& index) {
     } else {
         // we assume that the path refers to a folder in the file system
         // populate children
-        MDir dir(path);
+        const auto dirAccess = mixxx::FileAccess(mixxx::FileInfo(path));
 
-        QFileInfoList all = dir.dir().entryInfoList(
-            QDir::Dirs | QDir::NoDotAndDotDot);
+        QFileInfoList all = dirAccess.info().toQDir().entryInfoList(
+                QDir::Dirs | QDir::NoDotAndDotDot);
 
         // loop through all the item and construct the children
         foreach (QFileInfo one, all) {
@@ -429,7 +430,7 @@ void BrowseFeature::onLazyChildExpandation(const QModelIndex& index) {
     // On Ubuntu 10.04, otherwise, this will draw an icon although the folder
     // has no subfolders
     if (!folders.empty()) {
-        m_childModel.insertTreeItemRows(std::move(folders), 0, index);
+        m_pSidebarModel->insertTreeItemRows(std::move(folders), 0, index);
     }
 }
 
@@ -459,15 +460,11 @@ void BrowseFeature::loadQuickLinks() {
 }
 
 QString BrowseFeature::extractNameFromPath(const QString& spath) {
-    QString path = spath.left(spath.count()-1);
-    int index = path.lastIndexOf("/");
-    QString name = (spath.count() > 1) ? path.mid(index+1) : spath;
-    return name;
+    return QDir(spath).dirName();
 }
 
 QStringList BrowseFeature::getDefaultQuickLinks() const {
     // Default configuration
-    QStringList mixxxMusicDirs = m_pTrackCollection->getDirectoryDAO().getDirs();
     QDir osMusicDir(QStandardPaths::writableLocation(
             QStandardPaths::MusicLocation));
     QDir osDocumentsDir(QStandardPaths::writableLocation(
@@ -487,12 +484,15 @@ QStringList BrowseFeature::getDefaultQuickLinks() const {
     bool osDownloadsDirIncluded = false;
     bool osDesktopDirIncluded = false;
     bool osDocumentsDirIncluded = false;
-    foreach (QString dirPath, mixxxMusicDirs) {
-        QDir dir(dirPath);
+    const auto rootDirs = m_pLibrary->trackCollectionManager()
+                                  ->internalCollection()
+                                  ->loadRootDirs();
+    for (mixxx::FileInfo fileInfo : rootDirs) {
         // Skip directories we don't have permission to.
-        if (!Sandbox::canAccessFile(dir)) {
+        if (!Sandbox::canAccess(&fileInfo)) {
             continue;
         }
+        const auto dir = fileInfo.toQDir();
         if (dir == osMusicDir) {
             osMusicDirIncluded = true;
         }
@@ -508,22 +508,20 @@ QStringList BrowseFeature::getDefaultQuickLinks() const {
         result << dir.canonicalPath() + "/";
     }
 
-    if (!osMusicDirIncluded && Sandbox::canAccessFile(osMusicDir)) {
+    if (!osMusicDirIncluded && Sandbox::canAccessDir(osMusicDir)) {
         result << osMusicDir.canonicalPath() + "/";
     }
 
     if (downloadsExists && !osDownloadsDirIncluded &&
-            Sandbox::canAccessFile(osDownloadsDir)) {
+            Sandbox::canAccessDir(osDownloadsDir)) {
         result << osDownloadsDir.canonicalPath() + "/";
     }
 
-    if (!osDesktopDirIncluded &&
-            Sandbox::canAccessFile(osDesktopDir)) {
+    if (!osDesktopDirIncluded && Sandbox::canAccessDir(osDesktopDir)) {
         result << osDesktopDir.canonicalPath() + "/";
     }
 
-    if (!osDocumentsDirIncluded &&
-            Sandbox::canAccessFile(osDocumentsDir)) {
+    if (!osDocumentsDirIncluded && Sandbox::canAccessDir(osDocumentsDir)) {
         result << osDocumentsDir.canonicalPath() + "/";
     }
 
