@@ -3,20 +3,15 @@
 #include <Shellapi.h>
 #include <Shlobj.h>
 #else
-#include <sys/types.h>
-#include <sys/stat.h>
 #include <dirent.h>
-#include <unistd.h>
-#include <errno.h>
 #endif
 
-#include <QFileInfo>
+#include <QFileInfoList>
 
 #include "library/browse/browsefeature.h"
 #include "library/browse/foldertreemodel.h"
 #include "library/treeitem.h"
 #include "moc_foldertreemodel.cpp"
-#include "util/file.h"
 
 FolderTreeModel::FolderTreeModel(QObject *parent)
         : TreeItemModel(parent) {
@@ -59,7 +54,7 @@ bool FolderTreeModel::directoryHasChildren(const QString& path) const {
     }
 
     // Acquire a security token for the path.
-    MDir dir(path);
+    const auto dirAccess = mixxx::FileAccess(mixxx::FileInfo(path));
 
     /*
      *  The following code is too expensive, general and SLOW since
@@ -90,7 +85,7 @@ bool FolderTreeModel::directoryHasChildren(const QString& path) const {
 
     std::string dot("."), dotdot("..");
     QByteArray byteArray = QFile::encodeName(path);
-    DIR *directory = opendir(byteArray);
+    DIR* directory = opendir(byteArray);
     int unknown_count = 0;
     int total_count = 0;
     if (directory != nullptr) {
