@@ -1,11 +1,9 @@
 #include "library/colordelegate.h"
 
-#include <QColor>
 #include <QPainter>
 #include <QStyle>
 #include <QTableView>
 
-#include "library/trackmodel.h"
 #include "moc_colordelegate.cpp"
 #include "util/color/rgbcolor.h"
 
@@ -30,21 +28,6 @@ void ColorDelegate::paintItem(
 
     // Draw a border if the color cell has focus
     if (option.state & QStyle::State_HasFocus) {
-        // This uses a color from the stylesheet:
-        // WTrackTableView {
-        //   qproperty-focusBorderColor: red;
-        // }
-        QPen borderPen(
-                m_pFocusBorderColor,
-                1,
-                Qt::SolidLine,
-                Qt::SquareCap);
-        painter->setPen(borderPen);
-        painter->setBrush(QBrush(Qt::transparent));
-        painter->drawRect(
-                option.rect.left(),
-                option.rect.top(),
-                option.rect.width() - 1,
-                option.rect.height() - 1);
+        drawBorder(painter, m_pFocusBorderColor, option.rect);
     }
 }
