@@ -1,16 +1,14 @@
 #pragma once
 
-#include <QItemSelection>
-
+#include "library/libraryview.h"
 #include "library/ui_dlghidden.h"
 #include "preferences/usersettings.h"
-#include "library/library.h"
-#include "library/libraryview.h"
-#include "controllers/keyboard/keyboardeventfilter.h"
 
 class WLibrary;
 class WTrackTableView;
 class HiddenTableModel;
+class Library;
+class KeyboardEventFilter;
 
 class DlgHidden : public QWidget, public Ui::DlgHidden, public LibraryView {
     Q_OBJECT
@@ -23,11 +21,13 @@ class DlgHidden : public QWidget, public Ui::DlgHidden, public LibraryView {
 
     void onShow() override;
     bool hasFocus() const override;
+    void setFocus() override;
     void onSearch(const QString& text) override;
     QString currentSearch();
+    void saveCurrentViewState() override;
+    bool restoreCurrentViewState() override;
 
   public slots:
-    void clicked();
     void selectAll();
     void selectionChanged(const QItemSelection&, const QItemSelection&);
 
