@@ -5,8 +5,14 @@
 #include "library/trackcollectionmanager.h"
 #include "moc_hiddentablemodel.cpp"
 
+namespace {
+
+const QString kModelName = "hidden:";
+
+} // anonymous namespace
+
 HiddenTableModel::HiddenTableModel(QObject* parent,
-                                   TrackCollectionManager* pTrackCollectionManager)
+        TrackCollectionManager* pTrackCollectionManager)
         : BaseSqlTableModel(parent, pTrackCollectionManager, "mixxx.db.model.missing") {
     setTableModel();
 }
@@ -14,21 +20,20 @@ HiddenTableModel::HiddenTableModel(QObject* parent,
 HiddenTableModel::~HiddenTableModel() {
 }
 
-void HiddenTableModel::setTableModel(int id) {
-    Q_UNUSED(id);
-    QSqlQuery query(m_database);
+void HiddenTableModel::setTableModel() {
     const QString tableName("hidden_songs");
 
     QStringList columns;
     columns << "library." + LIBRARYTABLE_ID;
-    QString filter("mixxx_deleted=1");
-    query.prepare("CREATE TEMPORARY VIEW IF NOT EXISTS " + tableName + " AS "
-                  "SELECT "
-                  + columns.join(",") +
-                  " FROM library "
-                  "INNER JOIN track_locations "
-                  "ON library.location=track_locations.id "
-                  "WHERE " + filter);
+
+    QSqlQuery query(m_database);
+    query.prepare(
+            "CREATE TEMPORARY VIEW IF NOT EXISTS " + tableName +
+            " AS SELECT " + columns.join(",") +
+            " FROM library "
+            "INNER JOIN track_locations "
+            "ON library.location=track_locations.id "
+            "WHERE mixxx_deleted=1");
     if (!query.exec()) {
         qDebug() << query.executedQuery() << query.lastError();
     }
@@ -65,34 +70,41 @@ void HiddenTableModel::unhideTracks(const QModelIndexList& indices) {
 
     m_pTrackCollectionManager->unhideTracks(trackIds);
 
-    // TODO(rryan) : do not select, instead route event to BTC and notify from
-    // there.
+    // TODO(rryan) : do not select, instead route event to BTC and notify from there.
     select(); // Repopulate the data model.
 }
 
 bool HiddenTableModel::isColumnInternal(int column) {
-    if (column == fieldIndex(ColumnCache::COLUMN_LIBRARYTABLE_ID) ||
+    return column == fieldIndex(ColumnCache::COLUMN_LIBRARYTABLE_ID) ||
             column == fieldIndex(ColumnCache::COLUMN_LIBRARYTABLE_PLAYED) ||
             column == fieldIndex(ColumnCache::COLUMN_LIBRARYTABLE_BPM_LOCK) ||
             column == fieldIndex(ColumnCache::COLUMN_LIBRARYTABLE_MIXXXDELETED) ||
-            column == fieldIndex(ColumnCache::COLUMN_LIBRARYTABLE_KEY_ID)||
+            column == fieldIndex(ColumnCache::COLUMN_LIBRARYTABLE_KEY_ID) ||
             column == fieldIndex(ColumnCache::COLUMN_TRACKLOCATIONSTABLE_FSDELETED) ||
             column == fieldIndex(ColumnCache::COLUMN_LIBRARYTABLE_COVERART_SOURCE) ||
             column == fieldIndex(ColumnCache::COLUMN_LIBRARYTABLE_COVERART_TYPE) ||
             column == fieldIndex(ColumnCache::COLUMN_LIBRARYTABLE_COVERART_LOCATION) ||
-            column == fieldIndex(ColumnCache::COLUMN_LIBRARYTABLE_COVERART_HASH)) {
-        return true;
-    }
-    return false;
+            column == fieldIndex(ColumnCache::COLUMN_LIBRARYTABLE_COVERART_COLOR) ||
+            column == fieldIndex(ColumnCache::COLUMN_LIBRARYTABLE_COVERART_DIGEST) ||
+            column == fieldIndex(ColumnCache::COLUMN_LIBRARYTABLE_COVERART_HASH);
 }
 
 // Override flags from BaseSqlModel since we don't want edit this model
-Qt::ItemFlags HiddenTableModel::flags(const QModelIndex &index) const {
+Qt::ItemFlags HiddenTableModel::flags(const QModelIndex& index) const {
     return readOnlyFlags(index);
 }
 
-TrackModel::CapabilitiesFlags HiddenTableModel::getCapabilities() const {
-    return TRACKMODELCAPS_NONE
-            | TRACKMODELCAPS_PURGE
-            | TRACKMODELCAPS_UNHIDE;
+TrackModel::Capabilities HiddenTableModel::getCapabilities() const {
+    return Capability::Purge |
+            Capability::Unhide |
+            Capability::RemoveFromDisk;
+}
+
+QString HiddenTableModel::modelKey(bool noSearch) const {
+    if (noSearch) {
+        return kModelName + m_tableName;
+    }
+    return kModelName + m_tableName +
+            QStringLiteral("#") +
+            currentSearch();
 }
