@@ -1,13 +1,14 @@
 #pragma once
 
 #include <QItemDelegate>
-#include <QModelIndex>
 #include <QObject>
-#include <QtSql>
 
 #include "library/basesqltablemodel.h"
 #include "library/trackmodel.h"
 
+class QModelIndex;
+typedef QList<QModelIndex> QModelIndexList;
+
 class MissingTableModel final : public BaseSqlTableModel {
     Q_OBJECT
   public:
@@ -18,6 +19,8 @@ class MissingTableModel final : public BaseSqlTableModel {
 
     bool isColumnInternal(int column) final;
     void purgeTracks(const QModelIndexList& indices) final;
-    Qt::ItemFlags flags(const QModelIndex &index) const final;
-    CapabilitiesFlags getCapabilities() const final;
+    Qt::ItemFlags flags(const QModelIndex& index) const final;
+    Capabilities getCapabilities() const final;
+
+    QString modelKey(bool noSearch) const override;
 };
