@@ -1,10 +1,11 @@
 #include "library/searchquery.h"
 
-#include <QtDebug>
+#include <QRegularExpression>
 
-#include "library/crate/crateschema.h"
 #include "library/dao/trackschema.h"
 #include "library/queryutil.h"
+#include "library/trackset/crate/crateschema.h"
+#include "library/trackset/crate/cratestorage.h" // for CrateTrackSelectResult
 #include "track/keyutils.h"
 #include "track/track.h"
 #include "util/db/dbconnection.h"
@@ -12,6 +13,17 @@
 
 namespace {
 
+const QRegularExpression kDurationRegex(QStringLiteral("^(\\d+)(m|:)?([0-5]?\\d)?s?$"));
+
+// The ordering of operator alternatives separated by '|' is crucial to avoid incomplete
+// partial matches, e.g. by capturing "<" + "=" + <arg>  instead of "<=" + <arg>!
+//
+// See also: https://perldoc.perl.org/perlre
+// > Alternatives are tried from left to right, so the first alternative found for which
+// > the entire expression matches, is the one that is chosen. This means that alternatives
+// > are not necessarily greedy.
+const QRegularExpression kNumericOperatorRegex(QStringLiteral("^(<=|>=|=|<|>)(.*)$"));
+
 QVariant getTrackValueForColumn(const TrackPointer& pTrack, const QString& column) {
     if (column == LIBRARYTABLE_ARTIST) {
         return pTrack->getArtist();
@@ -51,6 +63,8 @@ QVariant getTrackValueForColumn(const TrackPointer& pTrack, const QString& colum
         return pTrack->getPlayCounter().isPlayed();
     } else if (column == LIBRARYTABLE_TIMESPLAYED) {
         return pTrack->getPlayCounter().getTimesPlayed();
+    } else if (column == LIBRARYTABLE_LAST_PLAYED_AT) {
+        return pTrack->getLastPlayedAt();
     } else if (column == LIBRARYTABLE_RATING) {
         return pTrack->getRating();
     } else if (column == LIBRARYTABLE_KEY) {
@@ -85,7 +99,7 @@ QString concatSqlClauses(
 } // namespace
 
 bool AndNode::match(const TrackPointer& pTrack) const {
-    for (const auto& pNode: m_nodes) {
+    for (const auto& pNode : m_nodes) {
         if (!pNode->match(pTrack)) {
             return false;
         }
@@ -98,7 +112,7 @@ bool AndNode::match(const TrackPointer& pTrack) const {
 QString AndNode::toSql() const {
     QStringList queryFragments;
     queryFragments.reserve(static_cast<int>(m_nodes.size()));
-    for (const auto& pNode: m_nodes) {
+    for (const auto& pNode : m_nodes) {
         QString sql = pNode->toSql();
         if (!sql.isEmpty()) {
             queryFragments << sql;
@@ -116,7 +130,7 @@ bool OrNode::match(const TrackPointer& pTrack) const {
         // the generated SQL query.
         return true;
     }
-    for (const auto& pNode: m_nodes) {
+    for (const auto& pNode : m_nodes) {
         if (pNode->match(pTrack)) {
             return true;
         }
@@ -127,7 +141,7 @@ bool OrNode::match(const TrackPointer& pTrack) const {
 QString OrNode::toSql() const {
     QStringList queryFragments;
     queryFragments.reserve(static_cast<int>(m_nodes.size()));
-    for (const auto& pNode: m_nodes) {
+    for (const auto& pNode : m_nodes) {
         QString sql = pNode->toSql();
         if (!sql.isEmpty()) {
             queryFragments << sql;
@@ -153,25 +167,33 @@ QString NotNode::toSql() const {
 }
 
 TextFilterNode::TextFilterNode(const QSqlDatabase& database,
-               const QStringList& sqlColumns,
-               const QString& argument)
+        const QStringList& sqlColumns,
+        const QString& argument,
+        const StringMatch matchMode)
         : m_database(database),
           m_sqlColumns(sqlColumns),
-          m_argument(argument) {
+          m_argument(argument),
+          m_matchMode(matchMode) {
     mixxx::DbConnection::makeStringLatinLow(&m_argument);
 }
 
 bool TextFilterNode::match(const TrackPointer& pTrack) const {
-    for (const auto& sqlColumn: m_sqlColumns) {
+    for (const auto& sqlColumn : m_sqlColumns) {
         QVariant value = getTrackValueForColumn(pTrack, sqlColumn);
-        if (!value.isValid() || !value.canConvert(QMetaType::QString)) {
+        if (!value.isValid() || !value.canConvert<QString>()) {
             continue;
         }
 
         QString strValue = value.toString();
         mixxx::DbConnection::makeStringLatinLow(&strValue);
-        if (strValue.contains(m_argument)) {
-            return true;
+        if (m_matchMode == StringMatch::Equals) {
+            if (strValue == m_argument) {
+                return true;
+            }
+        } else {
+            if (strValue.contains(m_argument)) {
+                return true;
+            }
         }
     }
     return false;
@@ -187,10 +209,19 @@ QString TextFilterNode::toSql() const {
             argument.append('_');
         }
     }
-    QString escapedArgument = escaper.escapeString(
-            kSqlLikeMatchAll + argument + kSqlLikeMatchAll);
+    QString escapedArgument;
+    // Using a switch-case without default case to get a compile-time -Wswitch warning
+    switch (m_matchMode) {
+    case StringMatch::Contains:
+        escapedArgument = escaper.escapeString(
+                kSqlLikeMatchAll + argument + kSqlLikeMatchAll);
+        break;
+    case StringMatch::Equals:
+        escapedArgument = escaper.escapeString(argument);
+        break;
+    }
     QStringList searchClauses;
-    for (const auto& sqlColumn: m_sqlColumns) {
+    for (const auto& sqlColumn : m_sqlColumns) {
         searchClauses << QString("%1 LIKE %2").arg(sqlColumn, escapedArgument);
     }
     return concatSqlClauses(searchClauses, "OR");
@@ -200,7 +231,7 @@ bool NullOrEmptyTextFilterNode::match(const TrackPointer& pTrack) const {
     if (!m_sqlColumns.isEmpty()) {
         // only use the major column
         QVariant value = getTrackValueForColumn(pTrack, m_sqlColumns.first());
-        if (!value.isValid() || !value.canConvert(QMetaType::QString)) {
+        if (!value.isValid() || !value.canConvert<QString>()) {
             return true;
         }
         return value.toString().isEmpty();
@@ -217,16 +248,16 @@ QString NullOrEmptyTextFilterNode::toSql() const {
 }
 
 CrateFilterNode::CrateFilterNode(const CrateStorage* pCrateStorage,
-                                 const QString& crateNameLike)
-    : m_pCrateStorage(pCrateStorage),
-      m_crateNameLike(crateNameLike),
-      m_matchInitialized(false) {
+        const QString& crateNameLike)
+        : m_pCrateStorage(pCrateStorage),
+          m_crateNameLike(crateNameLike),
+          m_matchInitialized(false) {
 }
 
 bool CrateFilterNode::match(const TrackPointer& pTrack) const {
     if (!m_matchInitialized) {
         CrateTrackSelectResult crateTracks(
-             m_pCrateStorage->selectTracksSortedByCrateNameLike(m_crateNameLike));
+                m_pCrateStorage->selectTracksSortedByCrateNameLike(m_crateNameLike));
 
         while (crateTracks.next()) {
             m_matchingTrackIds.push_back(crateTracks.trackId());
@@ -239,14 +270,14 @@ bool CrateFilterNode::match(const TrackPointer& pTrack) const {
 }
 
 QString CrateFilterNode::toSql() const {
-    return QString("id IN (%1)").arg(
-            m_pCrateStorage->formatQueryForTrackIdsByCrateNameLike(m_crateNameLike));
+    return QString("id IN (%1)")
+            .arg(m_pCrateStorage->formatQueryForTrackIdsByCrateNameLike(
+                    m_crateNameLike));
 }
 
-
 NoCrateFilterNode::NoCrateFilterNode(const CrateStorage* pCrateStorage)
-    : m_pCrateStorage(pCrateStorage),
-      m_matchInitialized(false) {
+        : m_pCrateStorage(pCrateStorage),
+          m_matchInitialized(false) {
 }
 
 bool NoCrateFilterNode::match(const TrackPointer& pTrack) const {
@@ -265,9 +296,9 @@ bool NoCrateFilterNode::match(const TrackPointer& pTrack) const {
 }
 
 QString NoCrateFilterNode::toSql() const {
-    return QString("%1 NOT IN (%2)").arg(
-            CRATETABLE_ID,
-            CrateStorage::formatQueryForTrackIdsWithCrate());
+    return QString("%1 NOT IN (%2)")
+            .arg(CRATETABLE_ID,
+                    CrateStorage::formatQueryForTrackIdsWithCrate());
 }
 
 NumericFilterNode::NumericFilterNode(const QStringList& sqlColumns)
@@ -293,10 +324,10 @@ void NumericFilterNode::init(QString argument) {
         return;
     }
 
-    QRegExp operatorMatcher("^(>|>=|=|<|<=)(.*)$");
-    if (operatorMatcher.indexIn(argument) != -1) {
-        m_operator = operatorMatcher.cap(1);
-        argument = operatorMatcher.cap(2);
+    QRegularExpressionMatch match = kNumericOperatorRegex.match(argument);
+    if (match.hasMatch()) {
+        m_operator = match.captured(1);
+        argument = match.captured(2);
     }
 
     bool parsed = false;
@@ -319,14 +350,14 @@ void NumericFilterNode::init(QString argument) {
     }
 }
 
-double NumericFilterNode::parse(const QString& arg, bool *ok) {
+double NumericFilterNode::parse(const QString& arg, bool* ok) {
     return arg.toDouble(ok);
 }
 
 bool NumericFilterNode::match(const TrackPointer& pTrack) const {
-    for (const auto& sqlColumn: m_sqlColumns) {
+    for (const auto& sqlColumn : m_sqlColumns) {
         QVariant value = getTrackValueForColumn(pTrack, sqlColumn);
-        if (!value.isValid() || !value.canConvert(QMetaType::Double)) {
+        if (!value.isValid() || !value.canConvert<double>()) {
             if (m_bNullQuery) {
                 return true;
             }
@@ -336,14 +367,14 @@ bool NumericFilterNode::match(const TrackPointer& pTrack) const {
         double dValue = value.toDouble();
         if (m_bOperatorQuery) {
             if ((m_operator == "=" && dValue == m_dOperatorArgument) ||
-                (m_operator == "<" && dValue < m_dOperatorArgument) ||
-                (m_operator == ">" && dValue > m_dOperatorArgument) ||
-                (m_operator == "<=" && dValue <= m_dOperatorArgument) ||
-                (m_operator == ">=" && dValue >= m_dOperatorArgument)) {
+                    (m_operator == "<" && dValue < m_dOperatorArgument) ||
+                    (m_operator == ">" && dValue > m_dOperatorArgument) ||
+                    (m_operator == "<=" && dValue <= m_dOperatorArgument) ||
+                    (m_operator == ">=" && dValue >= m_dOperatorArgument)) {
                 return true;
             }
         } else if (m_bRangeQuery && dValue >= m_dRangeLow &&
-                   dValue <= m_dRangeHigh) {
+                dValue <= m_dRangeHigh) {
             return true;
         }
     }
@@ -352,7 +383,7 @@ bool NumericFilterNode::match(const TrackPointer& pTrack) const {
 
 QString NumericFilterNode::toSql() const {
     if (m_bNullQuery) {
-        for (const auto& sqlColumn: m_sqlColumns) {
+        for (const auto& sqlColumn : m_sqlColumns) {
             // only use the major column
             return QString("%1 IS NULL").arg(sqlColumn);
         }
@@ -361,16 +392,19 @@ QString NumericFilterNode::toSql() const {
 
     if (m_bOperatorQuery) {
         QStringList searchClauses;
-        for (const auto& sqlColumn: m_sqlColumns) {
-            searchClauses << QString("%1 %2 %3").arg(
-                sqlColumn, m_operator, QString::number(m_dOperatorArgument));
+        for (const auto& sqlColumn : m_sqlColumns) {
+            searchClauses << QString("%1 %2 %3")
+                                     .arg(sqlColumn,
+                                             m_operator,
+                                             QString::number(
+                                                     m_dOperatorArgument));
         }
         return concatSqlClauses(searchClauses, "OR");
     }
 
     if (m_bRangeQuery) {
         QStringList searchClauses;
-        for (const auto& sqlColumn: m_sqlColumns) {
+        for (const auto& sqlColumn : m_sqlColumns) {
             searchClauses << QString(QStringLiteral("%1 BETWEEN %2 AND %3"))
                                      .arg(sqlColumn,
                                              QString::number(m_dRangeLow),
@@ -390,7 +424,7 @@ bool NullNumericFilterNode::match(const TrackPointer& pTrack) const {
     if (!m_sqlColumns.isEmpty()) {
         // only use the major column
         QVariant value = getTrackValueForColumn(pTrack, m_sqlColumns.first());
-        if (!value.isValid() || !value.canConvert(QMetaType::Double)) {
+        if (!value.isValid() || !value.canConvert<double>()) {
             return true;
         }
     }
@@ -405,7 +439,6 @@ QString NullNumericFilterNode::toSql() const {
     return QString();
 }
 
-
 DurationFilterNode::DurationFilterNode(
         const QStringList& sqlColumns, const QString& argument)
         : NumericFilterNode(sqlColumns) {
@@ -415,8 +448,8 @@ DurationFilterNode::DurationFilterNode(
 }
 
 double DurationFilterNode::parse(const QString& arg, bool* ok) {
-    QRegExp regex("^(\\d*)(m|:)?([0-6]?\\d)?s?$");
-    if (regex.indexIn(arg) == -1) {
+    QRegularExpressionMatch match = kDurationRegex.match(arg);
+    if (!match.hasMatch()) {
         *ok = false;
         return 0;
     }
@@ -425,15 +458,14 @@ double DurationFilterNode::parse(const QString& arg, bool* ok) {
     // seconds are in the 4th entry. If you don't believe me or this doesn't
     // work anymore because we changed our Qt version just have a look at caps.
     // -- (kain88, Aug 2014)
-    QStringList caps = regex.capturedTexts();
     double m = 0;
     double s = 0;
     // if only a number is entered parse as seconds
-    if (caps.at(3).isEmpty() && caps.at(2).isEmpty()) {
-        s = caps.at(1).toDouble(ok);
+    if (match.captured(3).isEmpty() && match.captured(2).isEmpty()) {
+        s = match.captured(1).toDouble(ok);
     } else {
-        m = caps.at(1).toDouble(ok);
-        s = caps.at(3).toDouble();
+        m = match.captured(1).toDouble(ok);
+        s = match.captured(3).toDouble();
     }
 
     if (!*ok) {
@@ -445,7 +477,7 @@ double DurationFilterNode::parse(const QString& arg, bool* ok) {
 }
 
 KeyFilterNode::KeyFilterNode(mixxx::track::io::key::ChromaticKey key,
-                             bool fuzzy) {
+        bool fuzzy) {
     if (fuzzy) {
         m_matchKeys = KeyUtils::getCompatibleKeys(key);
     } else {
@@ -459,7 +491,7 @@ bool KeyFilterNode::match(const TrackPointer& pTrack) const {
 
 QString KeyFilterNode::toSql() const {
     QStringList searchClauses;
-    for (const auto& matchKey: m_matchKeys) {
+    for (const auto& matchKey : m_matchKeys) {
         searchClauses << QString("key_id IS %1").arg(QString::number(matchKey));
     }
     return concatSqlClauses(searchClauses, "OR");
@@ -482,7 +514,6 @@ QString YearFilterNode::toSql() const {
     }
 
     if (m_bRangeQuery) {
-        QStringList rangeClauses;
         return QString(
                 QStringLiteral("CAST(substr(year,1,4) AS INTEGER) BETWEEN %1 AND %2"))
                 .arg(QString::number(m_dRangeLow),
