@@ -2,31 +2,37 @@
 #define SEARCHQUERY_H
 
 #include <QList>
-#include <QRegExp>
 #include <QSqlDatabase>
 #include <QString>
 #include <QStringList>
 #include <utility>
 #include <vector>
 
-#include "library/crate/cratestorage.h"
 #include "proto/keys.pb.h"
 #include "track/track_decl.h"
 #include "util/assert.h"
 #include "util/memory.h"
 
+class CrateStorage;
+class TrackId;
+
 const QString kMissingFieldSearchTerm = "\"\""; // "" searches for an empty string
 
+enum class StringMatch {
+    Contains = 0,
+    Equals,
+};
+
 class QueryNode {
   public:
     QueryNode(const QueryNode&) = delete; // prevent copying
-    virtual ~QueryNode() {}
+    virtual ~QueryNode() = default;
 
     virtual bool match(const TrackPointer& pTrack) const = 0;
     virtual QString toSql() const = 0;
 
   protected:
-    QueryNode() {}
+    QueryNode() = default;
 };
 
 class GroupNode : public QueryNode {
@@ -58,7 +64,7 @@ class AndNode : public GroupNode {
 class NotNode : public QueryNode {
   public:
     explicit NotNode(std::unique_ptr<QueryNode> pNode)
-        : m_pNode(std::move(pNode)) {
+            : m_pNode(std::move(pNode)) {
         DEBUG_ASSERT(m_pNode);
     }
 
@@ -72,8 +78,9 @@ class NotNode : public QueryNode {
 class TextFilterNode : public QueryNode {
   public:
     TextFilterNode(const QSqlDatabase& database,
-                   const QStringList& sqlColumns,
-                   const QString& argument);
+            const QStringList& sqlColumns,
+            const QString& argument,
+            const StringMatch matchMode = StringMatch::Contains);
 
     bool match(const TrackPointer& pTrack) const override;
     QString toSql() const override;
@@ -82,12 +89,13 @@ class TextFilterNode : public QueryNode {
     QSqlDatabase m_database;
     QStringList m_sqlColumns;
     QString m_argument;
+    StringMatch m_matchMode;
 };
 
 class NullOrEmptyTextFilterNode : public QueryNode {
   public:
     NullOrEmptyTextFilterNode(const QSqlDatabase& database,
-                   const QStringList& sqlColumns)
+            const QStringList& sqlColumns)
             : m_database(database),
               m_sqlColumns(sqlColumns) {
     }
@@ -100,11 +108,10 @@ class NullOrEmptyTextFilterNode : public QueryNode {
     QStringList m_sqlColumns;
 };
 
-
 class CrateFilterNode : public QueryNode {
   public:
     CrateFilterNode(const CrateStorage* pCrateStorage,
-                    const QString& crateNameLike);
+            const QString& crateNameLike);
 
     bool match(const TrackPointer& pTrack) const override;
     QString toSql() const override;
@@ -147,7 +154,7 @@ class NumericFilterNode : public QueryNode {
     // derived classes.
     void init(QString argument);
 
-    virtual double parse(const QString& arg, bool *ok);
+    virtual double parse(const QString& arg, bool* ok);
 
     QStringList m_sqlColumns;
     bool m_bOperatorQuery;
