@@ -1,32 +1,42 @@
 #pragma once
 
-#include <QRegExp>
+#include <QRegularExpression>
 #include <QString>
-#include <QtSql>
+#include <memory>
 
 #include "library/searchquery.h"
-#include "library/trackcollection.h"
 #include "util/class.h"
 
+class TrackCollection;
+class QueryNode;
+class AndNode;
+
 class SearchQueryParser {
   public:
     explicit SearchQueryParser(TrackCollection* pTrackCollection, QStringList searchColumns);
 
-    virtual ~SearchQueryParser();
-
     void setSearchColumns(QStringList searchColumns);
 
     std::unique_ptr<QueryNode> parseQuery(
             const QString& query,
             const QString& extraFilter) const;
 
+    /// splits the query into a list of terms
+    static QStringList splitQueryIntoWords(const QString& query);
+    /// checks if the changed search query is less specific then the original term
+    static bool queryIsLessSpecific(const QString& original, const QString& changed);
 
   private:
     void parseTokens(QStringList tokens,
                      AndNode* pQuery) const;
 
-    QString getTextArgument(QString argument,
-                            QStringList* tokens) const;
+    struct TextArgumentResult {
+        QString argument;
+        StringMatch mode;
+    };
+
+    TextArgumentResult getTextArgument(QString argument,
+            QStringList* tokens) const;
 
     TrackCollection* m_pTrackCollection;
     QStringList m_queryColumns;
@@ -37,11 +47,11 @@ class SearchQueryParser {
     QStringList m_allFilters;
     QHash<QString, QStringList> m_fieldToSqlColumns;
 
-    QRegExp m_fuzzyMatcher;
-    QRegExp m_textFilterMatcher;
-    QRegExp m_crateFilterMatcher;
-    QRegExp m_numericFilterMatcher;
-    QRegExp m_specialFilterMatcher;
+    QRegularExpression m_fuzzyMatcher;
+    QRegularExpression m_textFilterMatcher;
+    QRegularExpression m_crateFilterMatcher;
+    QRegularExpression m_numericFilterMatcher;
+    QRegularExpression m_specialFilterMatcher;
 
     DISALLOW_COPY_AND_ASSIGN(SearchQueryParser);
 };
