@@ -1,8 +1,7 @@
 #include "library/trackcollection.h"
 
-#include <QApplication>
-
 #include "library/basetrackcache.h"
+#include "library/trackset/crate/crate.h"
 #include "moc_trackcollection.cpp"
 #include "track/globaltrackcache.h"
 #include "util/assert.h"
@@ -75,6 +74,7 @@ void TrackCollection::connectDatabase(const QSqlDatabase& database) {
     DEBUG_ASSERT_QOBJECT_THREAD_AFFINITY(this);
 
     kLogger.info() << "Connecting database";
+    DEBUG_ASSERT(database.isOpen());
     m_database = database;
     m_trackDao.initialize(database);
     m_playlistDao.initialize(database);
@@ -141,34 +141,41 @@ QWeakPointer<BaseTrackCache> TrackCollection::disconnectTrackSource() {
     return pWeakPtr;
 }
 
-bool TrackCollection::addDirectory(const QString& dir) {
+QList<mixxx::FileInfo> TrackCollection::loadRootDirs(bool skipInvalidOrMissing) const {
+    return m_directoryDao.loadAllDirectories(skipInvalidOrMissing);
+}
+
+bool TrackCollection::addDirectory(const mixxx::FileInfo& rootDir) {
     DEBUG_ASSERT_QOBJECT_THREAD_AFFINITY(this);
 
     SqlTransaction transaction(m_database);
-    switch (m_directoryDao.addDirectory(dir)) {
-    case SQL_ERROR:
-        return false;
-    case ALREADY_WATCHING:
-        return true;
-    case ALL_FINE:
+    switch (m_directoryDao.addDirectory(rootDir)) {
+    case DirectoryDAO::AddResult::Ok:
         transaction.commit();
         return true;
+    case DirectoryDAO::AddResult::AlreadyWatching:
+        return true;
+    case DirectoryDAO::AddResult::InvalidOrMissingDirectory:
+    case DirectoryDAO::AddResult::SqlError:
+        return false;
     default:
         DEBUG_ASSERT("unreachable");
     }
     return false;
 }
 
-bool TrackCollection::removeDirectory(const QString& dir) {
+bool TrackCollection::removeDirectory(const mixxx::FileInfo& rootDir) {
     DEBUG_ASSERT_QOBJECT_THREAD_AFFINITY(this);
 
     SqlTransaction transaction(m_database);
-    switch (m_directoryDao.removeDirectory(dir)) {
-    case SQL_ERROR:
-        return false;
-    case ALL_FINE:
+    switch (m_directoryDao.removeDirectory(rootDir)) {
+    case DirectoryDAO::RemoveResult::Ok:
         transaction.commit();
         return true;
+    case DirectoryDAO::RemoveResult::NotFound:
+        return true;
+    case DirectoryDAO::RemoveResult::SqlError:
+        return false;
     default:
         DEBUG_ASSERT("unreachable");
     }
@@ -184,7 +191,7 @@ void TrackCollection::relocateDirectory(const QString& oldDir, const QString& ne
     // have permission so that we can access the folder on future runs. We need
     // to canonicalize the path so we first wrap the directory string with a
     // QDir.
-    Sandbox::createSecurityToken(QDir(newDir));
+    Sandbox::createSecurityTokenForDir(QDir(newDir));
 
     SqlTransaction transaction(m_database);
     QList<RelocatedTrack> relocatedTracks =
@@ -203,9 +210,11 @@ void TrackCollection::relocateDirectory(const QString& oldDir, const QString& ne
 }
 
 QList<TrackId> TrackCollection::resolveTrackIds(
-        const QList<TrackFile>& trackFiles,
+        const QList<mixxx::FileInfo>& trackFiles,
         TrackDAO::ResolveTrackIdFlags flags) {
-    QList<TrackId> trackIds = m_trackDao.resolveTrackIds(trackFiles, flags);
+    QList<TrackId> trackIds = m_trackDao.resolveTrackIds(
+            trackFiles,
+            flags);
     if (flags & TrackDAO::ResolveTrackIdFlag::UnhideHidden) {
         unhideTracks(trackIds);
     }
@@ -213,8 +222,9 @@ QList<TrackId> TrackCollection::resolveTrackIds(
 }
 
 QList<TrackId> TrackCollection::resolveTrackIdsFromUrls(
-        const QList<QUrl>& urls, bool addMissing) {
-    QList<TrackFile> files = DragAndDropHelper::supportedTracksFromUrls(urls, false, true);
+        const QList<QUrl>& urls,
+        bool addMissing) {
+    QList<mixxx::FileInfo> files = DragAndDropHelper::supportedTracksFromUrls(urls, false, true);
     if (files.isEmpty()) {
         return QList<TrackId>();
     }
@@ -229,14 +239,14 @@ QList<TrackId> TrackCollection::resolveTrackIdsFromUrls(
 
 QList<TrackId> TrackCollection::resolveTrackIdsFromLocations(
         const QList<QString>& locations) {
-    QList<TrackFile> trackFiles;
-    trackFiles.reserve(locations.size());
+    QList<mixxx::FileInfo> fileInfos;
+    fileInfos.reserve(locations.size());
     for (const QString& location : locations) {
-        trackFiles.append(TrackFile(location));
+        fileInfos.append(mixxx::FileInfo(location));
     }
-    return resolveTrackIds(trackFiles,
-            TrackDAO::ResolveTrackIdFlag::UnhideHidden
-                    | TrackDAO::ResolveTrackIdFlag::AddMissing);
+    return resolveTrackIds(
+            fileInfos,
+            TrackDAO::ResolveTrackIdFlag::UnhideHidden | TrackDAO::ResolveTrackIdFlag::AddMissing);
 }
 
 bool TrackCollection::hideTracks(const QList<TrackId>& trackIds) {
@@ -247,7 +257,7 @@ bool TrackCollection::hideTracks(const QList<TrackId>& trackIds) {
     for (const auto& trackId: trackIds) {
         QSet<int> playlistIds;
         m_playlistDao.getPlaylistsTrackIsIn(trackId, &playlistIds);
-        for (const auto& playlistId : qAsConst(playlistIds)) {
+        for (const auto& playlistId : std::as_const(playlistIds)) {
             if (m_playlistDao.getHiddenType(playlistId) != PlaylistDAO::PLHT_SET_LOG) {
                 allPlaylistIds.insert(playlistId);
             }
@@ -521,10 +531,10 @@ bool TrackCollection::updateAutoDjCrate(
     return updateCrate(crate);
 }
 
-void TrackCollection::saveTrack(Track* pTrack) {
+bool TrackCollection::saveTrack(Track* pTrack) const {
     DEBUG_ASSERT_QOBJECT_THREAD_AFFINITY(this);
 
-    m_trackDao.saveTrack(pTrack);
+    return m_trackDao.saveTrack(pTrack);
 }
 
 TrackPointer TrackCollection::getTrackById(
