@@ -69,15 +69,14 @@ void TreeItem::insertChild(int row, std::unique_ptr<TreeItem> pChild) {
 }
 
 void TreeItem::initFeatureRecursively(LibraryFeature* pFeature) {
-    DEBUG_ASSERT(!m_pFeature ||
-            m_pFeature == pFeature);
     DEBUG_ASSERT(!m_pParent ||
             m_pParent->m_pFeature == pFeature);
     if (m_pFeature == pFeature) {
         return;
     }
+    DEBUG_ASSERT(!m_pFeature);
     m_pFeature = pFeature;
-    for (auto* pChild : qAsConst(m_children)) {
+    for (auto* pChild : std::as_const(m_children)) {
         pChild->initFeatureRecursively(pFeature);
     }
 }
