@@ -1,13 +1,14 @@
 #pragma once
 
+#include <QIcon>
 #include <QList>
 #include <QString>
 #include <QVariant>
 
-#include "library/libraryfeature.h"
-
+#include "util/assert.h"
 #include "util/memory.h"
 
+class LibraryFeature;
 
 class TreeItem final {
     struct PrivateRootTag {};
@@ -91,7 +92,7 @@ class TreeItem final {
     // take ownership of children items
     void insertChildren(int row, std::vector<std::unique_ptr<TreeItem>>&& children);
     void removeChildren(int row, int count);
-
+    void insertChild(int row, std::unique_ptr<TreeItem> pChild);
 
     /////////////////////////////////////////////////////////////////////////
     // Payload
@@ -131,7 +132,6 @@ class TreeItem final {
             QString label = QString(),
             QVariant data = QVariant());
 
-    void insertChild(int row, std::unique_ptr<TreeItem> pChild);
     void initFeatureRecursively(LibraryFeature* pFeature);
 
     // The library feature is inherited from the parent.
