@@ -130,8 +130,10 @@ QModelIndex TreeItemModel::parent(const QModelIndex& index) const {
 
     TreeItem *childItem = static_cast<TreeItem*>(index.internalPointer());
     TreeItem *parentItem = childItem->parent();
-    if (parentItem == getRootItem()) {
+    if (!parentItem) {
         return QModelIndex();
+    } else if (parentItem == getRootItem()) {
+        return createIndex(0, 0, getRootItem());
     } else {
         return createIndex(parentItem->parentRow(), 0, parentItem);
     }
@@ -162,6 +164,10 @@ TreeItem* TreeItemModel::setRootItem(std::unique_ptr<TreeItem> pRootItem) {
     return getRootItem();
 }
 
+const QModelIndex TreeItemModel::getRootIndex() {
+    return createIndex(0, 0, getRootItem());
+}
+
 /**
  * Before you can resize the data model dynamically by using 'insertRows' and 'removeRows'
  * make sure you have initialized.
@@ -177,7 +183,7 @@ void TreeItemModel::insertTreeItemRows(
     TreeItem* pParentItem = getItem(parent);
     DEBUG_ASSERT(pParentItem != nullptr);
 
-    beginInsertRows(parent, position, position + rows.size() - 1);
+    beginInsertRows(parent, position, position + static_cast<int>(rows.size()) - 1);
     pParentItem->insertChildren(position, std::move(rows));
     endInsertRows();
 }
