@@ -3,7 +3,6 @@
 #include <QAbstractItemModel>
 #include <QModelIndex>
 #include <QVariant>
-#include <QList>
 
 #include "util/memory.h"
 
@@ -39,6 +38,8 @@ class TreeItemModel : public QAbstractItemModel {
     TreeItem* getRootItem() const {
         return m_pRootItem.get();
     }
+    /// Returns the QModelIndex of the Root element.
+    const QModelIndex getRootIndex();
 
     // Return the underlying TreeItem.
     // If the index is invalid, the root item is returned.
