@@ -1,19 +1,15 @@
-#ifndef MIXER_SAMPLER_H
-#define MIXER_SAMPLER_H
+#pragma once
 
 #include "mixer/basetrackplayer.h"
 
 class Sampler : public BaseTrackPlayerImpl {
     Q_OBJECT
   public:
-    Sampler(QObject* pParent,
+    Sampler(PlayerManager* pParent,
             UserSettingsPointer pConfig,
-            EngineMaster* pMixingEngine,
+            EngineMixer* pMixingEngine,
             EffectsManager* pEffectsManager,
-            VisualsManager* pVisualsManager,
             EngineChannel::ChannelOrientation defaultOrientation,
-            const QString& group);
+            const ChannelHandleAndGroup& handleGroup);
     ~Sampler() override = default;
 };
-
-#endif /* MIXER_SAMPLER_H */
