@@ -10,7 +10,9 @@ class MixxxApplication : public QApplication {
     MixxxApplication(int& argc, char** argv);
     ~MixxxApplication() override = default;
 
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     bool notify(QObject*, QEvent*) override;
+#endif
 
   private:
     bool touchIsRightButton();
