@@ -2,7 +2,6 @@
 
 #include <QList>
 #include <QString>
-#include <QWidget>
 
 #include "analyzer/plugins/analyzerplugin.h"
 #include "preferences/beatdetectionsettings.h"
@@ -10,6 +9,8 @@
 #include "preferences/dialog/ui_dlgprefbeatsdlg.h"
 #include "preferences/usersettings.h"
 
+class QWidget;
+
 class DlgPrefBeats : public DlgPreferencePage, public Ui::DlgBeatsDlg {
     Q_OBJECT
   public:
