@@ -1,26 +1,17 @@
 #include "preferences/dialog/dlgprefdeck.h"
 
-#include <QDir>
 #include <QDoubleSpinBox>
-#include <QList>
-#include <QLocale>
-#include <QToolTip>
-#include <QWidget>
 
 #include "control/controlobject.h"
 #include "control/controlproxy.h"
 #include "defs_urls.h"
 #include "engine/controls/ratecontrol.h"
-#include "engine/enginebuffer.h"
+#include "engine/sync/enginesync.h"
 #include "mixer/basetrackplayer.h"
-#include "mixer/playerinfo.h"
 #include "mixer/playermanager.h"
-#include "mixxx.h"
 #include "moc_dlgprefdeck.cpp"
 #include "preferences/usersettings.h"
-#include "util/compatibility.h"
 #include "util/duration.h"
-#include "widget/wnumberpos.h"
 
 namespace {
 constexpr int kDefaultRateRangePercent = 8;
@@ -33,30 +24,31 @@ constexpr double kDefaultPermanentRateChangeFine = 0.05;
 constexpr int kDefaultRateRampSensitivity = 250;
 // bool kDefaultCloneDeckOnLoad is defined in header file to make it available
 // to playermanager.cpp
+const QString kAppGroup = QStringLiteral("[App]");
 } // namespace
 
-DlgPrefDeck::DlgPrefDeck(QWidget* parent,
-        UserSettingsPointer pConfig)
+DlgPrefDeck::DlgPrefDeck(QWidget* parent, UserSettingsPointer pConfig)
         : DlgPreferencePage(parent),
           m_pConfig(pConfig),
           m_pControlTrackTimeDisplay(std::make_unique<ControlObject>(
                   ConfigKey("[Controls]", "ShowDurationRemaining"))),
           m_pControlTrackTimeFormat(std::make_unique<ControlObject>(
                   ConfigKey("[Controls]", "TimeFormat"))),
-          m_pNumDecks(
-                  make_parented<ControlProxy>("[Master]", "num_decks", this)),
+          m_pNumDecks(make_parented<ControlProxy>(
+                  kAppGroup, QStringLiteral("num_decks"), this)),
           m_pNumSamplers(make_parented<ControlProxy>(
-                  "[Master]", "num_samplers", this)),
+                  kAppGroup, QStringLiteral("num_samplers"), this)),
           m_iNumConfiguredDecks(0),
           m_iNumConfiguredSamplers(0) {
     setupUi(this);
     // Create text color for the cue mode link "?" to the manual
     createLinkColor();
 
-    m_pNumDecks->connectValueChanged(this, [=](double value){slotNumDecksChanged(value);});
+    m_pNumDecks->connectValueChanged(this, [=, this](double value) { slotNumDecksChanged(value); });
     slotNumDecksChanged(m_pNumDecks->get(), true);
 
-    m_pNumSamplers->connectValueChanged(this, [=](double value){slotNumSamplersChanged(value);});
+    m_pNumSamplers->connectValueChanged(
+            this, [=, this](double value) { slotNumSamplersChanged(value); });
     slotNumSamplersChanged(m_pNumSamplers->get(), true);
 
     // Set default value in config file and control objects, if not present
@@ -74,7 +66,7 @@ DlgPrefDeck::DlgPrefDeck(QWidget* parent,
     const int cueModeIndex = cueDefaultIndexByData(cueDefaultValue);
     ComboBoxCueMode->setCurrentIndex(cueModeIndex);
     slotCueModeCombobox(cueModeIndex);
-    for (ControlProxy* pControl : qAsConst(m_cueControls)) {
+    for (ControlProxy* pControl : std::as_const(m_cueControls)) {
         pControl->set(static_cast<int>(m_cueMode));
     }
     connect(ComboBoxCueMode,
@@ -154,16 +146,6 @@ DlgPrefDeck::DlgPrefDeck(QWidget* parent,
     comboBoxTimeFormat->setCurrentIndex(
                 comboBoxTimeFormat->findData(time_format));
 
-    // Override Playing Track on Track Load
-    // The check box reflects the opposite of the config value
-    m_bDisallowTrackLoadToPlayingDeck = !m_pConfig->getValue(
-            ConfigKey("[Controls]", "AllowTrackLoadToPlayingDeck"), false);
-    checkBoxDisallowLoadToPlayingDeck->setChecked(m_bDisallowTrackLoadToPlayingDeck);
-    connect(checkBoxDisallowLoadToPlayingDeck,
-            &QCheckBox::toggled,
-            this,
-            &DlgPrefDeck::slotDisallowTrackLoadToPlayingDeckCheckbox);
-
     comboBoxLoadPoint->addItem(tr("Intro start"), static_cast<int>(SeekOnLoadMode::IntroStart));
     comboBoxLoadPoint->addItem(tr("Main cue"), static_cast<int>(SeekOnLoadMode::MainCue));
     comboBoxLoadPoint->addItem(tr("First sound (skip silence)"), static_cast<int>(SeekOnLoadMode::FirstSound));
@@ -179,6 +161,31 @@ DlgPrefDeck::DlgPrefDeck(QWidget* parent,
             this,
             &DlgPrefDeck::slotSetTrackLoadMode);
 
+    comboBoxLoadWhenDeckPlaying->addItem(
+            tr("Reject"), static_cast<int>(LoadWhenDeckPlaying::Reject));
+    comboBoxLoadWhenDeckPlaying->addItem(tr("Allow, but stop deck"),
+            static_cast<int>(LoadWhenDeckPlaying::AllowButStopDeck));
+    comboBoxLoadWhenDeckPlaying->addItem(tr("Allow, play from load point"),
+            static_cast<int>(LoadWhenDeckPlaying::Allow));
+    int loadWhenDeckPlaying;
+    if (m_pConfig->exists(kConfigKeyLoadWhenDeckPlaying)) {
+        loadWhenDeckPlaying = m_pConfig->getValueString(kConfigKeyLoadWhenDeckPlaying).toInt();
+    } else {
+        // upgrade from older versions
+        if (m_pConfig->getValue(kConfigKeyAllowTrackLoadToPlayingDeck, false)) {
+            loadWhenDeckPlaying = static_cast<int>(LoadWhenDeckPlaying::Allow);
+        } else {
+            loadWhenDeckPlaying = static_cast<int>(kDefaultLoadWhenDeckPlaying);
+        }
+    }
+    comboBoxLoadWhenDeckPlaying->setCurrentIndex(
+            comboBoxLoadWhenDeckPlaying->findData(loadWhenDeckPlaying));
+    m_loadWhenDeckPlaying = static_cast<LoadWhenDeckPlaying>(loadWhenDeckPlaying);
+    connect(comboBoxLoadWhenDeckPlaying,
+            QOverload<int>::of(&QComboBox::currentIndexChanged),
+            this,
+            &DlgPrefDeck::slotLoadWhenDeckPlayingIndexChanged);
+
     // This option was introduced in Mixxx 2.3 with the intro & outro cues.
     // If the user has set main cue points with the intention of starting tracks
     // from those points, enable this option. With Denon and Numark CueModes,
@@ -253,9 +260,7 @@ DlgPrefDeck::DlgPrefDeck(QWidget* parent,
     }
     setRateRangeForAllDecks(m_iRateRangePercent);
 
-    //
     // Key lock mode
-    //
     connect(buttonGroupKeyLockMode,
             QOverload<QAbstractButton*>::of(&QButtonGroup::buttonClicked),
             this,
@@ -264,13 +269,11 @@ DlgPrefDeck::DlgPrefDeck(QWidget* parent,
     m_keylockMode = static_cast<KeylockMode>(
         m_pConfig->getValue(ConfigKey("[Controls]", "keylockMode"),
                             static_cast<int>(KeylockMode::LockOriginalKey)));
-    for (ControlProxy* pControl : qAsConst(m_keylockModeControls)) {
+    for (ControlProxy* pControl : std::as_const(m_keylockModeControls)) {
         pControl->set(static_cast<double>(m_keylockMode));
     }
 
-    //
     // Key unlock mode
-    //
     connect(buttonGroupKeyUnlockMode,
             QOverload<QAbstractButton*>::of(&QButtonGroup::buttonClicked),
             this,
@@ -279,25 +282,27 @@ DlgPrefDeck::DlgPrefDeck(QWidget* parent,
     m_keyunlockMode = static_cast<KeyunlockMode>(
         m_pConfig->getValue(ConfigKey("[Controls]", "keyunlockMode"),
         static_cast<int>(KeyunlockMode::ResetLockedKey)));
-    for (ControlProxy* pControl : qAsConst(m_keyunlockModeControls)) {
+    for (ControlProxy* pControl : std::as_const(m_keyunlockModeControls)) {
         pControl->set(static_cast<int>(m_keyunlockMode));
     }
 
-    //
     // Cue Mode
-    //
-
     // Add "(?)" with a manual link to the label
-    labelCueMode->setText(labelCueMode->text() + QStringLiteral(" ") +
+    labelCueMode->setText(labelCueMode->text() + QChar(' ') +
             coloredLinkString(
                     m_pLinkColor,
                     QStringLiteral("(?)"),
                     MIXXX_MANUAL_CUE_MODES_URL));
 
-    //
-    // Speed / Pitch reset configuration
-    //
+    // Sync Mode
+    // Add "(?)" with a manual link to the label
+    labelSyncMode->setText(labelSyncMode->text() + QChar(' ') +
+            coloredLinkString(
+                    m_pLinkColor,
+                    QStringLiteral("(?)"),
+                    MIXXX_MANUAL_SYNC_MODES_URL));
 
+    // Speed / Pitch reset configuration
     // Update "reset speed" and "reset pitch" check boxes
     // TODO: All defaults should only be set in slotResetToDefaults.
     int configSPAutoReset = m_pConfig->getValue<int>(
@@ -319,16 +324,7 @@ DlgPrefDeck::DlgPrefDeck(QWidget* parent,
     // Ramping Temporary Rate Change configuration
     //
 
-    // Rate Ramp Sensitivity slider & spinbox
-    connect(SliderRateRampSensitivity,
-            QOverload<int>::of(&QAbstractSlider::valueChanged),
-            SpinBoxRateRampSensitivity,
-            QOverload<int>::of(&QSpinBox::setValue));
-    connect(SpinBoxRateRampSensitivity,
-            QOverload<int>::of(&QSpinBox::valueChanged),
-            SliderRateRampSensitivity,
-            QOverload<int>::of(&QAbstractSlider::setValue));
-
+    // Rate Ramp Sensitivity slider
     m_iRateRampSensitivity =
             m_pConfig->getValue(ConfigKey("[Controls]", "RateRampSensitivity"),
                     kDefaultRateRampSensitivity);
@@ -347,10 +343,6 @@ DlgPrefDeck::DlgPrefDeck(QWidget* parent,
             &QRadioButton::toggled,
             SliderRateRampSensitivity,
             &QWidget::setEnabled);
-    connect(radioButtonRateRampModeLinear,
-            &QRadioButton::toggled,
-            SpinBoxRateRampSensitivity,
-            &QWidget::setEnabled);
     // Enable/disable temporary rate spinboxes when abrupt ramping is selected
     connect(radioButtonRateRampModeStepping,
             &QRadioButton::toggled,
@@ -433,9 +425,6 @@ void DlgPrefDeck::slotUpdate() {
 
     slotSetTrackTimeDisplay(m_pControlTrackTimeDisplay->get());
 
-    checkBoxDisallowLoadToPlayingDeck->setChecked(!m_pConfig->getValue(
-            ConfigKey("[Controls]", "AllowTrackLoadToPlayingDeck"), false));
-
     checkBoxCloneDeckOnLoadDoubleTap->setChecked(m_pConfig->getValue(
             ConfigKey("[Controls]", "CloneDeckOnLoadDoubleTap"), true));
 
@@ -454,6 +443,16 @@ void DlgPrefDeck::slotUpdate() {
     index = ComboBoxCueMode->findData(static_cast<int>(cueMode));
     ComboBoxCueMode->setCurrentIndex(index);
 
+    const EngineSync::SyncLockAlgorithm syncLockAlgorithm =
+            static_cast<EngineSync::SyncLockAlgorithm>(m_pConfig->getValue<int>(
+                    ConfigKey(kBpmConfigGroup, kSyncLockAlgorithmConfigKey),
+                    EngineSync::SyncLockAlgorithm::PREFER_SOFT_LEADER));
+    if (syncLockAlgorithm == EngineSync::SyncLockAlgorithm::PREFER_SOFT_LEADER) {
+        radioButtonSoftLeader->setChecked(true);
+    } else {
+        radioButtonLockBpm->setChecked(true);
+    }
+
     KeylockMode keylockMode =
             static_cast<KeylockMode>(static_cast<int>(m_keylockModeControls[0]->get()));
     if (keylockMode == KeylockMode::LockCurrentKey) {
@@ -512,14 +511,15 @@ void DlgPrefDeck::slotResetToDefaults() {
     // 8% Rate Range
     ComboBoxRateRange->setCurrentIndex(ComboBoxRateRange->findData(kDefaultRateRangePercent));
 
-    // Don't load tracks into playing decks.
-    checkBoxDisallowLoadToPlayingDeck->setChecked(true);
-
     // Clone decks by double-tapping Load button.
     checkBoxCloneDeckOnLoadDoubleTap->setChecked(kDefaultCloneDeckOnLoad);
+
     // Mixxx cue mode
     ComboBoxCueMode->setCurrentIndex(0);
 
+    // What to do if someone loads into a playing deck
+    comboBoxLoadWhenDeckPlaying->setCurrentIndex(static_cast<int>(kDefaultLoadWhenDeckPlaying));
+
     // Load at intro start
     comboBoxLoadPoint->setCurrentIndex(
             comboBoxLoadPoint->findData(static_cast<int>(SeekOnLoadMode::IntroStart)));
@@ -538,6 +538,8 @@ void DlgPrefDeck::slotResetToDefaults() {
     checkBoxResetSpeed->setChecked(false);
     checkBoxResetPitch->setChecked(true);
 
+    radioButtonSoftLeader->setChecked(true);
+
     radioButtonOriginalKey->setChecked(true);
     radioButtonResetUnlockedKey->setChecked(true);
 }
@@ -551,7 +553,7 @@ void DlgPrefDeck::slotRateRangeComboBox(int index) {
 }
 
 void DlgPrefDeck::setRateRangeForAllDecks(int rangePercent) {
-    for (ControlProxy* pControl : qAsConst(m_rateRangeControls)) {
+    for (ControlProxy* pControl : std::as_const(m_rateRangeControls)) {
         pControl->set(rangePercent / 100.0);
     }
 }
@@ -566,14 +568,14 @@ void DlgPrefDeck::setRateDirectionForAllDecks(bool inverted) {
     if (inverted) {
         rateDirectionMultiplier = kRateDirectionInverted;
     }
-    for (ControlProxy* pControl : qAsConst(m_rateDirectionControls)) {
+    for (ControlProxy* pControl : std::as_const(m_rateDirectionControls)) {
         pControl->set(rateDirectionMultiplier);
     }
 
     // If the rate slider direction setting has changed,
     // multiply the rate by -1 so the current sound does not change.
     if (rateDirectionMultiplier != oldRateDirectionMultiplier) {
-        for (ControlProxy* pControl : qAsConst(m_rateControls)) {
+        for (ControlProxy* pControl : std::as_const(m_rateControls)) {
             pControl->set(-1 * pControl->get());
         }
     }
@@ -595,10 +597,6 @@ void DlgPrefDeck::slotKeyUnlockModeSelected(QAbstractButton* pressedButton) {
     }
 }
 
-void DlgPrefDeck::slotDisallowTrackLoadToPlayingDeckCheckbox(bool checked) {
-    m_bDisallowTrackLoadToPlayingDeck = checked;
-}
-
 void DlgPrefDeck::slotCueModeCombobox(int index) {
     m_cueMode = static_cast<CueMode>(ComboBoxCueMode->itemData(index).toInt());
 }
@@ -669,6 +667,11 @@ void DlgPrefDeck::slotSetTrackLoadMode(int comboboxIndex) {
             comboBoxLoadPoint->itemData(comboboxIndex).toInt());
 }
 
+void DlgPrefDeck::slotLoadWhenDeckPlayingIndexChanged(int comboboxIndex) {
+    m_loadWhenDeckPlaying = static_cast<LoadWhenDeckPlaying>(
+            comboBoxLoadWhenDeckPlaying->itemData(comboboxIndex).toInt());
+}
+
 void DlgPrefDeck::slotApply() {
     m_pConfig->set(ConfigKey("[Controls]", "SetIntroStartAtMainCue"),
             ConfigValue(m_bSetIntroStartAtMainCue));
@@ -683,13 +686,12 @@ void DlgPrefDeck::slotApply() {
     m_pConfig->setValue(ConfigKey("[Controls]", "TimeFormat"), timeFormat);
 
     // Set cue mode for every deck
-    for (ControlProxy* pControl : qAsConst(m_cueControls)) {
+    for (ControlProxy* pControl : std::as_const(m_cueControls)) {
         pControl->set(static_cast<int>(m_cueMode));
     }
     m_pConfig->setValue(ConfigKey("[Controls]", "CueDefault"), static_cast<int>(m_cueMode));
 
-    m_pConfig->setValue(ConfigKey("[Controls]", "AllowTrackLoadToPlayingDeck"),
-                        !m_bDisallowTrackLoadToPlayingDeck);
+    m_pConfig->setValue(kConfigKeyLoadWhenDeckPlaying, static_cast<int>(m_loadWhenDeckPlaying));
 
     m_pConfig->setValue(ConfigKey("[Controls]", "CueRecall"), static_cast<int>(m_seekOnLoadMode));
     m_pConfig->setValue(ConfigKey("[Controls]", "CloneDeckOnLoadDoubleTap"),
@@ -717,17 +719,25 @@ void DlgPrefDeck::slotApply() {
     m_pConfig->set(ConfigKey("[Controls]", "SpeedAutoReset"),
                    ConfigValue(configSPAutoReset));
 
+    if (radioButtonSoftLeader->isChecked()) {
+        m_pConfig->setValue(ConfigKey(kBpmConfigGroup, kSyncLockAlgorithmConfigKey),
+                static_cast<int>(EngineSync::SyncLockAlgorithm::PREFER_SOFT_LEADER));
+    } else {
+        m_pConfig->setValue(ConfigKey(kBpmConfigGroup, kSyncLockAlgorithmConfigKey),
+                static_cast<int>(EngineSync::SyncLockAlgorithm::PREFER_LOCK_BPM));
+    }
+
     m_pConfig->setValue(ConfigKey("[Controls]", "keylockMode"),
                         static_cast<int>(m_keylockMode));
     // Set key lock behavior for every group
-    for (ControlProxy* pControl : qAsConst(m_keylockModeControls)) {
+    for (ControlProxy* pControl : std::as_const(m_keylockModeControls)) {
         pControl->set(static_cast<double>(m_keylockMode));
     }
 
     m_pConfig->setValue(ConfigKey("[Controls]", "keyunlockMode"),
                         static_cast<int>(m_keyunlockMode));
     // Set key un-lock behavior for every group
-    for (ControlProxy* pControl : qAsConst(m_keyunlockModeControls)) {
+    for (ControlProxy* pControl : std::as_const(m_keyunlockModeControls)) {
         pControl->set(static_cast<double>(m_keyunlockMode));
     }
 
