@@ -2,10 +2,12 @@
 
 #include <QMessageBox>
 #include <QtDebug>
+#include <algorithm>
+#include <vector>
 
 #include "control/controlproxy.h"
 #include "engine/enginebuffer.h"
-#include "engine/enginemaster.h"
+#include "engine/enginemixer.h"
 #include "mixer/playermanager.h"
 #include "moc_dlgprefsound.cpp"
 #include "preferences/dialog/dlgprefsounditem.h"
@@ -13,17 +15,34 @@
 #include "util/rlimit.h"
 #include "util/scopedoverridecursor.h"
 
-/**
- * Construct a new sound preferences pane. Initializes and populates all the
- * all the controls to the values obtained from SoundManager.
- */
+namespace {
+
+const QString kAppGroup = QStringLiteral("[App]");
+
+bool soundItemAlreadyExists(const AudioPath& output, const QWidget& widget) {
+    for (const QObject* pObj : widget.children()) {
+        const auto* pItem = qobject_cast<const DlgPrefSoundItem*>(pObj);
+        if (!pItem || pItem->type() != output.getType()) {
+            continue;
+        }
+        if (!AudioPath::isIndexed(pItem->type()) || pItem->index() == output.getIndex()) {
+            return true;
+        }
+    }
+    return false;
+}
+
+} // namespace
+
+/// Construct a new sound preferences pane. Initializes and populates
+/// all the controls to the values obtained from SoundManager.
 DlgPrefSound::DlgPrefSound(QWidget* pParent,
-        SoundManager* pSoundManager,
+        std::shared_ptr<SoundManager> pSoundManager,
         UserSettingsPointer pSettings)
         : DlgPreferencePage(pParent),
           m_pSoundManager(pSoundManager),
           m_pSettings(pSettings),
-          m_config(pSoundManager),
+          m_config(pSoundManager.get()),
           m_settingsModified(false),
           m_bLatencyChanged(false),
           m_bSkipConfigClear(true),
@@ -32,7 +51,7 @@ DlgPrefSound::DlgPrefSound(QWidget* pParent,
     // Create text color for the wiki links
     createLinkColor();
 
-    connect(m_pSoundManager,
+    connect(m_pSoundManager.get(),
             &SoundManager::devicesUpdated,
             this,
             &DlgPrefSound::refreshDevices);
@@ -83,20 +102,21 @@ DlgPrefSound::DlgPrefSound(QWidget* pParent,
             &DlgPrefSound::engineClockChanged);
 
     keylockComboBox->clear();
-    for (int i = 0; i < EngineBuffer::KEYLOCK_ENGINE_COUNT; ++i) {
-        keylockComboBox->addItem(
-                EngineBuffer::getKeylockEngineName(
-                        static_cast<EngineBuffer::KeylockEngine>(i)));
+    for (const auto engine : EngineBuffer::kKeylockEngines) {
+        if (EngineBuffer::isKeylockEngineAvailable(engine)) {
+            keylockComboBox->addItem(
+                    EngineBuffer::getKeylockEngineName(engine), QVariant::fromValue(engine));
+        }
     }
 
     m_pLatencyCompensation = new ControlProxy("[Master]", "microphoneLatencyCompensation", this);
-    m_pMasterDelay = new ControlProxy("[Master]", "delay", this);
+    m_pMainDelay = new ControlProxy("[Master]", "delay", this);
     m_pHeadDelay = new ControlProxy("[Master]", "headDelay", this);
     m_pBoothDelay = new ControlProxy("[Master]", "boothDelay", this);
 
     latencyCompensationSpinBox->setValue(m_pLatencyCompensation->get());
     latencyCompensationWarningLabel->setWordWrap(true);
-    masterDelaySpinBox->setValue(m_pMasterDelay->get());
+    mainDelaySpinBox->setValue(m_pMainDelay->get());
     headDelaySpinBox->setValue(m_pHeadDelay->get());
     boothDelaySpinBox->setValue(m_pBoothDelay->get());
 
@@ -104,10 +124,10 @@ DlgPrefSound::DlgPrefSound(QWidget* pParent,
             QOverload<double>::of(&QDoubleSpinBox::valueChanged),
             this,
             &DlgPrefSound::latencyCompensationSpinboxChanged);
-    connect(masterDelaySpinBox,
+    connect(mainDelaySpinBox,
             QOverload<double>::of(&QDoubleSpinBox::valueChanged),
             this,
-            &DlgPrefSound::masterDelaySpinboxChanged);
+            &DlgPrefSound::mainDelaySpinboxChanged);
     connect(headDelaySpinBox,
             QOverload<double>::of(&QDoubleSpinBox::valueChanged),
             this,
@@ -118,12 +138,12 @@ DlgPrefSound::DlgPrefSound(QWidget* pParent,
             &DlgPrefSound::boothDelaySpinboxChanged);
 
     m_pMicMonitorMode = new ControlProxy("[Master]", "talkover_mix", this);
-    micMonitorModeComboBox->addItem(tr("Master output only"),
-        QVariant(static_cast<int>(EngineMaster::MicMonitorMode::MASTER)));
-    micMonitorModeComboBox->addItem(tr("Master and booth outputs"),
-        QVariant(static_cast<int>(EngineMaster::MicMonitorMode::MASTER_AND_BOOTH)));
+    micMonitorModeComboBox->addItem(tr("Main output only"),
+            QVariant(static_cast<int>(EngineMixer::MicMonitorMode::Main)));
+    micMonitorModeComboBox->addItem(tr("Main and booth outputs"),
+            QVariant(static_cast<int>(EngineMixer::MicMonitorMode::MainAndBooth)));
     micMonitorModeComboBox->addItem(tr("Direct monitor (recording and broadcasting only)"),
-        QVariant(static_cast<int>(EngineMaster::MicMonitorMode::DIRECT_MONITOR)));
+            QVariant(static_cast<int>(EngineMixer::MicMonitorMode::DirectMonitor)));
     int modeIndex = micMonitorModeComboBox->findData(
         static_cast<int>(m_pMicMonitorMode->get()));
     micMonitorModeComboBox->setCurrentIndex(modeIndex);
@@ -163,7 +183,7 @@ DlgPrefSound::DlgPrefSound(QWidget* pParent,
 
     connect(queryButton, &QAbstractButton::clicked, this, &DlgPrefSound::queryClicked);
 
-    connect(m_pSoundManager,
+    connect(m_pSoundManager.get(),
             &SoundManager::outputRegistered,
             this,
             [this](const AudioOutput& output, AudioSource* source) {
@@ -172,7 +192,7 @@ DlgPrefSound::DlgPrefSound(QWidget* pParent,
                 loadSettings();
             });
 
-    connect(m_pSoundManager,
+    connect(m_pSoundManager.get(),
             &SoundManager::inputRegistered,
             this,
             [this](const AudioInput& input, AudioDestination* dest) {
@@ -181,37 +201,37 @@ DlgPrefSound::DlgPrefSound(QWidget* pParent,
                 loadSettings();
             });
 
-    m_pMasterAudioLatencyOverloadCount =
-            new ControlProxy("[Master]", "audio_latency_overload_count", this);
-    m_pMasterAudioLatencyOverloadCount->connectValueChanged(this, &DlgPrefSound::bufferUnderflow);
+    m_pAudioLatencyOverloadCount =
+            new ControlProxy(kAppGroup, QStringLiteral("audio_latency_overload_count"), this);
+    m_pAudioLatencyOverloadCount->connectValueChanged(this, &DlgPrefSound::bufferUnderflow);
 
-    m_pMasterLatency = new ControlProxy("[Master]", "latency", this);
-    m_pMasterLatency->connectValueChanged(this, &DlgPrefSound::masterLatencyChanged);
+    m_pOutputLatencyMs = new ControlProxy(kAppGroup, QStringLiteral("output_latency_ms"), this);
+    m_pOutputLatencyMs->connectValueChanged(this, &DlgPrefSound::outputLatencyChanged);
 
-    // TODO: remove this option by automatically disabling/enabling the master mix
-    // when recording, broadcasting, headphone, and master outputs are enabled/disabled
-    m_pMasterEnabled = new ControlProxy("[Master]", "enabled", this);
-    masterMixComboBox->addItem(tr("Disabled"));
-    masterMixComboBox->addItem(tr("Enabled"));
-    masterMixComboBox->setCurrentIndex(m_pMasterEnabled->toBool() ? 1 : 0);
-    connect(masterMixComboBox,
+    // TODO: remove this option by automatically disabling/enabling the main mix
+    // when recording, broadcasting, headphone, and main outputs are enabled/disabled
+    m_pMainEnabled = new ControlProxy("[Master]", "enabled", this);
+    mainMixComboBox->addItem(tr("Disabled"));
+    mainMixComboBox->addItem(tr("Enabled"));
+    mainMixComboBox->setCurrentIndex(m_pMainEnabled->toBool() ? 1 : 0);
+    connect(mainMixComboBox,
             QOverload<int>::of(&QComboBox::currentIndexChanged),
             this,
-            &DlgPrefSound::masterMixChanged);
-    m_pMasterEnabled->connectValueChanged(this, &DlgPrefSound::masterEnabledChanged);
+            &DlgPrefSound::mainMixChanged);
+    m_pMainEnabled->connectValueChanged(this, &DlgPrefSound::mainEnabledChanged);
 
-    m_pMasterMonoMixdown = new ControlProxy("[Master]", "mono_mixdown", this);
-    masterOutputModeComboBox->addItem(tr("Stereo"));
-    masterOutputModeComboBox->addItem(tr("Mono"));
-    masterOutputModeComboBox->setCurrentIndex(m_pMasterMonoMixdown->toBool() ? 1 : 0);
-    connect(masterOutputModeComboBox,
+    m_pMainMonoMixdown = new ControlProxy("[Master]", "mono_mixdown", this);
+    mainOutputModeComboBox->addItem(tr("Stereo"));
+    mainOutputModeComboBox->addItem(tr("Mono"));
+    mainOutputModeComboBox->setCurrentIndex(m_pMainMonoMixdown->toBool() ? 1 : 0);
+    connect(mainOutputModeComboBox,
             QOverload<int>::of(&QComboBox::currentIndexChanged),
             this,
-            &DlgPrefSound::masterOutputModeComboBoxChanged);
-    m_pMasterMonoMixdown->connectValueChanged(this, &DlgPrefSound::masterMonoMixdownChanged);
+            &DlgPrefSound::mainOutputModeComboBoxChanged);
+    m_pMainMonoMixdown->connectValueChanged(this, &DlgPrefSound::mainMonoMixdownChanged);
 
     m_pKeylockEngine =
-            new ControlProxy("[Master]", "keylock_engine", this);
+            new ControlProxy(kAppGroup, QStringLiteral("keylock_engine"), this);
 
 #ifdef __LINUX__
     qDebug() << "RLimit Cur " << RLimit::getCurRtPrio();
@@ -232,21 +252,7 @@ DlgPrefSound::DlgPrefSound(QWidget* pParent,
     realtimeHint->hide();
 #endif // __LINUX__
 
-    // Set the focus policy for QComboBoxes (and wide QDoubleSpinBoxes) and
-    // connect them to the custom event filter below so they don't accept focus
-    // when we scroll the preferences page.
-    QObjectList objList = children();
-    for (int i = 0; i < objList.length(); ++i) {
-        QComboBox* combo = qobject_cast<QComboBox*>(objList[i]);
-        QDoubleSpinBox* spin = qobject_cast<QDoubleSpinBox*>(objList[i]);
-        if (combo) {
-            combo->setFocusPolicy(Qt::StrongFocus);
-            combo->installEventFilter(this);
-        } else if (spin) {
-            spin->setFocusPolicy(Qt::StrongFocus);
-            spin->installEventFilter(this);
-        }
-    }
+    setScrollSafeGuardForAllInputWidgets(this);
 
     hardwareGuide->setText(
             tr("The %1 lists sound cards and controllers you may want to "
@@ -257,29 +263,8 @@ DlgPrefSound::DlgPrefSound(QWidget* pParent,
                             MIXXX_WIKI_HARDWARE_COMPATIBILITY_URL)));
 }
 
-DlgPrefSound::~DlgPrefSound() {
-    delete m_pLatencyCompensation;
-}
-
-// Catch scroll events over comboboxes and pass them to the scroll area instead.
-bool DlgPrefSound::eventFilter(QObject* obj, QEvent* e) {
-    if (e->type() == QEvent::Wheel) {
-        // Reject scrolling only if widget is unfocused.
-        // Object to widget cast is needed to check the focus state.
-        QComboBox* combo = qobject_cast<QComboBox*>(obj);
-        QDoubleSpinBox* spin = qobject_cast<QDoubleSpinBox*>(obj);
-        if ((combo && !combo->hasFocus()) || (spin && !spin->hasFocus())) {
-            QApplication::sendEvent(verticalLayout_2, e);
-            return true;
-        }
-    }
-    return QObject::eventFilter(obj, e);
-}
-
-/**
- * Slot called when the preferences dialog is opened or this pane is
- * selected.
- */
+/// Slot called when the preferences dialog is opened or this pane is
+/// selected.
 void DlgPrefSound::slotUpdate() {
     // this is unfortunate, because slotUpdate is called every time
     // we change to this pane, we lose changed and unapplied settings
@@ -292,9 +277,7 @@ void DlgPrefSound::slotUpdate() {
     m_settingsModified = false;
 }
 
-/**
- * Slot called when the Apply or OK button is pressed.
- */
+/// Slot called when the Apply or OK button is pressed.
 void DlgPrefSound::slotApply() {
     if (!m_settingsModified) {
         return;
@@ -304,17 +287,19 @@ void DlgPrefSound::slotApply() {
     m_config.clearOutputs();
     emit writePaths(&m_config);
 
-    SoundDeviceError err = SOUNDDEVICE_ERROR_OK;
+    SoundDeviceStatus status = SoundDeviceStatus::Ok;
     {
         ScopedWaitCursor cursor;
-        m_pKeylockEngine->set(keylockComboBox->currentIndex());
+        const auto keylockEngine =
+                keylockComboBox->currentData().value<EngineBuffer::KeylockEngine>();
+        m_pKeylockEngine->set(static_cast<double>(keylockEngine));
         m_pSettings->set(ConfigKey("[Master]", "keylock_engine"),
-                       ConfigValue(keylockComboBox->currentIndex()));
+                ConfigValue(static_cast<int>(keylockEngine)));
 
-        err = m_pSoundManager->setConfig(m_config);
+        status = m_pSoundManager->setConfig(m_config);
     }
-    if (err != SOUNDDEVICE_ERROR_OK) {
-        QString error = m_pSoundManager->getLastErrorMessage(err);
+    if (status != SoundDeviceStatus::Ok) {
+        QString error = m_pSoundManager->getLastErrorMessage(status);
         QMessageBox::warning(nullptr, tr("Configuration error"), error);
     } else {
         m_settingsModified = false;
@@ -330,91 +315,89 @@ QUrl DlgPrefSound::helpUrl() const {
     return QUrl(MIXXX_MANUAL_SOUND_URL);
 }
 
-/**
- * Initializes (and creates) all the path items. Each path item widget allows
- * the user to input a sound device name and channel number given a description
- * of what will be done with that info. Inputs and outputs are grouped by tab,
- * and each path item has an identifier (Master, Headphones, ...) and an index,
- * if necessary.
- */
+/// Initializes (and creates) all the path items. Each path item widget allows
+/// the user to input a sound device name and channel number given a description
+/// of what will be done with that info. Inputs and outputs are grouped by tab,
+/// and each path item has an identifier (Master, Headphones, ...) and an index,
+/// if necessary.
 void DlgPrefSound::initializePaths() {
-    foreach (AudioOutput out, m_pSoundManager->registeredOutputs()) {
-        if (!out.isHidden()) {
-            addPath(out);
+    // Pre-sort paths so they're added in the order they'll appear later on
+    // so Tab key order matches order in layout:
+    // * by AudioPathType
+    // * identical types by index
+    auto sortFilterAdd = [this]<typename T>(const QList<T>& l) {
+        // we use a vec of ref_wrappers since copying the path is unnecessary
+        // and we really just want to change the order
+        auto ref_vec_to_sort = std::vector<std::reference_wrapper<const T>>(l.begin(), l.end());
+        std::sort(ref_vec_to_sort.begin(), ref_vec_to_sort.end());
+        for (const T& path : ref_vec_to_sort) {
+            if (!path.isHidden()) {
+                addPath(path);
+            }
         }
-    }
-    foreach (AudioInput in, m_pSoundManager->registeredInputs()) {
-        addPath(in);
-    }
+    };
+
+    sortFilterAdd(m_pSoundManager->registeredOutputs());
+    sortFilterAdd(m_pSoundManager->registeredInputs());
 }
 
 void DlgPrefSound::addPath(const AudioOutput& output) {
     // if we already know about this output, don't make a new entry
-    foreach (QObject *obj, outputTab->children()) {
-        DlgPrefSoundItem *item = qobject_cast<DlgPrefSoundItem*>(obj);
-        if (item) {
-            if (item->type() == output.getType()) {
-                if (AudioPath::isIndexed(item->type())) {
-                    if (item->index() == output.getIndex()) {
-                        return;
-                    }
-                } else {
-                    return;
-                }
-            }
-        }
-    }
 
-    DlgPrefSoundItem *toInsert;
-    AudioPathType type = output.getType();
-    if (AudioPath::isIndexed(type)) {
-        toInsert = new DlgPrefSoundItem(outputTab, type,
-            m_outputDevices, false, output.getIndex());
-    } else {
-        toInsert = new DlgPrefSoundItem(outputTab, type,
-            m_outputDevices, false);
+    if (soundItemAlreadyExists(output, *outputTab)) {
+        return;
     }
-    connect(this, &DlgPrefSound::refreshOutputDevices, toInsert, &DlgPrefSoundItem::refreshDevices);
-    insertItem(toInsert, outputVLayout);
-    connectSoundItem(toInsert);
+    AudioPathType type = output.getType();
+    // TODO who owns this?
+    DlgPrefSoundItem* pSoundItem = new DlgPrefSoundItem(outputTab,
+            type,
+            m_outputDevices,
+            false,
+            AudioPath::isIndexed(type) ? output.getIndex() : 0);
+    insertItem(pSoundItem, outputVLayout);
+    connectSoundItem(pSoundItem);
+
+    setScrollSafeGuardForAllInputWidgets(pSoundItem);
 }
 
 void DlgPrefSound::addPath(const AudioInput& input) {
-    DlgPrefSoundItem *toInsert;
-    // if we already know about this input, don't make a new entry
-    foreach (QObject *obj, inputTab->children()) {
-        DlgPrefSoundItem *item = qobject_cast<DlgPrefSoundItem*>(obj);
-        if (item) {
-            if (item->type() == input.getType()) {
-                if (AudioPath::isIndexed(item->type())) {
-                    if (item->index() == input.getIndex()) {
-                        return;
-                    }
-                } else {
-                    return;
-                }
-            }
-        }
+    if (soundItemAlreadyExists(input, *inputTab)) {
+        return;
     }
     AudioPathType type = input.getType();
-    if (AudioPath::isIndexed(type)) {
-        toInsert = new DlgPrefSoundItem(inputTab, type,
-            m_inputDevices, true, input.getIndex());
-    } else {
-        toInsert = new DlgPrefSoundItem(inputTab, type,
-            m_inputDevices, true);
-    }
-    connect(this, &DlgPrefSound::refreshInputDevices, toInsert, &DlgPrefSoundItem::refreshDevices);
-    insertItem(toInsert, inputVLayout);
-    connectSoundItem(toInsert);
+    // TODO: who owns this?
+    DlgPrefSoundItem* pSoundItem = new DlgPrefSoundItem(inputTab,
+            type,
+            m_inputDevices,
+            true,
+            AudioPath::isIndexed(type) ? input.getIndex() : 0);
+    connectSoundItem(pSoundItem);
+    insertItem(pSoundItem, inputVLayout);
+
+    setScrollSafeGuardForAllInputWidgets(pSoundItem);
 }
 
-void DlgPrefSound::connectSoundItem(DlgPrefSoundItem *item) {
-    connect(item, &DlgPrefSoundItem::settingChanged, this, &DlgPrefSound::deviceSettingChanged);
-    connect(this, &DlgPrefSound::loadPaths, item, &DlgPrefSoundItem::loadPath);
-    connect(this, &DlgPrefSound::writePaths, item, &DlgPrefSoundItem::writePath);
-    connect(this, &DlgPrefSound::updatingAPI, item, &DlgPrefSoundItem::save);
-    connect(this, &DlgPrefSound::updatedAPI, item, &DlgPrefSoundItem::reload);
+void DlgPrefSound::connectSoundItem(DlgPrefSoundItem* pItem) {
+    connect(pItem,
+            &DlgPrefSoundItem::selectedDeviceChanged,
+            this,
+            &DlgPrefSound::deviceChanged);
+    connect(pItem,
+            &DlgPrefSoundItem::selectedChannelsChanged,
+            this,
+            &DlgPrefSound::deviceChannelsChanged);
+    connect(this, &DlgPrefSound::loadPaths, pItem, &DlgPrefSoundItem::loadPath);
+    connect(this, &DlgPrefSound::writePaths, pItem, &DlgPrefSoundItem::writePath);
+    if (pItem->isInput()) {
+        connect(this, &DlgPrefSound::refreshInputDevices, pItem, &DlgPrefSoundItem::refreshDevices);
+    } else {
+        connect(this,
+                &DlgPrefSound::refreshOutputDevices,
+                pItem,
+                &DlgPrefSoundItem::refreshDevices);
+    }
+    connect(this, &DlgPrefSound::updatingAPI, pItem, &DlgPrefSoundItem::save);
+    connect(this, &DlgPrefSound::updatedAPI, pItem, &DlgPrefSoundItem::reload);
 }
 
 void DlgPrefSound::insertItem(DlgPrefSoundItem *pItem, QVBoxLayout *pLayout) {
@@ -422,7 +405,7 @@ void DlgPrefSound::insertItem(DlgPrefSoundItem *pItem, QVBoxLayout *pLayout) {
     for (pos = 0; pos < pLayout->count() - 1; ++pos) {
         DlgPrefSoundItem *pOther(qobject_cast<DlgPrefSoundItem*>(
             pLayout->itemAt(pos)->widget()));
-        if (!pOther) {
+        if (!pOther) { // not a sound item, skip
             continue;
         }
         if (pItem->type() < pOther->type()) {
@@ -436,18 +419,14 @@ void DlgPrefSound::insertItem(DlgPrefSoundItem *pItem, QVBoxLayout *pLayout) {
     pLayout->insertWidget(pos, pItem);
 }
 
-/**
- * Convenience overload to load settings from the SoundManagerConfig owned by
- * SoundManager.
- */
+/// Convenience overload to load settings from the SoundManagerConfig owned by
+/// SoundManager.
 void DlgPrefSound::loadSettings() {
     loadSettings(m_pSoundManager->getConfig());
 }
 
-/**
- * Loads the settings in the given SoundManagerConfig into the dialog.
- */
-void DlgPrefSound::loadSettings(const SoundManagerConfig &config) {
+/// Loads the settings in the given SoundManagerConfig into the dialog.
+void DlgPrefSound::loadSettings(const SoundManagerConfig& config) {
     m_loading = true; // so settingsChanged ignores all our modifications here
     m_config = config;
     int apiIndex = apiComboBox->findData(m_config.getAPI());
@@ -492,22 +471,56 @@ void DlgPrefSound::loadSettings(const SoundManagerConfig &config) {
         engineClockComboBox->setCurrentIndex(0);
     }
 
-    // Default keylock is Rubberband.
-    int keylock_engine = m_pSettings->getValue(
-            ConfigKey("[Master]", "keylock_engine"), 1);
-    keylockComboBox->setCurrentIndex(keylock_engine);
+    // Default keylock engine is Rubberband Faster (v2)
+    const auto keylockEngine = static_cast<EngineBuffer::KeylockEngine>(
+            m_pSettings->getValue(ConfigKey("[Master]", "keylock_engine"),
+                    static_cast<int>(EngineBuffer::defaultKeylockEngine())));
+    const auto keylockEngineVariant = QVariant::fromValue(keylockEngine);
+    const int index = keylockComboBox->findData(keylockEngineVariant);
+    if (index >= 0) {
+        keylockComboBox->setCurrentIndex(index);
+    } else {
+        keylockComboBox->addItem(
+                EngineBuffer::getKeylockEngineName(keylockEngine), keylockEngineVariant);
+        keylockComboBox->setCurrentIndex(keylockComboBox->count() - 1);
+    }
 
+    // Collect selected I/O channel indices for all non-empty device comboboxes
+    // in order to allow auto-selecting free channels when different devices are
+    // selected later on, when a different device is selected for any I/O.
+    m_selectedOutputChannelIndices.clear();
+    m_selectedInputChannelIndices.clear();
+    for (auto* ch : std::as_const(outputTab->children())) {
+        DlgPrefSoundItem* pItem = qobject_cast<DlgPrefSoundItem*>(ch);
+        if (pItem) {
+            auto id = pItem->getDeviceId();
+            if (id == SoundDeviceId()) {
+                continue;
+            }
+            m_selectedOutputChannelIndices.insert(pItem,
+                    QPair<SoundDeviceId, int>(id, pItem->getChannelIndex()));
+        }
+    }
+    for (auto* ch : std::as_const(inputTab->children())) {
+        DlgPrefSoundItem* pItem = qobject_cast<DlgPrefSoundItem*>(ch);
+        if (pItem) {
+            auto id = pItem->getDeviceId();
+            if (id == SoundDeviceId()) {
+                continue;
+            }
+            m_selectedInputChannelIndices.insert(pItem,
+                    QPair<SoundDeviceId, int>(id, pItem->getChannelIndex()));
+        }
+    }
     m_loading = false;
     // DlgPrefSoundItem has it's own inhibit flag
     emit loadPaths(m_config);
 }
 
-/**
- * Slot called when the user selects a different API, or the
- * software changes it programmatically (for instance, when it
- * loads a value from SoundManager). Refreshes the device lists
- * for the new API and pushes those to the path items.
- */
+/// Slot called when the user selects a different API, or the
+/// software changes it programmatically (for instance, when it
+/// loads a value from SoundManager). Refreshes the device lists
+/// for the new API and pushes those to the path items.
 void DlgPrefSound::apiChanged(int index) {
     m_config.setAPI(apiComboBox->itemData(index).toString());
     refreshDevices();
@@ -519,23 +532,15 @@ void DlgPrefSound::apiChanged(int index) {
     // For bigger buffers the user has to manually match the value with Jack.
     // TODO(Be): Get the buffer size from JACK and update audioBufferComboBox.
     // PortAudio as off v19.7.0 does not have a way to get the buffer size from JACK.
-    if (m_config.getAPI() == MIXXX_PORTAUDIO_JACK_STRING) {
-        sampleRateComboBox->setEnabled(false);
-        deviceSyncComboBox->setEnabled(false);
-        engineClockComboBox->setEnabled(false);
-
-    } else {
-        sampleRateComboBox->setEnabled(true);
-        deviceSyncComboBox->setEnabled(true);
-        engineClockComboBox->setEnabled(true);
-    }
+    bool enable = m_config.getAPI() == MIXXX_PORTAUDIO_JACK_STRING ? false : true;
+    sampleRateComboBox->setEnabled(enable);
+    deviceSyncComboBox->setEnabled(enable);
+    engineClockComboBox->setEnabled(enable);
     updateAudioBufferSizes(sampleRateComboBox->currentIndex());
 }
 
-/**
- * Updates the list of APIs, trying to keep the API and device selections
- * constant if possible.
- */
+/// Updates the list of APIs, trying to keep the API and device selections
+/// constant if possible.
 void DlgPrefSound::updateAPIs() {
     QString currentAPI(apiComboBox->itemData(apiComboBox->currentIndex()).toString());
     emit updatingAPI();
@@ -552,10 +557,8 @@ void DlgPrefSound::updateAPIs() {
     emit updatedAPI();
 }
 
-/**
- * Slot called when the sample rate combo box changes to update the
- * sample rate in the config.
- */
+/// Slot called when the sample rate combo box changes to update the
+/// sample rate in the config.
 void DlgPrefSound::sampleRateChanged(int index) {
     m_config.setSampleRate(
             sampleRateComboBox->itemData(index).toUInt());
@@ -564,10 +567,8 @@ void DlgPrefSound::sampleRateChanged(int index) {
     checkLatencyCompensation();
 }
 
-/**
- * Slot called when the latency combo box is changed to update the
- * latency in the config.
- */
+/// Slot called when the latency combo box is changed to update the
+/// latency in the config.
 void DlgPrefSound::audioBufferChanged(int index) {
     m_config.setAudioBufferSizeIndex(
             audioBufferComboBox->itemData(index).toUInt());
@@ -651,10 +652,8 @@ void DlgPrefSound::updateAudioBufferSizes(int sampleRateIndex) {
     }
 }
 
-/**
- * Slot called when device lists go bad to refresh them, or the API
- * just changes and we need to display new devices.
- */
+/// Slot called when device lists go bad to refresh them, or the API
+/// just changes and we need to display new devices.
 void DlgPrefSound::refreshDevices() {
     if (m_config.getAPI() == SoundManagerConfig::kDefaultAPI) {
         m_outputDevices.clear();
@@ -669,11 +668,9 @@ void DlgPrefSound::refreshDevices() {
     emit refreshInputDevices(m_inputDevices);
 }
 
-/**
- * Called when any of the combo boxes in this dialog are changed. Enables the
- * apply button and marks that settings have been changed so that
- * DlgPrefSound::slotApply knows to apply them.
- */
+/// Called when any of the combo boxes in this dialog are changed. Enables the
+/// apply button and marks that settings have been changed so that
+/// DlgPrefSound::slotApply knows to apply them.
 void DlgPrefSound::settingChanged() {
     if (m_loading) {
         return; // doesn't count if we're just loading prefs
@@ -681,38 +678,93 @@ void DlgPrefSound::settingChanged() {
     m_settingsModified = true;
 }
 
-void DlgPrefSound::deviceSettingChanged() {
+void DlgPrefSound::deviceChanged() {
     if (m_loading) {
         return;
     }
+
+    DlgPrefSoundItem* pItem = qobject_cast<DlgPrefSoundItem*>(sender());
+    if (!pItem) {
+        return;
+    }
+    QHash<DlgPrefSoundItem*, QPair<SoundDeviceId, int>>* channels;
+    if (pItem->isInput()) {
+        channels = &m_selectedInputChannelIndices;
+    } else {
+        channels = &m_selectedOutputChannelIndices;
+    }
+    auto id = pItem->getDeviceId();
+    if (id == SoundDeviceId()) {
+        if (channels->contains(pItem)) {
+            channels->remove(pItem);
+        }
+    } else {
+        QList<int> selectedChannelsForDevice;
+        QHashIterator<DlgPrefSoundItem*, QPair<SoundDeviceId, int>> it(
+                pItem->isInput()
+                        ? m_selectedInputChannelIndices
+                        : m_selectedOutputChannelIndices);
+        while (it.hasNext()) {
+            it.next();
+            if (it.value().first == id) {
+                selectedChannelsForDevice.append(it.value().second);
+            }
+        }
+        pItem->selectFirstUnusedChannelIndex(selectedChannelsForDevice);
+    }
+
     checkLatencyCompensation();
     m_settingsModified = true;
 }
 
-/**
- * Slot called when the "Query Devices" button is clicked.
- */
+void DlgPrefSound::deviceChannelsChanged() {
+    if (m_loading) {
+        return;
+    }
+    DlgPrefSoundItem* pItem = qobject_cast<DlgPrefSoundItem*>(sender());
+    if (!pItem) {
+        return;
+    }
+    auto id = pItem->getDeviceId();
+    int index = pItem->getChannelIndex();
+    if (id != SoundDeviceId()) {
+        if (pItem->isInput()) {
+            m_selectedInputChannelIndices.insert(pItem, QPair<SoundDeviceId, int>(id, index));
+        } else {
+            m_selectedOutputChannelIndices.insert(pItem, QPair<SoundDeviceId, int>(id, index));
+        }
+    }
+
+    checkLatencyCompensation();
+    m_settingsModified = true;
+}
+
+/// Slot called when the "Query Devices" button is clicked.
 void DlgPrefSound::queryClicked() {
     ScopedWaitCursor cursor;
     m_pSoundManager->clearAndQueryDevices();
     updateAPIs();
 }
 
-/**
- * Slot called when the "Reset to Defaults" button is clicked.
- */
+/// Slot called when the "Reset to Defaults" button is clicked.
 void DlgPrefSound::slotResetToDefaults() {
-    SoundManagerConfig newConfig(m_pSoundManager);
-    newConfig.loadDefaults(m_pSoundManager, SoundManagerConfig::ALL);
+    SoundManagerConfig newConfig(m_pSoundManager.get());
+    newConfig.loadDefaults(m_pSoundManager.get(), SoundManagerConfig::ALL);
     loadSettings(newConfig);
-    keylockComboBox->setCurrentIndex(EngineBuffer::RUBBERBAND);
-    m_pKeylockEngine->set(EngineBuffer::RUBBERBAND);
 
-    masterMixComboBox->setCurrentIndex(1);
-    m_pMasterEnabled->set(1.0);
+    const auto keylockEngine = EngineBuffer::defaultKeylockEngine();
+    const int index = keylockComboBox->findData(QVariant::fromValue(keylockEngine));
+    DEBUG_ASSERT(index >= 0);
+    if (index >= 0) {
+        keylockComboBox->setCurrentIndex(index);
+    }
+    m_pKeylockEngine->set(static_cast<double>(keylockEngine));
 
-    masterDelaySpinBox->setValue(0.0);
-    m_pMasterDelay->set(0.0);
+    mainMixComboBox->setCurrentIndex(1);
+    m_pMainEnabled->set(1.0);
+
+    mainDelaySpinBox->setValue(0.0);
+    m_pMainDelay->set(0.0);
 
     headDelaySpinBox->setValue(0.0);
     m_pHeadDelay->set(0.0);
@@ -720,13 +772,13 @@ void DlgPrefSound::slotResetToDefaults() {
     boothDelaySpinBox->setValue(0.0);
     m_pBoothDelay->set(0.0);
 
-    // Enable talkover master output
+    // Enable talkover main output
     m_pMicMonitorMode->set(
-        static_cast<double>(
-            static_cast<int>(EngineMaster::MicMonitorMode::MASTER)));
+            static_cast<double>(
+                    static_cast<int>(EngineMixer::MicMonitorMode::Main)));
     micMonitorModeComboBox->setCurrentIndex(
-        micMonitorModeComboBox->findData(
-            static_cast<int>(EngineMaster::MicMonitorMode::MASTER)));
+            micMonitorModeComboBox->findData(
+                    static_cast<int>(EngineMixer::MicMonitorMode::Main)));
 
     latencyCompensationSpinBox->setValue(latencyCompensationSpinBox->minimum());
 
@@ -738,7 +790,7 @@ void DlgPrefSound::bufferUnderflow(double count) {
     update();
 }
 
-void DlgPrefSound::masterLatencyChanged(double latency) {
+void DlgPrefSound::outputLatencyChanged(double latency) {
     currentLatency->setText(QString("%1 ms").arg(latency));
     update();
 }
@@ -748,8 +800,8 @@ void DlgPrefSound::latencyCompensationSpinboxChanged(double value) {
     checkLatencyCompensation();
 }
 
-void DlgPrefSound::masterDelaySpinboxChanged(double value) {
-    m_pMasterDelay->set(value);
+void DlgPrefSound::mainDelaySpinboxChanged(double value) {
+    m_pMainDelay->set(value);
 }
 
 void DlgPrefSound::headDelaySpinboxChanged(double value) {
@@ -760,28 +812,28 @@ void DlgPrefSound::boothDelaySpinboxChanged(double value) {
     m_pBoothDelay->set(value);
 }
 
-void DlgPrefSound::masterMixChanged(int value) {
-    m_pMasterEnabled->set(value);
+void DlgPrefSound::mainMixChanged(int value) {
+    m_pMainEnabled->set(value);
 }
 
-void DlgPrefSound::masterEnabledChanged(double value) {
-    const bool masterEnabled = (value != 0);
-    masterMixComboBox->setCurrentIndex(masterEnabled ? 1 : 0);
+void DlgPrefSound::mainEnabledChanged(double value) {
+    const bool mainEnabled = (value != 0);
+    mainMixComboBox->setCurrentIndex(mainEnabled ? 1 : 0);
 }
 
-void DlgPrefSound::masterOutputModeComboBoxChanged(int value) {
-    m_pMasterMonoMixdown->set((double)value);
+void DlgPrefSound::mainOutputModeComboBoxChanged(int value) {
+    m_pMainMonoMixdown->set((double)value);
 }
 
-void DlgPrefSound::masterMonoMixdownChanged(double value) {
-    const bool masterMonoMixdownEnabled = (value != 0);
-    masterOutputModeComboBox->setCurrentIndex(masterMonoMixdownEnabled ? 1 : 0);
+void DlgPrefSound::mainMonoMixdownChanged(double value) {
+    const bool mainMonoMixdownEnabled = (value != 0);
+    mainOutputModeComboBox->setCurrentIndex(mainMonoMixdownEnabled ? 1 : 0);
 }
 
 void DlgPrefSound::micMonitorModeComboBoxChanged(int value) {
-    EngineMaster::MicMonitorMode newMode =
-        static_cast<EngineMaster::MicMonitorMode>(
-            micMonitorModeComboBox->itemData(value).toInt());
+    EngineMixer::MicMonitorMode newMode =
+            static_cast<EngineMixer::MicMonitorMode>(
+                    micMonitorModeComboBox->itemData(value).toInt());
 
     m_pMicMonitorMode->set(static_cast<double>(newMode));
 
@@ -789,9 +841,9 @@ void DlgPrefSound::micMonitorModeComboBoxChanged(int value) {
 }
 
 void DlgPrefSound::checkLatencyCompensation() {
-    EngineMaster::MicMonitorMode configuredMicMonitorMode =
-        static_cast<EngineMaster::MicMonitorMode>(
-            static_cast<int>(m_pMicMonitorMode->get()));
+    EngineMixer::MicMonitorMode configuredMicMonitorMode =
+            static_cast<EngineMixer::MicMonitorMode>(
+                    static_cast<int>(m_pMicMonitorMode->get()));
 
     // Do not clear the SoundManagerConfig on startup, from slotApply, or from slotUpdate
     if (!m_bSkipConfigClear) {
@@ -803,29 +855,31 @@ void DlgPrefSound::checkLatencyCompensation() {
 
     if (m_config.hasMicInputs() && !m_config.hasExternalRecordBroadcast()) {
         micMonitorModeComboBox->setEnabled(true);
-        if (configuredMicMonitorMode == EngineMaster::MicMonitorMode::DIRECT_MONITOR) {
+        if (configuredMicMonitorMode == EngineMixer::MicMonitorMode::DirectMonitor) {
             latencyCompensationSpinBox->setEnabled(true);
-            QString warningIcon(
-                    "<html>"
-                    "<img "
-                    "src=':/images/preferences/ic_preferences_warning.svg' "
-                    "width='20' height='20'>"
-                    "</html> ");
             QString lineBreak("<br/>");
             // TODO(Be): Make the "User Manual" text link to the manual.
             if (m_pLatencyCompensation->get() == 0.0) {
-                latencyCompensationWarningLabel->setText(
-                      warningIcon +
-                      tr("Microphone inputs are out of time in the record & broadcast signal compared to what you hear.") + lineBreak +
-                      tr("Measure round trip latency and enter it above for Microphone Latency Compensation to align microphone timing.") + lineBreak +
-                      tr("Refer to the Mixxx User Manual for details.") + "</html>");
+                latencyCompensationWarningLabel->setText(kWarningIconHtmlString +
+                        tr("Microphone inputs are out of time in the record & "
+                           "broadcast signal compared to what you hear.") +
+                        lineBreak +
+                        tr("Measure round trip latency and enter it above for "
+                           "Microphone Latency Compensation to align "
+                           "microphone timing.") +
+                        lineBreak +
+                        tr("Refer to the Mixxx User Manual for details.") +
+                        "</html>");
                 latencyCompensationWarningLabel->show();
             } else if (m_bLatencyChanged) {
-                latencyCompensationWarningLabel->setText(
-                  warningIcon +
-                  tr("Configured latency has changed.") + lineBreak +
-                  tr("Remeasure round trip latency and enter it above for Microphone Latency Compensation to align microphone timing.") + lineBreak +
-                  tr("Refer to the Mixxx User Manual for details.") + "</html>");
+                latencyCompensationWarningLabel->setText(kWarningIconHtmlString +
+                        tr("Configured latency has changed.") + lineBreak +
+                        tr("Remeasure round trip latency and enter it above "
+                           "for Microphone Latency Compensation to align "
+                           "microphone timing.") +
+                        lineBreak +
+                        tr("Refer to the Mixxx User Manual for details.") +
+                        "</html>");
                 latencyCompensationWarningLabel->show();
             } else {
                 latencyCompensationWarningLabel->hide();
