@@ -53,14 +53,16 @@ LaunchImage::LaunchImage(QWidget* pParent, const QString& styleSheet)
     hbox->addStretch();
 }
 
-void LaunchImage::progress(int value) {
+void LaunchImage::progress(int value, const QString& serviceName) {
     m_pProgressBar->setValue(value);
+    // TODO: show serviceName
+    Q_UNUSED(serviceName);
 }
 
 void LaunchImage::paintEvent(QPaintEvent *)
 {
     QStyleOption opt;
-    opt.init(this);
+    opt.initFrom(this);
     QPainter p(this);
     style()->drawPrimitive(QStyle::PE_Widget, &opt, &p, this);
 }
