@@ -2,7 +2,6 @@
 
 #include <QDomElement>
 #include <QList>
-#include <QMutex>
 #include <QObject>
 #include <QSet>
 #include <QString>
@@ -85,6 +84,8 @@ class LegacySkinParser : public QObject, public SkinParser {
     QWidget* parseEngineKey(const QDomElement& node);
     QWidget* parseBeatSpinBox(const QDomElement& node);
     QWidget* parseEffectChainName(const QDomElement& node);
+    QWidget* parseEffectChainPresetButton(const QDomElement& node);
+    QWidget* parseEffectChainPresetSelector(const QDomElement& node);
     QWidget* parseEffectName(const QDomElement& node);
     QWidget* parseEffectParameterName(const QDomElement& node);
     QWidget* parseEffectParameterKnob(const QDomElement& node);
