@@ -40,6 +40,8 @@ void Tooltips::addStandardTooltips() {
     QString loopActive = "(" + tr("loop active") + ")";
     QString loopInactive = "(" + tr("loop inactive") + ")";
     QString effectsWithinUnit = tr("Effects within the chain must be enabled to hear them.");
+    QString resetWithRightAndDoubleClick =
+            QString("%1 / %2: %3").arg(rightClick, doubleClick, resetToDefault);
 
     add("waveform_overview")
             << tr("Waveform Overview")
@@ -83,37 +85,37 @@ void Tooltips::addStandardTooltips() {
     add("pregain")
             << tr("Gain")
             << tr("Adjusts the pre-fader gain of the track (to avoid clipping).")
-            << QString("%1: %2").arg(rightClick, resetToDefault);
+            << resetWithRightAndDoubleClick;
 
     QString clippingHelp = tr("(too loud for the hardware and is being distorted).");
-    add("channel_PeakIndicator")
+    add("channel_peak_indicator")
             << tr("Channel Peak Indicator")
             << tr("Indicates when the signal on the channel is clipping,")
             << clippingHelp;
 
-    add("channel_PeakIndicatorL")
+    add("channel_peak_indicator_left")
             << tr("Channel L Peak Indicator")
             << tr("Indicates when the left signal on the channel is clipping,")
             << clippingHelp;
 
-    add("channel_PeakIndicatorR")
+    add("channel_peak_indicator_right")
             << tr("Channel R Peak Indicator")
             << tr("Indicates when the right signal on the channel is clipping,")
             << clippingHelp;
 
-    add("master_PeakIndicator")
-            << tr("Master Peak Indicator")
-            << tr("Indicates when the signal on the master output is clipping,")
+    add("main_peak_indicator")
+            << tr("Main Output Peak Indicator")
+            << tr("Indicates when the signal on the main output is clipping,")
             << clippingHelp;
 
-    add("master_PeakIndicatorL")
-            << tr("Master L Peak Indicator")
-            << tr("Indicates when the left signal on the master output is clipping,")
+    add("main_peak_indicator_left")
+            << tr("Main Output L Peak Indicator")
+            << tr("Indicates when the left signal on the main output is clipping,")
             << clippingHelp;
 
-    add("master_PeakIndicatorR")
-            << tr("Master R Peak Indicator")
-            << tr("Indicates when the right signal on the master output is clipping,")
+    add("main_peak_indicator_right")
+            << tr("Main Output R Peak Indicator")
+            << tr("Indicates when the right signal on the main output is clipping,")
             << clippingHelp;
 
     add("channel_VuMeter")
@@ -132,7 +134,7 @@ void Tooltips::addStandardTooltips() {
             << tr("Microphone Volume Meter")
             << tr("Shows the current microphone volume.");
 
-    add("microphone_PeakIndicator")
+    add("microphone_peak_indicator")
             << tr("Microphone Peak Indicator")
             << tr("Indicates when the signal on the microphone is clipping,")
             << clippingHelp;
@@ -141,7 +143,7 @@ void Tooltips::addStandardTooltips() {
             << tr("Auxiliary Volume Meter")
             << tr("Shows the current auxiliary volume.");
 
-    add("auxiliary_PeakIndicator")
+    add("auxiliary_peak_indicator")
             << tr("Auxiliary Peak Indicator")
             << tr("Indicates when the signal on the auxiliary is clipping,")
             << clippingHelp;
@@ -150,7 +152,7 @@ void Tooltips::addStandardTooltips() {
             << tr("Sampler Volume Meter")
             << tr("Shows the current sampler volume.");
 
-    add("sampler_PeakIndicator")
+    add("sampler_peak_indicator")
             << tr("Sampler Peak Indicator")
             << tr("Indicates when the signal on the sampler is clipping,")
             << clippingHelp;
@@ -159,70 +161,70 @@ void Tooltips::addStandardTooltips() {
             << tr("Preview Deck Volume Meter")
             << tr("Shows the current Preview Deck volume.");
 
-    add("preview_PeakIndicator")
+    add("preview_peak_indicator")
             << tr("Preview Deck Peak Indicator")
             << tr("Indicates when the signal on the Preview Deck is clipping,")
             << clippingHelp;
 
-    add("master_VuMeterL")
-            << tr("Master Channel L Volume Meter")
-            << tr("Shows the current master volume for the left channel.");
+    add("main_VuMeterL")
+            << tr("Main Channel L Volume Meter")
+            << tr("Shows the current volume for the left channel of the main output.");
 
-    add("master_VuMeterR")
-            << tr("Master Channel R Volume Meter")
-            << tr("Shows the current master volume for the right channel.");
+    add("main_VuMeterR")
+            << tr("Main Channel R Volume Meter")
+            << tr("Shows the current volume for the right channel of the main output.");
 
     add("channel_volume")
             << tr("Volume Control")
             << tr("Adjusts the volume of the selected channel.")
-            << QString("%1: %2").arg(rightClick, resetToDefault);
+            << resetWithRightAndDoubleClick;
 
     // Legacy control.
-    add("master_volume")
-            << tr("Master Volume")
-            << tr("Adjusts the master output volume.")
-            << QString("%1: %2").arg(rightClick, resetToDefault);
+    add("main_volume")
+            << tr("Main Output Gain")
+            << tr("Adjusts the main output gain.")
+            << resetWithRightAndDoubleClick;
 
-    add("master_gain")
-            << tr("Master Gain")
-            << tr("Adjusts the master output gain.")
-            << QString("%1: %2").arg(rightClick, resetToDefault);
+    add("main_gain")
+            << tr("Main Output Gain")
+            << tr("Adjusts the main output gain.")
+            << resetWithRightAndDoubleClick;
 
     add("booth_gain")
             << tr("Booth Gain")
             << tr("Adjusts the booth output gain.")
-            << QString("%1: %2").arg(rightClick, resetToDefault);
+            << resetWithRightAndDoubleClick;
 
     add("crossfader")
             << tr("Crossfader")
-            << tr("Determines the master output by fading between the left and right channels.")
-            << QString("%1: %2").arg(rightClick, resetToDefault)
+            << tr("Determines the main output by fading between the left and right channels.")
+            << resetWithRightAndDoubleClick
             << tr("Change the crossfader curve in Preferences -> Crossfader");
 
     add("balance")
             << tr("Balance")
-            << tr("Adjusts the left/right channel balance on the master output.")
-            << QString("%1: %2").arg(rightClick, resetToDefault);
+            << tr("Adjusts the left/right channel balance on the main output.")
+            << resetWithRightAndDoubleClick;
 
     // Legacy control.
     add("headphone_volume")
             << tr("Headphone Volume")
             << tr("Adjusts the headphone output volume.")
-            << QString("%1: %2").arg(rightClick, resetToDefault);
+            << resetWithRightAndDoubleClick;
 
     add("headphone_gain")
             << tr("Headphone Gain")
             << tr("Adjusts the headphone output gain.")
-            << QString("%1: %2").arg(rightClick, resetToDefault);
+            << resetWithRightAndDoubleClick;
 
     add("headMix")
             << tr("Headphone Mix")
-            << tr("Crossfades the headphone output between the master mix and cueing (PFL or Pre-Fader Listening) signal.")
-            << QString("%1: %2").arg(rightClick, resetToDefault);
+            << tr("Crossfades the headphone output between the main mix and cueing (PFL or Pre-Fader Listening) signal.")
+            << resetWithRightAndDoubleClick;
 
     add("headSplit")
             << tr("Headphone Split Cue")
-            << tr("If activated, the master signal plays in the right channel, while the cueing signal "
+            << tr("If activated, the main mix signal plays in the right channel, while the cueing signal "
                   "plays in the left channel.")
             << tr("Adjust the Headphone Mix so in the left channel is not the pure cueing signal.");
 
@@ -284,27 +286,27 @@ void Tooltips::addStandardTooltips() {
 
     add("show_vumeters")
             << tr("Volume Meters")
-            << tr("Show/hide volume meters for channels and master output.");
+            << tr("Show/hide volume meters for channels and main output.");
 
     add("microphone_volume")
             << tr("Microphone Volume")
             << tr("Adjusts the microphone volume.")
-            << QString("%1: %2").arg(rightClick, resetToDefault);
+            << resetWithRightAndDoubleClick;
 
     add("microphone_pregain")
             << tr("Microphone Gain")
             << tr("Adjusts the pre-fader microphone gain.")
-            << QString("%1: %2").arg(rightClick, resetToDefault);
+            << resetWithRightAndDoubleClick;
 
     add("auxiliary_pregain")
             << tr("Auxiliary Gain")
             << tr("Adjusts the pre-fader auxiliary gain.")
-            << QString("%1: %2").arg(rightClick, resetToDefault);
+            << resetWithRightAndDoubleClick;
 
     add("microphone_talkover")
             << tr("Microphone Talk-Over")
             << tr("Hold-to-talk or short click for latching to")
-            << tr("mix microphone input into the master output.");
+            << tr("mix microphone input into the main output.");
 
     add("talkover_duck_mode")
             << tr("Microphone Talkover Mode")
@@ -321,7 +323,7 @@ void Tooltips::addStandardTooltips() {
             << tr("Auto: Sets how much to reduce the music volume when the volume of active microphones rises above threshold.")
             << tr("Manual: Sets how much to reduce the music volume, when talkover is activated regardless of volume of microphone inputs.");
 
-    QString changeAmount = tr("Change the step-size in the Preferences -> Interface menu.");
+    QString changeAmount = tr("Change the step-size in the Preferences -> Decks menu.");
     add("rate_perm_up_rate_perm_up_small")
             << tr("Raise Pitch")
             << QString("%1: %2").arg(leftClick, tr("Sets the pitch higher."))
@@ -349,17 +351,17 @@ void Tooltips::addStandardTooltips() {
     add("filterLow")
             << tr("Low EQ")
             << tr("Adjusts the gain of the low EQ filter.")
-            << QString("%1: %2").arg(rightClick, resetToDefault);
+            << resetWithRightAndDoubleClick;
 
     add("filterMid")
             << tr("Mid EQ")
             << tr("Adjusts the gain of the mid EQ filter.")
-            << QString("%1: %2").arg(rightClick, resetToDefault);
+            << resetWithRightAndDoubleClick;
 
     add("filterHigh")
             << tr("High EQ")
             << tr("Adjusts the gain of the high EQ filter.")
-            << QString("%1: %2").arg(rightClick, resetToDefault);
+            << resetWithRightAndDoubleClick;
 
     QString eqKillLatch = tr("Hold-to-kill or short click for latching.");
     add("filterHighKill")
@@ -444,11 +446,23 @@ void Tooltips::addStandardTooltips() {
             << tr("Toggling keylock during playback may result in a momentary audio glitch.");
 
     add("hotcue_toggle")
-        <<tr("Changes the number of hotcue buttons displayed in the deck");
+            << tr("Changes the number of hotcue buttons displayed in the deck");
+
+    // Show Loop Controls
+    add("show_loop_controls")
+            << tr("Toggle visibility of Loop Controls");
+
+    // Show Beatjump Controls
+    add("show_beatjump_controls")
+            << tr("Toggle visibility of Beatjump Controls");
 
     // Show Rate Control
     add("rate_toggle")
-        <<tr("Toggle visibility of Rate Control");
+            << tr("Toggle visibility of Rate Control");
+
+    // Show Key Controls
+    add("show_key_controls")
+            << tr("Toggle visibility of Key Controls");
 
     // Used in cue/hotcue/loop tooltips below.
     QString quantizeSnap = tr("If quantize is enabled, snaps to the nearest beat.");
@@ -518,12 +532,12 @@ void Tooltips::addStandardTooltips() {
 
     add("mute")
             << tr("Mute")
-            << tr("Mutes the selected channel's audio in the master output.");
+            << tr("Mutes the selected channel's audio in the main output.");
 
-    add("master_enable")
-            << tr("Master enable")
+    add("main_enable")
+            << tr("Main mix enable")
             << tr("Hold or short click for latching to "
-                  "mix this input into the master output.");
+                  "mix this input into the main output.");
 
     add("back_start")
             << tr("Fast Rewind")
@@ -538,7 +552,7 @@ void Tooltips::addStandardTooltips() {
     // Ghetto-Sync (TM)
     add("beatsync_beatsync_tempo")
             << tr("Old Synchronize")
-            << tr("(This skin should be updated to use Master Sync!)")
+            << tr("(This skin should be updated to use Sync Lock!)")
             << QString("%1: %2").arg(leftClick, tr("Syncs the tempo (BPM) and phase to that of the other track, "
                                                    "if BPM is detected on both."))
             << QString("%1: %2").arg(rightClick, tr("Syncs the tempo (BPM) to that of the other track, "
@@ -549,8 +563,8 @@ void Tooltips::addStandardTooltips() {
 
     // Awesome-Sync (TM)
     add("sync_enabled")
-            << tr("Enable Master Sync")
-            << tr("Tap to sync the tempo to other playing tracks or the master clock.")
+            << tr("Enable Sync Lock")
+            << tr("Tap to sync the tempo to other playing tracks or the sync leader.")
             << tr("Hold for at least a second to enable sync lock for this deck.")
             << tr("Decks with sync locked will all play at the same tempo, and decks that also have "
                   "quantize enabled will always have their beats lined up.");
@@ -562,19 +576,21 @@ void Tooltips::addStandardTooltips() {
                                                    "from the other track. Requires a detected key on both involved decks."))
             << QString("%1: %2").arg(rightClick, tr("Resets the key to the original track key."));
 
-    add("sync_master")
-            << tr("Enable Sync Clock Master")
-            << tr("When enabled, this device will serve as the master clock for all other decks.");
+    add("sync_leader")
+            << tr("Enable Sync Leader")
+            << tr("When enabled, this device will serve as the sync leader for all other decks.")
+            << tr("This is relevant when a dynamic tempo track is loaded to a sync leader deck."
+                  "In that case, other synced devices will adopt the changing tempo.");
 
     add("rate")
             << tr("Speed Control")
             << tr("Changes the track playback speed (affects both the tempo and the pitch). If keylock is enabled, only the tempo is affected.")
-            << QString("%1: %2").arg(rightClick, resetToDefault);
+            << resetWithRightAndDoubleClick;
 
     add("pitch")
             << tr("Pitch Control")
             << tr("Changes the track pitch independent of the tempo.")
-            << QString("%1: %2").arg(rightClick, resetToDefault);
+            << resetWithRightAndDoubleClick;
 
     add("pitch_up")
             << tr("Pitch Control")
@@ -591,8 +607,7 @@ void Tooltips::addStandardTooltips() {
     add("pitch_adjust")
             << tr("Pitch Adjust")
             << tr("Adjust the pitch in addition to the speed slider pitch.")
-            << QString("%1: %2").arg(rightClick, resetToDefault);
-
+            << resetWithRightAndDoubleClick;
 
     add("rate_display")
             << tr("Pitch Rate")
@@ -606,9 +621,13 @@ void Tooltips::addStandardTooltips() {
             << tr("Repeat")
             << tr("When active the track will repeat if you go past the end or reverse before the start.");
 
-    add("eject")
-            << tr("Eject")
-            << tr("Ejects track from the player.");
+    add("eject") << tr("Eject") << tr("Ejects track from the player.")
+                 << tr("Un-ejects when no track is loaded, i.e. reloads the "
+                       "track that was ejected last (of any deck).")
+                 << QString("%1: %2").arg(doubleClick,
+                            "Reloads the last replaced track. "
+                            "If no track is loaded reloads the second-last "
+                            "ejected track.");
 
     add("hotcue") << tr("Hotcue")
                   << QString("%1: %2").arg(leftClick,
@@ -801,6 +820,15 @@ void Tooltips::addStandardTooltips() {
             << tr("Displays the musical key of the loaded track.")
             << trackTags;
 
+    add("track_comment")
+            << tr("Track Comment")
+            << tr("Displays the comment tag of the loaded track.")
+            << trackTags + "\n"
+            << dropTracksHere
+            << dragItem
+            << QString("%1: %2").arg(doubleClick, trackProperties)
+            << QString("%1: %2").arg(rightClick, trackMenu);
+
     add("text")
             << tr("Track Artist/Title")
             << tr("Displays the artist and title of the loaded track.")
@@ -887,7 +915,7 @@ void Tooltips::addStandardTooltips() {
             << tr("Adjust the mixing of the dry (input) signal with the wet (output) signal of the effect unit")
             << tr("D/W mode: Crossfade between dry and wet")
             << tr("D+W mode: Add wet to dry")
-            << QString("%1: %2").arg(rightClick, resetToDefault);
+            << resetWithRightAndDoubleClick;
 
     add("EffectUnit_mix_mode")
             << tr("Mix Mode")
@@ -900,7 +928,11 @@ void Tooltips::addStandardTooltips() {
     add("EffectUnit_super1")
             << tr("Super Knob")
             << tr("Controls the Meta Knob of all effects in this unit together.")
-            << QString("%1: %2").arg(rightClick, resetToDefault);
+            << resetWithRightAndDoubleClick;
+
+    add("EffectUnit_chain_preset_menu")
+            << tr("Effect Chain Preset Settings")
+            << tr("Show the effect chain settings menu for this unit.");
 
     add("EffectUnit_next_chain")
             << tr("Next Chain")
@@ -924,9 +956,9 @@ void Tooltips::addStandardTooltips() {
             << tr("Route the headphone channel through this effect unit.")
             << effectsWithinUnit;
 
-    add("EffectUnit_master_enabled")
+    add("EffectUnit_main_enabled")
             << tr("Assign Effect Unit")
-            << tr("Route the master mix through this effect unit.")
+            << tr("Route the main mix through this effect unit.")
             << effectsWithinUnit;
 
     add("EffectUnit_BusLeft_enabled")
@@ -983,7 +1015,7 @@ void Tooltips::addStandardTooltips() {
     add("EffectSlot_metaknob")
             << tr("Meta Knob")
             << tr("Controls linked parameters of this effect")
-            << QString("%1: %2").arg(rightClick, resetToDefault);
+            << resetWithRightAndDoubleClick;
 
     add("EffectSlot_focus")
             << tr("Effect Focus Button")
@@ -994,7 +1026,7 @@ void Tooltips::addStandardTooltips() {
     add("EffectSlot_parameter")
             << tr("Effect Parameter")
             << tr("Adjusts a parameter of the effect.")
-            << QString("%1: %2").arg(rightClick, resetToDefault);
+            << resetWithRightAndDoubleClick;
 
     add("EffectSlot_parameter_link_type")
             << tr("Meta Knob Link")
@@ -1018,7 +1050,7 @@ void Tooltips::addStandardTooltips() {
     add("QuickEffectRack_super1")
             << tr("Quick Effect Super Knob")
             << tr("Quick Effect Super Knob (control linked effect parameters).")
-            << QString("%1: %2").arg(rightClick, resetToDefault)
+            << resetWithRightAndDoubleClick
             << tr("Hint: Change the default Quick Effect mode in Preferences -> Equalizers.");
 
     add("QuickEffectRack_enabled")
@@ -1030,7 +1062,7 @@ void Tooltips::addStandardTooltips() {
     add("EqualizerRack_effect_parameter")
             << tr("Equalizer Parameter")
             << tr("Adjusts the gain of the EQ filter.")
-            << QString("%1: %2").arg(rightClick, resetToDefault)
+            << resetWithRightAndDoubleClick
             << tr("Hint: Change the default EQ mode in Preferences -> Equalizers.");
 
     add("EqualizerRack_effect_button_parameter")
