@@ -3,7 +3,6 @@
 #include <QtDebug>
 
 #include "control/controlobject.h"
-#include "control/controlproxy.h"
 #include "engine/sidechain/enginenetworkstream.h"
 #include "float.h"
 #include "moc_sounddevicenetwork.cpp"
@@ -18,7 +17,7 @@
 #include "waveform/visualplayposition.h"
 
 namespace {
-const int kNetworkLatencyFrames = 8192; // 185 ms @ 44100 Hz
+constexpr int kNetworkLatencyFrames = 8192; // 185 ms @ 44100 Hz
 // Related chunk sizes:
 // Mp3 frames = 1152 samples
 // Ogg frames = 64 to 8192 samples.
@@ -28,6 +27,8 @@ const int kNetworkLatencyFrames = 8192; // 185 ms @ 44100 Hz
 // which is 185 @ 44100 ms and twice the maximum of the max mixxx audio buffer
 
 const mixxx::Logger kLogger("SoundDeviceNetwork");
+
+const QString kAppGroup = QStringLiteral("[App]");
 } // namespace
 
 SoundDeviceNetwork::SoundDeviceNetwork(
@@ -37,6 +38,7 @@ SoundDeviceNetwork::SoundDeviceNetwork(
         : SoundDevice(config, sm),
           m_pNetworkStream(pNetworkStream),
           m_inputDrift(false),
+          m_audioLatencyUsage(kAppGroup, QStringLiteral("audio_latency_usage")),
           m_framesSinceAudioLatencyUsageUpdate(0),
           m_denormals(false),
           m_targetTime(0) {
@@ -47,15 +49,12 @@ SoundDeviceNetwork::SoundDeviceNetwork(
     m_strDisplayName = QObject::tr("Network stream");
     m_iNumInputChannels = pNetworkStream->getNumInputChannels();
     m_iNumOutputChannels = pNetworkStream->getNumOutputChannels();
-
-    m_pMasterAudioLatencyUsage = std::make_unique<ControlProxy>("[Master]",
-            "audio_latency_usage");
 }
 
 SoundDeviceNetwork::~SoundDeviceNetwork() {
 }
 
-SoundDeviceError SoundDeviceNetwork::open(bool isClkRefDevice, int syncBuffers) {
+SoundDeviceStatus SoundDeviceNetwork::open(bool isClkRefDevice, int syncBuffers) {
     Q_UNUSED(syncBuffers);
     kLogger.debug() << "open:" << m_deviceId.name;
 
@@ -89,9 +88,9 @@ SoundDeviceError SoundDeviceNetwork::open(bool isClkRefDevice, int syncBuffers)
 
         // Update the samplerate and latency ControlObjects, which allow the
         // waveform view to properly correct for the latency.
-        ControlObject::set(ConfigKey("[Master]", "latency"),
+        ControlObject::set(ConfigKey(kAppGroup, QStringLiteral("output_latency_ms")),
                 requestedBufferTime.toDoubleMillis());
-        ControlObject::set(ConfigKey("[Master]", "samplerate"), m_dSampleRate);
+        ControlObject::set(ConfigKey(kAppGroup, QStringLiteral("samplerate")), m_dSampleRate);
 
         // Network stream was just started above so we have to wait until
         // we can pass one chunk.
@@ -104,14 +103,15 @@ SoundDeviceError SoundDeviceNetwork::open(bool isClkRefDevice, int syncBuffers)
         kLogger.debug() << "Maximum:" << framesPerBuffer << "frames/buffer @"
                         << m_dSampleRate << "Hz =" << requestedBufferTime.formatMillisWithUnit();
     }
-    return SOUNDDEVICE_ERROR_OK;
+
+    return SoundDeviceStatus::Ok;
 }
 
 bool SoundDeviceNetwork::isOpen() const {
     return (m_inputFifo != nullptr || m_outputFifo != nullptr);
 }
 
-SoundDeviceError SoundDeviceNetwork::close() {
+SoundDeviceStatus SoundDeviceNetwork::close() {
     //kLogger.debug() << "close:" << getInternalName();
     m_pNetworkStream->stopStream();
     if (m_pThread) {
@@ -123,7 +123,7 @@ SoundDeviceError SoundDeviceNetwork::close() {
     m_outputFifo.reset();
     m_inputFifo.reset();
 
-    return SOUNDDEVICE_ERROR_OK;
+    return SoundDeviceStatus::Ok;
 }
 
 QString SoundDeviceNetwork::getError() const {
@@ -441,7 +441,7 @@ void SoundDeviceNetwork::callbackProcessClkRef() {
         m_denormals = true;
         // This disables the denormals calculations, to avoid a
         // performance penalty of ~20
-        // https://bugs.launchpad.net/mixxx/+bug/1404401
+        // https://github.com/mixxxdj/mixxx/issues/7747
 #ifdef __SSE__
         if (!_MM_GET_DENORMALS_ZERO_MODE()) {
             qDebug() << "SSE: Enabling denormals to zero mode";
@@ -514,11 +514,11 @@ void SoundDeviceNetwork::updateAudioLatencyUsage(SINT framesPerBuffer) {
     if (m_framesSinceAudioLatencyUsageUpdate
             > (m_dSampleRate / CPU_USAGE_UPDATE_RATE)) {
         double secInAudioCb = m_timeInAudioCallback.toDoubleSeconds();
-        m_pMasterAudioLatencyUsage->set(secInAudioCb /
+        m_audioLatencyUsage.set(secInAudioCb /
                 (m_framesSinceAudioLatencyUsageUpdate / m_dSampleRate));
         m_timeInAudioCallback = mixxx::Duration::empty();
         m_framesSinceAudioLatencyUsageUpdate = 0;
-        //qDebug() << m_pMasterAudioLatencyUsage->get();
+        // qDebug() << m_audioLatencyUsage->get();
     }
 
     qint64 currentTime = m_pNetworkStream->getInputStreamTimeUs();
