@@ -5,12 +5,13 @@
 #include <QString>
 #include <memory>
 
+#include "control/pollingcontrolproxy.h"
 #include "soundio/sounddevice.h"
 #include "util/duration.h"
+#include "util/fifo.h"
 #include "util/performancetimer.h"
 
 class SoundManager;
-class ControlProxy;
 
 class SoundDevicePortAudio : public SoundDevice {
   public:
@@ -21,9 +22,9 @@ class SoundDevicePortAudio : public SoundDevice {
             unsigned int devIndex);
     ~SoundDevicePortAudio() override;
 
-    SoundDeviceError open(bool isClkRefDevice, int syncBuffers) override;
+    SoundDeviceStatus open(bool isClkRefDevice, int syncBuffers) override;
     bool isOpen() const override;
-    SoundDeviceError close() override;
+    SoundDeviceStatus close() override;
     void readProcess(SINT framesPerBuffer) override;
     void writeProcess(SINT framesPerBuffer) override;
     QString getError() const override;
@@ -74,7 +75,7 @@ class SoundDevicePortAudio : public SoundDevice {
     QString m_lastError;
     // Whether we have set the thread priority to realtime or not.
     bool m_bSetThreadPriority;
-    ControlProxy* m_pMasterAudioLatencyUsage;
+    PollingControlProxy m_audioLatencyUsage;
     mixxx::Duration m_timeInAudioCallback;
     int m_framesSinceAudioLatencyUsageUpdate;
     int m_syncBuffers;
