@@ -3,15 +3,12 @@
 #include <portaudio.h>
 
 #include <QLibrary>
-#include <QtDebug>
+#include <QThread>
 #include <cstring> // for memcpy and strcmp
 
 #include "control/controlobject.h"
-#include "control/controlproxy.h"
-#include "engine/enginebuffer.h"
-#include "engine/enginemaster.h"
+#include "engine/enginemixer.h"
 #include "engine/sidechain/enginenetworkstream.h"
-#include "engine/sidechain/enginesidechain.h"
 #include "moc_soundmanager.cpp"
 #include "soundio/sounddevice.h"
 #include "soundio/sounddevicenetwork.h"
@@ -19,10 +16,9 @@
 #include "soundio/sounddeviceportaudio.h"
 #include "soundio/soundmanagerutil.h"
 #include "util/cmdlineargs.h"
-#include "util/compatibility.h"
+#include "util/compatibility/qatomic.h"
 #include "util/defs.h"
 #include "util/sample.h"
-#include "util/sleep.h"
 #include "util/versionstore.h"
 #include "vinylcontrol/defs_vinylcontrol.h"
 
@@ -30,6 +26,8 @@ typedef PaError (*SetJackClientName)(const char *name);
 
 namespace {
 
+const QString kAppGroup = QStringLiteral("[App]");
+
 #define CPU_OVERLOAD_DURATION 500 // in ms
 
 struct DeviceMode {
@@ -39,19 +37,21 @@ struct DeviceMode {
 };
 
 #ifdef __LINUX__
-const unsigned int kSleepSecondsAfterClosingDevice = 5;
+constexpr unsigned int kSleepSecondsAfterClosingDevice = 5;
 #endif
 } // anonymous namespace
 
 SoundManager::SoundManager(UserSettingsPointer pConfig,
-        EngineMaster* pMaster)
-        : m_pMaster(pMaster),
+        EngineMixer* pEngineMixer)
+        : m_pEngineMixer(pEngineMixer),
           m_pConfig(pConfig),
           m_paInitialized(false),
           m_config(this),
           m_pErrorDevice(nullptr),
           m_underflowHappened(0),
-          m_underflowUpdateCount(0) {
+          m_underflowUpdateCount(0),
+          m_audioLatencyOverloadCount(kAppGroup, QStringLiteral("audio_latency_overload_count")),
+          m_audioLatencyOverload(kAppGroup, QStringLiteral("audio_latency_overload")) {
     // TODO(xxx) some of these ControlObject are not needed by soundmanager, or are unused here.
     // It is possible to take them out?
     m_pControlObjectSoundStatusCO = new ControlObject(
@@ -61,12 +61,6 @@ SoundManager::SoundManager(UserSettingsPointer pConfig,
     m_pControlObjectVinylControlGainCO = new ControlObject(
             ConfigKey(VINYL_PREF_KEY, "gain"));
 
-    m_pMasterAudioLatencyOverloadCount = new ControlProxy("[Master]",
-            "audio_latency_overload_count");
-
-    m_pMasterAudioLatencyOverload = new ControlProxy("[Master]",
-            "audio_latency_overload");
-
     //Hack because PortAudio samplerate enumeration is slow as hell on Linux (ALSA dmix sucks, so we can't blame PortAudio)
     m_samplerates.push_back(44100);
     m_samplerates.push_back(48000);
@@ -101,8 +95,6 @@ SoundManager::~SoundManager() {
 
     delete m_pControlObjectSoundStatusCO;
     delete m_pControlObjectVinylControlGainCO;
-    delete m_pMasterAudioLatencyOverloadCount;
-    delete m_pMasterAudioLatencyOverload;
 }
 
 QList<SoundDevicePointer> SoundManager::getDeviceList(
@@ -152,7 +144,7 @@ void SoundManager::closeDevices(bool sleepAfterClosing) {
     //qDebug() << "SoundManager::closeDevices()";
 
     bool closed = false;
-    for (const auto& pDevice: qAsConst(m_devices)) {
+    for (const auto& pDevice : std::as_const(m_devices)) {
         if (pDevice->isOpen()) {
             // NOTE(rryan): As of 2009 (?) it has been safe to close() a SoundDevice
             // while callbacks are active.
@@ -165,21 +157,21 @@ void SoundManager::closeDevices(bool sleepAfterClosing) {
 #ifdef __LINUX__
         // Sleep for 5 sec to allow asynchronously sound APIs like "pulse" to free
         // its resources as well
-        sleep(kSleepSecondsAfterClosingDevice);
+        QThread::sleep(kSleepSecondsAfterClosingDevice);
 #endif
     }
 
     // TODO(rryan): Should we do this before SoundDevice::close()? No! Because
     // then the callback may be running when we call
     // onInputDisconnected/onOutputDisconnected.
-    for (const auto& pDevice: qAsConst(m_devices)) {
+    for (const auto& pDevice : std::as_const(m_devices)) {
         for (const auto& in: pDevice->inputs()) {
             // Need to tell all registered AudioDestinations for this AudioInput
             // that the input was disconnected.
             for (auto it = m_registeredDestinations.constFind(in);
                  it != m_registeredDestinations.constEnd() && it.key() == in; ++it) {
                 it.value()->onInputUnconfigured(in);
-                m_pMaster->onInputDisconnected(in);
+                m_pEngineMixer->onInputDisconnected(in);
             }
         }
         for (const auto& out: pDevice->outputs()) {
@@ -321,13 +313,13 @@ void SoundManager::queryDevicesMixxx() {
     m_devices.append(currentDevice);
 }
 
-SoundDeviceError SoundManager::setupDevices() {
+SoundDeviceStatus SoundManager::setupDevices() {
     // NOTE(rryan): Big warning: This function is concurrent with calls to
     // pushBuffer and onDeviceOutputCallback until closeDevices() below.
 
     qDebug() << "SoundManager::setupDevices()";
     m_pControlObjectSoundStatusCO->set(SOUNDMANAGER_CONNECTING);
-    SoundDeviceError err = SOUNDDEVICE_ERROR_OK;
+    SoundDeviceStatus status = SoundDeviceStatus::Ok;
     // NOTE(rryan): Do not clear m_pClkRefDevice here. If we didn't touch the
     // SoundDevice that is the clock reference, then it is safe to leave it as
     // it was. Clearing it causes the engine to stop being processed which
@@ -353,9 +345,9 @@ SoundDeviceError SoundManager::setupDevices() {
 
     // Instead of clearing m_pClkRefDevice and then assigning it directly,
     // compute the new one then atomically hand off below.
-    SoundDevicePointer pNewMasterClockRef;
+    SoundDevicePointer pNewMainClockRef;
 
-    m_pMasterAudioLatencyOverloadCount->set(0);
+    m_audioLatencyOverloadCount.set(0);
 
     // load with all configured devices.
     // all found devices are removed below
@@ -365,7 +357,7 @@ SoundDeviceError SoundManager::setupDevices() {
     QVector<DeviceMode> toOpen;
     bool haveOutput = false;
     // loop over all available devices
-    for (const auto& pDevice: qAsConst(m_devices)) {
+    for (const auto& pDevice : std::as_const(m_devices)) {
         DeviceMode mode = {pDevice, false, false};
         pDevice->clearInputs();
         pDevice->clearOutputs();
@@ -376,8 +368,8 @@ SoundDeviceError SoundManager::setupDevices() {
             // TODO(bkgood) look into allocating this with the frames per
             // buffer value from SMConfig
             AudioInputBuffer aib(in, SampleUtil::alloc(MAX_BUFFER_LEN));
-            err = pDevice->addInput(aib);
-            if (err != SOUNDDEVICE_ERROR_OK) {
+            status = pDevice->addInput(aib);
+            if (status != SoundDeviceStatus::Ok) {
                 SampleUtil::free(aib.getBuffer());
                 goto closeAndError;
             }
@@ -390,7 +382,7 @@ SoundDeviceError SoundManager::setupDevices() {
                     it != m_registeredDestinations.end() && it.key() == in;
                     ++it) {
                 it.value()->onInputConfigured(in);
-                m_pMaster->onInputConnected(in);
+                m_pEngineMixer->onInputConnected(in);
             }
         }
         QList<AudioOutput> outputs =
@@ -398,19 +390,19 @@ SoundDeviceError SoundManager::setupDevices() {
 
         // Statically connect the Network Device to the Sidechain
         if (pDevice->getDeviceId().name == kNetworkDeviceInternalName) {
-            AudioOutput out(AudioPath::RECORD_BROADCAST, 0, 2, 0);
+            AudioOutput out(AudioPathType::RecordBroadcast, 0, 2, 0);
             outputs.append(out);
             if (m_config.getForceNetworkClock() && !jackApiUsed()) {
-                pNewMasterClockRef = pDevice;
+                pNewMainClockRef = pDevice;
             }
         }
 
-        for (const auto& out: qAsConst(outputs)) {
+        for (const auto& out : std::as_const(outputs)) {
             mode.isOutput = true;
             if (pDevice->getDeviceId().name != kNetworkDeviceInternalName) {
                 haveOutput = true;
             }
-            // following keeps us from asking for a channel buffer EngineMaster
+            // following keeps us from asking for a channel buffer EngineMixer
             // doesn't have -- bkgood
             const CSAMPLE* pBuffer = m_registeredSources.value(out)->buffer(out);
             if (pBuffer == nullptr) {
@@ -419,18 +411,18 @@ SoundDeviceError SoundManager::setupDevices() {
             }
 
             AudioOutputBuffer aob(out, pBuffer);
-            err = pDevice->addOutput(aob);
-            if (err != SOUNDDEVICE_ERROR_OK) {
+            status = pDevice->addOutput(aob);
+            if (status != SoundDeviceStatus::Ok) {
                 goto closeAndError;
             }
 
             if (!m_config.getForceNetworkClock() || jackApiUsed()) {
-                if (out.getType() == AudioOutput::MASTER) {
-                    pNewMasterClockRef = pDevice;
-                } else if ((out.getType() == AudioOutput::DECK ||
-                            out.getType() == AudioOutput::BUS)
-                        && !pNewMasterClockRef) {
-                    pNewMasterClockRef = pDevice;
+                if (out.getType() == AudioPathType::Main) {
+                    pNewMainClockRef = pDevice;
+                } else if ((out.getType() == AudioPathType::Deck ||
+                                   out.getType() == AudioPathType::Bus) &&
+                        !pNewMainClockRef) {
+                    pNewMainClockRef = pDevice;
                 }
             }
 
@@ -456,9 +448,9 @@ SoundDeviceError SoundManager::setupDevices() {
 
         // If we have not yet set a clock source then we use the first
         // output pDevice
-        if (pNewMasterClockRef.isNull() &&
+        if (pNewMainClockRef.isNull() &&
                 (!haveOutput || mode.isOutput)) {
-            pNewMasterClockRef = pDevice;
+            pNewMainClockRef = pDevice;
             qWarning() << "Output sound device clock reference not set! Using"
                        << pDevice->getDisplayName();
         }
@@ -469,8 +461,8 @@ SoundDeviceError SoundManager::setupDevices() {
         if (CmdlineArgs::Instance().getSafeMode() && syncBuffers == 0) {
             syncBuffers = 2;
         }
-        err = pDevice->open(pNewMasterClockRef == pDevice, syncBuffers);
-        if (err != SOUNDDEVICE_ERROR_OK) {
+        status = pDevice->open(pNewMainClockRef == pDevice, syncBuffers);
+        if (status != SoundDeviceStatus::Ok) {
             goto closeAndError;
         }
         devicesNotFound.remove(pDevice->getDeviceId());
@@ -482,8 +474,8 @@ SoundDeviceError SoundManager::setupDevices() {
         }
     }
 
-    if (pNewMasterClockRef) {
-        qDebug() << "Using" << pNewMasterClockRef->getDisplayName()
+    if (pNewMainClockRef) {
+        qDebug() << "Using" << pNewMainClockRef->getDisplayName()
                  << "as output sound device clock reference";
     } else {
         qWarning() << "No output devices opened, no clock reference device set";
@@ -502,16 +494,16 @@ SoundDeviceError SoundManager::setupDevices() {
     // returns OK if we were able to open all the devices the user wanted
     if (devicesNotFound.isEmpty()) {
         emit devicesSetup();
-        return SOUNDDEVICE_ERROR_OK;
+        return SoundDeviceStatus::Ok;
     }
     m_pErrorDevice = SoundDevicePointer(
             new SoundDeviceNotFound(devicesNotFound.constBegin()->name));
-    return SOUNDDEVICE_ERROR_DEVICE_COUNT;
+    return SoundDeviceStatus::ErrorDeviceCount;
 
 closeAndError:
     const bool sleepAfterClosing = false;
     closeDevices(sleepAfterClosing);
-    return err;
+    return status;
 }
 
 SoundDevicePointer SoundManager::getErrorDevice() const {
@@ -526,7 +518,7 @@ QString SoundManager::getErrorDeviceName() const {
     return tr("a device");
 }
 
-QString SoundManager::getLastErrorMessage(SoundDeviceError err) const {
+QString SoundManager::getLastErrorMessage(SoundDeviceStatus status) const {
     QString error;
     QString deviceName(tr("a device"));
     QString detailedError(tr("An unknown error occurred"));
@@ -535,8 +527,8 @@ QString SoundManager::getLastErrorMessage(SoundDeviceError err) const {
         deviceName = pDevice->getDisplayName();
         detailedError = pDevice->getError();
     }
-    switch (err) {
-    case SOUNDDEVICE_ERROR_DUPLICATE_OUTPUT_CHANNEL:
+    switch (status) {
+    case SoundDeviceStatus::ErrorDuplicateOutputChannel:
         error = tr("Two outputs cannot share channels on \"%1\"").arg(deviceName);
         break;
     default:
@@ -550,8 +542,8 @@ SoundManagerConfig SoundManager::getConfig() const {
     return m_config;
 }
 
-SoundDeviceError SoundManager::setConfig(const SoundManagerConfig& config) {
-    SoundDeviceError err = SOUNDDEVICE_ERROR_OK;
+SoundDeviceStatus SoundManager::setConfig(const SoundManagerConfig& config) {
+    SoundDeviceStatus status = SoundDeviceStatus::Ok;
     m_config = config;
     checkConfig();
 
@@ -567,11 +559,11 @@ SoundDeviceError SoundManager::setConfig(const SoundManagerConfig& config) {
     m_pConfig->set(ConfigKey("[Soundcard]","Samplerate"),
                    ConfigValue(static_cast<int>(m_config.getSampleRate())));
 
-    err = setupDevices();
-    if (err == SOUNDDEVICE_ERROR_OK) {
+    status = setupDevices();
+    if (status == SoundDeviceStatus::Ok) {
         m_config.writeToDisk();
     }
-    return err;
+    return status;
 }
 
 void SoundManager::checkConfig() {
@@ -591,9 +583,9 @@ void SoundManager::checkConfig() {
 }
 
 void SoundManager::onDeviceOutputCallback(const SINT iFramesPerBuffer) {
-    // Produce a block of samples for output. EngineMaster expects stereo
+    // Produce a block of samples for output. EngineMixer expects stereo
     // samples so multiply iFramesPerBuffer by 2.
-    m_pMaster->process(iFramesPerBuffer * 2);
+    m_pEngineMixer->process(iFramesPerBuffer * 2);
 }
 
 void SoundManager::pushInputBuffers(const QList<AudioInputBuffer>& inputs,
@@ -692,9 +684,9 @@ int SoundManager::getConfiguredDeckCount() const {
 void SoundManager::processUnderflowHappened(SINT framesPerBuffer) {
     if (m_underflowUpdateCount == 0) {
         if (atomicLoadRelaxed(m_underflowHappened)) {
-            m_pMasterAudioLatencyOverload->set(1.0);
-            m_pMasterAudioLatencyOverloadCount->set(
-                    m_pMasterAudioLatencyOverloadCount->get() + 1);
+            m_audioLatencyOverload.set(1.0);
+            m_audioLatencyOverloadCount.set(
+                    m_audioLatencyOverloadCount.get() + 1);
             m_underflowUpdateCount = CPU_OVERLOAD_DURATION *
                     m_config.getSampleRate() / framesPerBuffer / 1000;
 
@@ -702,7 +694,7 @@ void SoundManager::processUnderflowHappened(SINT framesPerBuffer) {
                                      // but that is OK, because we count only
                                      // 1 underflow each 500 ms
         } else {
-            m_pMasterAudioLatencyOverload->set(0.0);
+            m_audioLatencyOverload.set(0.0);
         }
     } else {
         --m_underflowUpdateCount;
