@@ -5,9 +5,9 @@
 #include <QObject>
 #include <QSharedPointer>
 #include <QString>
-#include <memory>
 
 #include "audio/types.h"
+#include "control/pollingcontrolproxy.h"
 #include "engine/sidechain/enginenetworkstream.h"
 #include "preferences/usersettings.h"
 #include "soundio/sounddevice.h"
@@ -15,14 +15,8 @@
 #include "util/cmdlineargs.h"
 #include "util/types.h"
 
-class EngineMaster;
-class AudioOutput;
-class AudioInput;
-class AudioSource;
-class AudioDestination;
+class EngineMixer;
 class ControlObject;
-class ControlProxy;
-class SoundDeviceNotFound;
 
 #define MIXXX_PORTAUDIO_JACK_STRING "JACK Audio Connection Kit"
 #define MIXXX_PORTAUDIO_ALSA_STRING "ALSA"
@@ -39,7 +33,7 @@ class SoundDeviceNotFound;
 class SoundManager : public QObject {
     Q_OBJECT
   public:
-    SoundManager(UserSettingsPointer pConfig, EngineMaster *_master);
+    SoundManager(UserSettingsPointer pConfig, EngineMixer* pEngineMixer);
     ~SoundManager() override;
 
     // Returns a list of all devices we've enumerated that match the provided
@@ -56,7 +50,7 @@ class SoundManager : public QObject {
 
     // Opens all the devices chosen by the user in the preferences dialog, and
     // establishes the proper connections between them and the mixing engine.
-    SoundDeviceError setupDevices();
+    SoundDeviceStatus setupDevices();
 
     // Playermanager will notify us when the number of decks changes.
     void setConfiguredDeckCount(int count);
@@ -64,7 +58,7 @@ class SoundManager : public QObject {
 
     SoundDevicePointer getErrorDevice() const;
     QString getErrorDeviceName() const;
-    QString getLastErrorMessage(SoundDeviceError err) const;
+    QString getLastErrorMessage(SoundDeviceStatus status) const;
 
     // Returns a list of samplerates we will attempt to support for a given API.
     QList<unsigned int> getSampleRates(const QString& api) const;
@@ -75,7 +69,7 @@ class SoundManager : public QObject {
     // Get a list of host APIs supported by PortAudio.
     QList<QString> getHostAPIList() const;
     SoundManagerConfig getConfig() const;
-    SoundDeviceError setConfig(const SoundManagerConfig& config);
+    SoundDeviceStatus setConfig(const SoundManagerConfig& config);
     void checkConfig();
 
     void onDeviceOutputCallback(const SINT iFramesPerBuffer);
@@ -129,7 +123,7 @@ class SoundManager : public QObject {
         return m_config.getAPI() == MIXXX_PORTAUDIO_JACK_STRING;
     }
 
-    EngineMaster *m_pMaster;
+    EngineMixer* m_pEngineMixer;
     UserSettingsPointer m_pConfig;
     bool m_paInitialized;
     mixxx::audio::SampleRate m_jackSampleRate;
@@ -148,6 +142,6 @@ class SoundManager : public QObject {
 
     QAtomicInt m_underflowHappened;
     int m_underflowUpdateCount;
-    ControlProxy* m_pMasterAudioLatencyOverloadCount;
-    ControlProxy* m_pMasterAudioLatencyOverload;
+    PollingControlProxy m_audioLatencyOverloadCount;
+    PollingControlProxy m_audioLatencyOverload;
 };
