@@ -1,21 +1,15 @@
 #include "sources/metadatasourcetaglib.h"
 
-#include <taglib/vorbisfile.h>
+#include <opusfile.h>
+#include <vorbisfile.h>
 
 #include <QFile>
-#include <QFileInfo>
-#include <QThread>
+#include <memory>
 
 #include "track/taglib/trackmetadata.h"
+#include "track/taglib/trackmetadata_common.h"
 #include "util/logger.h"
-#include "util/memory.h"
-#if (TAGLIB_HAS_OPUSFILE)
-#include <taglib/opusfile.h>
-#endif
-
-#if defined(__WINDOWS__)
-#include <Windows.h>
-#endif
+#include "util/safelywritablefile.h"
 
 namespace mixxx {
 
@@ -23,22 +17,6 @@ namespace {
 
 Logger kLogger("MetadataSourceTagLib");
 
-// TODO(uklotzde): Add a configurable option in the user settings
-const bool kExportTrackMetadataIntoTemporaryFile = true;
-
-// Appended to the original file name of the temporary file used for writing
-const QString kSafelyWritableTempFileSuffix = QStringLiteral("_temp");
-
-// Appended to the original file name for renaming and before deleting this
-// file. Should not be longer than kSafelyWritableTempFileSuffix to avoid
-// potential failures caused by exceeded path length.
-const QString kSafelyWritableOrigFileSuffix = QStringLiteral("_orig");
-
-#if defined(__WINDOWS__)
-const int kWindowsSharingViolationMaxRetries = 5;
-const int kWindowsSharingViolationSleepBeforeNextRetryMillis = 100;
-#endif
-
 // Workaround for missing functionality in TagLib 1.11.x that
 // doesn't support to read text chunks from AIFF files.
 // See also:
@@ -84,38 +62,46 @@ class AiffFile : public TagLib::RIFF::AIFF::File {
     }
 };
 
-inline QDateTime getMetadataSynchronized(const QFileInfo& fileInfo) {
-    return fileInfo.lastModified();
-}
-
 } // anonymous namespace
 
 std::pair<MetadataSourceTagLib::ImportResult, QDateTime>
 MetadataSourceTagLib::afterImport(ImportResult importResult) const {
-    return std::make_pair(importResult, getMetadataSynchronized(QFileInfo(m_fileName)));
+    const auto sourceSynchronizedAt =
+            MetadataSource::getFileSynchronizedAt(QFile(m_fileName));
+    DEBUG_ASSERT(sourceSynchronizedAt.isValid() ||
+            importResult != ImportResult::Succeeded);
+    return std::make_pair(importResult, sourceSynchronizedAt);
 }
 
 std::pair<MetadataSourceTagLib::ExportResult, QDateTime>
 MetadataSourceTagLib::afterExport(ExportResult exportResult) const {
-    return std::make_pair(exportResult, getMetadataSynchronized(QFileInfo(m_fileName)));
+    const auto sourceSynchronizedAt =
+            MetadataSource::getFileSynchronizedAt(QFile(m_fileName));
+    DEBUG_ASSERT(sourceSynchronizedAt.isValid() ||
+            exportResult != ExportResult::Succeeded);
+    return std::make_pair(exportResult, sourceSynchronizedAt);
 }
 
 std::pair<MetadataSource::ImportResult, QDateTime>
 MetadataSourceTagLib::importTrackMetadataAndCoverImage(
         TrackMetadata* pTrackMetadata,
-        QImage* pCoverImage) const {
+        QImage* pCoverImage,
+        bool resetMissingTagMetadata) const {
     VERIFY_OR_DEBUG_ASSERT(pTrackMetadata || pCoverImage) {
         kLogger.warning()
                 << "Nothing to import"
                 << "from file" << m_fileName
-                << "with type" << m_fileType;
+                << "of type" << m_fileType;
         return afterImport(ImportResult::Unavailable);
     }
     if (kLogger.traceEnabled()) {
         kLogger.trace() << "Importing"
-                        << ((pTrackMetadata && pCoverImage) ? "track metadata and cover art" : (pTrackMetadata ? "track metadata" : "cover art"))
+                        << ((pTrackMetadata && pCoverImage)
+                                           ? "track metadata and cover art"
+                                           : (pTrackMetadata ? "track metadata"
+                                                             : "cover art"))
                         << "from file" << m_fileName
-                        << "with type" << m_fileType;
+                        << "of type" << m_fileType;
     }
 
     // Rationale: If a file contains different types of tags only
@@ -134,20 +120,21 @@ MetadataSourceTagLib::importTrackMetadataAndCoverImage(
         if (taglib::hasID3v2Tag(file)) {
             const TagLib::ID3v2::Tag* pTag = file.ID3v2Tag();
             DEBUG_ASSERT(pTag);
-            taglib::id3v2::importTrackMetadataFromTag(pTrackMetadata, *pTag);
+            taglib::id3v2::importTrackMetadataFromTag(
+                    pTrackMetadata, *pTag, resetMissingTagMetadata);
             taglib::id3v2::importCoverImageFromTag(pCoverImage, *pTag);
             return afterImport(ImportResult::Succeeded);
         } else if (taglib::hasAPETag(file)) {
             const TagLib::APE::Tag* pTag = file.APETag();
             DEBUG_ASSERT(pTag);
-            taglib::ape::importTrackMetadataFromTag(pTrackMetadata, *pTag);
+            taglib::ape::importTrackMetadataFromTag(pTrackMetadata, *pTag, resetMissingTagMetadata);
             taglib::ape::importCoverImageFromTag(pCoverImage, *pTag);
             return afterImport(ImportResult::Succeeded);
         } else if (taglib::hasID3v1Tag(file)) {
             // Note (TagLib 1.1.11): TagLib::MPEG::File::tag() returns a
             // valid pointer even if neither an ID3v2 nor an ID3v1 tag is
             // present!
-            // See also: https://bugs.launchpad.net/mixxx/+bug/1865957
+            // See also: https://github.com/mixxxdj/mixxx/issues/9891
             const TagLib::Tag* pTag = file.tag();
             if (pTag) {
                 taglib::importTrackMetadataFromTag(pTrackMetadata, *pTag);
@@ -164,7 +151,7 @@ MetadataSourceTagLib::importTrackMetadataAndCoverImage(
         if (taglib::hasMP4Tag(file)) {
             const TagLib::MP4::Tag* pTag = file.tag();
             DEBUG_ASSERT(pTag);
-            taglib::mp4::importTrackMetadataFromTag(pTrackMetadata, *pTag);
+            taglib::mp4::importTrackMetadataFromTag(pTrackMetadata, *pTag, resetMissingTagMetadata);
             taglib::mp4::importCoverImageFromTag(pCoverImage, *pTag);
             return afterImport(ImportResult::Succeeded);
         }
@@ -181,13 +168,17 @@ MetadataSourceTagLib::importTrackMetadataAndCoverImage(
         if (taglib::hasXiphComment(file)) {
             TagLib::Ogg::XiphComment* pTag = file.xiphComment();
             DEBUG_ASSERT(pTag);
-            taglib::xiph::importTrackMetadataFromTag(pTrackMetadata, *pTag, taglib::FileType::FLAC);
+            taglib::xiph::importTrackMetadataFromTag(pTrackMetadata,
+                    *pTag,
+                    taglib::FileType::FLAC,
+                    resetMissingTagMetadata);
             coverImageImported = taglib::xiph::importCoverImageFromTag(pCoverImage, *pTag);
             importSucceeded = true;
         } else if (taglib::hasID3v2Tag(file)) {
             const TagLib::ID3v2::Tag* pTag = file.ID3v2Tag();
             DEBUG_ASSERT(pTag);
-            taglib::id3v2::importTrackMetadataFromTag(pTrackMetadata, *pTag);
+            taglib::id3v2::importTrackMetadataFromTag(
+                    pTrackMetadata, *pTag, resetMissingTagMetadata);
             coverImageImported = taglib::id3v2::importCoverImageFromTag(pCoverImage, *pTag);
             importSucceeded = true;
         }
@@ -214,13 +205,15 @@ MetadataSourceTagLib::importTrackMetadataAndCoverImage(
         }
         TagLib::Ogg::XiphComment* pTag = file.tag();
         if (pTag) {
-            taglib::xiph::importTrackMetadataFromTag(pTrackMetadata, *pTag, taglib::FileType::OGG);
+            taglib::xiph::importTrackMetadataFromTag(pTrackMetadata,
+                    *pTag,
+                    taglib::FileType::OGG,
+                    resetMissingTagMetadata);
             taglib::xiph::importCoverImageFromTag(pCoverImage, *pTag);
             return afterImport(ImportResult::Succeeded);
         }
         break;
     }
-#if (TAGLIB_HAS_OPUSFILE)
     case taglib::FileType::OPUS: {
         TagLib::Ogg::Opus::File file(TAGLIB_FILENAME_FROM_QSTRING(m_fileName));
         if (!taglib::readAudioPropertiesFromFile(pTrackMetadata, file)) {
@@ -228,13 +221,15 @@ MetadataSourceTagLib::importTrackMetadataAndCoverImage(
         }
         TagLib::Ogg::XiphComment* pTag = file.tag();
         if (pTag) {
-            taglib::xiph::importTrackMetadataFromTag(pTrackMetadata, *pTag, taglib::FileType::OPUS);
+            taglib::xiph::importTrackMetadataFromTag(pTrackMetadata,
+                    *pTag,
+                    taglib::FileType::OPUS,
+                    resetMissingTagMetadata);
             taglib::xiph::importCoverImageFromTag(pCoverImage, *pTag);
             return afterImport(ImportResult::Succeeded);
         }
         break;
     }
-#endif // TAGLIB_HAS_OPUSFILE
     case taglib::FileType::WV: {
         TagLib::WavPack::File file(TAGLIB_FILENAME_FROM_QSTRING(m_fileName));
         if (!taglib::readAudioPropertiesFromFile(pTrackMetadata, file)) {
@@ -243,7 +238,7 @@ MetadataSourceTagLib::importTrackMetadataAndCoverImage(
         if (taglib::hasAPETag(file)) {
             const TagLib::APE::Tag* pTag = file.APETag();
             DEBUG_ASSERT(pTag);
-            taglib::ape::importTrackMetadataFromTag(pTrackMetadata, *pTag);
+            taglib::ape::importTrackMetadataFromTag(pTrackMetadata, *pTag, resetMissingTagMetadata);
             taglib::ape::importCoverImageFromTag(pCoverImage, *pTag);
             return afterImport(ImportResult::Succeeded);
         }
@@ -255,13 +250,10 @@ MetadataSourceTagLib::importTrackMetadataAndCoverImage(
             break;
         }
         if (taglib::hasID3v2Tag(file)) {
-#if (TAGLIB_HAS_WAV_ID3V2TAG)
             const TagLib::ID3v2::Tag* pTag = file.ID3v2Tag();
-#else
-            const TagLib::ID3v2::Tag* pTag = file.tag();
-#endif
             DEBUG_ASSERT(pTag);
-            taglib::id3v2::importTrackMetadataFromTag(pTrackMetadata, *pTag);
+            taglib::id3v2::importTrackMetadataFromTag(
+                    pTrackMetadata, *pTag, resetMissingTagMetadata);
             taglib::id3v2::importCoverImageFromTag(pCoverImage, *pTag);
             return afterImport(ImportResult::Succeeded);
         } else if (file.hasInfoTag()) {
@@ -280,7 +272,8 @@ MetadataSourceTagLib::importTrackMetadataAndCoverImage(
         if (taglib::hasID3v2Tag(file)) {
             const TagLib::ID3v2::Tag* pTag = file.tag();
             DEBUG_ASSERT(pTag);
-            taglib::id3v2::importTrackMetadataFromTag(pTrackMetadata, *pTag);
+            taglib::id3v2::importTrackMetadataFromTag(
+                    pTrackMetadata, *pTag, resetMissingTagMetadata);
             taglib::id3v2::importCoverImageFromTag(pCoverImage, *pTag);
             return afterImport(ImportResult::Succeeded);
         } else if (file.importTrackMetadataFromTextChunks(pTrackMetadata)) {
@@ -471,7 +464,7 @@ class OggTagSaver : public TagSaver {
         (TAGLIB_PATCH_VERSION == 1)
         // TagLib 1.11.1 suffers from a serious bug that corrupts OGG files
         // when writing tags: https://github.com/taglib/taglib/issues/864
-        // Launchpad issue: https://bugs.launchpad.net/mixxx/+bug/1833190
+        // Launchpad issue: https://github.com/mixxxdj/mixxx/issues/9680
         Q_UNUSED(pFile);
         Q_UNUSED(trackMetadata);
         kLogger.warning() << "Skipping export of metadata into Ogg file due to "
@@ -489,7 +482,6 @@ class OggTagSaver : public TagSaver {
     bool m_modifiedTags;
 };
 
-#if (TAGLIB_HAS_OPUSFILE)
 class OpusTagSaver : public TagSaver {
   public:
     OpusTagSaver(const QString& fileName, const TrackMetadata& trackMetadata)
@@ -517,7 +509,6 @@ class OpusTagSaver : public TagSaver {
     TagLib::Ogg::Opus::File m_file;
     bool m_modifiedTags;
 };
-#endif // TAGLIB_HAS_OPUSFILE
 
 class WavPackTagSaver : public TagSaver {
   public:
@@ -576,7 +567,6 @@ class WavTagSaver : public TagSaver {
         if (pFile->isOpen()) {
             TagLib::RIFF::Info::Tag* pInfoTag = nullptr;
             // Write into all available tags
-#if (TAGLIB_HAS_WAV_ID3V2TAG)
             if (pFile->hasID3v2Tag()) {
                 modifiedTags |= taglib::id3v2::exportTrackMetadataIntoTag(
                         pFile->ID3v2Tag(), trackMetadata);
@@ -590,11 +580,6 @@ class WavTagSaver : public TagSaver {
                 pInfoTag = pFile->InfoTag();
                 DEBUG_ASSERT(pInfoTag);
             }
-#else
-            modifiedTags |= taglib::id3v2::exportTrackMetadataIntoTag(pFile->tag(), trackMetadata);
-            pInfoTag = pFile->InfoTag();
-            DEBUG_ASSERT(pInfoTag);
-#endif
             modifiedTags |= exportTrackMetadataIntoRIFFTag(pInfoTag, trackMetadata);
         }
         return modifiedTags;
@@ -629,276 +614,6 @@ class AiffTagSaver : public TagSaver {
     bool m_modifiedTags;
 };
 
-/**
- * When writing the tags in-place directly into the original file
- * an intermediate failure might corrupt this precious file. For
- * example this might occur if the application crashes or is quit
- * unexpectedly, if the original file becomes unavailable while
- * writing by disconnecting a drive, if the file system is running
- * out of free space, or if an unexpected driver or hardware failure
- * occurs.
- *
- * To reduce the risk of corrupting the original file all write
- * operations are performed on a temporary file that is created
- * as an exact copy of the original file. Only after all write
- * operations have finished successfully the original file is
- * replaced with the temporary file.
- */
-class SafelyWritableFile final {
-  public:
-    SafelyWritableFile(QString origFileName, bool useTemporaryFile) {
-        // Both file names remain uninitialized until all prerequisite operations
-        // in the constructor have been completed successfully. Otherwise failure
-        // to create the temporary file will not be handled correctly!
-        // See also: https://bugs.launchpad.net/mixxx/+bug/1815305
-        DEBUG_ASSERT(m_origFileName.isNull());
-        DEBUG_ASSERT(m_tempFileName.isNull());
-        if (!QFileInfo(origFileName).isWritable()) {
-            kLogger.warning()
-                    << "Failed to prepare file for writing:"
-                    << origFileName
-                    << "is not writable.";
-            // Abort constructor
-            return;
-        }
-        if (useTemporaryFile) {
-            QString tempFileName = origFileName + kSafelyWritableTempFileSuffix;
-            QFile origFile(origFileName);
-            if (!origFile.copy(tempFileName)) {
-                kLogger.warning()
-                        << origFile.errorString()
-                        << "- Failed to clone original into temporary file before writing:"
-                        << origFileName
-                        << "->"
-                        << tempFileName;
-                // Abort constructor
-                return;
-            }
-            QFile tempFile(tempFileName);
-            DEBUG_ASSERT(tempFile.exists());
-            // Both file sizes are expected to be equal after successfully
-            // copying the file contents.
-            VERIFY_OR_DEBUG_ASSERT(origFile.size() == tempFile.size()) {
-                kLogger.warning()
-                        << "Failed to verify size after cloning original into temporary file before writing:"
-                        << origFile.size()
-                        << "<>"
-                        << tempFile.size();
-                // Cleanup
-                if (tempFile.exists() && !tempFile.remove()) {
-                    kLogger.warning()
-                            << tempFile.errorString()
-                            << "- Failed to remove temporary file:"
-                            << tempFileName;
-                }
-                // Abort constructor
-                return;
-            }
-            // Successfully cloned original into temporary file for writing - finish initialization
-            m_origFileName = std::move(origFileName);
-            m_tempFileName = std::move(tempFileName);
-        } else {
-            // Directly write into original file - finish initialization
-            m_origFileName = std::move(origFileName);
-            DEBUG_ASSERT(m_tempFileName.isNull());
-        }
-    }
-    ~SafelyWritableFile() {
-        cancel();
-    }
-
-    const QString& fileName() const {
-        if (m_tempFileName.isNull()) {
-            // If m_tempFileName has not been initialized then no temporary
-            // copy was requested in the constructor.
-            return m_origFileName;
-        } else {
-            return m_tempFileName;
-        }
-    }
-
-    bool isReady() const {
-        return !fileName().isEmpty();
-    }
-
-    bool commit() {
-        if (m_tempFileName.isNull()) {
-            return true; // nothing to do
-        }
-        QString backupFileName = m_origFileName + kSafelyWritableOrigFileSuffix;
-#ifdef __WINDOWS__
-        // After Mixxx has closed the track file, the indexer or virus scanner
-        // might kick in and fail ReplaceFileW() with a sharing violation when
-        // replacing the original file with the one with the updated metadata.
-        int i = 0;
-        for (; i < kWindowsSharingViolationMaxRetries; ++i) {
-            if (ReplaceFileW(
-                        reinterpret_cast<LPCWSTR>(m_origFileName.utf16()),
-                        reinterpret_cast<LPCWSTR>(m_tempFileName.utf16()),
-                        reinterpret_cast<LPCWSTR>(backupFileName.utf16()),
-                        REPLACEFILE_IGNORE_MERGE_ERRORS | REPLACEFILE_IGNORE_ACL_ERRORS,
-                        nullptr,
-                        nullptr)) {
-                // Success, break retry loop
-                break;
-            } else {
-                DWORD error = GetLastError();
-                switch (error) {
-                case ERROR_UNABLE_TO_MOVE_REPLACEMENT:
-                    // The m_tempFileName file could not be renamed. m_origFileName
-                    // file and m_tempFileName file retain their original file names.
-                    kLogger.critical()
-                            << "Unable to rename replacement file"
-                            << m_tempFileName
-                            << "->"
-                            << m_origFileName;
-                    return false;
-                case ERROR_UNABLE_TO_MOVE_REPLACEMENT_2:
-                    // The m_tempFileName file could not be moved. The m_tempFileName file still exists
-                    // under its original name; however, it has inherited the file streams and
-                    // attributes from the file it is replacing. The m_origFileName file still exists.
-                    kLogger.critical()
-                            << "Unable to move replacement file"
-                            << m_tempFileName
-                            << "->"
-                            << m_origFileName;
-                    return false;
-                case ERROR_UNABLE_TO_REMOVE_REPLACED:
-                    // The replaced file could not be deleted. The replaced and replacement files
-                    // retain their original file names.
-                    kLogger.critical()
-                            << "Unable to remove"
-                            << m_origFileName
-                            << "before replacing by"
-                            << m_tempFileName;
-                    return false;
-                case ERROR_SHARING_VIOLATION:
-                    // The process cannot access the file because it is being used by another process.
-                    kLogger.warning()
-                            << "Unable to replace"
-                            << m_origFileName
-                            << "by"
-                            << m_tempFileName
-                            << "because it is used by another process";
-                    QThread::msleep(kWindowsSharingViolationSleepBeforeNextRetryMillis);
-                    continue; // Retry
-                case ERROR_ACCESS_DENIED:
-                    kLogger.critical()
-                            << "Unable to replace"
-                            << m_origFileName
-                            << "by"
-                            << m_tempFileName
-                            << "Access is denied";
-                    return false;
-                default:
-                    // If any other error is returned, such as ERROR_INVALID_PARAMETER, the replaced
-                    // and replacement files will retain their original file names. In this scenario,
-                    // a backup file does not exist and it is not guaranteed that the replacement file
-                    // will have inherited all of the attributes and streams of the replaced file.
-                    kLogger.critical()
-                            << "Error"
-                            << error
-                            << "during replacing"
-                            << m_origFileName
-                            << "by"
-                            << m_tempFileName;
-                    return false;
-                }
-            }
-        }
-        QFile backupFile(backupFileName);
-        if (backupFile.exists()) {
-            if (!backupFile.remove()) {
-                kLogger.warning()
-                        << backupFile.errorString()
-                        << "- Failed to remove backup file after writing:"
-                        << backupFile.fileName();
-                return false;
-            }
-        }
-        if (i >= kWindowsSharingViolationMaxRetries) {
-            // We have given up after the maximum retries in the loop above.
-            return false;
-        }
-#else
-        QFile newFile(m_tempFileName);
-        if (!newFile.exists()) {
-            kLogger.warning()
-                    << "Temporary file not found:"
-                    << newFile.fileName();
-            return false;
-        }
-        QFile oldFile(m_origFileName);
-        if (oldFile.exists()) {
-            DEBUG_ASSERT(!QFile::exists(backupFileName)); // very unlikely, otherwise renaming fails
-            if (!oldFile.rename(backupFileName)) {
-                kLogger.critical()
-                        << oldFile.errorString()
-                        << "- Failed to rename the original file for backup before writing:"
-                        << oldFile.fileName()
-                        << "->"
-                        << backupFileName;
-                return false;
-            }
-        }
-        DEBUG_ASSERT(!QFile::exists(m_origFileName));
-        if (!newFile.rename(m_origFileName)) {
-            kLogger.critical()
-                    << newFile.errorString()
-                    << "- Failed to rename temporary file after writing:"
-                    << newFile.fileName()
-                    << "->"
-                    << m_origFileName;
-            if (oldFile.exists()) {
-                // Try to restore the original file
-                if (!oldFile.rename(m_origFileName)) {
-                    // Undo operation failed
-                    kLogger.warning()
-                            << oldFile.errorString()
-                            << "- Both the original and the temporary file are still available:"
-                            << oldFile.fileName()
-                            << newFile.fileName();
-                }
-                return false;
-            }
-        }
-        if (oldFile.exists()) {
-            if (!oldFile.remove()) {
-                kLogger.warning()
-                        << oldFile.errorString()
-                        << "- Failed to remove backup file after writing:"
-                        << oldFile.fileName();
-                return false;
-            }
-        }
-#endif
-        // Prevent any further interaction and file access
-        m_origFileName = QString();
-        m_tempFileName = QString();
-        return true;
-    }
-
-    void cancel() {
-        if (m_tempFileName.isNull()) {
-            return; // nothing to do
-        }
-        QFile tempFile(m_tempFileName);
-        if (tempFile.exists() && !tempFile.remove()) {
-            kLogger.warning()
-                    << tempFile.errorString()
-                    << "- Failed to remove temporary file:"
-                    << m_tempFileName;
-        }
-        // Prevent any further interaction and file access
-        m_origFileName = QString();
-        m_tempFileName = QString();
-    }
-
-  private:
-    QString m_origFileName;
-    QString m_tempFileName;
-};
-
 } // anonymous namespace
 
 std::pair<MetadataSource::ExportResult, QDateTime>
@@ -909,7 +624,8 @@ MetadataSourceTagLib::exportTrackMetadata(
     // to identify the file that is affected.
     kLogger.info() << "Exporting track metadata into file:" << m_fileName;
 
-    SafelyWritableFile safelyWritableFile(m_fileName, kExportTrackMetadataIntoTemporaryFile);
+    SafelyWritableFile safelyWritableFile(m_fileName,
+            SafelyWritableFile::SafetyMode::Edit);
     if (!safelyWritableFile.isReady()) {
         kLogger.warning()
                 << "Unable to export track metadata into file"
@@ -936,12 +652,10 @@ MetadataSourceTagLib::exportTrackMetadata(
         pTagSaver = std::make_unique<OggTagSaver>(safelyWritableFile.fileName(), trackMetadata);
         break;
     }
-#if (TAGLIB_HAS_OPUSFILE)
     case taglib::FileType::OPUS: {
         pTagSaver = std::make_unique<OpusTagSaver>(safelyWritableFile.fileName(), trackMetadata);
         break;
     }
-#endif // TAGLIB_HAS_OPUSFILE
     case taglib::FileType::WV: {
         pTagSaver = std::make_unique<WavPackTagSaver>(safelyWritableFile.fileName(), trackMetadata);
         break;
