@@ -1,10 +1,12 @@
+#ifdef __BROADCAST__
+
+#include "preferences/broadcastsettings.h"
+
 #include <QFile>
 #include <QString>
 
+#include "preferences/broadcastsettingsmodel.h"
 #include "test/mixxxtest.h"
-#include "preferences/broadcastsettings.h"
-//#include "broadcast/defs_broadcast.h"
-//#include "defs_urls.h"
 
 namespace {
 
@@ -102,3 +104,5 @@ TEST_F(BroadcastSettingsTest, AddRemoveUpdateFromModel) {
 }
 
 } // namespace
+
+#endif // __BROADCAST__
