@@ -6,37 +6,28 @@
 #include "database/mixxxdb.h"
 #include "library/trackcollection.h"
 #include "library/trackcollectionmanager.h"
-#include "test/mixxxtest.h"
+#include "test/mixxxdbtest.h"
+#include "test/soundsourceproviderregistration.h"
 #include "util/db/dbconnectionpooled.h"
 #include "util/db/dbconnectionpooler.h"
 
-class LibraryTest : public MixxxTest {
+class LibraryTest : public MixxxDbTest, SoundSourceProviderRegistration {
   protected:
     LibraryTest();
     ~LibraryTest() override = default;
 
-    const mixxx::DbConnectionPoolPtr& dbConnectionPool() const {
-        return m_dbConnectionPooler;
-    }
-
-    QSqlDatabase dbConnection() const {
-        return mixxx::DbConnectionPooled(m_dbConnectionPooler);
-    }
-
-    TrackCollectionManager* trackCollections() const {
+    TrackCollectionManager* trackCollectionManager() const {
         return m_pTrackCollectionManager.get();
     }
 
     TrackCollection* internalCollection() const {
-        return trackCollections()->internalCollection();
+        return trackCollectionManager()->internalCollection();
     }
 
     TrackPointer getOrAddTrackByLocation(
             const QString& trackLocation) const;
 
   private:
-    const MixxxDb m_mixxxDb;
-    const mixxx::DbConnectionPooler m_dbConnectionPooler;
     const std::unique_ptr<TrackCollectionManager> m_pTrackCollectionManager;
     ControlObject m_keyNotationCO;
 };
