@@ -3,7 +3,9 @@
 #include <QDir>
 #include <QtDebug>
 
+#include "library/searchquery.h"
 #include "library/searchqueryparser.h"
+#include "library/trackset/crate/crate.h"
 #include "test/librarytest.h"
 #include "track/track.h"
 #include "util/assert.h"
@@ -178,6 +180,32 @@ TEST_F(SearchQueryParserTest, TextFilter) {
         qPrintable(pQuery->toSql()));
 }
 
+TEST_F(SearchQueryParserTest, TextFilterEquals) {
+    m_parser.setSearchColumns({"artist", "album"});
+    auto pQuery(m_parser.parseQuery("comment:=\"asdf\"", QString()));
+
+    TrackPointer pTrack(Track::newTemporary());
+    pTrack->setComment("test ASDF test");
+    EXPECT_FALSE(pQuery->match(pTrack));
+    pTrack->setComment("ASDF");
+    EXPECT_TRUE(pQuery->match(pTrack));
+
+    EXPECT_STREQ(
+            qPrintable(QString("comment LIKE 'asdf'")),
+            qPrintable(pQuery->toSql()));
+
+    // Incomplete quoting should use StringMatch::Contains,
+    // i.e. equal to 'comment:asdf'
+    pQuery = m_parser.parseQuery("comment:=\"asdf", QString());
+
+    pTrack->setComment("test ASDF test");
+    EXPECT_TRUE(pQuery->match(pTrack));
+
+    EXPECT_STREQ(
+            qPrintable(QString("comment LIKE '%asdf%'")),
+            qPrintable(pQuery->toSql()));
+}
+
 TEST_F(SearchQueryParserTest, TextFilterEmpty) {
     m_parser.setSearchColumns({"artist", "album"});
     // An empty argument should pass everything.
@@ -631,6 +659,12 @@ TEST_F(SearchQueryParserTest, HumanReadableDurationSearchWithOperators) {
     EXPECT_STREQ(
         qPrintable(QString("duration >= 63")),
         qPrintable(pQuery->toSql()));
+
+    // Seconds out of range
+    pQuery = m_parser.parseQuery("duration:>=1:60", QString());
+    pTrack->setDuration(60);
+    EXPECT_FALSE(pQuery->match(pTrack));
+    EXPECT_TRUE(pQuery->toSql().isEmpty());
 }
 
 TEST_F(SearchQueryParserTest, HumanReadableDurationSearchwithRangeFilter) {
@@ -695,10 +729,10 @@ TEST_F(SearchQueryParserTest, CrateFilter) {
     auto pQuery(m_parser.parseQuery(QString("crate: %1").arg(searchTerm), QString()));
 
     // locations for test tracks
-    const QString kTrackALocationTest(QDir::currentPath() %
-                  "/src/test/id3-test-data/cover-test-jpg.mp3");
-    const QString kTrackBLocationTest(QDir::currentPath() %
-                  "/src/test/id3-test-data/cover-test-png.mp3");
+    const QString kTrackALocationTest(getTestDir().filePath(
+            QStringLiteral("id3-test-data/cover-test-jpg.mp3")));
+    const QString kTrackBLocationTest(getTestDir().filePath(
+            QStringLiteral("id3-test-data/cover-test-png.mp3")));
 
     // Create new crate and add it to the collection
     Crate testCrate;
@@ -734,12 +768,12 @@ TEST_F(SearchQueryParserTest, ShortCrateFilter) {
     auto pQuery(m_parser.parseQuery(QString("%1").arg(searchTerm), QString()));
 
     // locations for test tracks
-    const QString kTrackALocationTest(QDir::currentPath() %
-                  "/src/test/id3-test-data/cover-test-jpg.mp3");
-    const QString kTrackBLocationTest(QDir::currentPath() %
-                  "/src/test/id3-test-data/cover-test-png.mp3");
-    const QString kTrackCLocationTest(QDir::currentPath() %
-                  "/src/test/id3-test-data/artist.mp3");
+    const QString kTrackALocationTest(getTestDir().filePath(
+            QStringLiteral("id3-test-data/cover-test-jpg.mp3")));
+    const QString kTrackBLocationTest(getTestDir().filePath(
+            QStringLiteral("id3-test-data/cover-test-png.mp3")));
+    const QString kTrackCLocationTest(
+            getTestDir().filePath(QStringLiteral("id3-test-data/artist.mp3")));
 
     // Create new crate and add it to the collection
     Crate testCrate;
@@ -766,7 +800,6 @@ TEST_F(SearchQueryParserTest, ShortCrateFilter) {
     EXPECT_TRUE(pQuery->match(pTrackC));
 }
 
-
 TEST_F(SearchQueryParserTest, CrateFilterEmpty) {
     // Empty should match everything
     auto pQuery(m_parser.parseQuery(QString("crate: "), QString()));
@@ -789,10 +822,10 @@ TEST_F(SearchQueryParserTest, CrateFilterQuote){
     auto pQuery(m_parser.parseQuery(QString("crate: \"%1\"").arg(searchTerm), QString()));
 
     // locations for test tracks
-    const QString kTrackALocationTest(QDir::currentPath() %
-                  "/src/test/id3-test-data/cover-test-jpg.mp3");
-    const QString kTrackBLocationTest(QDir::currentPath() %
-                  "/src/test/id3-test-data/cover-test-png.mp3");
+    const QString kTrackALocationTest(getTestDir().filePath(
+            QStringLiteral("id3-test-data/cover-test-jpg.mp3")));
+    const QString kTrackBLocationTest(getTestDir().filePath(
+            QStringLiteral("id3-test-data/cover-test-png.mp3")));
 
     // Create new crate and add it to the collection
     Crate testCrate;
@@ -832,10 +865,10 @@ TEST_F(SearchQueryParserTest, CrateFilterWithOther){
     auto pQuery(m_parser.parseQuery(QString("crate: %1 artist: asdf").arg(searchTerm), QString()));
 
     // locations for test tracks
-    const QString kTrackALocationTest(QDir::currentPath() %
-                  "/src/test/id3-test-data/cover-test-jpg.mp3");
-    const QString kTrackBLocationTest(QDir::currentPath() %
-                  "/src/test/id3-test-data/cover-test-png.mp3");
+    const QString kTrackALocationTest(getTestDir().filePath(
+            QStringLiteral("id3-test-data/cover-test-jpg.mp3")));
+    const QString kTrackBLocationTest(getTestDir().filePath(
+            QStringLiteral("id3-test-data/cover-test-png.mp3")));
 
     // Create new crate and add it to the collection
     Crate testCrate;
@@ -868,7 +901,7 @@ TEST_F(SearchQueryParserTest, CrateFilterWithOther){
 
 TEST_F(SearchQueryParserTest, CrateFilterWithCrateFilterAndNegation){
     // User's search term
-    QString searchTermA = "testA'1"; // Also a test if "'" is escaped lp1789728
+    QString searchTermA = "testA'1"; // Also a test if "'" is escaped #9419
     QString searchTermAEsc = "testA''1";
     QString searchTermB = "testB";
 
@@ -878,10 +911,10 @@ TEST_F(SearchQueryParserTest, CrateFilterWithCrateFilterAndNegation){
             QString()));
 
     // locations for test tracks
-    const QString kTrackALocationTest(QDir::currentPath() %
-                  "/src/test/id3-test-data/cover-test-jpg.mp3");
-    const QString kTrackBLocationTest(QDir::currentPath() %
-                  "/src/test/id3-test-data/cover-test-png.mp3");
+    const QString kTrackALocationTest(getTestDir().filePath(
+            QStringLiteral("id3-test-data/cover-test-jpg.mp3")));
+    const QString kTrackBLocationTest(getTestDir().filePath(
+            QStringLiteral("id3-test-data/cover-test-png.mp3")));
 
     // Create new crates and add them to the collection
     Crate testCrateA;
@@ -930,3 +963,105 @@ TEST_F(SearchQueryParserTest, CrateFilterWithCrateFilterAndNegation){
                             ") AND (NOT (" + m_crateFilterQuery.arg(searchTermB) + "))"),
                  qPrintable(pQueryB->toSql()));
 }
+
+TEST_F(SearchQueryParserTest, SplitQueryIntoWords) {
+    QStringList rv = SearchQueryParser::splitQueryIntoWords(QString("a test b"));
+    QStringList ex = QStringList() << "a"
+                                   << "test"
+                                   << "b";
+    qDebug() << rv << ex;
+    EXPECT_EQ(rv, ex);
+
+    QStringList rv2 = SearchQueryParser::splitQueryIntoWords(QString("a \"test ' b\" x"));
+    QStringList ex2 = QStringList() << "a"
+                                    << "\"test ' b\""
+                                    << "x";
+    qDebug() << rv2 << ex2;
+    EXPECT_EQ(rv2, ex2);
+
+    QStringList rv3 = SearchQueryParser::splitQueryIntoWords(QString("a x"));
+    QStringList ex3 = QStringList() << "a"
+                                    << "x";
+    qDebug() << rv3 << ex3;
+    EXPECT_EQ(rv3, ex3);
+
+    QStringList rv4 = SearchQueryParser::splitQueryIntoWords(
+            QString("a crate:x title:\"S p A C e\" ~key:2m"));
+    QStringList ex4 = QStringList() << "a"
+                                    << "crate:x"
+                                    << "title:\"S p A C e\""
+                                    << "~key:2m";
+    qDebug() << rv4 << ex4;
+    EXPECT_EQ(rv4, ex4);
+}
+
+TEST_F(SearchQueryParserTest, QueryIsLessSpecific) {
+    // Generate a file name for the temporary file
+    EXPECT_TRUE(SearchQueryParser::queryIsLessSpecific(
+            QStringLiteral("searchme"),
+            QStringLiteral("searchm")));
+
+    EXPECT_TRUE(SearchQueryParser::queryIsLessSpecific(
+            QStringLiteral("A B C"),
+            QStringLiteral("A C")));
+
+    EXPECT_FALSE(SearchQueryParser::queryIsLessSpecific(
+            QStringLiteral("A B C"),
+            QStringLiteral("A D C")));
+
+    EXPECT_TRUE(SearchQueryParser::queryIsLessSpecific(
+            QStringLiteral("A D C"),
+            QStringLiteral("A D C ")));
+
+    EXPECT_TRUE(SearchQueryParser::queryIsLessSpecific(
+            QStringLiteral("A D  C "),
+            QStringLiteral("A D C")));
+
+    EXPECT_FALSE(SearchQueryParser::queryIsLessSpecific(
+            QStringLiteral("A B  C"),
+            QStringLiteral("A D C")));
+
+    EXPECT_TRUE(SearchQueryParser::queryIsLessSpecific(
+            QStringLiteral("A D C"),
+            QStringLiteral("A D C ")));
+
+    EXPECT_TRUE(SearchQueryParser::queryIsLessSpecific(
+            QStringLiteral("A D  C "),
+            QStringLiteral("A D C")));
+
+    EXPECT_FALSE(SearchQueryParser::queryIsLessSpecific(
+            QStringLiteral("A B  C"),
+            QStringLiteral("A D C")));
+
+    EXPECT_TRUE(SearchQueryParser::queryIsLessSpecific(
+            QStringLiteral("Abba1 Abba2 Abb"),
+            QStringLiteral("Abba1 Abba Abb")));
+
+    EXPECT_FALSE(SearchQueryParser::queryIsLessSpecific(
+            QStringLiteral("Abba1 Abba2 Abb"),
+            QStringLiteral("Abba1 Aba Abb")));
+
+    EXPECT_TRUE(SearchQueryParser::queryIsLessSpecific(
+            QStringLiteral("Abba1"),
+            QLatin1String("")));
+
+    EXPECT_TRUE(SearchQueryParser::queryIsLessSpecific(
+            QStringLiteral("Abba1"),
+            QStringLiteral("bba")));
+
+    EXPECT_TRUE(SearchQueryParser::queryIsLessSpecific(
+            QStringLiteral("crate:abc"),
+            QStringLiteral("crate:ab")));
+
+    EXPECT_FALSE(SearchQueryParser::queryIsLessSpecific(
+            QStringLiteral("crate:\"a b c\""),
+            QStringLiteral("crate:\"a c\"")));
+
+    EXPECT_FALSE(SearchQueryParser::queryIsLessSpecific(
+            QStringLiteral("-crate:\"a b c\""),
+            QStringLiteral("crate:\"a b c\"")));
+
+    EXPECT_FALSE(SearchQueryParser::queryIsLessSpecific(
+            QStringLiteral("-crate:\"a b c\""),
+            QStringLiteral("crate:\"a b c\"")));
+}
