@@ -1,10 +1,11 @@
 #include <gtest/gtest.h>
-#include <taglib/textidentificationframe.h>
-#include <taglib/tstring.h>
+#include <textidentificationframe.h>
+#include <tstring.h>
 
 #include <QDir>
 #include <QtDebug>
 
+#include "test/mixxxtest.h"
 #include "track/serato/markers.h"
 #include "util/memory.h"
 
@@ -177,12 +178,16 @@ TEST_F(SeratoMarkersTest, ParseEntry) {
 }
 
 TEST_F(SeratoMarkersTest, ParseMarkersDataMP3) {
-    parseMarkersDataInDirectory(QDir("src/test/serato/data/mp3/markers_"),
+    parseMarkersDataInDirectory(
+            QDir(MixxxTest::getOrInitTestDir().filePath(
+                    QStringLiteral("serato/data/mp3/markers_"))),
             mixxx::taglib::FileType::MP3);
 }
 
 TEST_F(SeratoMarkersTest, ParseMarkersDataMP4) {
-    parseMarkersDataInDirectory(QDir("src/test/serato/data/mp4/markers_"),
+    parseMarkersDataInDirectory(
+            QDir(MixxxTest::getOrInitTestDir().filePath(
+                    QStringLiteral("serato/data/mp4/markers_"))),
             mixxx::taglib::FileType::MP4);
 }
 
