@@ -2,17 +2,17 @@
 #include <QTemporaryFile>
 #include <QtDebug>
 
+#include "analyzer/analyzersilence.h"
 #include "sources/audiosourcestereoproxy.h"
 #include "sources/soundsourceproxy.h"
 #include "test/mixxxtest.h"
+#include "test/soundsourceproviderregistration.h"
 #include "track/track.h"
 #include "track/trackmetadata.h"
 #include "util/samplebuffer.h"
 
 namespace {
 
-const QDir kTestDir(QDir::current().absoluteFilePath("src/test/id3-test-data"));
-
 const SINT kBufferSizes[] = {
         256,
         512,
@@ -36,7 +36,7 @@ const CSAMPLE kMaxDecodingError = 0.01f;
 
 } // anonymous namespace
 
-class SoundSourceProxyTest : public MixxxTest {
+class SoundSourceProxyTest : public MixxxTest, SoundSourceProviderRegistration {
   protected:
     static QStringList getFileNameSuffixes() {
         QStringList availableFileNameSuffixes;
@@ -63,7 +63,7 @@ class SoundSourceProxyTest : public MixxxTest {
                 << ".wv";
 
         QStringList supportedFileNameSuffixes;
-        for (const auto& fileNameSuffix : qAsConst(availableFileNameSuffixes)) {
+        for (const auto& fileNameSuffix : std::as_const(availableFileNameSuffixes)) {
             // We need to check for the whole file name here!
             if (SoundSourceProxy::isFileNameSupported(fileNameSuffix)) {
                 supportedFileNameSuffixes << fileNameSuffix;
@@ -76,11 +76,13 @@ class SoundSourceProxyTest : public MixxxTest {
         return supportedFileNameSuffixes;
     }
 
-    static QStringList getFilePaths() {
+    QStringList getFilePaths() {
         QStringList filePaths;
         const QStringList fileNameSuffixes = getFileNameSuffixes();
         for (const auto& fileNameSuffix : fileNameSuffixes) {
-            filePaths.append(kTestDir.absoluteFilePath("cover-test" + fileNameSuffix));
+            filePaths.append(getTestDir().filePath(
+                    QStringLiteral("id3-test-data/cover-test") +
+                    fileNameSuffix));
         }
         return filePaths;
     }
@@ -118,8 +120,8 @@ class SoundSourceProxyTest : public MixxxTest {
             const CSAMPLE* actual,
             const char* errorMessage) {
         for (SINT i = 0; i < size; ++i) {
-            EXPECT_NEAR(expected[i], actual[i],
-                    kMaxDecodingError) << errorMessage;
+            EXPECT_NEAR(expected[i], actual[i], kMaxDecodingError)
+                    << "i=" << i << " " << errorMessage;
         }
     }
 
@@ -218,10 +220,14 @@ TEST_F(SoundSourceProxyTest, openEmptyFile) {
 }
 
 TEST_F(SoundSourceProxyTest, readArtist) {
-    auto pTrack = Track::newTemporary(TrackFile(
-            kTestDir, "artist.mp3"));
+    auto pTrack = Track::newTemporary(
+            getTestDir().filePath(QStringLiteral("id3-test-data/artist.mp3")));
     SoundSourceProxy proxy(pTrack);
-    proxy.updateTrackFromSource();
+    EXPECT_EQ(
+            SoundSourceProxy::UpdateTrackFromSourceResult::MetadataImportedAndUpdated,
+            proxy.updateTrackFromSource(
+                    SoundSourceProxy::UpdateTrackFromSourceMode::Once,
+                    SyncTrackMetadataParams{}));
     EXPECT_EQ("Test Artist", pTrack->getArtist());
 }
 
@@ -229,43 +235,68 @@ TEST_F(SoundSourceProxyTest, readNoTitle) {
     // We need to verify every track has at least a title to not have empty lines in the library
 
     // Test a file with no metadata
-    auto pTrack1 = Track::newTemporary(TrackFile(
-            kTestDir, "empty.mp3"));
+    auto pTrack1 = Track::newTemporary(
+            getTestDir().filePath(QStringLiteral("id3-test-data/empty.mp3")));
     SoundSourceProxy proxy1(pTrack1);
-    proxy1.updateTrackFromSource();
+    EXPECT_EQ(
+            SoundSourceProxy::UpdateTrackFromSourceResult::MetadataImportedAndUpdated,
+            proxy1.updateTrackFromSource(
+                    SoundSourceProxy::UpdateTrackFromSourceMode::Once,
+                    SyncTrackMetadataParams{}));
     EXPECT_EQ("empty", pTrack1->getTitle());
 
     // Test a reload also works
     pTrack1->setTitle("");
-    proxy1.updateTrackFromSource(SoundSourceProxy::ImportTrackMetadataMode::Again);
+    EXPECT_EQ(
+            SoundSourceProxy::UpdateTrackFromSourceResult::MetadataImportedAndUpdated,
+            proxy1.updateTrackFromSource(
+                    SoundSourceProxy::UpdateTrackFromSourceMode::Always,
+                    SyncTrackMetadataParams{}));
     EXPECT_EQ("empty", pTrack1->getTitle());
 
     // Test a file with other metadata but no title
-    auto pTrack2 = Track::newTemporary(TrackFile(
-            kTestDir, "cover-test-png.mp3"));
+    auto pTrack2 = Track::newTemporary(
+            getTestDir().filePath(QStringLiteral("id3-test-data/cover-test-png.mp3")));
     SoundSourceProxy proxy2(pTrack2);
-    proxy2.updateTrackFromSource();
+    EXPECT_EQ(
+            SoundSourceProxy::UpdateTrackFromSourceResult::MetadataImportedAndUpdated,
+            proxy2.updateTrackFromSource(
+                    SoundSourceProxy::UpdateTrackFromSourceMode::Once,
+                    SyncTrackMetadataParams{}));
     EXPECT_EQ("cover-test-png", pTrack2->getTitle());
 
     // Test a reload also works
     pTrack2->setTitle("");
-    proxy2.updateTrackFromSource(SoundSourceProxy::ImportTrackMetadataMode::Again);
+    EXPECT_EQ(
+            SoundSourceProxy::UpdateTrackFromSourceResult::MetadataImportedAndUpdated,
+            proxy2.updateTrackFromSource(
+                    SoundSourceProxy::UpdateTrackFromSourceMode::Always,
+                    SyncTrackMetadataParams{}));
     EXPECT_EQ("cover-test-png", pTrack2->getTitle());
 
     // Test a file with a title
-    auto pTrack3 = Track::newTemporary(TrackFile(
-            kTestDir, "cover-test-jpg.mp3"));
+    auto pTrack3 = Track::newTemporary(
+            getTestDir().filePath(QStringLiteral("id3-test-data/cover-test-jpg.mp3")));
     SoundSourceProxy proxy3(pTrack3);
-    proxy3.updateTrackFromSource();
+    EXPECT_EQ(
+            SoundSourceProxy::UpdateTrackFromSourceResult::MetadataImportedAndUpdated,
+            proxy3.updateTrackFromSource(
+                    SoundSourceProxy::UpdateTrackFromSourceMode::Once,
+                    SyncTrackMetadataParams{}));
     EXPECT_EQ("test22kMono", pTrack3->getTitle());
 }
 
 TEST_F(SoundSourceProxyTest, TOAL_TPE2) {
-    auto pTrack = Track::newTemporary(TrackFile(
-            kTestDir, "TOAL_TPE2.mp3"));
+    auto pTrack = Track::newTemporary(
+            getTestDir().filePath(QStringLiteral("id3-test-data/TOAL_TPE2.mp3")));
     SoundSourceProxy proxy(pTrack);
     mixxx::TrackMetadata trackMetadata;
-    EXPECT_EQ(mixxx::MetadataSource::ImportResult::Succeeded, proxy.importTrackMetadata(&trackMetadata));
+    // Both resetMissingTagMetadata = false/true have the same effect
+    constexpr auto resetMissingTagMetadata = false;
+    EXPECT_EQ(mixxx::MetadataSource::ImportResult::Succeeded,
+            proxy.importTrackMetadataAndCoverImage(
+                         &trackMetadata, nullptr, resetMissingTagMetadata)
+                    .first);
     EXPECT_EQ("TITLE2", trackMetadata.getTrackInfo().getArtist());
     EXPECT_EQ("ARTIST", trackMetadata.getAlbumInfo().getTitle());
     EXPECT_EQ("TITLE", trackMetadata.getAlbumInfo().getArtist());
@@ -274,7 +305,7 @@ TEST_F(SoundSourceProxyTest, TOAL_TPE2) {
 }
 
 TEST_F(SoundSourceProxyTest, seekForwardBackward) {
-    const SINT kReadFrameCount = 10000;
+    constexpr SINT kReadFrameCount = 10000;
 
     const QStringList filePaths = getFilePaths();
     for (const auto& filePath : filePaths) {
@@ -484,7 +515,7 @@ TEST_F(SoundSourceProxyTest, skipAndRead) {
 }
 
 TEST_F(SoundSourceProxyTest, seekBoundaries) {
-    const SINT kReadFrameCount = 1000;
+    constexpr SINT kReadFrameCount = 1000;
     const QStringList filePaths = getFilePaths();
     for (const auto& filePath : filePaths) {
         ASSERT_TRUE(SoundSourceProxy::isFileNameSupported(filePath));
@@ -597,7 +628,7 @@ TEST_F(SoundSourceProxyTest, seekBoundaries) {
 }
 
 TEST_F(SoundSourceProxyTest, readBeyondEnd) {
-    const SINT kReadFrameCount = 1000;
+    constexpr SINT kReadFrameCount = 1000;
     const QStringList filePaths = getFilePaths();
     for (const auto& filePath : filePaths) {
         ASSERT_TRUE(SoundSourceProxy::isFileNameSupported(filePath));
@@ -717,6 +748,130 @@ TEST_F(SoundSourceProxyTest, regressionTestCachingReaderChunkJumpForward) {
     }
 }
 
+TEST_F(SoundSourceProxyTest, firstSoundTest) {
+    constexpr SINT kReadFrameCount = 2000;
+
+    struct RefFirstSound {
+        QString path;
+        SINT firstSoundSample;
+    };
+
+    RefFirstSound refs[] = {{QStringLiteral("cover-test.aiff"), 1166},
+            {QStringLiteral("cover-test-alac.caf"), 1166},
+            {QStringLiteral("cover-test.flac"), 1166},
+            {QStringLiteral("cover-test-itunes-12.3.0-aac.m4a"),
+#if defined(__WINDOWS__) || defined(__FAAD__)
+                    1390}, // Media Foundation 10.0.17763.2989
+                           // Media Foundation 10.0.20348.1
+                           // Nero FAAD2 2.64
+#else
+                    1166}, // FFmpeg 4.2.7-0ubuntu0.1
+                           // FFmpeg 4.4.2-0ubuntu0.22.04.1
+                           // FFmpeg 5.1.2 windows
+                           // CoreAudio Version 11.7.8 (Build 20G1351)
+                           // CoreAusio Version 12.6.7 (Build 21G651)
+#endif
+            // 1168 FFmpeg 4.2.7-0ubuntu0.1
+
+            {QStringLiteral("cover-test-ffmpeg-aac.m4a"),
+#if defined(__WINDOWS__)
+                    3160}, // Media Foundation 10.0.17763.2989
+                           // Media Foundation 10.0.20348.1
+#else
+                    1112}, // FFmpeg 4.2.7-0ubuntu0.1
+                           // FFmpeg 4.4.2-0ubuntu0.22.04.1
+                           // FFmpeg 5.1.2 windows
+                           // CoreAudio Version 11.7.8 (Build 20G1351)
+                           // CoreAusio Version 12.6.7 (Build 21G651)
+                           // Nero FAAD2 2.64
+#endif
+
+            {QStringLiteral("cover-test-itunes-12.7.0-alac.m4a"), 1166},
+            {QStringLiteral("cover-test-png.mp3"),
+#if defined(__LINUX__)
+                    1752}, // MAD: MPEG Audio Decoder 0.15.1 (beta) NDEBUG FPM_64BIT
+#elif defined(__WINDOWS__)
+                    1752}, // MAD: MPEG Audio Decoder 0.15.1 (beta) NDEBUG FPM_DEFAULT
+#else
+                                    0}, // CoreAudio Version 11.7.8 (Build 20G1351)
+#endif
+
+            {QStringLiteral("cover-test-vbr.mp3"),
+#if defined(__LINUX__) || defined(__WINDOWS__)
+                    3376}, // MAD: MPEG Audio Decoder 0.15.1 (beta) NDEBUG FPM_64BIT
+#else
+                    2318}, // CoreAudio Version 11.7.8 (Build 20G1351)
+#endif
+            // 3326 MAD: MPEG Audio Decoder 0.15.1 (beta) NDEBUG FPM_DEFAULT
+            // No offset compared to FPM_64BIT builds but rounding differences
+            // https://github.com/mixxxdj/mixxx/issues/11888
+            // 1166 FFmpeg
+
+            {QStringLiteral("cover-test.ogg"), 1166},
+            {QStringLiteral("cover-test.opus"), 1268},
+            {QStringLiteral("cover-test.wav"), 1166},
+            {QStringLiteral("cover-test.wav"), 1166},
+            {QStringLiteral("cover-test.wv"), 1166}};
+
+    for (const auto& ref : refs) {
+        QString filePath = getTestDir().filePath(
+                QStringLiteral("id3-test-data/") +
+                ref.path);
+        ASSERT_TRUE(SoundSourceProxy::isFileNameSupported(filePath));
+        const auto fileUrl = QUrl::fromLocalFile(filePath);
+        const auto providerRegistrations =
+                SoundSourceProxy::allProviderRegistrationsForUrl(fileUrl);
+        for (const auto& providerRegistration : providerRegistrations) {
+            mixxx::AudioSourcePointer pContReadSource = openAudioSource(
+                    filePath,
+                    providerRegistration.getProvider());
+
+            // Obtaining an AudioSource may fail for unsupported file formats,
+            // even if the corresponding file extension is supported, e.g.
+            // AAC vs. ALAC in .m4a files
+            if (!pContReadSource) {
+                // skip test file
+                continue;
+            }
+            mixxx::SampleBuffer contReadData(
+                    pContReadSource->getSignalInfo().frames2samples(kReadFrameCount));
+
+            SINT contFrameIndex = pContReadSource->frameIndexMin();
+            while (pContReadSource->frameIndexRange().containsIndex(contFrameIndex)) {
+                const auto readFrameIndexRange =
+                        mixxx::IndexRange::forward(contFrameIndex, kReadFrameCount);
+                // Read next chunk of frames for Cont source without seeking
+                const auto contSampleFrames =
+                        pContReadSource->readSampleFrames(
+                                mixxx::WritableSampleFrames(
+                                        readFrameIndexRange,
+                                        mixxx::SampleBuffer::WritableSlice(contReadData)));
+                ASSERT_FALSE(contSampleFrames.frameIndexRange().empty());
+                ASSERT_TRUE(contSampleFrames.frameIndexRange().isSubrangeOf(readFrameIndexRange));
+                ASSERT_EQ(contSampleFrames.frameIndexRange().start(), readFrameIndexRange.start());
+                contFrameIndex += contSampleFrames.frameLength();
+
+                const SINT sampleCount =
+                        pContReadSource->getSignalInfo().frames2samples(
+                                contSampleFrames.frameLength());
+
+                auto samples = std::span<const CSAMPLE>(&contReadData[0], sampleCount);
+
+                const SINT firstSoundSample = AnalyzerSilence::findFirstSoundInChunk(samples);
+                if (firstSoundSample < static_cast<SINT>(samples.size())) {
+                    EXPECT_EQ(firstSoundSample, ref.firstSoundSample)
+                            << filePath.toStdString() << " "
+                            << providerRegistration.getProvider()
+                                       ->getDisplayName()
+                                       .toStdString();
+                    break;
+                }
+            }
+            break;
+        }
+    }
+}
+
 TEST_F(SoundSourceProxyTest, getTypeFromFile) {
     QTemporaryDir tempDir;
     ASSERT_TRUE(tempDir.isValid());
@@ -728,48 +883,206 @@ TEST_F(SoundSourceProxyTest, getTypeFromFile) {
             tempDir.filePath("file_with_empty_suffix.");
     const QString filePathWithUnknownSuffix =
             tempDir.filePath("file_with.unknown_suffix");
-    // TODO: Currently, our SoundSource::getTypeFromFile() can not detect the
-    // file type of files with a known but wrong file extension properly, so
-    // this test needs to be disabled.
-    //const QString filePathWithWrongSuffix =
-    //        tempDir.filePath("file_with_wrong_suffix.wav");
+    const QString filePathWithWrongSuffix =
+            tempDir.filePath("file_with_wrong_suffix.wav");
     const QString filePathWithUppercaseAndLeadingTrailingWhitespaceSuffix =
             tempDir.filePath("file_with_uppercase_suffix. MP3 ");
 
     // Create the temporary files by copying an existing file
-    const QString validFilePath = kTestDir.absoluteFilePath(QStringLiteral("empty.mp3"));
+    const QString validFilePath = getTestDir().filePath(QStringLiteral("id3-test-data/empty.mp3"));
     mixxxtest::copyFile(validFilePath, filePathWithoutSuffix);
     mixxxtest::copyFile(validFilePath, filePathWithEmptySuffix);
     mixxxtest::copyFile(validFilePath, filePathWithUnknownSuffix);
-    //mixxxtest::copyFile(validFilePath, filePathWithWrongSuffix);
+    mixxxtest::copyFile(validFilePath, filePathWithWrongSuffix);
     mixxxtest::copyFile(validFilePath, filePathWithUppercaseAndLeadingTrailingWhitespaceSuffix);
 
-    ASSERT_STREQ(qPrintable("mp3"), qPrintable(mixxx::SoundSource::getTypeFromFile(validFilePath)));
+    ASSERT_STREQ(qPrintable("mp3"),
+            qPrintable(mixxx::SoundSource::getTypeFromFile(
+                    QFileInfo(validFilePath))));
 
     EXPECT_STREQ(qPrintable("mp3"),
             qPrintable(mixxx::SoundSource::getTypeFromFile(
-                    filePathWithoutSuffix)));
+                    QFileInfo(filePathWithoutSuffix))));
     EXPECT_STREQ(qPrintable("mp3"),
             qPrintable(mixxx::SoundSource::getTypeFromFile(
-                    filePathWithEmptySuffix)));
+                    QFileInfo(filePathWithEmptySuffix))));
     EXPECT_STREQ(qPrintable("mp3"),
             qPrintable(mixxx::SoundSource::getTypeFromFile(
-                    filePathWithUnknownSuffix)));
-    //EXPECT_STREQ(qPrintable("mp3"),
-    //        qPrintable(mixxx::SoundSource::getTypeFromFile(
-    //                filePathWithWrongSuffix)));
+                    QFileInfo(filePathWithUnknownSuffix))));
     EXPECT_STREQ(qPrintable("mp3"),
             qPrintable(mixxx::SoundSource::getTypeFromFile(
-                    filePathWithUppercaseAndLeadingTrailingWhitespaceSuffix)));
+                    QFileInfo(filePathWithWrongSuffix))));
+    EXPECT_STREQ(qPrintable("mp3"),
+            qPrintable(mixxx::SoundSource::getTypeFromFile(
+                    QFileInfo(filePathWithUppercaseAndLeadingTrailingWhitespaceSuffix))));
 }
 
 TEST_F(SoundSourceProxyTest, getTypeFromMissingFile) {
-    const QFileInfo missingFileWithUppercaseSuffixAndLeadingTrailingWhitespaceSuffix =
-            kTestDir.absoluteFilePath(QStringLiteral("missing_file. AIFF "));
+    // Also verify that the shortened suffix ".aif" (case-insensitive) is
+    // mapped to file type "aiff", independent of whether the file exists or not!
+    const QFileInfo missingFileWithUppercaseSuffix(
+            getTestDir().filePath(QStringLiteral("id3-test-data/missing_file.AIF")));
 
-    ASSERT_FALSE(missingFileWithUppercaseSuffixAndLeadingTrailingWhitespaceSuffix.exists());
+    ASSERT_FALSE(missingFileWithUppercaseSuffix.exists());
 
     EXPECT_STREQ(qPrintable("aiff"),
             qPrintable(mixxx::SoundSource::getTypeFromFile(
-                    missingFileWithUppercaseSuffixAndLeadingTrailingWhitespaceSuffix)));
+                    QFileInfo(missingFileWithUppercaseSuffix))));
+}
+
+TEST_F(SoundSourceProxyTest, getTypeFromAiffFile) {
+    QTemporaryDir tempDir;
+    ASSERT_TRUE(tempDir.isValid());
+
+    // Verify that both suffixes are supported
+    ASSERT_TRUE(SoundSourceProxy::isFileSuffixSupported(QStringLiteral("aif")));
+    ASSERT_TRUE(SoundSourceProxy::isFileSuffixSupported(QStringLiteral("aiff")));
+
+    const QString aiffFilePath =
+            getTestDir().filePath(QStringLiteral("id3-test-data/cover-test.aiff"));
+
+    ASSERT_TRUE(QFileInfo::exists(aiffFilePath));
+    ASSERT_STREQ(qPrintable("aiff"),
+            qPrintable(mixxx::SoundSource::getTypeFromFile(
+                    QFileInfo(aiffFilePath))));
+
+    const QString aiffFilePathWithShortenedSuffix =
+            tempDir.filePath(QStringLiteral("cover-test.aif"));
+    mixxxtest::copyFile(aiffFilePath, aiffFilePathWithShortenedSuffix);
+    ASSERT_TRUE(QFileInfo::exists(aiffFilePathWithShortenedSuffix));
+
+    EXPECT_STREQ(qPrintable("aiff"),
+            qPrintable(mixxx::SoundSource::getTypeFromFile(
+                    QFileInfo(aiffFilePathWithShortenedSuffix))));
+}
+
+TEST_F(SoundSourceProxyTest, updateTrackFromSourceFileMissing) {
+    QTemporaryDir tempDir;
+    ASSERT_TRUE(tempDir.isValid());
+
+    ASSERT_TRUE(SoundSourceProxy::isFileTypeSupported(QStringLiteral("mp3")));
+
+    const QString missingFilePath =
+            tempDir.filePath(QStringLiteral("missing.mp3"));
+
+    auto pTrack = Track::newTemporary(missingFilePath);
+    ASSERT_EQ(SoundSourceProxy(pTrack).updateTrackFromSource(
+                      SoundSourceProxy::UpdateTrackFromSourceMode::Once,
+                      SyncTrackMetadataParams{}),
+            SoundSourceProxy::UpdateTrackFromSourceResult::MetadataImportFailed);
+}
+
+TEST_F(SoundSourceProxyTest, handleWrongFileSuffix) {
+    QTemporaryDir tempDir;
+    ASSERT_TRUE(tempDir.isValid());
+
+    ASSERT_TRUE(SoundSourceProxy::isFileTypeSupported(QStringLiteral("aiff")));
+    ASSERT_TRUE(SoundSourceProxy::isFileTypeSupported(QStringLiteral("mp3")));
+
+    const QString contentFileType = QStringLiteral("mp3");
+    const QString wrongFileType = QStringLiteral("aiff");
+
+    const QString contentFileTypePath =
+            getTestDir().filePath(QStringLiteral("id3-test-data/cover-test-png.mp3"));
+    const QString wrongFileTypePath =
+            tempDir.filePath(QStringLiteral("cover-test.aiff"));
+    mixxxtest::copyFile(contentFileTypePath, wrongFileTypePath);
+
+    auto pTrack = Track::newTemporary(wrongFileTypePath);
+    ASSERT_TRUE(pTrack->getType().isEmpty());
+
+    ASSERT_EQ(SoundSourceProxy(pTrack).updateTrackFromSource(
+                      SoundSourceProxy::UpdateTrackFromSourceMode::Once,
+                      SyncTrackMetadataParams{}),
+            SoundSourceProxy::UpdateTrackFromSourceResult::MetadataImportedAndUpdated);
+    EXPECT_STREQ(qPrintable(contentFileType), qPrintable(pTrack->getType()));
+
+    // Change the file type back to the wrong file type after the initial import
+    // and update from source once again to verify that the wrong type gets updated
+    // to the correct type.
+    pTrack->setType(wrongFileType);
+    // The re-import of metadata should be skipped with UpdateTrackFromSourceMode::Once
+    // and updateTrackFromSource() is supposed to return false.
+    ASSERT_EQ(SoundSourceProxy(pTrack).updateTrackFromSource(
+                      SoundSourceProxy::UpdateTrackFromSourceMode::Once,
+                      SyncTrackMetadataParams{}),
+            SoundSourceProxy::UpdateTrackFromSourceResult::NotUpdated);
+    // But even though updateTrackFromSource() returned false the wrong file type
+    // should have been fixed.
+    EXPECT_STREQ(qPrintable(contentFileType), qPrintable(pTrack->getType()));
+}
+
+TEST_F(SoundSourceProxyTest, fileTypeWithCorrespondingSuffix) {
+    const QString fileTypesWithCorrespondingSuffixes[] = {
+        QStringLiteral("3gp"),
+        QStringLiteral("3g2"),
+        QStringLiteral("aac"),
+        QStringLiteral("ac3"),
+        QStringLiteral("aiff"),
+        // Test fails for file type "caf" supported by SoundSourceSndfile
+        //QStringLiteral("caf"),
+        QStringLiteral("flac"),
+        QStringLiteral("it"),
+        QStringLiteral("m4a"),
+        QStringLiteral("m4v"),
+        QStringLiteral("med"),
+#if !defined(__APPLE__)
+        // Test fails on macOS for file type "mj2" supported by SoundSourceFFmpeg
+        QStringLiteral("mj2"),
+#endif
+        QStringLiteral("med"),
+        QStringLiteral("mod"),
+        QStringLiteral("mov"),
+        QStringLiteral("mp2"),
+        QStringLiteral("mp3"),
+        QStringLiteral("mp4"),
+        QStringLiteral("ogg"),
+        // Test fails for file type "okt" supported by SoundSourceModPlug
+        //QStringLiteral("okt"),
+        QStringLiteral("opus"),
+        QStringLiteral("s3m"),
+        QStringLiteral("stm"),
+        QStringLiteral("wav"),
+        QStringLiteral("wma"),
+        QStringLiteral("wv"),
+        QStringLiteral("xm"),
+    };
+    for (const auto& fileType : fileTypesWithCorrespondingSuffixes) {
+        // We only need to check the actually supported file types.
+        // The result for unsupported file types is undefined.
+        if (SoundSourceProxy::isFileTypeSupported(fileType)) {
+            qInfo() << "Suffixes for file type"
+                    << fileType
+                    << ':'
+                    << SoundSourceProxy::getFileSuffixesForFileType(fileType);
+            EXPECT_TRUE(SoundSourceProxy::getFileSuffixesForFileType(fileType).contains(fileType));
+        }
+    }
+}
+
+TEST_F(SoundSourceProxyTest, fileSuffixWithDifferingType) {
+    const std::tuple<QString, QString> fileSuffixesWithDifferingTypes[] = {
+            {QStringLiteral("aif"), QStringLiteral("aiff")},
+    };
+    for (const auto& [fileSuffix, fileType] : fileSuffixesWithDifferingTypes) {
+        EXPECT_EQ(SoundSourceProxy::isFileTypeSupported(fileType),
+                SoundSourceProxy::isFileSuffixSupported(fileSuffix));
+    }
+}
+
+TEST_F(SoundSourceProxyTest, freeModeGarbage) {
+    // Try to load a file with an insane bitrate in the garbage before the real frame.
+    QString filePath = getTestDir().filePath(
+            QStringLiteral("id3-test-data/free_mode_garbage.mp3"));
+    ASSERT_TRUE(SoundSourceProxy::isFileNameSupported(filePath));
+    const auto fileUrl = QUrl::fromLocalFile(filePath);
+    const auto providerRegistrations =
+            SoundSourceProxy::allProviderRegistrationsForUrl(fileUrl);
+    for (const auto& providerRegistration : providerRegistrations) {
+        mixxx::AudioSourcePointer pContReadSource = openAudioSource(
+                filePath,
+                providerRegistration.getProvider());
+        ASSERT_TRUE(pContReadSource != nullptr);
+        break;
+    }
 }
