@@ -22,27 +22,34 @@ Keys KeyFactory::loadKeysFromByteArray(const QString& keysVersion,
 }
 
 // static
-Keys KeyFactory::makeBasicKeys(mixxx::track::io::key::ChromaticKey global_key,
-                               mixxx::track::io::key::Source source) {
+Keys KeyFactory::makeBasicKeys(
+        mixxx::track::io::key::ChromaticKey global_key,
+        mixxx::track::io::key::Source source) {
     KeyMap key_map;
     key_map.set_global_key(global_key);
+    QString global_key_text = KeyUtils::keyToString(global_key, KeyUtils::KeyNotation::Custom);
+    key_map.set_global_key_text(global_key_text.toStdString());
     key_map.set_source(source);
     return Keys(key_map);
 }
 
 // static
-Keys KeyFactory::makeBasicKeysFromText(const QString& global_key_text,
-                                       mixxx::track::io::key::Source source) {
+Keys KeyFactory::makeBasicKeysNormalized(
+        const QString& global_key_text,
+        mixxx::track::io::key::Source source) {
+    mixxx::track::io::key::ChromaticKey global_key = KeyUtils::guessKeyFromText(global_key_text);
+    return KeyFactory::makeBasicKeys(global_key, source);
+}
+
+// static
+Keys KeyFactory::makeBasicKeysKeepText(
+        const QString& global_key_text,
+        mixxx::track::io::key::Source source) {
     KeyMap key_map;
     key_map.set_source(source);
-    mixxx::track::io::key::ChromaticKey global_key = KeyUtils::guessKeyFromText(
-        global_key_text);
-    if (global_key != mixxx::track::io::key::INVALID) {
-        key_map.set_global_key(global_key);
-    } else {
-        // If we couldn't understand the key, save it as text.
-        key_map.set_global_key_text(global_key_text.toStdString());
-    }
+    key_map.set_global_key_text(global_key_text.toStdString());
+    mixxx::track::io::key::ChromaticKey global_key = KeyUtils::guessKeyFromText(global_key_text);
+    key_map.set_global_key(global_key);
     return Keys(key_map);
 }
 
@@ -90,7 +97,7 @@ Keys KeyFactory::makePreferredKeys(
 
     if (version == KEY_MAP_VERSION) {
         KeyMap key_map;
-        for (const auto* it = key_changes.constBegin();
+        for (auto it = key_changes.constBegin();
                 it != key_changes.constEnd();
                 ++it) {
             // Key position is in frames. Do not accept fractional frames.
