@@ -1,32 +1,38 @@
 #include "track/keyutils.h"
 
 #include <QMap>
-#include <QMutexLocker>
 #include <QPair>
-#include <QRegExp>
+#include <QRegularExpression>
 #include <QtDebug>
-#include <array>
 
-#include "util/math.h"
+#include "util/compatibility/qmutex.h"
 
 using mixxx::track::io::key::ChromaticKey;
 using mixxx::track::io::key::ChromaticKey_IsValid;
 
 namespace {
 
+// All of these regular expressions need exact matching.
+// QRegularExpression::anchoredPattern cannot be used because Mixxx supports Qt < 5.15, so
+// instead do what QRegularExpression::anchoredPattern by wrapping each regex in \\A(?: ... )\\z
+
 // OpenKey notation, the numbers 1-12 followed by d (dur, major) or m (moll, minor).
-const QString s_openKeyPattern = QStringLiteral("^\\s*(1[0-2]|[1-9])([dm])\\s*$");
+const QRegularExpression s_openKeyRegex(QStringLiteral(
+        "\\A(?:^\\s*(1[0-2]|[1-9])([dm])\\s*$)\\z"));
 
 // Lancelot notation, the numbers 1-12 followed by A (minor) or B(I) (major).
 // or "I", "L", "M", "D", "P", "C" for the advanced modes
-const QString s_lancelotKeyPattern = QStringLiteral("^\\s*0*(1[0-2]|[1-9])([ABILMDPC])\\s*$");
+const QRegularExpression s_lancelotKeyRegex(
+        QStringLiteral("\\A(?:^\\s*0*(1[0-2]|[1-9])([ABILMDPC])\\s*$)\\z"));
 constexpr std::string_view s_lancelotMajorModes = "BILM";
 
 // a-g followed by any number of sharps or flats, optionally followed by
 // a scale mode spec (m = minor, min, maj ..)
 // Note: ## = x exists: https://jadebultitude.com/double-sharp-sign-in-music
-const QString s_keyPattern = QString::fromUtf8(
-        "^\\s*([a-g])([#♯b♭]*) *([a-z]{3}.*|m)?\\s*$");
+const QRegularExpression s_keyRegex(
+        QString::fromUtf8(
+                "\\A(?:^\\s*([a-g])([#♯b♭]*) *([a-z]{3}.*|m)?\\s*$)\\z"),
+        QRegularExpression::CaseInsensitiveOption);
 
 const QString s_sharpSymbol = QString::fromUtf8("♯");
 //static const QString s_flatSymbol = QString::fromUtf8("♭");
@@ -253,7 +259,7 @@ QString KeyUtils::keyDebugName(ChromaticKey key) {
 
 // static
 void KeyUtils::setNotation(const QMap<ChromaticKey, QString>& notation) {
-    QMutexLocker locker(&s_notationMutex);
+    const auto locker = lockMutex(&s_notationMutex);
     s_notation = notation;
     s_reverseNotation.clear();
 
@@ -275,9 +281,10 @@ QString KeyUtils::keyToString(ChromaticKey key,
     }
 
     if (notation == KeyNotation::Custom) {
-        // The default value for notation is KeyUtils::KeyNotation::Custom, so this executes when the function is
-        // called without a notation specified after KeyUtils::setNotation has set up s_notation.
-        QMutexLocker locker(&s_notationMutex);
+        // The default value for notation is KeyUtils::KeyNotation::Custom, so
+        // this executes when the function is called without a notation specified
+        // after KeyUtils::setNotation has set up s_notation.
+        const auto locker = lockMutex(&s_notationMutex);
         auto it = s_notation.constFind(key);
         if (it != s_notation.constEnd()) {
             return it.value();
@@ -307,8 +314,8 @@ QString KeyUtils::keyToString(ChromaticKey key,
 }
 
 // static
-QString KeyUtils::getGlobalKeyText(const Keys& keys, KeyNotation notation) {
-    const mixxx::track::io::key::ChromaticKey globalKey(keys.getGlobalKey());
+QString KeyUtils::formatGlobalKey(const Keys& keys, KeyNotation notation) {
+    const mixxx::track::io::key::ChromaticKey globalKey = keys.getGlobalKey();
     if (globalKey != mixxx::track::io::key::INVALID) {
         return keyToString(globalKey, notation);
     } else {
@@ -333,32 +340,33 @@ ChromaticKey KeyUtils::guessKeyFromText(const QString& text) {
 
     // Try using the user's custom notation.
     {
-        QMutexLocker locker(&s_notationMutex);
+        const auto locker = lockMutex(&s_notationMutex);
         auto it = s_reverseNotation.constFind(text);
         if (it != s_reverseNotation.constEnd()) {
             return it.value();
         }
     }
 
-    QRegExp openKeyMatcher(s_openKeyPattern);
-    if (openKeyMatcher.exactMatch(trimmed)) {
+    QRegularExpressionMatch openKeyMatch = s_openKeyRegex.match(trimmed);
+    if (openKeyMatch.hasMatch()) {
         bool ok = false;
-        int openKeyNumber = openKeyMatcher.cap(1).toInt(&ok);
+        int openKeyNumber = openKeyMatch.captured(1).toInt(&ok);
 
         // Regex should mean this never happens.
         if (!ok || openKeyNumber < 1 || openKeyNumber > 12) {
             return mixxx::track::io::key::INVALID;
         }
 
-        bool major = openKeyMatcher.cap(2).compare("d") == 0;
+        bool major = openKeyMatch.captured(2)
+                             .compare("d") == 0;
 
         return openKeyNumberToKey(openKeyNumber, major);
     }
 
-    QRegExp lancelotKeyMatcher(s_lancelotKeyPattern);
-    if (lancelotKeyMatcher.exactMatch(trimmed)) {
+    QRegularExpressionMatch lancelotMatch = s_lancelotKeyRegex.match(trimmed);
+    if (lancelotMatch.hasMatch()) {
         bool ok = false;
-        int lancelotNumber = lancelotKeyMatcher.cap(1).toInt(&ok);
+        int lancelotNumber = lancelotMatch.captured(1).toInt(&ok);
 
         // Regex should mean this never happens.
         if (!ok || lancelotNumber < 1 || lancelotNumber > 12) {
@@ -367,24 +375,24 @@ ChromaticKey KeyUtils::guessKeyFromText(const QString& text) {
 
         int openKeyNumber = lancelotNumberToOpenKeyNumber(lancelotNumber);
 
-        const QChar lancelotScaleMode = lancelotKeyMatcher.cap(2).at(0);
+        const QChar lancelotScaleMode = lancelotMatch.captured(2).at(0);
         bool major = (s_lancelotMajorModes.find(lancelotScaleMode.toLatin1()) != std::string::npos);
 
         return openKeyNumberToKey(openKeyNumber, major);
     }
 
-    QRegExp keyMatcher(s_keyPattern, Qt::CaseInsensitive);
-    if (keyMatcher.exactMatch(trimmed)) {
+    QRegularExpressionMatch keyMatch = s_keyRegex.match(trimmed);
+    if (keyMatch.hasMatch()) {
         // Take the first letter, lowercase it and subtract 'a' and we get a
         // number between 0-6. Look up the major key associated with that letter
         // from s_letterToMajorKey. Upper-case means major, lower-case means
         // minor. Then apply the sharps or flats to the key.
-        QChar letter = keyMatcher.cap(1).at(0);
+        QChar letter = keyMatch.captured(1).at(0);
         int letterIndex = letter.toLower().toLatin1() - 'a';
         bool major = letter.isUpper();
 
         // Now apply sharps and flats to the letter key.
-        QString adjustments = keyMatcher.cap(2);
+        QString adjustments = keyMatch.captured(2);
         int steps = 0;
         for (const auto* it = adjustments.constBegin();
                 it != adjustments.constEnd();
@@ -392,7 +400,7 @@ ChromaticKey KeyUtils::guessKeyFromText(const QString& text) {
             steps += (*it == '#' || *it == s_sharpSymbol[0]) ? 1 : -1;
         }
 
-        QString scale = keyMatcher.cap(3);
+        QString scale = keyMatch.captured(3);
         // we override major if a scale definition exists
         if (!scale.isEmpty()) {
             if (scale.compare("m", Qt::CaseInsensitive) == 0) {
