@@ -22,10 +22,10 @@
 namespace {
 
 QString getPrimaryDecoderNameForFilePath(const QString& filePath) {
-    const QString fileExtension =
-            mixxx::SoundSource::getTypeFromFile(filePath);
+    const QString fileType =
+            mixxx::SoundSource::getTypeFromFile(QFileInfo(filePath));
     const mixxx::SoundSourceProviderPointer pPrimaryProvider =
-            SoundSourceProxy::getPrimaryProviderForFileExtension(fileExtension);
+            SoundSourceProxy::getPrimaryProviderForFileType(fileType);
     if (pPrimaryProvider) {
         return pPrimaryProvider->getDisplayName();
     } else {
@@ -292,7 +292,7 @@ void SeratoTags::setCueInfos(const QList<CueInfo>& cueInfos, double timingOffset
     // ensure that each hotcue number is unique (by using a map), apply the
     // timing offset and split up cues and loops.
     QList<CueInfo> cueList;
-    for (const CueInfo& cueInfo : qAsConst(cueInfos)) {
+    for (const CueInfo& cueInfo : std::as_const(cueInfos)) {
         if (!cueInfo.getHotCueIndex()) {
             continue;
         }
@@ -359,7 +359,7 @@ std::optional<RgbColor::optional_t> SeratoTags::getTrackColor() const {
             pStoredColor->toDisplayedColor());
 }
 
-void SeratoTags::setTrackColor(RgbColor::optional_t color) {
+void SeratoTags::setTrackColor(const RgbColor::optional_t& color) {
     auto storedColor = SeratoStoredTrackColor::fromDisplayedColor(color);
     m_seratoMarkers.setTrackColor(storedColor);
     m_seratoMarkers2.setTrackColor(storedColor);
