@@ -1,8 +1,6 @@
 #pragma once
 
 #include <QList>
-#include <QMutex>
-#include <QMutexLocker>
 #include <QObject>
 #include <QUrl>
 
@@ -12,17 +10,18 @@
 #include "track/cue.h"
 #include "track/cueinfoimporter.h"
 #include "track/track_decl.h"
-#include "track/trackfile.h"
 #include "track/trackrecord.h"
-#include "util/sandbox.h"
+#include "util/color/predefinedcolorpalettes.h"
+#include "util/compatibility/qmutex.h"
+#include "util/fileaccess.h"
+#include "util/memory.h"
 #include "waveform/waveform.h"
 
 class Track : public QObject {
     Q_OBJECT
 
   public:
-    Track(TrackFile fileInfo,
-            SecurityTokenPointer pSecurityToken,
+    Track(mixxx::FileAccess fileAccess,
             TrackId trackId = TrackId());
     Track(const Track&) = delete;
     ~Track() override;
@@ -31,61 +30,78 @@ class Track : public QObject {
     // testing purposes. The resulting track will neither be stored
     // in the database nor will the metadata of the corresponding file
     // be updated.
-    // Use SoundSourceProxy::importTemporaryTrack() for importing files
-    // to ensure that the file will not be written while reading it!
     static TrackPointer newTemporary(
-            TrackFile fileInfo = TrackFile(),
-            SecurityTokenPointer pSecurityToken = SecurityTokenPointer());
+            mixxx::FileAccess fileAccess = mixxx::FileAccess());
+    static TrackPointer newTemporary(
+            const QString& filePath) {
+        return newTemporary(mixxx::FileAccess(mixxx::FileInfo(filePath)));
+    }
+    static TrackPointer newTemporary(
+            const QDir& dir,
+            const QString& file) {
+        return newTemporary(mixxx::FileAccess(mixxx::FileInfo(dir, file)));
+    }
+
     // Creates a dummy instance only for testing purposes.
     static TrackPointer newDummy(
-            TrackFile fileInfo,
+            const QString& filePath,
             TrackId trackId);
 
-    Q_PROPERTY(QString artist READ getArtist WRITE setArtist)
-    Q_PROPERTY(QString title READ getTitle WRITE setTitle)
-    Q_PROPERTY(QString album READ getAlbum WRITE setAlbum)
-    Q_PROPERTY(QString albumArtist READ getAlbumArtist WRITE setAlbumArtist)
-    Q_PROPERTY(QString genre READ getGenre WRITE setGenre)
-    Q_PROPERTY(QString composer READ getComposer WRITE setComposer)
-    Q_PROPERTY(QString grouping READ getGrouping WRITE setGrouping)
-    Q_PROPERTY(QString year READ getYear WRITE setYear)
-    Q_PROPERTY(QString track_number READ getTrackNumber WRITE setTrackNumber)
-    Q_PROPERTY(QString track_total READ getTrackTotal WRITE setTrackTotal)
-    Q_PROPERTY(int times_played READ getTimesPlayed)
-    Q_PROPERTY(QString comment READ getComment WRITE setComment)
-    Q_PROPERTY(double bpm READ getBpm)
-    Q_PROPERTY(QString bpmFormatted READ getBpmText STORED false)
-    Q_PROPERTY(QString key READ getKeyText WRITE setKeyText)
-    Q_PROPERTY(double duration READ getDuration)
-    Q_PROPERTY(QString durationFormatted READ getDurationTextSeconds STORED false)
-    Q_PROPERTY(QString durationFormattedCentiseconds READ getDurationTextCentiseconds STORED false)
-    Q_PROPERTY(QString durationFormattedMilliseconds READ getDurationTextMilliseconds STORED false)
-    Q_PROPERTY(QString info READ getInfo STORED false)
-    Q_PROPERTY(QString titleInfo READ getTitleInfo STORED false)
+    Q_PROPERTY(QString artist READ getArtist WRITE setArtist NOTIFY artistChanged)
+    Q_PROPERTY(QString title READ getTitle WRITE setTitle NOTIFY titleChanged)
+    Q_PROPERTY(QString album READ getAlbum WRITE setAlbum NOTIFY albumChanged)
+    Q_PROPERTY(QString albumArtist READ getAlbumArtist WRITE setAlbumArtist
+                    NOTIFY albumArtistChanged)
+    Q_PROPERTY(QString genre READ getGenre STORED false NOTIFY genreChanged)
+#if defined(__EXTRA_METADATA__)
+    Q_PROPERTY(QString mood READ getMood STORED false NOTIFY moodChanged)
+#endif // __EXTRA_METADATA__
+    Q_PROPERTY(QString composer READ getComposer WRITE setComposer NOTIFY composerChanged)
+    Q_PROPERTY(QString grouping READ getGrouping WRITE setGrouping NOTIFY groupingChanged)
+    Q_PROPERTY(QString year READ getYear WRITE setYear NOTIFY yearChanged)
+    Q_PROPERTY(QString trackNumber READ getTrackNumber WRITE setTrackNumber
+                    NOTIFY trackNumberChanged)
+    Q_PROPERTY(QString trackTotal READ getTrackTotal WRITE setTrackTotal NOTIFY trackTotalChanged)
+    Q_PROPERTY(int timesPlayed READ getTimesPlayed NOTIFY timesPlayedChanged)
+    Q_PROPERTY(QString comment READ getComment WRITE setComment NOTIFY commentChanged)
+    Q_PROPERTY(double bpm READ getBpm NOTIFY bpmChanged)
+    Q_PROPERTY(QString bpmText READ getBpmText STORED false NOTIFY bpmChanged)
+    Q_PROPERTY(QString keyText READ getKeyText WRITE setKeyText NOTIFY keyChanged)
+    Q_PROPERTY(double duration READ getDuration NOTIFY durationChanged)
+    Q_PROPERTY(QString durationTextSeconds READ getDurationTextSeconds
+                    STORED false NOTIFY durationChanged)
+    Q_PROPERTY(QString durationTextCentiseconds READ getDurationTextCentiseconds
+                    STORED false NOTIFY durationChanged)
+    Q_PROPERTY(QString durationTextMilliseconds READ getDurationTextMilliseconds
+                    STORED false NOTIFY durationChanged)
+    Q_PROPERTY(QString info READ getInfo STORED false NOTIFY infoChanged)
+    Q_PROPERTY(QString titleInfo READ getTitleInfo STORED false NOTIFY infoChanged)
+    Q_PROPERTY(QDateTime sourceSynchronizedAt READ getSourceSynchronizedAt STORED false)
 
-    TrackFile getFileInfo() const {
-        // Copying TrackFile/QFileInfo is thread-safe (implicit sharing), no locking needed.
-        return m_fileInfo;
-    }
-    SecurityTokenPointer getSecurityToken() const {
-        // Copying a QSharedPointer is thread-safe, no locking needed.
-        return m_pSecurityToken;
+    mixxx::FileInfo getFileInfo() const {
+        // Copying mixxx::FileInfo based on QFileInfo is thread-safe due to implicit sharing,
+        // i.e. no locking needed.
+        static_assert(mixxx::FileInfo::isQFileInfo());
+        return m_fileAccess.info();
     }
 
     TrackId getId() const;
 
     // Returns absolute path to the file, including the filename.
     QString getLocation() const {
-        return m_fileInfo.location();
-    }
-    // The (refreshed) canonical location
-    QString getCanonicalLocation() const;
-    bool checkFileExists() const {
-        return m_fileInfo.checkFileExists();
+        const auto fileInfo = getFileInfo();
+        if (!fileInfo.hasLocation()) {
+            return {};
+        }
+        return fileInfo.location();
     }
 
-    // File/format type
-    void setType(const QString&);
+    /// Set the file type
+    ///
+    /// Returns the old type to allow the caller to report if it has changed.
+    QString setType(const QString& newType);
+
+    /// Get the file type
     QString getType() const;
 
     // Get number of channels
@@ -99,13 +115,9 @@ class Track : public QObject {
 
     void setDuration(mixxx::Duration duration);
     void setDuration(double duration);
-    double getDuration() const {
-        return getDuration(DurationRounding::NONE);
-    }
+    double getDuration() const;
     // Returns the duration rounded to seconds
-    int getDurationInt() const {
-        return static_cast<int>(getDuration(DurationRounding::SECONDS));
-    }
+    int getDurationSecondsInt() const;
     // Returns the duration formatted as a string (H:MM:SS or H:MM:SS.cc or H:MM:SS.mmm)
     QString getDurationText(mixxx::Duration::Precision precision) const;
 
@@ -121,13 +133,15 @@ class Track : public QObject {
     }
 
     // Sets the BPM if not locked.
-    bool trySetBpm(double bpm);
+    bool trySetBpm(double bpmValue) {
+        return trySetBpm(mixxx::Bpm(bpmValue));
+    }
+    bool trySetBpm(mixxx::Bpm bpm);
 
-    double getBpm() const {
-        const QMutexLocker lock(&m_qMutex);
-        return getBpmWhileLocked().getValue();
+    double getBpm() const;
+    QString getBpmText() const {
+        return mixxx::Bpm::displayValueText(getBpm());
     }
-    QString getBpmText() const;
 
     // A track with a locked BPM will not be re-analyzed by the beats or bpm
     // analyzer.
@@ -135,12 +149,21 @@ class Track : public QObject {
     bool isBpmLocked() const;
 
     void setReplayGain(const mixxx::ReplayGain&);
+    // Adjust ReplayGain by multiplying the given gain amount.
+    void adjustReplayGainFromPregain(double);
+    // Returns ReplayGain
     mixxx::ReplayGain getReplayGain() const;
 
-    // Indicates if the metadata has been parsed from file tags.
-    bool isMetadataSynchronized() const;
-    // Only used by a free function in TrackDAO!
-    void setMetadataSynchronized(bool metadataSynchronized);
+    /// Checks if the internal metadata is in-sync with the
+    /// metadata stored in file tags.
+    bool checkSourceSynchronized() const;
+
+    // The date/time of the last import or export of metadata
+    void setSourceSynchronizedAt(const QDateTime& sourceSynchronizedAt);
+    void resetSourceSynchronizedAt() {
+        setSourceSynchronizedAt(QDateTime{});
+    }
+    QDateTime getSourceSynchronizedAt() const;
 
     void setDateAdded(const QDateTime& dateAdded);
     QDateTime getDateAdded() const;
@@ -161,13 +184,14 @@ class Track : public QObject {
     // following the TDRC ID3v2.4 frame or if not exists, TYER + TDAT.
     QString getYear() const;
     void setYear(const QString&);
-
-    QString getGenre() const;
-    void setGenre(const QString&);
+    // Returns the track color
     mixxx::RgbColor::optional_t getColor() const;
-    void setColor(mixxx::RgbColor::optional_t);
+    void setColor(const mixxx::RgbColor::optional_t&);
     QString getComment() const;
     void setComment(const QString&);
+    void clearComment() {
+        setComment(QString());
+    }
     QString getComposer() const;
     void setComposer(const QString&);
     QString getGrouping() const;
@@ -180,6 +204,40 @@ class Track : public QObject {
     void setTrackNumber(const QString&);
     void setTrackTotal(const QString&);
 
+    /// Return the genre as text
+    QString getGenre() const;
+
+    /// Update the genre text.
+    ///
+    /// Returns true if track metadata has been updated and false
+    /// otherwise.
+    ///
+    /// TODO: Update the corresponding custom tags by splitting
+    /// the text according to the given tag mapping configuration.
+    /// All existing custom genre tags with their associated score
+    /// will be replaced.
+    bool updateGenre(
+            /*TODO: const mixxx::TaggingConfig& config,*/
+            const QString& genre);
+
+#if defined(__EXTRA_METADATA__)
+    /// Return the mood as text
+    QString getMood() const;
+
+    /// Update the mood text.
+    ///
+    /// Returns true if track metadata has been updated and false
+    /// otherwise.
+    ///
+    /// TODO: Update the corresponding custom tags by splitting
+    /// the text according to the given tag mapping configuration.
+    /// All existing custom mood tags with their associated score
+    /// will be replaced.
+    bool updateMood(
+            /*TODO: const mixxx::TaggingConfig& config,*/
+            const QString& mood);
+#endif // __EXTRA_METADATA__
+
     PlayCounter getPlayCounter() const;
     void setPlayCounter(const PlayCounter& playCounter);
     void resetPlayCounter(int iTimesPlayed = 0) {
@@ -187,12 +245,18 @@ class Track : public QObject {
     }
     // Sets played status and increments or decrements the play count
     void updatePlayCounter(bool bPlayed = true);
+    // Sets played status but leaves play count untouched
+    void updatePlayedStatusKeepPlayCount(bool bPlayed);
 
     // Only required for the times_played property
     int getTimesPlayed() const {
         return getPlayCounter().getTimesPlayed();
     }
 
+    QDateTime getLastPlayedAt() const {
+        return getPlayCounter().getLastPlayedAt();
+    }
+
     int getRating() const;
     void setRating(int);
     void resetRating() {
@@ -220,17 +284,36 @@ class Track : public QObject {
     ConstWaveformPointer getWaveformSummary() const;
     void setWaveformSummary(ConstWaveformPointer pWaveform);
 
-    // Get the track's main cue point
-    CuePosition getCuePoint() const;
+    /// Get the track's main cue point
+    mixxx::audio::FramePos getMainCuePosition() const;
     // Set the track's main cue point
-    void setCuePoint(CuePosition cue);
+    void setMainCuePosition(mixxx::audio::FramePos position);
     /// Shift all cues by a constant offset
-    void shiftCuePositionsMillis(double milliseconds);
+    void shiftCuePositionsMillis(mixxx::audio::FrameDiff_t milliseconds);
     // Call when analysis is done.
     void analysisFinished();
 
     // Calls for managing the track's cue points
-    CuePointer createAndAddCue();
+    CuePointer createAndAddCue(
+            mixxx::CueType type,
+            int hotCueIndex,
+            mixxx::audio::FramePos startPosition,
+            mixxx::audio::FramePos endPosition,
+            mixxx::RgbColor color = mixxx::PredefinedColorPalettes::kDefaultCueColor);
+    CuePointer createAndAddCue(
+            mixxx::CueType type,
+            int hotCueIndex,
+            double startPositionSamples,
+            double endPositionSamples,
+            mixxx::RgbColor color = mixxx::PredefinedColorPalettes::kDefaultCueColor) {
+        return createAndAddCue(type,
+                hotCueIndex,
+                mixxx::audio::FramePos::fromEngineSamplePosMaybeInvalid(
+                        startPositionSamples),
+                mixxx::audio::FramePos::fromEngineSamplePosMaybeInvalid(
+                        endPositionSamples),
+                color);
+    }
     CuePointer findCueByType(mixxx::CueType type) const; // NOTE: Cannot be used for hotcues.
     CuePointer findCueById(DbId id) const;
     void removeCue(const CuePointer& pCue);
@@ -256,7 +339,7 @@ class Track : public QObject {
             std::unique_ptr<mixxx::CueInfoImporter> pCueInfoImporter);
     ImportStatus getCueImportStatus() const;
 
-    bool isDirty();
+    bool isDirty() const;
 
     // Get the track's Beats list
     mixxx::BeatsPointer getBeats() const;
@@ -288,30 +371,24 @@ class Track : public QObject {
     void setCoverInfo(const CoverInfoRelative& coverInfo);
     CoverInfoRelative getCoverInfo() const;
     CoverInfo getCoverInfoWithLocation() const;
-    // Verify the cover image hash and update it if necessary.
-    // If the corresponding image has already been loaded it
-    // could be provided as a parameter to avoid reloading
-    // if actually needed.
-    bool refreshCoverImageHash(
-            const QImage& loadedImage = QImage());
 
-    quint16 getCoverHash() const;
-
-    // Set/get track metadata and cover art (optional) all at once.
-    void importMetadata(
+    /// Set track metadata after importing from the source.
+    ///
+    /// The timestamp tracks when metadata has last been synchronized
+    /// with file tags, either by importing or exporting the metadata.
+    void replaceMetadataFromSource(
             mixxx::TrackMetadata importedMetadata,
-            const QDateTime& metadataSynchronized = QDateTime());
-    // Merge additional metadata that is not (yet) stored in the database
-    // and only available from file tags.
-    void mergeImportedMetadata(
-            const mixxx::TrackMetadata& importedMetadata);
+            const QDateTime& sourceSynchronizedAt);
 
-    void readTrackMetadata(
-            mixxx::TrackMetadata* pTrackMetadata,
-            bool* pMetadataSynchronized = nullptr) const;
-    void readTrackRecord(
-            mixxx::TrackRecord* pTrackRecord,
+    mixxx::TrackMetadata getMetadata(
+            mixxx::TrackRecord::SourceSyncStatus*
+                    pSourceSyncStatus = nullptr) const;
+
+    mixxx::TrackRecord getRecord(
             bool* pDirty = nullptr) const;
+    bool replaceRecord(
+            mixxx::TrackRecord newRecord,
+            mixxx::BeatsPointer pOptionalBeats = nullptr);
 
     // Mark the track dirty if it isn't already.
     void markDirty();
@@ -333,16 +410,38 @@ class Track : public QObject {
             const mixxx::audio::StreamInfo& streamInfo);
 
   signals:
+    void artistChanged(const QString&);
+    void titleChanged(const QString&);
+    void albumChanged(const QString&);
+    void albumArtistChanged(const QString&);
+    void genreChanged(const QString&);
+#if defined(__EXTRA_METADATA__)
+    void moodChanged(const QString&);
+#endif // __EXTRA_METADATA__
+    void composerChanged(const QString&);
+    void groupingChanged(const QString&);
+    void yearChanged(const QString&);
+    void trackNumberChanged(const QString&);
+    void trackTotalChanged(const QString&);
+    void commentChanged(const QString&);
+    void bpmChanged();
+    void keyChanged();
+    void timesPlayedChanged();
+    void durationChanged();
+    void infoChanged();
+
     void waveformUpdated();
     void waveformSummaryUpdated();
     void coverArtUpdated();
-    void bpmUpdated(double bpm);
     void beatsUpdated();
-    void keyUpdated(double key);
-    void keysUpdated();
     void replayGainUpdated(mixxx::ReplayGain replayGain);
-    void colorUpdated(mixxx::RgbColor::optional_t color);
+    // This signal indicates that ReplayGain is being adjusted, and pregains should be
+    // adjusted in the opposite direction to compensate (no audible change).
+    void replayGainAdjusted(const mixxx::ReplayGain&);
+    void colorUpdated(const mixxx::RgbColor::optional_t& color);
+    void ratingUpdated(int rating);
     void cuesUpdated();
+    void loopRemove();
     void analyzed();
 
     void changed(TrackId trackId);
@@ -353,31 +452,30 @@ class Track : public QObject {
     void slotCueUpdated();
 
   private:
-    // Set a unique identifier for the track. Only used by
-    // GlobalTrackCacheResolver!
+    /// Set a unique identifier for the track.
+    /// Only used by GlobalTrackCacheResolver when the track is saved to db for the first time
     void initId(TrackId id);
-    // Reset the unique identifier after purged from library
-    // which undos a previous add. Only used by
-    // GlobalTrackCacheResolver!
+    /// Remove the TrackId.
+    /// Only used by GlobalTrackCacheResolver when the track is purged from the library
     void resetId();
 
-    void relocate(
-            TrackFile fileInfo,
-            SecurityTokenPointer pSecurityToken = SecurityTokenPointer());
+    void relocate(mixxx::FileAccess fileAccess);
 
     // Set whether the TIO is dirty or not and unlock before emitting
     // any signals. This must only be called from member functions
     // while the TIO is locked.
-    void markDirtyAndUnlock(QMutexLocker* pLock) {
+    void markDirtyAndUnlock(QT_RECURSIVE_MUTEX_LOCKER* pLock) {
         setDirtyAndUnlock(pLock, true);
     }
-    void setDirtyAndUnlock(QMutexLocker* pLock, bool bDirty);
+    void setDirtyAndUnlock(QT_RECURSIVE_MUTEX_LOCKER* pLock, bool bDirty);
 
-    void afterKeysUpdated(QMutexLocker* pLock);
-    void emitKeysUpdated(mixxx::track::io::key::ChromaticKey newKey);
+    void afterKeysUpdated(QT_RECURSIVE_MUTEX_LOCKER* pLock);
 
-    void afterBeatsAndBpmUpdated(QMutexLocker* pLock);
-    void emitBeatsAndBpmUpdated(mixxx::Bpm newBpm);
+    void afterBeatsAndBpmUpdated(QT_RECURSIVE_MUTEX_LOCKER* pLock);
+    void emitBeatsAndBpmUpdated();
+
+    /// Emits a changed signal for each Q_PROPERTY
+    void emitChangedSignalsForAllMetadata();
 
     /// Sets beats and returns a boolean to indicate if BPM/Beats were updated.
     /// Only supposed to be called while the caller guards this a lock.
@@ -398,36 +496,37 @@ class Track : public QObject {
     bool importPendingCueInfosWhileLocked();
 
     mixxx::Bpm getBpmWhileLocked() const;
-    bool trySetBpmWhileLocked(double bpmValue);
+    bool trySetBpmWhileLocked(mixxx::Bpm bpm);
     bool trySetBeatsWhileLocked(
             mixxx::BeatsPointer pBeats,
             bool lockBpmAfterSet = false);
 
     bool trySetBeatsMarkDirtyAndUnlock(
-            QMutexLocker* pLock,
+            QT_RECURSIVE_MUTEX_LOCKER* pLock,
             mixxx::BeatsPointer pBeats,
             bool lockBpmAfterSet);
     bool tryImportPendingBeatsMarkDirtyAndUnlock(
-            QMutexLocker* pLock,
+            QT_RECURSIVE_MUTEX_LOCKER* pLock,
             bool lockBpmAfterSet);
 
     void setCuePointsMarkDirtyAndUnlock(
-            QMutexLocker* pLock,
+            QT_RECURSIVE_MUTEX_LOCKER* pLock,
             const QList<CuePointer>& cuePoints);
     void importPendingCueInfosMarkDirtyAndUnlock(
-            QMutexLocker* pLock);
+            QT_RECURSIVE_MUTEX_LOCKER* pLock);
 
-    enum class DurationRounding {
-        SECONDS, // rounded to full seconds
-        NONE     // unmodified
-    };
-    double getDuration(DurationRounding rounding) const;
+    /// Merge additional metadata that is not (yet) stored in the database
+    /// and only available from file tags.
+    ///
+    /// Returns true if the track has been modified and false otherwise.
+    bool mergeExtraMetadataFromSource(
+            const mixxx::TrackMetadata& importedMetadata);
 
     bool exportSeratoMetadata();
 
     ExportTrackMetadataResult exportMetadata(
-            mixxx::MetadataSourcePointer pMetadataSource,
-            UserSettingsPointer pConfig);
+            const mixxx::MetadataSource& metadataSource,
+            const SyncTrackMetadataParams& syncParams);
 
     // Information about the actual properties of the
     // audio stream is only available after opening the
@@ -435,23 +534,17 @@ class Track : public QObject {
     // stream info of the track need to be updated to reflect
     // these values.
     bool hasStreamInfoFromSource() const {
-        QMutexLocker lock(&m_qMutex);
+        const auto locked = lockMutex(&m_qMutex);
         return m_record.hasStreamInfoFromSource();
     }
     void updateStreamInfoFromSource(
             mixxx::audio::StreamInfo&& streamInfo);
 
     // Mutex protecting access to object
-#if QT_VERSION >= QT_VERSION_CHECK(5, 14, 0)
-    mutable QRecursiveMutex m_qMutex;
-#else
-    mutable QMutex m_qMutex;
-#endif
+    mutable QT_RECURSIVE_MUTEX m_qMutex;
 
     // The file
-    mutable TrackFile m_fileInfo;
-
-    SecurityTokenPointer m_pSecurityToken;
+    mixxx::FileAccess m_fileAccess;
 
     mixxx::TrackRecord m_record;
 
@@ -477,6 +570,18 @@ class Track : public QObject {
     std::unique_ptr<mixxx::CueInfoImporter> m_pCueInfoImporterPending;
 
     friend class TrackDAO;
+    void setHeaderParsedFromTrackDAO(bool headerParsed) {
+        // Always operating on a newly created, exclusive instance! No need
+        // to lock the mutex.
+        DEBUG_ASSERT(!m_record.m_headerParsed);
+        m_record.m_headerParsed = headerParsed;
+    }
+    /// Set the genre text WITHOUT updating the corresponding custom tags.
+    ///
+    /// TODO: Remove and populate TrackRecord from the database instead.
+    void setGenreFromTrackDAO(
+            const QString& genre);
+
     friend class GlobalTrackCache;
     friend class GlobalTrackCacheResolver;
     friend class SoundSourceProxy;
