@@ -9,6 +9,8 @@ class ThreadCpuTimer {
     mixxx::Duration elapsed() const;
     mixxx::Duration restart();
   private:
+#if defined(Q_OS_UNIX)
     qint64 t1;
     qint64 t2;
+#endif
 };
