@@ -1,12 +1,16 @@
-#include <QtDebug>
-#include <limits.h>
-
 #include "vinylcontrol/vinylcontrolxwax.h"
-#include "util/timer.h"
-#include "control/controlproxy.h"
+
+#include <QtDebug>
+
 #include "control/controlobject.h"
-#include "util/math.h"
+#include "control/controlproxy.h"
+#include "moc_vinylcontrolxwax.cpp"
 #include "util/defs.h"
+#include "util/math.h"
+#include "util/timer.h"
+#include "vinylcontrol/defs_vinylcontrol.h"
+#include "vinylcontrol/steadypitch.h"
+#include "vinylcontrol/vinylsignalquality.h"
 
 /****** TODO *******
    Stuff to maybe implement here
@@ -21,7 +25,7 @@ constexpr int kChannels = 2;
 } // namespace
 
 // Sample threshold below which we consider there to be no signal.
-const double kMinSignal = 75.0 / SAMPLE_MAXIMUM;
+constexpr double kMinSignal = 75.0 / SAMPLE_MAXIMUM;
 
 bool VinylControlXwax::s_bLUTInitialized = false;
 QMutex VinylControlXwax::s_xwaxLUTMutex;
@@ -29,10 +33,12 @@ QMutex VinylControlXwax::s_xwaxLUTMutex;
 VinylControlXwax::VinylControlXwax(UserSettingsPointer pConfig, const QString& group)
         : VinylControl(pConfig, group),
           m_dVinylPositionOld(0.0),
-          m_pWorkBuffer(new short[MAX_BUFFER_LEN]),
+          m_pWorkBuffer(MAX_BUFFER_LEN),
           m_workBufferSize(MAX_BUFFER_LEN),
-          m_iQualPos(0),
-          m_iQualFilled(0),
+          m_iQualityRingIndex(0),
+          m_iQualityRingFilled(0),
+          m_iQualityLastPosition(-1),
+          m_dQualityLastPitch(0.0),
           m_iPosition(-1),
           m_bAtRecordEnd(false),
           m_bForceResync(false),
@@ -42,7 +48,6 @@ VinylControlXwax::VinylControlXwax(UserSettingsPointer pConfig, const QString& g
           m_dOldDuration(0.0),
           m_dOldDurationInaccurate(-1.0),
           m_bWasReversed(false),
-          m_pPitchRing(nullptr),
           m_iPitchRingSize(0),
           m_iPitchRingPos(0),
           m_iPitchRingFilled(0),
@@ -59,8 +64,9 @@ VinylControlXwax::VinylControlXwax(UserSettingsPointer pConfig, const QString& g
           m_dUiUpdateTime(-1.0) {
     // TODO(rryan): Should probably live in VinylControlManager since it's not
     // specific to a VC deck.
-    signalenabled->slotSet(m_pConfig->getValueString(
-        ConfigKey(VINYL_PREF_KEY, "show_signal_quality")).toInt());
+    signalenabled->set(m_pConfig->getValueString(
+                                        ConfigKey(VINYL_PREF_KEY, "show_signal_quality"))
+                               .toInt());
 
     // Get the vinyl type and speed.
     QString strVinylType = m_pConfig->getValueString(
@@ -70,28 +76,33 @@ VinylControlXwax::VinylControlXwax(UserSettingsPointer pConfig, const QString& g
 
     // libxwax indexes by C-strings so we pass libxwax string literals so we
     // don't have to deal with freeing the strings later
-    char* timecode = nullptr;
+    const char* timecode = nullptr;
 
     if (strVinylType == MIXXX_VINYL_SERATOCV02VINYLSIDEA) {
-        timecode = (char*)"serato_2a";
-    }
-    else if (strVinylType == MIXXX_VINYL_SERATOCV02VINYLSIDEB) {
-        timecode = (char*)"serato_2b";
+        timecode = MIXXX_VINYL_SERATOCV02VINYLSIDEA_XWAX_NAME;
+    } else if (strVinylType == MIXXX_VINYL_SERATOCV02VINYLSIDEB) {
+        timecode = MIXXX_VINYL_SERATOCV02VINYLSIDEB_XWAX_NAME;
     } else if (strVinylType == MIXXX_VINYL_SERATOCD) {
-        timecode = (char*)"serato_cd";
+        timecode = MIXXX_VINYL_SERATOCD_XWAX_NAME;
         m_bCDControl = true;
         // Set up very sensitive steady monitors for CDJs.
         m_pSteadySubtle = new SteadyPitch(0.06, true);
         m_pSteadyGross = new SteadyPitch(0.25, true);
     } else if (strVinylType == MIXXX_VINYL_TRAKTORSCRATCHSIDEA) {
-        timecode = (char*)"traktor_a";
+        timecode = MIXXX_VINYL_TRAKTORSCRATCHSIDEA_XWAX_NAME;
     } else if (strVinylType == MIXXX_VINYL_TRAKTORSCRATCHSIDEB) {
-        timecode = (char*)"traktor_b";
+        timecode = MIXXX_VINYL_TRAKTORSCRATCHSIDEB_XWAX_NAME;
     } else if (strVinylType == MIXXX_VINYL_MIXVIBESDVS) {
-        timecode = (char*)"mixvibes_v2";
+        timecode = MIXXX_VINYL_MIXVIBESDVS_XWAX_NAME;
+    } else if (strVinylType == MIXXX_VINYL_MIXVIBES7INCH) {
+        timecode = MIXXX_VINYL_MIXVIBES7INCH_XWAX_NAME;
+    } else if (strVinylType == MIXXX_VINYL_PIONEERA) {
+        timecode = MIXXX_VINYL_PIONEERA_XWAX_NAME;
+    } else if (strVinylType == MIXXX_VINYL_PIONEERB) {
+        timecode = MIXXX_VINYL_PIONEERB_XWAX_NAME;
     } else {
-        qDebug() << "Unknown vinyl type, defaulting to serato_2a";
-        timecode = (char*)"serato_2a";
+        qDebug() << "Unknown vinyl type, defaulting to" << MIXXX_VINYL_DEFAULT_XWAX_NAME;
+        timecode = MIXXX_VINYL_DEFAULT_XWAX_NAME;
     }
 
     // If we didn't set up the steady monitors already (not CDJ), do it now.
@@ -104,8 +115,9 @@ VinylControlXwax::VinylControlXwax(UserSettingsPointer pConfig, const QString& g
 
     timecode_def* tc_def = timecoder_find_definition(timecode);
     if (tc_def == nullptr) {
-        qDebug() << "Error finding timecode definition for " << timecode << ", defaulting to serato_2a";
-        timecode = (char*)"serato_2a";
+        qDebug() << "Error finding timecode definition for " << timecode
+                 << ", defaulting to" << MIXXX_VINYL_DEFAULT_XWAX_NAME;
+        timecode = MIXXX_VINYL_DEFAULT_XWAX_NAME;
         tc_def = timecoder_find_definition(timecode);
     }
 
@@ -117,7 +129,7 @@ VinylControlXwax::VinylControlXwax(UserSettingsPointer pConfig, const QString& g
     }
 
     double latency = ControlObject::get(
-            ConfigKey("[Master]", "latency"));
+            ConfigKey(QStringLiteral("[App]"), QStringLiteral("output_latency_ms")));
     if (latency <= 0 || latency > 200) {
         qDebug() << "Failed to get sane latency, assuming 20 as a reasonable value";
         latency = 20;
@@ -129,7 +141,7 @@ VinylControlXwax::VinylControlXwax(UserSettingsPointer pConfig, const QString& g
     // Set pitch ring size to 1/4 of one revolution -- a full revolution adds
     // too much stickiness to the pitch.
     m_iPitchRingSize = static_cast<int>(60000 / (rpm * latency * 4));
-    m_pPitchRing = new double[m_iPitchRingSize];
+    m_pPitchRing.resize(m_iPitchRingSize);
 
     qDebug() << "Xwax Vinyl control starting with a sample rate of:" << iSampleRate;
     qDebug() << "Building timecode lookup tables for" << strVinylType << "with speed" << strVinylSpeed;
@@ -153,8 +165,6 @@ VinylControlXwax::VinylControlXwax(UserSettingsPointer pConfig, const QString& g
 VinylControlXwax::~VinylControlXwax() {
     delete m_pSteadySubtle;
     delete m_pSteadyGross;
-    delete [] m_pPitchRing;
-    delete [] m_pWorkBuffer;
 
     // Cleanup xwax nicely
     timecoder_monitor_clear(&timecoder);
@@ -201,8 +211,7 @@ void VinylControlXwax::analyzeSamples(CSAMPLE* pSamples, size_t nFrames) {
     size_t samplesSize = nFrames * kChannels;
 
     if (samplesSize > m_workBufferSize) {
-        delete [] m_pWorkBuffer;
-        m_pWorkBuffer = new short[samplesSize];
+        m_pWorkBuffer.resize(samplesSize);
         m_workBufferSize = samplesSize;
     }
 
@@ -221,7 +230,7 @@ void VinylControlXwax::analyzeSamples(CSAMPLE* pSamples, size_t nFrames) {
 
     // Submit the samples to the xwax timecode processor. The size argument is
     // in stereo frames.
-    timecoder_submit(&timecoder, m_pWorkBuffer, nFrames);
+    timecoder_submit(&timecoder, m_pWorkBuffer.data(), nFrames);
 
     bool bHaveSignal = fabs(pSamples[0]) + fabs(pSamples[1]) > kMinSignal;
     //qDebug() << "signal?" << bHaveSignal;
@@ -230,15 +239,16 @@ void VinylControlXwax::analyzeSamples(CSAMPLE* pSamples, size_t nFrames) {
     //        and make that get called when any options get changed in the preferences dialog, rather than
     //        polling every time we get a buffer.
 
-
     // Check if vinyl control is enabled...
     m_bIsEnabled = enabled && checkEnabled(m_bIsEnabled, enabled->toBool());
 
+    double dVinylPitch = timecoder_get_pitch(&timecoder);
+
     if(bHaveSignal) {
         // Always analyze the input samples
         m_iPosition = timecoder_get_position(&timecoder, nullptr);
         //Notify the UI if the timecode quality is good
-        establishQuality(m_iPosition != -1);
+        establishQuality(dVinylPitch);
     }
 
     //are we even playing and enabled at all?
@@ -246,8 +256,6 @@ void VinylControlXwax::analyzeSamples(CSAMPLE* pSamples, size_t nFrames) {
         return;
     }
 
-    double dVinylPitch = timecoder_get_pitch(&timecoder);
-
     // Has a new track been loaded? Currently we use track duration which is
     // integer seconds in the song. However, for calculations we need the
     // higher-accuracy duration found by dividing the track samples by the
@@ -293,7 +301,7 @@ void VinylControlXwax::analyzeSamples(CSAMPLE* pSamples, size_t nFrames) {
         //to absolute mode (would cause sudden track skip)
         if (reportedPlayButton && reportedMode == MIXXX_VCMODE_ABSOLUTE) {
             m_iVCMode = MIXXX_VCMODE_RELATIVE;
-            mode->slotSet((double)m_iVCMode);
+            mode->set((double)m_iVCMode);
         } else {
             // go ahead and switch
             m_iVCMode = reportedMode;
@@ -305,14 +313,14 @@ void VinylControlXwax::analyzeSamples(CSAMPLE* pSamples, size_t nFrames) {
         //if we are out of error mode...
         if (vinylStatus->get() == VINYL_STATUS_ERROR &&
                 m_iVCMode == MIXXX_VCMODE_RELATIVE) {
-            vinylStatus->slotSet(VINYL_STATUS_OK);
+            vinylStatus->set(VINYL_STATUS_OK);
         }
     }
 
     //if looping has been enabled, don't allow absolute mode
     if (loopEnabled->toBool() && m_iVCMode == MIXXX_VCMODE_ABSOLUTE) {
         m_iVCMode = MIXXX_VCMODE_RELATIVE;
-        mode->slotSet((double)m_iVCMode);
+        mode->set((double)m_iVCMode);
     }
 
     // Don't allow cueing mode to be enabled in absolute mode.
@@ -358,9 +366,9 @@ void VinylControlXwax::analyzeSamples(CSAMPLE* pSamples, size_t nFrames) {
             //ok, it's still valid, blink
             if ((reportedPlayButton && (int)(filePosition * 2.0) % 2) ||
                     (!reportedPlayButton && (int)(m_iPosition / 500.0) % 2)) {
-                vinylStatus->slotSet(VINYL_STATUS_WARNING);
+                vinylStatus->set(VINYL_STATUS_WARNING);
             } else {
-                vinylStatus->slotSet(VINYL_STATUS_DISABLED);
+                vinylStatus->set(VINYL_STATUS_DISABLED);
             }
         }
     }
@@ -411,8 +419,8 @@ void VinylControlXwax::analyzeSamples(CSAMPLE* pSamples, size_t nFrames) {
                             m_group, "LoadSelectedTrack", this);
                 }
 
-                m_pControlTrackLoader->slotSet(1.0);
-                m_pControlTrackLoader->slotSet(0.0); // I think I have to do this...
+                m_pControlTrackLoader->set(1.0);
+                m_pControlTrackLoader->set(0.0); // I think I have to do this...
 
                 // if position is known and safe then no track select mode
                 m_bTrackSelectMode = false;
@@ -630,19 +638,19 @@ void VinylControlXwax::analyzeSamples(CSAMPLE* pSamples, size_t nFrames) {
             //resetSteadyPitch(dVinylPitch, filePosition);
             // Notify the UI that the timecode quality is garbage/missing.
             m_fTimecodeQuality = 0.0f;
-            m_iPitchRingPos = 0;
-            m_iPitchRingFilled = 0;
-            m_iQualPos = 0;
-            m_iQualFilled = 0;
+            m_dQualityLastPitch = 0.0;
+            m_iQualityLastPosition = -1;
+            m_iQualityRingIndex = 0;
+            m_iQualityRingFilled = 0;
             m_bForceResync = true;
-            vinylStatus->slotSet(VINYL_STATUS_OK);
+            vinylStatus->set(VINYL_STATUS_OK);
         }
     }
 }
 
 void VinylControlXwax::enableRecordEndMode() {
     qDebug() << "record end, setting constant mode";
-    vinylStatus->slotSet(VINYL_STATUS_WARNING);
+    vinylStatus->set(VINYL_STATUS_WARNING);
     enableConstantMode();
     m_bAtRecordEnd = true;
 }
@@ -650,7 +658,7 @@ void VinylControlXwax::enableRecordEndMode() {
 void VinylControlXwax::enableConstantMode() {
     m_iOldVCMode = m_iVCMode;
     m_iVCMode = MIXXX_VCMODE_CONSTANT;
-    mode->slotSet((double)m_iVCMode);
+    mode->set((double)m_iVCMode);
     togglePlayButton(true);
     double rate = m_pVCRate->get();
     m_pRateRatio->set(fabs(rate));
@@ -660,22 +668,22 @@ void VinylControlXwax::enableConstantMode() {
 void VinylControlXwax::enableConstantMode(double rate) {
     m_iOldVCMode = m_iVCMode;
     m_iVCMode = MIXXX_VCMODE_CONSTANT;
-    mode->slotSet((double)m_iVCMode);
+    mode->set((double)m_iVCMode);
     togglePlayButton(true);
     m_pRateRatio->set(fabs(rate));
     m_pVCRate->set(rate);
 }
 
 void VinylControlXwax::disableRecordEndMode() {
-    vinylStatus->slotSet(VINYL_STATUS_OK);
+    vinylStatus->set(VINYL_STATUS_OK);
     m_bAtRecordEnd = false;
     m_iVCMode = MIXXX_VCMODE_RELATIVE;
-    mode->slotSet((double)m_iVCMode);
+    mode->set((double)m_iVCMode);
 }
 
 void VinylControlXwax::togglePlayButton(bool on) {
     if (m_bIsEnabled && (playButton->get() > 0) != on) {
-        playButton->slotSet((float)on);  //and we all float on all right
+        playButton->set((float)on); //and we all float on all right
     }
 }
 
@@ -685,8 +693,8 @@ void VinylControlXwax::doTrackSelection(bool valid_pos, double pitch, double pos
     //track will be selected when the needle is moved back to play area
     //track selection can be cancelled by loading a track manually
 
-    const int SELECT_INTERVAL = 150;
-    const double NOPOS_SPEED = 0.50;
+    constexpr int SELECT_INTERVAL = 150;
+    constexpr double NOPOS_SPEED = 0.50;
 
     if (m_pControlTrackSelector == nullptr) {
         // this isn't done in the constructor because this object
@@ -736,9 +744,9 @@ double VinylControlXwax::checkSteadyPitch(double pitch, double time) {
         return 0;
     }
     if (m_pSteadyGross->check(pitch, time) < 0.5) {
-        scratching->slotSet(1.0);
+        scratching->set(1.0);
     } else {
-        scratching->slotSet(0.0);
+        scratching->set(0.0);
     }
     return m_pSteadySubtle->check(pitch, time);
 }
@@ -747,15 +755,15 @@ double VinylControlXwax::checkSteadyPitch(double pitch, double time) {
 void VinylControlXwax::syncPosition() {
     //qDebug() << "sync position" << m_dVinylPosition / m_dOldDuration;
     // VinylPos in seconds / total length of song.
-    vinylSeek->slotSet(m_dVinylPosition / m_dOldDuration);
+    vinylSeek->set(m_dVinylPosition / m_dOldDuration);
 }
 
 bool VinylControlXwax::checkEnabled(bool was, bool is) {
     // if we're not enabled, but the last object was, try turning ourselves on
     // XXX: is this just a race that's working right now?
     if (!is && wantenabled->get() > 0) {
-        enabled->slotSet(true);
-        wantenabled->slotSet(false); //don't try to do this over and over
+        enabled->set(true);
+        wantenabled->set(false); //don't try to do this over and over
         return true; //optimism!
     }
 
@@ -777,9 +785,9 @@ bool VinylControlXwax::checkEnabled(bool was, bool is) {
     }
 
     if (is && !was) {
-        vinylStatus->slotSet(VINYL_STATUS_OK);
+        vinylStatus->set(VINYL_STATUS_OK);
     } else if (!is) {
-        vinylStatus->slotSet(VINYL_STATUS_DISABLED);
+        vinylStatus->set(VINYL_STATUS_DISABLED);
     }
 
     return is;
@@ -793,22 +801,75 @@ bool VinylControlXwax::uiUpdateTime(double now) {
     return false;
 }
 
-void VinylControlXwax::establishQuality(bool quality_sample) {
-    m_bQualityRing[m_iQualPos] = quality_sample;
-    if (m_iQualFilled < QUALITY_RING_SIZE) {
-        m_iQualFilled++;
+int VinylControlXwax::getPositionQuality() {
+    int positionQualityPercent;
+
+    if (m_iPosition == -1) {
+        // No position code - happens when control vinyl is spinning slow (or during scratching) or the setup is incorrect
+        positionQualityPercent = 0;
+    } else if ((m_iPosition <= m_iQualityLastPosition - 5) ||
+            (m_iPosition >= m_iQualityLastPosition + 5)) {
+        // Position code changed not by more than 5. This indicates a normal spinning control vinyl.
+        positionQualityPercent = 100;
+    } else {
+        // Position code changed by more than 5. This indicates a fast spinning control vinyl, a jumping needle or a false signal.
+        // This covers also the case of m_iQualityLastPosition == -1
+        positionQualityPercent = 50;
     }
 
-    int quality = 0;
-    for (int i = 0; i < m_iQualFilled; ++i) {
-        if (m_bQualityRing[i]) {
-            quality++;
+    m_iQualityLastPosition = m_iPosition;
+
+    return positionQualityPercent;
+}
+
+int VinylControlXwax::getPitchQuality(double& pitch) {
+    int pitchQualityPercent;
+
+    if (m_iQualityRingFilled == 0) {
+        m_dQualityLastPitch = pitch;
+        pitchQualityPercent = 0;
+        return pitchQualityPercent; // First value - but at least two values needed for calculation
+    }
+
+    double pitchDifference = pitch - m_dQualityLastPitch;
+    m_dQualityLastPitch = pitch;
+
+    if (pitchDifference != 0) {
+        double pitchStability = std::fabs(pitch / pitchDifference);
+
+        if (pitchStability < 3.0) {
+            // Unlikely that this pitch difference is from a proper set up control vinyl
+            pitchQualityPercent = 0;
+        } else if (pitchStability > 6.0) {
+            // Typical pitch difference for normal spinning control vinyl
+            pitchQualityPercent = 100;
+        } else {
+            // Weak pitch difference for a control vinyl slow spinning ,or for scratching
+            pitchQualityPercent = 75;
         }
+    } else {
+        // Unlikely case of two identical float values for pitch
+        pitchQualityPercent = 0;
+    }
+    return pitchQualityPercent;
+}
+
+void VinylControlXwax::establishQuality(double& pitch) {
+    m_iQualityRing[m_iQualityRingIndex] = getPositionQuality() + getPitchQuality(pitch);
+    m_fTimecodeQuality = std::max<float>(0.0,
+            std::min<float>(1.0,
+                    static_cast<float>(std::accumulate(m_iQualityRing,
+                            m_iQualityRing + m_iQualityRingFilled,
+                            0)) /
+                            2.0f / 100.0f /
+                            static_cast<float>(
+                                    m_iQualityRingFilled))); // Two information in percent per datapoint
+
+    if (m_iQualityRingFilled < QUALITY_RING_SIZE) {
+        m_iQualityRingFilled++;
     }
 
-    m_fTimecodeQuality = static_cast<float>(quality) /
-            static_cast<float>(m_iQualFilled);
-    m_iQualPos = (m_iQualPos + 1) % QUALITY_RING_SIZE;
+    m_iQualityRingIndex = (m_iQualityRingIndex + 1) % QUALITY_RING_SIZE;
 }
 
 float VinylControlXwax::getAngle() {
