@@ -1,16 +1,14 @@
 #include "waveform/renderers/glvsynctestrenderer.h"
 #if !defined(QT_NO_OPENGL) && !defined(QT_OPENGL_ES_2)
 
-#include "track/track.h"
 #include "util/performancetimer.h"
 #include "waveform/renderers/waveformwidgetrenderer.h"
 #include "waveform/waveform.h"
-#include "waveform/waveformwidgetfactory.h"
 
 GLVSyncTestRenderer::GLVSyncTestRenderer(
         WaveformWidgetRenderer* waveformWidgetRenderer)
-    : WaveformRendererSignalBase(waveformWidgetRenderer),
-      m_drawcount(0) {
+        : GLWaveformRendererSignal(waveformWidgetRenderer),
+          m_drawcount(0) {
 }
 
 GLVSyncTestRenderer::~GLVSyncTestRenderer() {
@@ -26,12 +24,9 @@ inline void setPoint(QPointF& point, qreal x, qreal y) {
 }
 
 void GLVSyncTestRenderer::draw(QPainter* painter, QPaintEvent* /*event*/) {
-    maybeInitializeGL();
-
     PerformanceTimer timer;
     //mixxx::Duration t5, t6, t7, t8, t9, t10, t11, t12, t13;
 
-
     timer.start();
 
     ConstWaveformPointer pWaveform = m_waveformRenderer->getWaveform();
