@@ -1,15 +1,11 @@
 #pragma once
 
-#include "waveform/renderers/glwaveformrenderer.h"
+#include "waveform/renderers/glwaveformrenderersignal.h"
 #if !defined(QT_NO_OPENGL) && !defined(QT_OPENGL_ES_2)
 
-#include "waveform/renderers/waveformrenderersignalbase.h"
+class QDomNode;
 
-QT_FORWARD_DECLARE_CLASS(QDomNode)
-class ControlObject;
-
-class GLWaveformRendererFilteredSignal : public WaveformRendererSignalBase,
-                                         public GLWaveformRenderer {
+class GLWaveformRendererFilteredSignal : public GLWaveformRendererSignal {
   public:
     explicit GLWaveformRendererFilteredSignal(
             WaveformWidgetRenderer* waveformWidgetRenderer);
