@@ -1,16 +1,19 @@
 #include "waveform/waveformwidgetfactory.h"
 
+#ifdef MIXXX_USE_QOPENGL
+#include <QOpenGLShaderProgram>
+#include <QOpenGLWindow>
+#else
 #include <QGLFormat>
 #include <QGLShaderProgram>
-#include <QGuiApplication>
+#endif
+
 #include <QOpenGLFunctions>
+#include <QRegularExpression>
 #include <QStringList>
-#include <QTime>
 #include <QWidget>
 #include <QWindow>
-#include <QtDebug>
 
-#include "control/controlpotmeter.h"
 #include "moc_waveformwidgetfactory.cpp"
 #include "util/cmdlineargs.h"
 #include "util/math.h"
@@ -20,6 +23,19 @@
 #include "waveform/sharedglcontext.h"
 #include "waveform/visualsmanager.h"
 #include "waveform/vsyncthread.h"
+#ifdef MIXXX_USE_QOPENGL
+#include "waveform/widgets/allshader/filteredwaveformwidget.h"
+#include "waveform/widgets/allshader/hsvwaveformwidget.h"
+#include "waveform/widgets/allshader/lrrgbwaveformwidget.h"
+#include "waveform/widgets/allshader/rgbwaveformwidget.h"
+#include "waveform/widgets/allshader/simplewaveformwidget.h"
+#else
+#include "waveform/widgets/qthsvwaveformwidget.h"
+#include "waveform/widgets/qtrgbwaveformwidget.h"
+#include "waveform/widgets/qtsimplewaveformwidget.h"
+#include "waveform/widgets/qtvsynctestwidget.h"
+#include "waveform/widgets/qtwaveformwidget.h"
+#endif
 #include "waveform/widgets/emptywaveformwidget.h"
 #include "waveform/widgets/glrgbwaveformwidget.h"
 #include "waveform/widgets/glsimplewaveformwidget.h"
@@ -27,16 +43,11 @@
 #include "waveform/widgets/glvsynctestwidget.h"
 #include "waveform/widgets/glwaveformwidget.h"
 #include "waveform/widgets/hsvwaveformwidget.h"
-#include "waveform/widgets/qthsvwaveformwidget.h"
-#include "waveform/widgets/qtrgbwaveformwidget.h"
-#include "waveform/widgets/qtsimplewaveformwidget.h"
-#include "waveform/widgets/qtvsynctestwidget.h"
-#include "waveform/widgets/qtwaveformwidget.h"
 #include "waveform/widgets/rgbwaveformwidget.h"
 #include "waveform/widgets/softwarewaveformwidget.h"
 #include "waveform/widgets/waveformwidgetabstract.h"
-#include "widget/wvumeter.h"
-#include "widget/wvumetergl.h"
+#include "widget/wvumeterbase.h"
+#include "widget/wvumeterlegacy.h"
 #include "widget/wwaveformviewer.h"
 
 namespace {
@@ -48,26 +59,17 @@ bool shouldRenderWaveform(WaveformWidgetAbstract* pWaveformWidget) {
         return false;
     }
 
-    auto* glw = qobject_cast<QGLWidget*>(pWaveformWidget->getWidget());
+    auto* glw = pWaveformWidget->getGLWidget();
     if (glw == nullptr) {
-        // Not a QGLWidget. We can simply use QWidget::isVisible.
+        // Not a WGLWidget. We can simply use QWidget::isVisible.
         auto* qwidget = qobject_cast<QWidget*>(pWaveformWidget->getWidget());
         return qwidget != nullptr && qwidget->isVisible();
     }
 
-    if (glw == nullptr || !glw->isValid() || !glw->isVisible()) {
-        return false;
-    }
-
-    // Strangely, a widget can have non-zero width/height, be valid and visible,
-    // yet still not show up on the screen. QWindow::isExposed tells us this.
-    const QWindow* window = glw->windowHandle();
-    if (window == nullptr || !window->isExposed()) {
-        return false;
-    }
-
-    return true;
+    return glw->shouldRender();
 }
+
+const QRegularExpression openGLVersionRegex(QStringLiteral("^(\\d+)\\.(\\d+).*$"));
 }  // anonymous namespace
 
 ///////////////////////////////////////////
@@ -104,7 +106,7 @@ WaveformWidgetFactory::WaveformWidgetFactory()
           m_configType(WaveformWidgetType::EmptyWaveform),
           m_config(nullptr),
           m_skipRender(false),
-          m_frameRate(30),
+          m_frameRate(60),
           m_endOfTrackWarningTime(30),
           m_defaultZoom(WaveformWidgetRenderer::s_waveformDefaultZoom),
           m_zoomSync(true),
@@ -118,13 +120,74 @@ WaveformWidgetFactory::WaveformWidgetFactory()
           m_pVisualsManager(nullptr),
           m_frameCnt(0),
           m_actualFrameRate(0),
-          m_vSyncType(0),
           m_playMarkerPosition(WaveformWidgetRenderer::s_defaultPlayMarkerPosition) {
     m_visualGain[All] = 1.0;
     m_visualGain[Low] = 1.0;
     m_visualGain[Mid] = 1.0;
     m_visualGain[High] = 1.0;
 
+#ifdef MIXXX_USE_QOPENGL
+    WGLWidget* widget = SharedGLContext::getWidget();
+    if (widget) {
+        widget->makeCurrentIfNeeded();
+        auto* pContext = QOpenGLContext::currentContext();
+        if (pContext) {
+            auto* glFunctions = pContext->functions();
+            glFunctions->initializeOpenGLFunctions();
+            QString versionString(QLatin1String(
+                    reinterpret_cast<const char*>(glFunctions->glGetString(GL_VERSION))));
+            QString vendorString(QLatin1String(
+                    reinterpret_cast<const char*>(glFunctions->glGetString(GL_VENDOR))));
+            QString rendererString = QString(QLatin1String(
+                    reinterpret_cast<const char*>(glFunctions->glGetString(GL_RENDERER))));
+            qDebug().noquote() << QStringLiteral(
+                    "OpenGL driver version string \"%1\", vendor \"%2\", "
+                    "renderer \"%3\"")
+                                          .arg(versionString, vendorString, rendererString);
+
+            GLint majorVersion, minorVersion = GL_INVALID_ENUM;
+            glFunctions->glGetIntegerv(GL_MAJOR_VERSION, &majorVersion);
+            glFunctions->glGetIntegerv(GL_MINOR_VERSION, &minorVersion);
+            if (majorVersion == GL_INVALID_ENUM || minorVersion == GL_INVALID_ENUM) {
+                // GL_MAJOR/MINOR_VERSION are not supported below OpenGL 3.0, so
+                // parse GL_VERSION string as a fallback.
+                // https://www.khronos.org/opengl/wiki/OpenGL_Context#OpenGL_version_number
+                auto match = openGLVersionRegex.match(versionString);
+                DEBUG_ASSERT(match.hasMatch());
+                majorVersion = match.captured(1).toInt();
+                minorVersion = match.captured(2).toInt();
+            }
+
+            qDebug().noquote()
+                    << QStringLiteral("Supported OpenGL version: %1.%2")
+                               .arg(QString::number(majorVersion), QString::number(minorVersion));
+
+            m_openGLShaderAvailable = QOpenGLShaderProgram::hasOpenGLShaderPrograms(pContext);
+
+            m_openGLVersion = pContext->isOpenGLES() ? "ES " : "";
+            m_openGLVersion += majorVersion == 0 ? QString("None") : versionString;
+
+            // Qt5 requires at least OpenGL 2.1 or OpenGL ES 2.0
+            if (pContext->isOpenGLES()) {
+                if (majorVersion * 100 + minorVersion >= 200) {
+                    m_openGlesAvailable = true;
+                }
+            } else {
+                if (majorVersion * 100 + minorVersion >= 201) {
+                    m_openGlAvailable = true;
+                }
+            }
+
+            if (!rendererString.isEmpty()) {
+                m_openGLVersion += " (" + rendererString + ")";
+            }
+        } else {
+            qDebug() << "QOpenGLContext::currentContext() returns nullptr";
+        }
+        widget->doneCurrent();
+        widget->hide();
+    }
+#else
     QGLWidget* pGlWidget = SharedGLContext::getWidget();
     if (pGlWidget && pGlWidget->isValid()) {
         // will be false if SafeMode is enabled
@@ -265,7 +328,7 @@ WaveformWidgetFactory::WaveformWidgetFactory()
 
         pGlWidget->hide();
     }
-
+#endif
     evaluateWidgets();
     m_time.start();
 }
@@ -296,8 +359,6 @@ bool WaveformWidgetFactory::setConfig(UserSettingsPointer config) {
                 ConfigValue(m_endOfTrackWarningTime));
     }
 
-    m_vSyncType = m_config->getValue(ConfigKey("[Waveform]","VSync"), 0);
-
     double defaultZoom = m_config->getValueString(ConfigKey("[Waveform]","DefaultZoom")).toDouble(&ok);
     if (ok) {
         setDefaultZoom(defaultZoom);
@@ -354,27 +415,27 @@ void WaveformWidgetFactory::destroyWidgets() {
     m_waveformWidgetHolders.clear();
 }
 
-void WaveformWidgetFactory::addVuMeter(WVuMeter* pVuMeter) {
+void WaveformWidgetFactory::addVuMeter(WVuMeterLegacy* pVuMeter) {
     // Do not hold the pointer to of timer listeners since they may be deleted.
     // We don't activate update() or repaint() directly so listener widgets
     // can decide whether to paint or not.
     connect(this,
             &WaveformWidgetFactory::waveformUpdateTick,
             pVuMeter,
-            &WVuMeter::maybeUpdate,
+            &WVuMeterLegacy::maybeUpdate,
             Qt::DirectConnection);
 }
 
-void WaveformWidgetFactory::addVuMeter(WVuMeterGL* pVuMeter) {
+void WaveformWidgetFactory::addVuMeter(WVuMeterBase* pVuMeter) {
     // WVuMeterGLs to be rendered and swapped from the vsync thread
     connect(this,
             &WaveformWidgetFactory::renderVuMeters,
             pVuMeter,
-            &WVuMeterGL::render);
+            &WVuMeterBase::render);
     connect(this,
             &WaveformWidgetFactory::swapVuMeters,
             pVuMeter,
-            &WVuMeterGL::swap);
+            &WVuMeterBase::swap);
 }
 
 void WaveformWidgetFactory::slotSkinLoaded() {
@@ -445,21 +506,6 @@ void WaveformWidgetFactory::setEndOfTrackWarningTime(int endTime) {
     }
 }
 
-void WaveformWidgetFactory::setVSyncType(int type) {
-    if (m_config) {
-        m_config->set(ConfigKey("[Waveform]","VSync"), ConfigValue((int)type));
-    }
-
-    m_vSyncType = type;
-    if (m_vsyncThread) {
-        m_vsyncThread->setVSyncType(type);
-    }
-}
-
-int WaveformWidgetFactory::getVSyncType() {
-    return m_vSyncType;
-}
-
 bool WaveformWidgetFactory::setWidgetType(WaveformWidgetType::Type type) {
     return setWidgetType(type, &m_type);
 }
@@ -477,6 +523,7 @@ bool WaveformWidgetFactory::setWidgetType(
         // type is acceptable
         *pCurrentType = type;
         if (m_config) {
+            m_configType = type;
             m_config->setValue(
                     ConfigKey("[Waveform]", "WaveformType"),
                     static_cast<int>(*pCurrentType));
@@ -487,6 +534,7 @@ bool WaveformWidgetFactory::setWidgetType(
     // fallback
     *pCurrentType = WaveformWidgetType::EmptyWaveform;
     if (m_config) {
+        m_configType = *pCurrentType;
         m_config->setValue(
                 ConfigKey("[Waveform]", "WaveformType"),
                 static_cast<int>(*pCurrentType));
@@ -498,6 +546,9 @@ bool WaveformWidgetFactory::setWidgetTypeFromConfig() {
     int empty = findHandleIndexFromType(WaveformWidgetType::EmptyWaveform);
     int desired = findHandleIndexFromType(m_configType);
     if (desired == -1) {
+        qDebug() << "WaveformWidgetFactory::setWidgetTypeFromConfig"
+                 << " - configured type" << static_cast<int>(m_configType)
+                 << "not found -- using 'EmptyWaveform'";
         desired = empty;
     }
     return setWidgetTypeFromHandle(desired, true);
@@ -505,7 +556,8 @@ bool WaveformWidgetFactory::setWidgetTypeFromConfig() {
 
 bool WaveformWidgetFactory::setWidgetTypeFromHandle(int handleIndex, bool force) {
     if (handleIndex < 0 || handleIndex >= m_waveformWidgetHandles.size()) {
-        qDebug() << "WaveformWidgetFactory::setWidgetType - invalid handle --> use of 'EmptyWaveform'";
+        qDebug() << "WaveformWidgetFactory::setWidgetTypeFromHandle"
+                    " - invalid handle --> using 'EmptyWaveform'";
         // fallback empty type
         setWidgetType(WaveformWidgetType::EmptyWaveform);
         return false;
@@ -513,7 +565,8 @@ bool WaveformWidgetFactory::setWidgetTypeFromHandle(int handleIndex, bool force)
 
     WaveformWidgetAbstractHandle& handle = m_waveformWidgetHandles[handleIndex];
     if (handle.m_type == m_type && !force) {
-        qDebug() << "WaveformWidgetFactory::setWidgetType - type already in use";
+        qDebug() << "WaveformWidgetFactory::setWidgetTypeFromHandle - type"
+                 << handle.getDisplayName() << "already in use";
         return true;
     }
 
@@ -521,7 +574,6 @@ bool WaveformWidgetFactory::setWidgetTypeFromHandle(int handleIndex, bool force)
     setWidgetType(handle.m_type);
 
     m_skipRender = true;
-    //qDebug() << "recreate start";
 
     //re-create/setup all waveform widgets
     for (auto& holder : m_waveformWidgetHolders) {
@@ -550,7 +602,6 @@ bool WaveformWidgetFactory::setWidgetTypeFromHandle(int handleIndex, bool force)
     }
 
     m_skipRender = false;
-    //qDebug() << "recreate done";
     return true;
 }
 
@@ -561,7 +612,7 @@ void WaveformWidgetFactory::setDefaultZoom(double zoom) {
         m_config->set(ConfigKey("[Waveform]","DefaultZoom"), ConfigValue(m_defaultZoom));
     }
 
-    for (const auto& holder : m_waveformWidgetHolders) {
+    for (const auto& holder : std::as_const(m_waveformWidgetHolders)) {
         holder.m_waveformViewer->setZoom(m_defaultZoom);
     }
 }
@@ -577,7 +628,7 @@ void WaveformWidgetFactory::setZoomSync(bool sync) {
     }
 
     double refZoom = m_waveformWidgetHolders[0].m_waveformWidget->getZoomFactor();
-    for (const auto& holder : m_waveformWidgetHolders) {
+    for (const auto& holder : std::as_const(m_waveformWidgetHolders)) {
         holder.m_waveformViewer->setZoom(refZoom);
     }
 }
@@ -588,7 +639,7 @@ void WaveformWidgetFactory::setDisplayBeatGridAlpha(int alpha) {
         return;
     }
 
-    for (const auto& holder : m_waveformWidgetHolders) {
+    for (const auto& holder : std::as_const(m_waveformWidgetHolders)) {
         holder.m_waveformWidget->setDisplayBeatGridAlpha(m_beatGridAlpha);
     }
 }
@@ -612,41 +663,34 @@ void WaveformWidgetFactory::setOverviewNormalized(bool normalize) {
 }
 
 void WaveformWidgetFactory::setPlayMarkerPosition(double position) {
-    //qDebug() << "setPlayMarkerPosition, position=" << position;
     m_playMarkerPosition = position;
     if (m_config) {
         m_config->setValue(ConfigKey("[Waveform]", "PlayMarkerPosition"), m_playMarkerPosition);
     }
 
-    for (const auto& holder : m_waveformWidgetHolders) {
+    for (const auto& holder : std::as_const(m_waveformWidgetHolders)) {
         holder.m_waveformWidget->setPlayMarkerPosition(m_playMarkerPosition);
     }
 }
 
 void WaveformWidgetFactory::notifyZoomChange(WWaveformViewer* viewer) {
     WaveformWidgetAbstract* pWaveformWidget = viewer->getWaveformWidget();
-    if (pWaveformWidget != nullptr && isZoomSync()) {
-        //qDebug() << "WaveformWidgetFactory::notifyZoomChange";
-        double refZoom = pWaveformWidget->getZoomFactor();
+    if (pWaveformWidget == nullptr || !isZoomSync()) {
+        return;
+    }
+    double refZoom = pWaveformWidget->getZoomFactor();
 
-        for (std::size_t i = 0; i < m_waveformWidgetHolders.size(); ++i) {
-            WaveformWidgetHolder& holder = m_waveformWidgetHolders[i];
-            if (holder.m_waveformViewer != viewer) {
-                holder.m_waveformViewer->setZoom(refZoom);
-            }
+    for (const auto& holder : std::as_const(m_waveformWidgetHolders)) {
+        if (holder.m_waveformViewer != viewer) {
+            holder.m_waveformViewer->setZoom(refZoom);
         }
     }
 }
 
-void WaveformWidgetFactory::render() {
+void WaveformWidgetFactory::renderSelf() {
     ScopedTimer t("WaveformWidgetFactory::render() %1waveforms",
             static_cast<int>(m_waveformWidgetHolders.size()));
 
-    //int paintersSetupTime0 = 0;
-    //int paintersSetupTime1 = 0;
-
-    //qDebug() << "render()" << m_vsyncThread->elapsed();
-
     if (!m_skipRender) {
         if (m_type) {   // no regular updates for an empty waveform
             // next rendered frame is displayed after next buffer swap and than after VSync
@@ -683,7 +727,7 @@ void WaveformWidgetFactory::render() {
             }
         }
 
-        // WSpinnys are also double-buffered QGLWidgets, like all the waveform
+        // WSpinnys are also double-buffered WGLWidgets, like all the waveform
         // renderers. Render all the WSpinny widgets now.
         emit renderSpinnies(m_vsyncThread);
         // Same for WVuMeterGL. Note that we are either using WVuMeter or WVuMeterGL.
@@ -710,10 +754,14 @@ void WaveformWidgetFactory::render() {
     m_pGuiTick->process();
 
     //qDebug() << "refresh end" << m_vsyncThread->elapsed();
+}
+
+void WaveformWidgetFactory::render() {
+    renderSelf();
     m_vsyncThread->vsyncSlotFinished();
 }
 
-void WaveformWidgetFactory::swap() {
+void WaveformWidgetFactory::swapSelf() {
     ScopedTimer t("WaveformWidgetFactory::swap() %1waveforms",
             static_cast<int>(m_waveformWidgetHolders.size()));
 
@@ -722,22 +770,21 @@ void WaveformWidgetFactory::swap() {
         if (m_type) {   // no regular updates for an empty waveform
             // Show rendered buffer from last render() run
             //qDebug() << "swap() start" << m_vsyncThread->elapsed();
-            for (const auto& holder : m_waveformWidgetHolders) {
+            for (const auto& holder : std::as_const(m_waveformWidgetHolders)) {
                 WaveformWidgetAbstract* pWaveformWidget = holder.m_waveformWidget;
 
                 // Don't swap invalid / invisible widgets or widgets with an
                 // unexposed window. Prevents continuous log spew of
                 // "QOpenGLContext::swapBuffers() called with non-exposed
-                // window, behavior is undefined" on Qt5. See Bug #1779487.
+                // window, behavior is undefined" on Qt5. See issue #9360.
                 if (!shouldRenderWaveform(pWaveformWidget)) {
                     continue;
                 }
-                QGLWidget* glw = qobject_cast<QGLWidget*>(pWaveformWidget->getWidget());
+                WGLWidget* glw = pWaveformWidget->getGLWidget();
                 if (glw != nullptr) {
-                    if (glw->context() != QGLContext::currentContext()) {
-                        glw->makeCurrent();
-                    }
+                    glw->makeCurrentIfNeeded();
                     glw->swapBuffers();
+                    glw->doneCurrent();
                 }
                 //qDebug() << "swap x" << m_vsyncThread->elapsed();
             }
@@ -749,17 +796,46 @@ void WaveformWidgetFactory::swap() {
         // If we are using WVuMeter, this does nothing
         emit swapVuMeters();
     }
-    //qDebug() << "swap end" << m_vsyncThread->elapsed();
+}
+
+void WaveformWidgetFactory::swap() {
+    swapSelf();
     m_vsyncThread->vsyncSlotFinished();
 }
 
+void WaveformWidgetFactory::swapAndRender() {
+    // used for PLL
+    WGLWidget* widget = SharedGLContext::getWidget();
+    widget->getOpenGLWindow()->update();
+
+    swapSelf();
+    renderSelf();
+
+    m_vsyncThread->vsyncSlotFinished();
+}
+
+void WaveformWidgetFactory::slotFrameSwapped() {
+#ifdef MIXXX_USE_QOPENGL
+    if (m_vsyncThread->pllInitializing()) {
+        // continuously trigger redraws during PLL init
+        WGLWidget* widget = SharedGLContext::getWidget();
+        widget->getOpenGLWindow()->update();
+    }
+    // update the phase-locked-loop
+    m_vsyncThread->updatePLL();
+#endif
+}
+
 WaveformWidgetType::Type WaveformWidgetFactory::autoChooseWidgetType() const {
-    if (m_openGlAvailable) {
-        if (m_openGLShaderAvailable) {
-            return WaveformWidgetType::GLSLRGBWaveform;
-        } else {
-            return WaveformWidgetType::GLRGBWaveform;
-        }
+    if (isOpenGlShaderAvailable()) {
+#ifndef MIXXX_USE_QOPENGL
+        return WaveformWidgetType::GLSLRGBWaveform;
+#else
+        return WaveformWidgetType::AllShaderRGBWaveform;
+#endif
+    }
+    if (isOpenGlAvailable() || isOpenGlesAvailable()) {
+        return WaveformWidgetType::GLRGBWaveform;
     }
     return WaveformWidgetType::RGBWaveform;
 }
@@ -771,15 +847,21 @@ void WaveformWidgetFactory::evaluateWidgets() {
         bool useOpenGl;
         bool useOpenGles;
         bool useOpenGLShaders;
-        bool developerOnly;
+        WaveformWidgetCategory category;
+
+        // this lambda needs its type specified explicitly,
+        // requiring it to be called with via `.operator()<WaveformT>()`
+        auto setWaveformVarsByType = [&]<typename WaveformT>() {
+            widgetName = buildWidgetDisplayName<WaveformT>();
+            useOpenGl = WaveformT::useOpenGl();
+            useOpenGles = WaveformT::useOpenGles();
+            useOpenGLShaders = WaveformT::useOpenGLShaders();
+            category = WaveformT::category();
+        };
 
         switch(type) {
         case WaveformWidgetType::EmptyWaveform:
-            widgetName = EmptyWaveformWidget::getWaveformWidgetName();
-            useOpenGl = EmptyWaveformWidget::useOpenGl();
-            useOpenGles = EmptyWaveformWidget::useOpenGles();
-            useOpenGLShaders = EmptyWaveformWidget::useOpenGLShaders();
-            developerOnly = EmptyWaveformWidget::developerOnly();
+            setWaveformVarsByType.operator()<EmptyWaveformWidget>();
             break;
         case WaveformWidgetType::SoftwareSimpleWaveform:
             continue; // //TODO(vrince):
@@ -791,112 +873,114 @@ void WaveformWidgetFactory::evaluateWidgets() {
             // https://bugs.launchpad.net/bugs/1928772
             continue;
 #else
-            widgetName = SoftwareWaveformWidget::getWaveformWidgetName();
-            useOpenGl = SoftwareWaveformWidget::useOpenGl();
-            useOpenGles = SoftwareWaveformWidget::useOpenGles();
-            useOpenGLShaders = SoftwareWaveformWidget::useOpenGLShaders();
-            developerOnly = SoftwareWaveformWidget::developerOnly();
+            setWaveformVarsByType.operator()<SoftwareWaveformWidget>();
             break;
 #endif
         case WaveformWidgetType::HSVWaveform:
 #ifdef __APPLE__
             continue;
 #else
-            widgetName = HSVWaveformWidget::getWaveformWidgetName();
-            useOpenGl = HSVWaveformWidget::useOpenGl();
-            useOpenGles = HSVWaveformWidget::useOpenGles();
-            useOpenGLShaders = HSVWaveformWidget::useOpenGLShaders();
-            developerOnly = HSVWaveformWidget::developerOnly();
+            setWaveformVarsByType.operator()<HSVWaveformWidget>();
             break;
 #endif
         case WaveformWidgetType::RGBWaveform:
 #ifdef __APPLE__
             continue;
 #else
-            widgetName = RGBWaveformWidget::getWaveformWidgetName();
-            useOpenGl = RGBWaveformWidget::useOpenGl();
-            useOpenGles = RGBWaveformWidget::useOpenGles();
-            useOpenGLShaders = RGBWaveformWidget::useOpenGLShaders();
-            developerOnly = RGBWaveformWidget::developerOnly();
+            setWaveformVarsByType.operator()<RGBWaveformWidget>();
             break;
 #endif
         case WaveformWidgetType::QtSimpleWaveform:
-            widgetName = QtSimpleWaveformWidget::getWaveformWidgetName();
-            useOpenGl = QtSimpleWaveformWidget::useOpenGl();
-            useOpenGles = QtSimpleWaveformWidget::useOpenGles();
-            useOpenGLShaders = QtSimpleWaveformWidget::useOpenGLShaders();
-            developerOnly = QtSimpleWaveformWidget::developerOnly();
+#ifdef MIXXX_USE_QOPENGL
+            continue;
+#else
+            setWaveformVarsByType.operator()<QtSimpleWaveformWidget>();
             break;
+#endif
         case WaveformWidgetType::QtWaveform:
-            widgetName = QtWaveformWidget::getWaveformWidgetName();
-            useOpenGl = QtWaveformWidget::useOpenGl();
-            useOpenGles = QtWaveformWidget::useOpenGles();
-            useOpenGLShaders = QtWaveformWidget::useOpenGLShaders();
-            developerOnly = QtWaveformWidget::developerOnly();
+#ifdef MIXXX_USE_QOPENGL
+            continue;
+#else
+            setWaveformVarsByType.operator()<QtWaveformWidget>();
             break;
+#endif
         case WaveformWidgetType::GLSimpleWaveform:
-            widgetName = GLSimpleWaveformWidget::getWaveformWidgetName();
-            useOpenGl = GLSimpleWaveformWidget::useOpenGl();
-            useOpenGles = GLSimpleWaveformWidget::useOpenGles();
-            useOpenGLShaders = GLSimpleWaveformWidget::useOpenGLShaders();
-            developerOnly = GLSimpleWaveformWidget::developerOnly();
+            setWaveformVarsByType.operator()<GLSimpleWaveformWidget>();
             break;
         case WaveformWidgetType::GLFilteredWaveform:
-            widgetName = GLWaveformWidget::getWaveformWidgetName();
-            useOpenGl = GLWaveformWidget::useOpenGl();
-            useOpenGles = GLWaveformWidget::useOpenGles();
-            useOpenGLShaders = GLWaveformWidget::useOpenGLShaders();
-            developerOnly = GLWaveformWidget::developerOnly();
+            setWaveformVarsByType.operator()<GLWaveformWidget>();
             break;
         case WaveformWidgetType::GLSLFilteredWaveform:
-            widgetName = GLSLFilteredWaveformWidget::getWaveformWidgetName();
-            useOpenGl = GLSLFilteredWaveformWidget::useOpenGl();
-            useOpenGles = GLSLFilteredWaveformWidget::useOpenGles();
-            useOpenGLShaders = GLSLFilteredWaveformWidget::useOpenGLShaders();
-            developerOnly = GLSLFilteredWaveformWidget::developerOnly();
+            setWaveformVarsByType.operator()<GLSLFilteredWaveformWidget>();
             break;
         case WaveformWidgetType::GLSLRGBWaveform:
-            widgetName = GLSLRGBWaveformWidget::getWaveformWidgetName();
-            useOpenGl = GLSLRGBWaveformWidget::useOpenGl();
-            useOpenGles = GLSLRGBWaveformWidget::useOpenGles();
-            useOpenGLShaders = GLSLRGBWaveformWidget::useOpenGLShaders();
-            developerOnly = GLSLRGBWaveformWidget::developerOnly();
+            setWaveformVarsByType.operator()<GLSLRGBWaveformWidget>();
+            break;
+        case WaveformWidgetType::GLSLRGBStackedWaveform:
+            setWaveformVarsByType.operator()<GLSLRGBStackedWaveformWidget>();
             break;
         case WaveformWidgetType::GLVSyncTest:
-            widgetName = GLVSyncTestWidget::getWaveformWidgetName();
-            useOpenGl = GLVSyncTestWidget::useOpenGl();
-            useOpenGles =  GLVSyncTestWidget::useOpenGles();
-            useOpenGLShaders = GLVSyncTestWidget::useOpenGLShaders();
-            developerOnly = GLVSyncTestWidget::developerOnly();
+            setWaveformVarsByType.operator()<GLVSyncTestWidget>();
             break;
         case WaveformWidgetType::GLRGBWaveform:
-            widgetName = GLRGBWaveformWidget::getWaveformWidgetName();
-            useOpenGl = GLRGBWaveformWidget::useOpenGl();
-            useOpenGles =  GLRGBWaveformWidget::useOpenGles();
-            useOpenGLShaders = GLRGBWaveformWidget::useOpenGLShaders();
-            developerOnly = GLRGBWaveformWidget::developerOnly();
+            setWaveformVarsByType.operator()<GLRGBWaveformWidget>();
             break;
         case WaveformWidgetType::QtVSyncTest:
-            widgetName = QtVSyncTestWidget::getWaveformWidgetName();
-            useOpenGl = QtVSyncTestWidget::useOpenGl();
-            useOpenGles =  QtVSyncTestWidget::useOpenGles();
-            useOpenGLShaders = QtVSyncTestWidget::useOpenGLShaders();
-            developerOnly = QtVSyncTestWidget::developerOnly();
+#ifdef MIXXX_USE_QOPENGL
+            continue;
+#else
+            setWaveformVarsByType.operator()<QtVSyncTestWidget>();
+#endif
             break;
         case WaveformWidgetType::QtHSVWaveform:
-            widgetName = QtHSVWaveformWidget::getWaveformWidgetName();
-            useOpenGl = QtHSVWaveformWidget::useOpenGl();
-            useOpenGles = QtHSVWaveformWidget::useOpenGles();
-            useOpenGLShaders = QtHSVWaveformWidget::useOpenGLShaders();
-            developerOnly = QtHSVWaveformWidget::developerOnly();
+#ifdef MIXXX_USE_QOPENGL
+            continue;
+#else
+            setWaveformVarsByType.operator()<QtHSVWaveformWidget>();
             break;
+#endif
         case WaveformWidgetType::QtRGBWaveform:
-            widgetName = QtRGBWaveformWidget::getWaveformWidgetName();
-            useOpenGl = QtRGBWaveformWidget::useOpenGl();
-            useOpenGles = QtRGBWaveformWidget::useOpenGles();
-            useOpenGLShaders = QtRGBWaveformWidget::useOpenGLShaders();
-            developerOnly = QtRGBWaveformWidget::developerOnly();
+#ifdef MIXXX_USE_QOPENGL
+            continue;
+#else
+            setWaveformVarsByType.operator()<QtRGBWaveformWidget>();
             break;
+#endif
+        case WaveformWidgetType::AllShaderRGBWaveform:
+#ifndef MIXXX_USE_QOPENGL
+            continue;
+#else
+            setWaveformVarsByType.operator()<allshader::RGBWaveformWidget>();
+            break;
+#endif
+        case WaveformWidgetType::AllShaderLRRGBWaveform:
+#ifndef MIXXX_USE_QOPENGL
+            continue;
+#else
+            setWaveformVarsByType.operator()<allshader::LRRGBWaveformWidget>();
+            break;
+#endif
+        case WaveformWidgetType::AllShaderFilteredWaveform:
+#ifndef MIXXX_USE_QOPENGL
+            continue;
+#else
+            setWaveformVarsByType.operator()<allshader::FilteredWaveformWidget>();
+            break;
+#endif
+        case WaveformWidgetType::AllShaderSimpleWaveform:
+#ifndef MIXXX_USE_QOPENGL
+            continue;
+#else
+            setWaveformVarsByType.operator()<allshader::SimpleWaveformWidget>();
+            break;
+#endif
+        case WaveformWidgetType::AllShaderHSVWaveform:
+#ifndef MIXXX_USE_QOPENGL
+            continue;
+#else
+            setWaveformVarsByType.operator()<allshader::HSVWaveformWidget>();
+            break;
+#endif
         default:
             DEBUG_ASSERT(!"Unexpected WaveformWidgetType");
             continue;
@@ -908,33 +992,22 @@ void WaveformWidgetFactory::evaluateWidgets() {
                 active = false;
             } else if (useOpenGLShaders && !isOpenGlShaderAvailable()) {
                 active = false;
-            } else {
-                if (useOpenGLShaders) {
-                    widgetName += " " + tr("(GLSL)");
-                } else if (useOpenGl) {
-                    widgetName += " " + tr("(GL)");
-                }
             }
         } else if (isOpenGlesAvailable()) {
             if (useOpenGl && !useOpenGles) {
                 active = false;
             } else if (useOpenGLShaders && !isOpenGlShaderAvailable()) {
                 active = false;
-            } else {
-                if (useOpenGLShaders) {
-                    widgetName += " " + tr("(GLSL ES)");
-                } else if (useOpenGles) {
-                    widgetName += " " + tr("(GL ES)");
-                }
             }
         } else {
-            // No sufficient GL supptor
+            // No sufficient GL support
             if (useOpenGles || useOpenGl || useOpenGLShaders) {
                 active = false;
             }
         }
 
-        if (developerOnly && !CmdlineArgs::Instance().getDeveloper()) {
+        if (category == WaveformWidgetCategory::DeveloperOnly &&
+                !CmdlineArgs::Instance().getDeveloper()) {
             active = false;
         }
 
@@ -942,7 +1015,7 @@ void WaveformWidgetFactory::evaluateWidgets() {
             // add new handle for each available widget type
             WaveformWidgetAbstractHandle handle;
             handle.m_displayString = widgetName;
-            handle.m_type = (WaveformWidgetType::Type)type;
+            handle.m_type = static_cast<WaveformWidgetType::Type>(type);
 
             m_waveformWidgetHandles.push_back(handle);
         }
@@ -967,12 +1040,6 @@ WaveformWidgetAbstract* WaveformWidgetFactory::createWaveformWidget(
         case WaveformWidgetType::RGBWaveform:
             widget = new RGBWaveformWidget(viewer->getGroup(), viewer);
             break;
-        case WaveformWidgetType::QtSimpleWaveform:
-            widget = new QtSimpleWaveformWidget(viewer->getGroup(), viewer);
-            break;
-        case WaveformWidgetType::QtWaveform:
-            widget = new QtWaveformWidget(viewer->getGroup(), viewer);
-            break;
         case WaveformWidgetType::GLSimpleWaveform:
             widget = new GLSimpleWaveformWidget(viewer->getGroup(), viewer);
             break;
@@ -988,9 +1055,35 @@ WaveformWidgetAbstract* WaveformWidgetFactory::createWaveformWidget(
         case WaveformWidgetType::GLSLRGBWaveform:
             widget = new GLSLRGBWaveformWidget(viewer->getGroup(), viewer);
             break;
+        case WaveformWidgetType::GLSLRGBStackedWaveform:
+            widget = new GLSLRGBStackedWaveformWidget(viewer->getGroup(), viewer);
+            break;
         case WaveformWidgetType::GLVSyncTest:
             widget = new GLVSyncTestWidget(viewer->getGroup(), viewer);
             break;
+#ifdef MIXXX_USE_QOPENGL
+        case WaveformWidgetType::AllShaderRGBWaveform:
+            widget = new allshader::RGBWaveformWidget(viewer->getGroup(), viewer);
+            break;
+        case WaveformWidgetType::AllShaderLRRGBWaveform:
+            widget = new allshader::LRRGBWaveformWidget(viewer->getGroup(), viewer);
+            break;
+        case WaveformWidgetType::AllShaderFilteredWaveform:
+            widget = new allshader::FilteredWaveformWidget(viewer->getGroup(), viewer);
+            break;
+        case WaveformWidgetType::AllShaderSimpleWaveform:
+            widget = new allshader::SimpleWaveformWidget(viewer->getGroup(), viewer);
+            break;
+        case WaveformWidgetType::AllShaderHSVWaveform:
+            widget = new allshader::HSVWaveformWidget(viewer->getGroup(), viewer);
+            break;
+#else
+        case WaveformWidgetType::QtSimpleWaveform:
+            widget = new QtSimpleWaveformWidget(viewer->getGroup(), viewer);
+            break;
+        case WaveformWidgetType::QtWaveform:
+            widget = new QtWaveformWidget(viewer->getGroup(), viewer);
+            break;
         case WaveformWidgetType::QtVSyncTest:
             widget = new QtVSyncTestWidget(viewer->getGroup(), viewer);
             break;
@@ -1000,6 +1093,7 @@ WaveformWidgetAbstract* WaveformWidgetFactory::createWaveformWidget(
         case WaveformWidgetType::QtRGBWaveform:
             widget = new QtRGBWaveformWidget(viewer->getGroup(), viewer);
             break;
+#endif
         default:
         //case WaveformWidgetType::SoftwareSimpleWaveform: TODO: (vrince)
         //case WaveformWidgetType::EmptyWaveform:
@@ -1032,13 +1126,27 @@ int WaveformWidgetFactory::findIndexOf(WWaveformViewer* viewer) const {
 }
 
 void WaveformWidgetFactory::startVSync(GuiTick* pGuiTick, VisualsManager* pVisualsManager) {
+    const auto vSyncMode = static_cast<VSyncThread::VSyncMode>(
+            m_config->getValue(ConfigKey("[Waveform]", "VSync"), 0));
+
     m_pGuiTick = pGuiTick;
     m_pVisualsManager = pVisualsManager;
-    m_vsyncThread = new VSyncThread(this);
+    m_vsyncThread = new VSyncThread(this, vSyncMode);
     m_vsyncThread->setObjectName(QStringLiteral("VSync"));
-    m_vsyncThread->setVSyncType(m_vSyncType);
     m_vsyncThread->setSyncIntervalTimeMicros(static_cast<int>(1e6 / m_frameRate));
 
+#ifdef MIXXX_USE_QOPENGL
+    if (m_vsyncThread->vsyncMode() == VSyncThread::ST_PLL) {
+        WGLWidget* widget = SharedGLContext::getWidget();
+        connect(widget->getOpenGLWindow(),
+                &QOpenGLWindow::frameSwapped,
+                this,
+                &WaveformWidgetFactory::slotFrameSwapped,
+                Qt::DirectConnection);
+        widget->show();
+    }
+#endif
+
     connect(m_vsyncThread,
             &VSyncThread::vsyncRender,
             this,
@@ -1047,11 +1155,15 @@ void WaveformWidgetFactory::startVSync(GuiTick* pGuiTick, VisualsManager* pVisua
             &VSyncThread::vsyncSwap,
             this,
             &WaveformWidgetFactory::swap);
+    connect(m_vsyncThread,
+            &VSyncThread::vsyncSwapAndRender,
+            this,
+            &WaveformWidgetFactory::swapAndRender);
 
     m_vsyncThread->start(QThread::NormalPriority);
 }
 
-void WaveformWidgetFactory::getAvailableVSyncTypes(QList<QPair<int, QString > >* pList) {
+void WaveformWidgetFactory::getAvailableVSyncTypes(QList<QPair<int, QString>>* pList) {
     m_vsyncThread->getAvailableVSyncTypes(pList);
 }
 
@@ -1066,10 +1178,86 @@ WaveformWidgetType::Type WaveformWidgetFactory::findTypeFromHandleIndex(int inde
 int WaveformWidgetFactory::findHandleIndexFromType(WaveformWidgetType::Type type) {
     int index = -1;
     for (int i = 0; i < m_waveformWidgetHandles.size(); i++) {
-        WaveformWidgetAbstractHandle& handle = m_waveformWidgetHandles[i];
+        const WaveformWidgetAbstractHandle& handle = m_waveformWidgetHandles[i];
         if (handle.m_type == type) {
             index = i;
         }
     }
     return index;
 }
+
+QString WaveformWidgetFactory::getDisplayNameFromType(WaveformWidgetType::Type type) {
+    for (const auto& handle : std::as_const(m_waveformWidgetHandles)) {
+        if (handle.m_type == type) {
+            return handle.m_displayString;
+        }
+    }
+    return QString();
+}
+
+template<typename WaveformT>
+QString WaveformWidgetFactory::buildWidgetDisplayName() const {
+    const bool isLegacy = WaveformT::category() == WaveformWidgetCategory::Legacy;
+    QStringList extras;
+    if (isLegacy) {
+        extras.push_back(tr("legacy"));
+    }
+    if (isOpenGlesAvailable()) {
+        if (WaveformT::useOpenGLShaders()) {
+            extras.push_back(QStringLiteral("GLSL ES"));
+        } else if (WaveformT::useOpenGles()) {
+            extras.push_back(QStringLiteral("GLES"));
+        }
+    } else if (isOpenGlAvailable()) {
+        if (WaveformT::useOpenGLShaders()) {
+            extras.push_back(QStringLiteral("GLSL"));
+        } else if (WaveformT::useOpenGl()) {
+            extras.push_back(QStringLiteral("GL"));
+        }
+    }
+    QString name = WaveformT::getWaveformWidgetName();
+    if (extras.isEmpty()) {
+        return name;
+    }
+    return QStringLiteral("%1 (%2)").arg(name, extras.join(QStringLiteral(", ")));
+}
+
+// static
+QSurfaceFormat WaveformWidgetFactory::getSurfaceFormat(UserSettingsPointer config) {
+    // The first call should pass the config to set the vsync mode. Subsequent
+    // calls will use the value as set on the first call.
+    static const auto vsyncMode = config->getValue(ConfigKey("[Waveform]", "VSync"), 0);
+
+    QSurfaceFormat format;
+    // Qt5 requires at least OpenGL 2.1 or OpenGL ES 2.0, default is 2.0
+    // format.setVersion(2, 1);
+    // Core and Compatibility contexts have been introduced in openGL 3.2
+    // From 3.0 to 3.1 we have implicit the Core profile and Before 3.0 we have the
+    // Compatibility profile
+    // format.setProfile(QSurfaceFormat::CoreProfile);
+
+    // setSwapInterval sets the application preferred swap interval
+    // in minimum number of video frames that are displayed before a buffer swap occurs
+    // - 0 will turn the vertical refresh syncing off
+    // - 1 (default) means swapping after drawig a video frame to the buffer
+    // - n means swapping after drawing n video frames to the buffer
+    //
+    // The vertical sync setting requested by the OpenGL application, can be overwritten
+    // if a user changes the "Wait for vertical refresh" setting in AMD graphic drivers
+    // for Windows.
+
+#if defined(__APPLE__)
+    // On OS X, syncing to vsync has good performance FPS-wise and
+    // eliminates tearing. (This is an comment from pre QOpenGLWindow times)
+    format.setSwapInterval(1);
+    (void)vsyncMode;
+#else
+    // It seems that on Windows (at least for some AMD drivers), the setting 1 is not
+    // not properly handled. We saw frame rates divided by exact integers, like it should
+    // be with values >1 (see https://github.com/mixxxdj/mixxx/issues/11617)
+    // Reported as https://bugreports.qt.io/browse/QTBUG-114882
+    // On Linux, horrible FPS were seen with "VSync off" before switching to QOpenGLWindow too
+    format.setSwapInterval(vsyncMode == VSyncThread::ST_PLL ? 1 : 0);
+#endif
+    return format;
+}
