@@ -1,6 +1,7 @@
 #pragma once
 
 #include <QObject>
+#include <QSurfaceFormat>
 #include <QVector>
 #include <vector>
 
@@ -8,11 +9,10 @@
 #include "skin/legacy/skincontext.h"
 #include "util/performancetimer.h"
 #include "util/singleton.h"
-#include "waveform/waveform.h"
 #include "waveform/widgets/waveformwidgettype.h"
 
-class WVuMeter;
-class WVuMeterGL;
+class WVuMeterLegacy;
+class WVuMeterBase;
 class WWaveformViewer;
 class WaveformWidgetAbstract;
 class VSyncThread;
@@ -98,6 +98,9 @@ class WaveformWidgetFactory : public QObject, public Singleton<WaveformWidgetFac
     }
     int findHandleIndexFromType(WaveformWidgetType::Type type);
 
+    /// Returns the desired surface format for the OpenGLWindow
+    static QSurfaceFormat getSurfaceFormat(UserSettingsPointer config = nullptr);
+
   protected:
     bool setWidgetType(
             WaveformWidgetType::Type type,
@@ -120,15 +123,13 @@ class WaveformWidgetFactory : public QObject, public Singleton<WaveformWidgetFac
     int isOverviewNormalized() const { return m_overviewNormalized;}
 
     const QVector<WaveformWidgetAbstractHandle> getAvailableTypes() const { return m_waveformWidgetHandles;}
-    void getAvailableVSyncTypes(QList<QPair<int, QString > >* list);
+    void getAvailableVSyncTypes(QList<QPair<int, QString>>* list);
     void destroyWidgets();
 
-    void addVuMeter(WVuMeter* pWidget);
-    void addVuMeter(WVuMeterGL* pWidget);
+    void addVuMeter(WVuMeterLegacy* pWidget);
+    void addVuMeter(WVuMeterBase* pWidget);
 
     void startVSync(GuiTick* pGuiTick, VisualsManager* pVisualsManager);
-    void setVSyncType(int vsType);
-    int getVSyncType();
 
     void setPlayMarkerPosition(double position);
     double getPlayMarkerPosition() const { return m_playMarkerPosition; }
@@ -157,13 +158,21 @@ class WaveformWidgetFactory : public QObject, public Singleton<WaveformWidgetFac
   private slots:
     void render();
     void swap();
+    void swapAndRender();
+    void slotFrameSwapped();
 
   private:
+    void renderSelf();
+    void swapSelf();
+
     void evaluateWidgets();
+    template<typename WaveformT>
+    QString buildWidgetDisplayName() const;
     WaveformWidgetAbstract* createWaveformWidget(WaveformWidgetType::Type type, WWaveformViewer* viewer);
     int findIndexOf(WWaveformViewer* viewer) const;
 
     WaveformWidgetType::Type findTypeFromHandleIndex(int index);
+    QString getDisplayNameFromType(WaveformWidgetType::Type type);
 
     //All type of available widgets
 
