@@ -1,10 +1,8 @@
 #pragma once
 
-#include <QGLWidget>
+#include "glwaveformwidgetabstract.h"
 
-#include "waveformwidgetabstract.h"
-
-class QtWaveformWidget : public QGLWidget, public WaveformWidgetAbstract {
+class QtWaveformWidget : public GLWaveformWidgetAbstract {
     Q_OBJECT
   public:
     QtWaveformWidget(const QString& group, QWidget* parent);
@@ -16,7 +14,9 @@ class QtWaveformWidget : public QGLWidget, public WaveformWidgetAbstract {
     static inline bool useOpenGl() { return true; }
     static inline bool useOpenGles() { return true; }
     static inline bool useOpenGLShaders() { return false; }
-    static inline bool developerOnly() { return false; }
+    static inline WaveformWidgetCategory category() {
+        return WaveformWidgetCategory::Legacy;
+    }
 
   protected:
     virtual void castToQWidget();
