@@ -2,6 +2,8 @@
 
 #include <QString>
 
+class QEvent;
+
 /// Mixin to mark a widget as a drop target for tracks.
 ///
 /// This class is *not* derived from QObject (inheriting from 2 QObject classes
@@ -20,6 +22,11 @@ class TrackDropTarget {
         emit trackDropped(filename, group); // clazy:exclude=incorrect-emit
     }
 
+    virtual bool handleDragAndDropEventFromWindow(QEvent* event) {
+        Q_UNUSED(event);
+        return false;
+    }
+
   signals:
     virtual void trackDropped(const QString& filename, const QString& group) = 0;
     virtual void cloneDeck(const QString& sourceGroup, const QString& targetGroup) = 0;
