@@ -1,9 +1,8 @@
 #pragma once
 
+#include <QByteArray>
 #include <QLabel>
-#include <QMouseEvent>
 #include <QPixmap>
-#include <QWidget>
 
 #include "track/track_decl.h"
 #include "util/parented_ptr.h"
@@ -16,15 +15,13 @@ class CoverInfoRelative;
 class WCoverArtLabel : public QLabel {
     Q_OBJECT
   public:
-    explicit WCoverArtLabel(QWidget* parent = nullptr);
+    explicit WCoverArtLabel(QWidget* parent = nullptr, WCoverArtMenu* m_pWCoverArtMenu = nullptr);
+
     ~WCoverArtLabel() override; // Verifies that the base destructor is virtual
 
     void setCoverArt(const CoverInfo& coverInfo, const QPixmap& px);
     void loadTrack(TrackPointer pTrack);
-
-  signals:
-    void coverInfoSelected(const CoverInfoRelative& coverInfo);
-    void reloadCoverArt();
+    void loadData(const QByteArray& data);
 
   protected:
     void mousePressEvent(QMouseEvent* event) override;
@@ -34,11 +31,14 @@ class WCoverArtLabel : public QLabel {
       void slotCoverMenu(const QPoint& pos);
 
   private:
-    const parented_ptr<WCoverArtMenu> m_pCoverMenu;
+    WCoverArtMenu* m_pCoverMenu;
+
     const parented_ptr<DlgCoverArtFullSize> m_pDlgFullSize;
 
     const QPixmap m_defaultCover;
 
+    QByteArray m_Data;
+
     TrackPointer m_pLoadedTrack;
 
     QPixmap m_loadedCover;
