@@ -1,11 +1,11 @@
 #pragma once
 
-#include <QAction>
 #include <QMenu>
-#include <QWidget>
-#include <QPixmap>
 
 #include "library/coverart.h"
+#include "library/export/coverartcopyworker.h"
+
+class QAction;
 
 // This class implements a context-menu with all CoverArt user actions. Callers
 // must call setCoverArt before calling exec or popup. This class does
@@ -27,6 +27,12 @@ class WCoverArtMenu : public QMenu {
   private slots:
     void slotChange();
     void slotUnset();
+    void slotStarted();
+    void slotAskOverwrite(const QString& coverArtAbsolutePath,
+            std::promise<CoverArtCopyWorker::OverwriteAnswer>* promise);
+    void slotCoverArtCopyFailed(const QString& errorMessage);
+    void slotCoverArtUpdated(const CoverInfoRelative& coverInfo);
+    void slotFinished();
 
   private:
     void createActions();
@@ -36,4 +42,7 @@ class WCoverArtMenu : public QMenu {
     QAction* m_pUnset;
 
     CoverInfo m_coverInfo;
+
+    QScopedPointer<CoverArtCopyWorker> m_worker;
+    bool m_isWorkerRunning;
 };
