@@ -1,20 +1,20 @@
 #include "widget/wlibrarysidebar.h"
 
-#include <QFileInfo>
 #include <QHeaderView>
-#include <QMimeData>
 #include <QUrl>
 #include <QtDebug>
 
 #include "library/sidebarmodel.h"
 #include "moc_wlibrarysidebar.cpp"
+#include "util/defs.h"
 #include "util/dnd.h"
 
-const int expand_time = 250;
+constexpr int expand_time = 250;
 
 WLibrarySidebar::WLibrarySidebar(QWidget* parent)
         : QTreeView(parent),
           WBaseWidget(this) {
+    qRegisterMetaType<FocusWidget>("FocusWidget");
     //Set some properties
     setHeaderHidden(true);
     setSelectionMode(QAbstractItemView::SingleSelection);
@@ -36,6 +36,8 @@ void WLibrarySidebar::contextMenuEvent(QContextMenuEvent *event) {
     if (!clickedIndex.isValid()) {
         return;
     }
+    // Use this instead of setCurrentIndex() to keep current selection
+    selectionModel()->setCurrentIndex(clickedIndex, QItemSelectionModel::NoUpdate);
     event->accept();
     emit rightClicked(event->globalPos(), clickedIndex);
     //}
@@ -49,9 +51,9 @@ void WLibrarySidebar::dragEnterEvent(QDragEnterEvent * event) {
         // drag so for now we accept all drags. Since almost every
         // LibraryFeature accepts all files in the drop and accepts playlist
         // drops we default to those flags to DragAndDropHelper.
-        QList<TrackFile> files = DragAndDropHelper::supportedTracksFromUrls(
+        QList<mixxx::FileInfo> fileInfos = DragAndDropHelper::supportedTracksFromUrls(
                 event->mimeData()->urls(), false, true);
-        if (!files.isEmpty()) {
+        if (!fileInfos.isEmpty()) {
             event->acceptProposedAction();
             return;
         }
@@ -64,7 +66,11 @@ void WLibrarySidebar::dragEnterEvent(QDragEnterEvent * event) {
 void WLibrarySidebar::dragMoveEvent(QDragMoveEvent * event) {
     //qDebug() << "dragMoveEvent" << event->mimeData()->formats();
     // Start a timer to auto-expand sections the user hovers on.
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+    QPoint pos = event->position().toPoint();
+#else
     QPoint pos = event->pos();
+#endif
     QModelIndex index = indexAt(pos);
     if (m_hoverIndex != index) {
         m_expandTimer.stop();
@@ -87,7 +93,12 @@ void WLibrarySidebar::dragMoveEvent(QDragMoveEvent * event) {
             if (sidebarModel) {
                 accepted = false;
                 for (const QUrl& url : urls) {
-                    QModelIndex destIndex = indexAt(event->pos());
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+                    QPoint pos = event->position().toPoint();
+#else
+                    QPoint pos = event->pos();
+#endif
+                    QModelIndex destIndex = indexAt(pos);
                     if (sidebarModel->dragMoveAccept(destIndex, url)) {
                         // We only need one URL to be valid for us
                         // to accept the whole drag...
@@ -141,7 +152,13 @@ void WLibrarySidebar::dropEvent(QDropEvent * event) {
             //eg. dragging a track from Windows Explorer onto the sidebar
             SidebarModel* sidebarModel = qobject_cast<SidebarModel*>(model());
             if (sidebarModel) {
-                QModelIndex destIndex = indexAt(event->pos());
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+                QPoint pos = event->position().toPoint();
+#else
+                QPoint pos = event->pos();
+#endif
+
+                QModelIndex destIndex = indexAt(pos);
                 // event->source() will return NULL if something is dropped from
                 // a different application
                 const QList<QUrl> urls = event->mimeData()->urls();
@@ -159,22 +176,19 @@ void WLibrarySidebar::dropEvent(QDropEvent * event) {
     }
 }
 
-
 void WLibrarySidebar::toggleSelectedItem() {
-    QModelIndexList selectedIndices = this->selectionModel()->selectedRows();
-    if (selectedIndices.size() > 0) {
-        QModelIndex index = selectedIndices.at(0);
+    QModelIndex index = selectedIndex();
+    if (index.isValid()) {
         // Activate the item so its content shows in the main library.
-        emit pressed(index);
+        emit clicked(index);
         // Expand or collapse the item as necessary.
         setExpanded(index, !isExpanded(index));
     }
 }
 
 bool WLibrarySidebar::isLeafNodeSelected() {
-    QModelIndexList selectedIndices = this->selectionModel()->selectedRows();
-    if (selectedIndices.size() > 0) {
-        QModelIndex index = selectedIndices.at(0);
+    QModelIndex index = selectedIndex();
+    if (index.isValid()) {
         if(!index.model()->hasChildren(index)) {
             return true;
         }
@@ -186,49 +200,210 @@ bool WLibrarySidebar::isLeafNodeSelected() {
     return false;
 }
 
+bool WLibrarySidebar::isChildIndexSelected(const QModelIndex& index) {
+    // qDebug() << "WLibrarySidebar::isChildIndexSelected" << index;
+    QModelIndex selIndex = selectedIndex();
+    if (!selIndex.isValid()) {
+        return false;
+    }
+    SidebarModel* sidebarModel = qobject_cast<SidebarModel*>(model());
+    VERIFY_OR_DEBUG_ASSERT(sidebarModel) {
+        // qDebug() << " >> model() is not SidebarModel";
+        return false;
+    }
+    QModelIndex translated = sidebarModel->translateChildIndex(index);
+    if (!translated.isValid()) {
+        // qDebug() << " >> index can't be translated";
+        return false;
+    }
+    return translated == selIndex;
+}
+
+bool WLibrarySidebar::isFeatureRootIndexSelected(LibraryFeature* pFeature) {
+    // qDebug() << "WLibrarySidebar::isFeatureRootIndexSelected";
+    QModelIndex selIndex = selectedIndex();
+    if (!selIndex.isValid()) {
+        return false;
+    }
+    SidebarModel* sidebarModel = qobject_cast<SidebarModel*>(model());
+    VERIFY_OR_DEBUG_ASSERT(sidebarModel) {
+        return false;
+    }
+    const QModelIndex rootIndex = sidebarModel->getFeatureRootIndex(pFeature);
+    return rootIndex == selIndex;
+}
+
+/// Invoked by actual keypresses (requires widget focus) and emulated keypresses
+/// sent by LibraryControl
 void WLibrarySidebar::keyPressEvent(QKeyEvent* event) {
-    if (event->key() == Qt::Key_Return) {
+    // TODO(XXX) Should first keyEvent ensure previous item has focus? I.e. if the selected
+    // item is not focused, require second press to perform the desired action.
+
+    // make the selected item the navigation starting point
+    focusSelectedIndex();
+
+    switch (event->key()) {
+    case Qt::Key_Return:
+        focusSelectedIndex();
         toggleSelectedItem();
         return;
-    } else if (event->key() == Qt::Key_Down ||
-            event->key() == Qt::Key_Up ||
-            event->key() == Qt::Key_PageDown ||
-            event->key() == Qt::Key_PageUp ||
-            event->key() == Qt::Key_End ||
-            event->key() == Qt::Key_Home) {
+    case Qt::Key_Down:
+    case Qt::Key_Up:
+    case Qt::Key_PageDown:
+    case Qt::Key_PageUp:
+    case Qt::Key_End:
+    case Qt::Key_Home: {
         // Let the tree view move up and down for us.
         QTreeView::keyPressEvent(event);
-        // But force the index to be activated/clicked after the selection
-        // changes. (Saves you from having to push "enter" after changing the
-        // selection.)
-        QModelIndexList selectedIndices = selectionModel()->selectedRows();
-        //Note: have to get the selected indices _after_ QTreeView::keyPressEvent()
-        if (selectedIndices.size() > 0) {
-            QModelIndex index = selectedIndices.at(0);
-            emit pressed(index);
+        // After the selection changed force-activate (click) the newly selected
+        // item to save us from having to push "Enter".
+        QModelIndex selIndex = selectedIndex();
+        if (!selIndex.isValid()) {
+            return;
+        }
+        // Ensure the new selection is visible even if it was already selected/
+        // focused, like when the topmost item was selected but out of sight and
+        // we pressed Up, Home or PageUp.
+        scrollTo(selIndex);
+        emit pressed(selIndex);
+        return;
+    }
+    case Qt::Key_Left: {
+        // If an expanded item is selected let QTreeView collapse it
+        QModelIndex selIndex = selectedIndex();
+        if (!selIndex.isValid()) {
+            return;
+        }
+        // collapse knot
+        if (isExpanded(selIndex)) {
+            QTreeView::keyPressEvent(event);
+            return;
+        }
+        // Else jump to its parent and activate it
+        QModelIndex parentIndex = selIndex.parent();
+        if (parentIndex.isValid()) {
+            selectIndex(parentIndex);
+            emit pressed(parentIndex);
         }
         return;
-    //} else if (event->key() == Qt::Key_Enter && (event->modifiers() & Qt::AltModifier)) {
-    //    // encoder click via "GoToItem"
-    //    qDebug() << "GoToItem";
-    //    TODO(xxx) decide what todo here instead of in librarycontrol
     }
+    case Qt::Key_Escape:
+        // Focus tracks table
+        emit setLibraryFocus(FocusWidget::TracksTable);
+        return;
+    case kRenameSidebarItemShortcutKey: { // F2
+        // Rename crate or playlist (internal, external, history)
+        QModelIndex selIndex = selectedIndex();
+        if (!selIndex.isValid()) {
+            return;
+        }
+        if (isExpanded(selIndex)) {
+            // Root views / knots can not be renamed
+            return;
+        }
+        emit renameItem(selIndex);
+        return;
+    }
+    case kHideRemoveShortcutKey: { // Del (macOS: Cmd+Backspace)
+        // Delete crate or playlist (internal, external, history)
+        if (event->modifiers() != kHideRemoveShortcutModifier) {
+            return;
+        }
+        QModelIndex selIndex = selectedIndex();
+        if (!selIndex.isValid()) {
+            return;
+        }
+        emit deleteItem(selIndex);
+        return;
+    }
+    default:
+        QTreeView::keyPressEvent(event);
+    }
+}
 
-    // Fall through to default handler.
-    QTreeView::keyPressEvent(event);
+void WLibrarySidebar::mousePressEvent(QMouseEvent* event) {
+    // handle right click only in contextMenuEvent() to not select the clicked index
+    if (event->buttons().testFlag(Qt::RightButton)) {
+        return;
+    }
+    QTreeView::mousePressEvent(event);
+}
+
+void WLibrarySidebar::focusInEvent(QFocusEvent* event) {
+    // Clear the current index, i.e. remove the focus indicator
+    selectionModel()->clearCurrentIndex();
+    QTreeView::focusInEvent(event);
 }
 
 void WLibrarySidebar::selectIndex(const QModelIndex& index) {
+    //qDebug() << "WLibrarySidebar::selectIndex" << index;
+    if (!index.isValid()) {
+        return;
+    }
     auto* pModel = new QItemSelectionModel(model());
     pModel->select(index, QItemSelectionModel::Select);
-    setSelectionModel(pModel);
-
+    if (selectionModel()) {
+        selectionModel()->deleteLater();
+    }
     if (index.parent().isValid()) {
         expand(index.parent());
     }
+    setSelectionModel(pModel);
+    setCurrentIndex(index);
     scrollTo(index);
 }
 
+/// Selects a child index from a feature and ensures visibility
+void WLibrarySidebar::selectChildIndex(const QModelIndex& index, bool selectItem) {
+    SidebarModel* sidebarModel = qobject_cast<SidebarModel*>(model());
+    VERIFY_OR_DEBUG_ASSERT(sidebarModel) {
+        qDebug() << "model() is not SidebarModel";
+        return;
+    }
+    QModelIndex translated = sidebarModel->translateChildIndex(index);
+    if (!translated.isValid()) {
+        return;
+    }
+
+    if (selectItem) {
+        auto* pModel = new QItemSelectionModel(sidebarModel);
+        pModel->select(translated, QItemSelectionModel::Select);
+        if (selectionModel()) {
+            selectionModel()->deleteLater();
+        }
+        setSelectionModel(pModel);
+        setCurrentIndex(translated);
+    }
+
+    QModelIndex parentIndex = translated.parent();
+    while (parentIndex.isValid()) {
+        expand(parentIndex);
+        parentIndex = parentIndex.parent();
+    }
+    scrollTo(translated, EnsureVisible);
+}
+
+QModelIndex WLibrarySidebar::selectedIndex() {
+    QModelIndexList selectedIndices = selectionModel()->selectedRows();
+    if (selectedIndices.isEmpty()) {
+        return QModelIndex();
+    }
+    QModelIndex selIndex = selectedIndices.first();
+    DEBUG_ASSERT(selIndex.isValid());
+    return selIndex;
+}
+
+/// Refocus the selected item after right-click
+void WLibrarySidebar::focusSelectedIndex() {
+    // After the context menu was activated (and closed, with or without clicking
+    // an action), the currentIndex is the right-clicked item.
+    // If if the currentIndex is not selected, make the selection the currentIndex
+    QModelIndex selIndex = selectedIndex();
+    if (selIndex.isValid() && selIndex != selectionModel()->currentIndex()) {
+        setCurrentIndex(selIndex);
+    }
+}
+
 bool WLibrarySidebar::event(QEvent* pEvent) {
     if (pEvent->type() == QEvent::ToolTip) {
         updateTooltip();
