@@ -1,35 +1,18 @@
-//
-// C++ Implementation: woverview
-//
-// Description:
-//
-//
-// Author: Tue Haste Andersen <haste@diku.dk>, (C) 2003
-//
-// Copyright: See COPYING file that comes with this distribution
-//
-//
-
 #include "woverview.h"
 
 #include <QBrush>
-#include <QMimeData>
 #include <QMouseEvent>
 #include <QPaintEvent>
 #include <QPainter>
-#include <QUrl>
-#include <QtDebug>
+#include <QVBoxLayout>
 
 #include "analyzer/analyzerprogress.h"
-#include "control/controlobject.h"
 #include "control/controlproxy.h"
 #include "engine/engine.h"
 #include "mixer/playermanager.h"
 #include "moc_woverview.cpp"
 #include "preferences/colorpalettesettings.h"
 #include "track/track.h"
-#include "util/color/color.h"
-#include "util/compatibility.h"
 #include "util/dnd.h"
 #include "util/duration.h"
 #include "util/math.h"
@@ -86,7 +69,9 @@ WOverview::WOverview(
     m_pPassthroughControl =
             new ControlProxy(m_group, "passthrough", this, ControlFlag::NoAssertIfMissing);
     m_pPassthroughControl->connectValueChanged(this, &WOverview::onPassthroughChange);
-    m_bPassthroughEnabled = static_cast<bool>(m_pPassthroughControl->get());
+    m_bPassthroughEnabled = m_pPassthroughControl->toBool();
+
+    m_pPassthroughLabel = new QLabel(this);
 
     setAcceptDrops(true);
 
@@ -96,18 +81,6 @@ WOverview::WOverview(
             this, &WOverview::onTrackAnalyzerProgress);
 
     connect(m_pCueMenuPopup.get(), &WCueMenuPopup::aboutToHide, this, &WOverview::slotCueMenuPopupAboutToHide);
-
-    m_pPassthroughLabel = new QLabel(this);
-    m_pPassthroughLabel->setObjectName("PassthroughLabel");
-    m_pPassthroughLabel->setAlignment(Qt::AlignLeft | Qt::AlignVCenter);
-    // Shown on the overview waveform when vinyl passthrough is enabled
-    m_pPassthroughLabel->setText(tr("Passthrough"));
-    m_pPassthroughLabel->hide();
-    QVBoxLayout *pPassthroughLayout = new QVBoxLayout(this);
-    pPassthroughLayout->setContentsMargins(0,0,0,0);
-    pPassthroughLayout->setAlignment(Qt::AlignLeft | Qt::AlignVCenter);
-    pPassthroughLayout->addWidget(m_pPassthroughLabel);
-    setLayout(pPassthroughLayout);
 }
 
 void WOverview::setup(const QDomNode& node, const SkinContext& context) {
@@ -151,7 +124,7 @@ void WOverview::setup(const QDomNode& node, const SkinContext& context) {
     m_endOfTrackColor = QColor(200, 25, 20);
     const QString endOfTrackColorName = context.selectString(node, "EndOfTrackColor");
     if (!endOfTrackColorName.isNull()) {
-        m_endOfTrackColor.setNamedColor(endOfTrackColorName);
+        m_endOfTrackColor = QColor(endOfTrackColorName);
         m_endOfTrackColor = WSkinColor::getCorrectColor(m_endOfTrackColor);
     }
 
@@ -162,16 +135,9 @@ void WOverview::setup(const QDomNode& node, const SkinContext& context) {
     auto colorPalette = colorPaletteSettings.getHotcueColorPalette();
     m_pCueMenuPopup->setColorPalette(colorPalette);
 
-    for (const auto& pMark: m_marks) {
-        if (pMark->isValid()) {
-            pMark->connectSamplePositionChanged(this,
-                    &WOverview::onMarkChanged);
-        }
-        if (pMark->hasVisible()) {
-            pMark->connectVisibleChanged(this,
-                    &WOverview::onMarkChanged);
-        }
-    }
+    m_marks.connectSamplePositionChanged(this, &WOverview::onMarkChanged);
+    m_marks.connectSampleEndPositionChanged(this, &WOverview::onMarkChanged);
+    m_marks.connectVisibleChanged(this, &WOverview::onMarkChanged);
 
     QDomNode child = node.firstChild();
     while (!child.isNull()) {
@@ -199,6 +165,27 @@ void WOverview::setup(const QDomNode& node, const SkinContext& context) {
         child = child.nextSibling();
     }
 
+    DEBUG_ASSERT(m_pPassthroughLabel != nullptr);
+    m_pPassthroughLabel->setAlignment(Qt::AlignLeft | Qt::AlignVCenter);
+    // Shown on the overview waveform when vinyl passthrough is enabled
+    m_pPassthroughLabel->setText(tr("Passthrough"));
+    m_pPassthroughLabel->setStyleSheet(
+            QStringLiteral("QLabel{"
+                           "font-family: 'Open Sans';"
+                           "font-size: %1px;"
+                           "font-weight: bold;"
+                           "color: %2;"
+                           "margin-left };")
+                    .arg(QString::number(int(m_iLabelFontSize * 1.5)),
+                            QColor(m_signalColors.getPassthroughLabelColor())
+                                    .name(QColor::HexRgb)));
+    m_pPassthroughLabel->hide();
+    QVBoxLayout* pPassthroughLayout = new QVBoxLayout(this);
+    pPassthroughLayout->setContentsMargins(m_iLabelFontSize, 0, 0, 0); // l, t, r, b
+    pPassthroughLayout->setAlignment(Qt::AlignLeft | Qt::AlignVCenter);
+    pPassthroughLayout->addWidget(m_pPassthroughLabel);
+    setLayout(pPassthroughLayout);
+
     QString orientationString = context.selectString(node, "Orientation").toLower();
     if (orientationString == "vertical") {
         m_orientation = Qt::Vertical;
@@ -208,8 +195,8 @@ void WOverview::setup(const QDomNode& node, const SkinContext& context) {
 
     m_bShowCueTimes = context.selectBool(node, "ShowCueTimes", true);
 
-    //qDebug() << "WOverview : m_marks" << m_marks.size();
-    //qDebug() << "WOverview : m_markRanges" << m_markRanges.size();
+    // qDebug() << "WOverview : std::as_const(m_marks)" << m_marks.size();
+    // qDebug() << "WOverview : m_markRanges" << m_markRanges.size();
     if (!m_connections.isEmpty()) {
         ControlParameterWidgetConnection* defaultConnection = m_connections.at(0);
         if (defaultConnection) {
@@ -402,7 +389,6 @@ void WOverview::onPassthroughChange(double v) {
 }
 
 void WOverview::updateCues(const QList<CuePointer> &loadedCues) {
-    m_marksToRender.clear();
     for (const CuePointer& currentCue : loadedCues) {
         const WaveformMarkPointer pMark = m_marks.getHotCueMark(currentCue->getHotCue());
 
@@ -414,7 +400,9 @@ void WOverview::updateCues(const QList<CuePointer> &loadedCues) {
             }
 
             int hotcueNumber = currentCue->getHotCue();
-            if (currentCue->getType() == mixxx::CueType::HotCue && hotcueNumber != Cue::kNoHotCue) {
+            if ((currentCue->getType() == mixxx::CueType::HotCue ||
+                        currentCue->getType() == mixxx::CueType::Loop) &&
+                    hotcueNumber != Cue::kNoHotCue) {
                 // Prepend the hotcue number to hotcues' labels
                 QString newLabel = currentCue->getLabel();
                 if (newLabel.isEmpty()) {
@@ -430,18 +418,7 @@ void WOverview::updateCues(const QList<CuePointer> &loadedCues) {
         }
     }
 
-    for (const auto& pMark : m_marks) {
-        if (pMark->isValid() && pMark->isVisible()) {
-            double samplePosition = pMark->getSamplePosition();
-            if (samplePosition != Cue::kNoPosition) {
-                // Create a stable key for sorting, because the WaveformMark's samplePosition is a
-                // ControlObject which can change at any time by other threads. Such a change causes
-                // another updateCues() call, rebuilding m_marksToRender.
-                auto key = WaveformMarkSortKey(samplePosition, pMark->getHotCue());
-                m_marksToRender.emplace(key, pMark);
-            }
-        }
-    }
+    m_marks.update();
 }
 
 // connecting the tracks cuesUpdated and onMarkChanged is not possible
@@ -453,9 +430,17 @@ void WOverview::receiveCuesUpdated() {
 void WOverview::mouseMoveEvent(QMouseEvent* e) {
     if (m_bLeftClickDragging) {
         if (m_orientation == Qt::Horizontal) {
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+            m_iPickupPos = math_clamp(e->position().x(), 0, width() - 1);
+#else
             m_iPickupPos = math_clamp(e->x(), 0, width() - 1);
+#endif
         } else {
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+            m_iPickupPos = math_clamp(e->position().y(), 0, height() - 1);
+#else
             m_iPickupPos = math_clamp(e->y(), 0, height() - 1);
+#endif
         }
     }
 
@@ -470,21 +455,7 @@ void WOverview::mouseMoveEvent(QMouseEvent* e) {
         return;
     }
 
-    m_pHoveredMark.clear();
-
-    // Non-hotcue marks (intro/outro cues, main cue, loop in/out) are sorted
-    // before hotcues in m_marksToRender so if there is a hotcue in the same
-    // location, the hotcue gets rendered on top. When right clicking, the
-    // the hotcue rendered on top must be assigned to m_pHoveredMark to show
-    // the CueMenuPopup. To accomplish this, m_marksToRender is iterated in
-    // reverse and the loop breaks as soon as m_pHoveredMark is set.
-    for (auto i = m_marksToRender.crbegin(); i != m_marksToRender.crend(); ++i) {
-        const WaveformMarkPointer& pMark = i->second;
-        if (pMark->contains(e->pos(), m_orientation)) {
-            m_pHoveredMark = pMark;
-            break;
-        }
-    }
+    m_pHoveredMark = m_marks.findHoveredMark(e->pos(), m_orientation);
 
     //qDebug() << "WOverview::mouseMoveEvent" << e->pos() << m_iPos;
     update();
@@ -492,6 +463,10 @@ void WOverview::mouseMoveEvent(QMouseEvent* e) {
 
 void WOverview::mouseReleaseEvent(QMouseEvent* e) {
     mouseMoveEvent(e);
+    if (m_bPassthroughEnabled) {
+        m_bLeftClickDragging = false;
+        return;
+    }
     //qDebug() << "WOverview::mouseReleaseEvent" << e->pos() << m_iPos << ">>" << dValue;
 
     if (e->button() == Qt::LeftButton) {
@@ -512,15 +487,27 @@ void WOverview::mouseReleaseEvent(QMouseEvent* e) {
 void WOverview::mousePressEvent(QMouseEvent* e) {
     //qDebug() << "WOverview::mousePressEvent" << e->pos();
     mouseMoveEvent(e);
+    if (m_bPassthroughEnabled) {
+        m_bLeftClickDragging = false;
+        return;
+    }
     double trackSamples = getTrackSamples();
     if (m_pCurrentTrack == nullptr || trackSamples <= 0) {
         return;
     }
     if (e->button() == Qt::LeftButton) {
         if (m_orientation == Qt::Horizontal) {
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+            m_iPickupPos = math_clamp(e->position().x(), 0, width() - 1);
+#else
             m_iPickupPos = math_clamp(e->x(), 0, width() - 1);
+#endif
         } else {
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+            m_iPickupPos = math_clamp(e->position().y(), 0, height() - 1);
+#else
             m_iPickupPos = math_clamp(e->y(), 0, height() - 1);
+#endif
         }
 
         if (m_pHoveredMark != nullptr) {
@@ -562,7 +549,11 @@ void WOverview::mousePressEvent(QMouseEvent* e) {
                     return;
                 } else {
                     m_pCueMenuPopup->setTrackAndCue(m_pCurrentTrack, pHoveredCue);
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+                    m_pCueMenuPopup->popup(e->globalPosition().toPoint());
+#else
                     m_pCueMenuPopup->popup(e->globalPos());
+#endif
                 }
             }
         }
@@ -830,9 +821,9 @@ void WOverview::drawMarks(QPainter* pPainter, const float offset, const float ga
 
     bool markHovered = false;
 
-    for (auto i = m_marksToRender.cbegin(); i != m_marksToRender.cend(); ++i) {
+    for (auto it = m_marks.cbegin(); it != m_marks.cend(); ++it) {
         PainterScope painterScope(pPainter);
-        const WaveformMarkPointer& pMark = i->second;
+        const WaveformMarkPointer& pMark = *it;
         double samplePosition = pMark->getSamplePosition();
         const float markPosition = math_clamp(
                 offset + static_cast<float>(samplePosition) * gain,
@@ -850,12 +841,33 @@ void WOverview::drawMarks(QPainter* pPainter, const float offset, const float ga
             bgLine.setLine(0.0, markPosition - 1.0, width(), markPosition - 1.0);
         }
 
+        QRectF rect;
+        double sampleEndPosition = pMark->getSampleEndPosition();
+        if (sampleEndPosition > 0) {
+            const float markEndPosition = math_clamp(
+                    offset + static_cast<float>(sampleEndPosition) * gain,
+                    0.0f,
+                    static_cast<float>(width()));
+
+            if (m_orientation == Qt::Horizontal) {
+                rect.setCoords(markPosition, 0, markEndPosition, height());
+            } else {
+                rect.setCoords(0, markPosition, width(), markEndPosition);
+            }
+        }
+
         pPainter->setPen(pMark->borderColor());
         pPainter->drawLine(bgLine);
 
         pPainter->setPen(pMark->fillColor());
         pPainter->drawLine(line);
 
+        if (rect.isValid()) {
+            QColor loopColor = pMark->fillColor();
+            loopColor.setAlphaF(0.5);
+            pPainter->fillRect(rect, loopColor);
+        }
+
         if (!pMark->m_text.isEmpty()) {
             Qt::Alignment halign = pMark->m_align & Qt::AlignHorizontal_Mask;
             Qt::Alignment valign = pMark->m_align & Qt::AlignVertical_Mask;
@@ -869,8 +881,8 @@ void WOverview::drawMarks(QPainter* pPainter, const float offset, const float ga
 
             if (pMark != m_pHoveredMark) {
                 float nextMarkPosition = -1.0f;
-                for (auto m = std::next(i); m != m_marksToRender.cend(); ++m) {
-                    const WaveformMarkPointer& otherMark = m->second;
+                for (auto m = std::next(it); m != m_marks.cend(); ++m) {
+                    const WaveformMarkPointer& otherMark = *m;
                     bool otherAtSameHeight = valign == (otherMark->m_align & Qt::AlignVertical_Mask);
                     // Hotcues always show at least their number.
                     bool otherHasLabel = !otherMark->m_text.isEmpty() || otherMark->getHotCue() != Cue::kNoHotCue;
@@ -936,7 +948,7 @@ void WOverview::drawMarks(QPainter* pPainter, const float offset, const float ga
                     m_labelTextColor,
                     m_labelBackgroundColor,
                     width(),
-                    getDevicePixelRatioF(this));
+                    devicePixelRatioF());
         }
 
         // Show cue position when hovered
@@ -984,7 +996,7 @@ void WOverview::drawMarks(QPainter* pPainter, const float offset, const float ga
                     m_labelTextColor,
                     m_labelBackgroundColor,
                     width(),
-                    getDevicePixelRatioF(this));
+                    devicePixelRatioF());
 
             QPointF timeDistancePoint(positionTextPoint.x(),
                     (fontMetrics.height() + height()) / 2);
@@ -996,7 +1008,7 @@ void WOverview::drawMarks(QPainter* pPainter, const float offset, const float ga
                     m_labelTextColor,
                     m_labelBackgroundColor,
                     width(),
-                    getDevicePixelRatioF(this));
+                    devicePixelRatioF());
             markHovered = true;
         }
     }
@@ -1091,7 +1103,7 @@ void WOverview::drawTimeRuler(QPainter* pPainter) {
                 m_labelTextColor,
                 m_labelBackgroundColor,
                 width(),
-                getDevicePixelRatioF(this));
+                devicePixelRatioF());
         m_timeRulerPositionLabel.draw(pPainter);
 
         QString timeDistanceText = mixxx::Duration::formatTime(fabs(timeDistance));
@@ -1107,7 +1119,7 @@ void WOverview::drawTimeRuler(QPainter* pPainter) {
                 m_labelTextColor,
                 m_labelBackgroundColor,
                 width(),
-                getDevicePixelRatioF(this));
+                devicePixelRatioF());
         m_timeRulerDistanceLabel.draw(pPainter);
     } else {
         m_timeRulerPositionLabel.clear();
@@ -1121,8 +1133,7 @@ void WOverview::drawMarkLabels(QPainter* pPainter, const float offset, const flo
     QFontMetricsF fontMetrics(markerFont);
 
     // Draw WaveformMark labels
-    for (const auto& pair : m_marksToRender) {
-        const WaveformMarkPointer& pMark = pair.second;
+    for (const auto& pMark : std::as_const(m_marks)) {
         if (m_pHoveredMark != nullptr && pMark != m_pHoveredMark) {
             if (pMark->m_label.intersects(m_pHoveredMark->m_label)) {
                 continue;
@@ -1180,7 +1191,7 @@ void WOverview::drawMarkLabels(QPainter* pPainter, const float offset, const flo
                     m_labelTextColor,
                     m_labelBackgroundColor,
                     width(),
-                    getDevicePixelRatioF(this));
+                    devicePixelRatioF());
 
             if (!(markRange.m_durationLabel.intersects(m_cuePositionLabel) || markRange.m_durationLabel.intersects(m_cueTimeDistanceLabel) || markRange.m_durationLabel.intersects(m_timeRulerPositionLabel) || markRange.m_durationLabel.intersects(m_timeRulerDistanceLabel))) {
                 markRange.m_durationLabel.draw(pPainter);
@@ -1205,17 +1216,7 @@ void WOverview::paintText(const QString& text, QPainter* pPainter) {
     QFont font = pPainter->font();
     QFontMetrics fm(font);
 
-    // TODO: The following use of QFontMetrics::width(const QString&, int) const
-    // is deprecated and should be replaced with
-    // QFontMetrics::horizontalAdvance(const QString&, int) const. However, the
-    // proposed alternative has just been introduced in Qt 5.11.
-    // Until the minimum required Qt version of Mixxx is increased, we need a
-    // version check here.
-    #if (QT_VERSION < QT_VERSION_CHECK(5, 11, 0))
-    int textWidth = fm.width(text);
-    #else
     int textWidth = fm.horizontalAdvance(text);
-    #endif
 
     if (textWidth > length()) {
         qreal pointSize = font.pointSizeF();
@@ -1243,8 +1244,8 @@ void WOverview::resizeEvent(QResizeEvent* pEvent) {
     Q_UNUSED(pEvent);
     // Play-position potmeters range from 0 to 1 but they allow out-of-range
     // sets. This is to give VC access to the pre-roll area.
-    const double kMaxPlayposRange = 1.0;
-    const double kMinPlayposRange = 0.0;
+    constexpr double kMaxPlayposRange = 1.0;
+    constexpr double kMinPlayposRange = 0.0;
 
     // Values of zero and one in normalized space.
     const double zero = (0.0 - kMinPlayposRange) / (kMaxPlayposRange - kMinPlayposRange);
@@ -1255,7 +1256,7 @@ void WOverview::resizeEvent(QResizeEvent* pEvent) {
     m_a = (length() - 1) / (one - zero);
     m_b = zero * m_a;
 
-    m_devicePixelRatio = getDevicePixelRatioF(this);
+    m_devicePixelRatio = devicePixelRatioF();
 
     m_waveformImageScaled = QImage();
     m_diffGain = 0;
