@@ -25,19 +25,31 @@ case "$1" in
 
         sudo apt-get update
 
-	  # If jackd2 is installed as per dpkg database, install libjack-jackd2-dev.
+        # If jackd2 is installed as per dpkg database, install libjack-jackd2-dev.
         # This avoids a package deadlock, resulting in jackd2 being removed, and jackd1 being installed,
         # to satisfy portaudio19-dev's need for a jackd dev package. In short, portaudio19-dev needs a
         # jackd dev library, so let's give it one..
         if [ "$(dpkg-query -W -f='${Status}' jackd2 2>/dev/null | grep -c "ok installed")" -eq 1 ];
         then
-         sudo apt-get install libjack-jackd2-dev;
+            sudo apt-get install libjack-jackd2-dev;
         fi
 
+        # Install a faster linker. Prefer mold, fall back to lld
+        if apt-cache show mold 2>%1 >/dev/null;
+        then
+            sudo apt-get install -y --no-install-recommends mold
+        else
+            if apt-cache show lld 2>%1 >/dev/null;
+            then
+                sudo apt-get install -y --no-install-recommends lld
+            fi
+        fi
 
         sudo apt-get install -y --no-install-recommends -- \
             ccache \
             cmake \
+            clazy \
+            clang-tidy \
             debhelper \
             devscripts \
             docbook-to-man \
@@ -45,18 +57,23 @@ case "$1" in
             fonts-open-sans \
             fonts-ubuntu \
             g++ \
+            lcov \
+            libbenchmark-dev \
             libchromaprint-dev \
             libdistro-info-perl \
             libebur128-dev \
             libfaad-dev \
             libfftw3-dev \
             libflac-dev \
+            libgmock-dev \
+            libgtest-dev \
             libhidapi-dev \
             libid3tag0-dev \
             liblilv-dev \
             libmad0-dev \
             libmodplug-dev \
             libmp3lame-dev \
+            libmsgsl-dev \
             libopus-dev \
             libopusfile-dev \
             libportmidi-dev \
@@ -72,13 +89,16 @@ case "$1" in
             libsqlite3-dev \
             libssl-dev \
             libtag1-dev \
+            libudev-dev \
             libupower-glib-dev \
             libusb-1.0-0-dev \
             libwavpack-dev \
+            lv2-dev \
             markdown \
             portaudio19-dev \
             protobuf-compiler \
             qt5keychain-dev \
+            qtdeclarative5-dev \
             qtscript5-dev \
             "${PACKAGES_EXTRA[@]}"
         ;;
